
/*  This file is part of the program psim.

    Copyright (C) 1994-1995, Andrew Cagney <cagney@highland.com.au>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
    --

    This file was generated by the program igen */

#ifndef _ICACHE_C_
#define _ICACHE_C_


#include "cpu.h"
#include "idecode.h"
#include "semantics.h"
#include "icache.h"
#ifdef HAVE_COMMON_FPU
#include "sim-inline.h"
#include "sim-fpu.h"
#endif
#include "support.h"


PSIM_INLINE_ICACHE\
(unsigned_word)
icache_illegal
(cpu *processor,
 instruction_word instruction,
 unsigned_word cia,
 idecode_cache *cache_entry)
{
  /* semantic routine */
#line 983 "./ppc-instructions"
  program_interrupt(processor, cia,
                    illegal_instruction_program_interrupt);
  error("Internal function must longjump\n");
  return 0;
#line 54 "icache.c"
}

PSIM_INLINE_ICACHE\
(unsigned_word)
icache_floating_point_unavailable
(cpu *processor,
 instruction_word instruction,
 unsigned_word cia,
 idecode_cache *cache_entry)
{
  /* semantic routine */
#line 991 "./ppc-instructions"
  floating_point_unavailable_interrupt(processor, cia);
  error("Internal function must longjump\n");
  return 0;
#line 70 "icache.c"
}


EXTERN_ICACHE\
(unsigned_word)
icache_instruction_call(cpu *processor,
                        instruction_word instruction,
                        unsigned_word cia,
                        idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_instruction_call
#undef MY_PREFIX
#define MY_PREFIX instruction_call
#line 1494 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode instruction_call\n"));
#line 87 "icache.c"

  /* validate: 0.0,6./,11./,16./,21./,31.1 */
  if (WITH_RESERVED_BITS && (instruction & 0xffffffff) != 0x1)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.0,6./,11./,16./,21./,31.1 #define get-values-from-icache|put-values-in-icache */
#line 96 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_instruction_call;

    /* semantic routine */
#line 1494 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics instruction_call\n"));
#line 106 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_instruction_call, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1494 "./ppc-instructions"
    {
      if (!os_emul_instruction_call(processor, cia, real_addr(cia, 1)))
        program_interrupt(processor, cia,
                          illegal_instruction_program_interrupt);
    }
#line 121 "icache.c"
    return nia;
    /* extraction: 0.0,6./,11./,16./,21./,31.1 #undef get-values-from-icache|put-values-in-icache */
#line 124 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Trap_Word_Immediate(cpu *processor,
                           instruction_word instruction,
                           unsigned_word cia,
                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Trap_Word_Immediate
#undef MY_PREFIX
#define MY_PREFIX Trap_Word_Immediate
#line 2898 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Trap Word Immediate\n"));
#line 143 "icache.c"

  /* validate: 0.3,6.TO,11.RA,16.SI */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0xc000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.3,6.TO,11.RA,16.SI #define get-values-from-icache|put-values-in-icache */
#line 2898 "./ppc-instructions"
    unsigned const TO UNUSED = cache_entry->crack.D.TO = EXTRACTED32(instruction, 6, 10); /* default in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 65 "./ppc-instructions"
    unsigned_word const EXTS_SI UNUSED = cache_entry->crack.D.EXTS_SI = ((signed_word)(signed16)instruction); /* in icache - extraction */
#line 163 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Trap_Word_Immediate;

    /* semantic routine */
#line 2898 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Trap Word Immediate\n"));
#line 173 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Trap_Word_Immediate, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2898 "./ppc-instructions"
    {
      signed_word a = EXTENDED(*rA);
      signed_word b = EXTS(SI);
      if ((a < b && TO_0_)
          || (a > b && TO_1_)
          || (a == b && TO_2_)
          || ((unsigned_word)a < (unsigned_word)b && TO_3_)
          || ((unsigned_word)a > (unsigned_word)b && TO_4_)
          )
        program_interrupt(processor, cia,
                          trap_program_interrupt);
    }
#line 195 "icache.c"
    return nia;
    /* extraction: 0.3,6.TO,11.RA,16.SI #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 199 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Multiply_Low_Immediate(cpu *processor,
                              instruction_word instruction,
                              unsigned_word cia,
                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Multiply_Low_Immediate
#undef MY_PREFIX
#define MY_PREFIX Multiply_Low_Immediate
#line 2688 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Multiply Low Immediate\n"));
#line 218 "icache.c"

  /* validate: 0.7,6.RT,11.RA,16.SI */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x1c000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.7,6.RT,11.RA,16.SI #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.D.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.D.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.D.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 65 "./ppc-instructions"
    unsigned_word const EXTS_SI UNUSED = cache_entry->crack.D.EXTS_SI = ((signed_word)(signed16)instruction); /* in icache - extraction */
#line 242 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Multiply_Low_Immediate;

    /* semantic routine */
#line 2688 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Multiply Low Immediate\n"));
#line 252 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Multiply_Low_Immediate, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2688 "./ppc-instructions"
    {
      signed_word prod = *rA * EXTS(SI);
      *rT = prod;
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK, 0/*Rc*/);
    }
#line 267 "icache.c"
    return nia;
    /* extraction: 0.7,6.RT,11.RA,16.SI #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 271 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Subtract_From_Immediate_Carrying(cpu *processor,
                                        instruction_word instruction,
                                        unsigned_word cia,
                                        idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Subtract_From_Immediate_Carrying
#undef MY_PREFIX
#define MY_PREFIX Subtract_From_Immediate_Carrying
#line 2575 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Subtract From Immediate Carrying\n"));
#line 290 "icache.c"

  /* validate: 0.8,6.RT,11.RA,16.SI */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x20000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.8,6.RT,11.RA,16.SI #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.D.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.D.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.D.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 65 "./ppc-instructions"
    unsigned_word const EXTS_SI UNUSED = cache_entry->crack.D.EXTS_SI = ((signed_word)(signed16)instruction); /* in icache - extraction */
#line 314 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Subtract_From_Immediate_Carrying;

    /* semantic routine */
#line 2575 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Subtract From Immediate Carrying\n"));
#line 324 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Subtract_From_Immediate_Carrying, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2575 "./ppc-instructions"
    {
      ALU_BEGIN(*rA);
      ALU_NOT;
      ALU_ADD(EXTS(SI));
      ALU_ADD(1);
      ALU_END(*rT, 1/*CA*/, 0/*OE*/, 0/*Rc*/);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK, 0/*Rc*/);
    }
#line 342 "icache.c"
    return nia;
    /* extraction: 0.8,6.RT,11.RA,16.SI #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 346 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Compare_Logical_Immediate(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Compare_Logical_Immediate
#undef MY_PREFIX
#define MY_PREFIX Compare_Logical_Immediate
#line 2834 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Compare Logical Immediate\n"));
#line 365 "icache.c"

  /* validate: 0.10,6.BF,9./,10.L,11.RA,16.UI */
  if (WITH_RESERVED_BITS && (instruction & 0xfc400000) != 0x28000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.10,6.BF,9./,10.L,11.RA,16.UI #define get-values-from-icache|put-values-in-icache */
#line 68 "./ppc-instructions"
    unsigned const BF UNUSED = cache_entry->crack.D.BF = EXTRACTED32(instruction, 6, 8); /* in icache - extraction */
#line 69 "./ppc-instructions"
    unsigned32 const BF_BITMASK UNUSED = cache_entry->crack.D.BF_BITMASK = (1 << BF); /* in icache - extraction */
#line 2834 "./ppc-instructions"
    unsigned const L UNUSED = cache_entry->crack.D.L = EXTRACTED32(instruction, 10, 10); /* default in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 2834 "./ppc-instructions"
    unsigned const UI UNUSED = cache_entry->crack.D.UI = EXTRACTED32(instruction, 16, 31); /* default in icache - extraction */
#line 389 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Compare_Logical_Immediate;

    /* semantic routine */
#line 2834 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Compare Logical Immediate\n"));
#line 399 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Compare_Logical_Immediate, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2834 "./ppc-instructions"
    {
      if (!is_64bit_mode && L)
        program_interrupt(processor, cia,
                          illegal_instruction_program_interrupt);
      else {
        unsigned_word a;
        unsigned_word b = UI;
        if (L == 0)
          a = MASKED(*rA, 32, 63);
        else
          a = *rA;
        CR_COMPARE(BF, a, b);
      }
      PPC_INSN_INT_CR(0, RA_BITMASK, BF_BITMASK);
    }
#line 424 "icache.c"
    return nia;
    /* extraction: 0.10,6.BF,9./,10.L,11.RA,16.UI #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 428 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Compare_Immediate(cpu *processor,
                         instruction_word instruction,
                         unsigned_word cia,
                         idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Compare_Immediate
#undef MY_PREFIX
#define MY_PREFIX Compare_Immediate
#line 2792 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Compare Immediate\n"));
#line 447 "icache.c"

  /* validate: 0.11,6.BF,9./,10.L,11.RA,16.SI */
  if (WITH_RESERVED_BITS && (instruction & 0xfc400000) != 0x2c000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.11,6.BF,9./,10.L,11.RA,16.SI #define get-values-from-icache|put-values-in-icache */
#line 68 "./ppc-instructions"
    unsigned const BF UNUSED = cache_entry->crack.D.BF = EXTRACTED32(instruction, 6, 8); /* in icache - extraction */
#line 69 "./ppc-instructions"
    unsigned32 const BF_BITMASK UNUSED = cache_entry->crack.D.BF_BITMASK = (1 << BF); /* in icache - extraction */
#line 2792 "./ppc-instructions"
    unsigned const L UNUSED = cache_entry->crack.D.L = EXTRACTED32(instruction, 10, 10); /* default in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 65 "./ppc-instructions"
    unsigned_word const EXTS_SI UNUSED = cache_entry->crack.D.EXTS_SI = ((signed_word)(signed16)instruction); /* in icache - extraction */
#line 471 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Compare_Immediate;

    /* semantic routine */
#line 2792 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Compare Immediate\n"));
#line 481 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Compare_Immediate, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2792 "./ppc-instructions"
    {
      if (!is_64bit_mode && L)
        program_interrupt(processor, cia,
                          illegal_instruction_program_interrupt);
      else {
        signed_word a;
        signed_word b = EXTS(SI);
        if (L == 0)
          a = EXTENDED(*rA);
        else
          a = *rA;
        CR_COMPARE(BF, a, b);
      }
      PPC_INSN_INT_CR(0, RA_BITMASK, BF_BITMASK);
    }
#line 506 "icache.c"
    return nia;
    /* extraction: 0.11,6.BF,9./,10.L,11.RA,16.SI #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 510 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Add_Immediate_Carrying(cpu *processor,
                              instruction_word instruction,
                              unsigned_word cia,
                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Add_Immediate_Carrying
#undef MY_PREFIX
#define MY_PREFIX Add_Immediate_Carrying
#line 2555 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Add Immediate Carrying\n"));
#line 529 "icache.c"

  /* validate: 0.12,6.RT,11.RA,16.SI */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x30000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.12,6.RT,11.RA,16.SI #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.D.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.D.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.D.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 65 "./ppc-instructions"
    unsigned_word const EXTS_SI UNUSED = cache_entry->crack.D.EXTS_SI = ((signed_word)(signed16)instruction); /* in icache - extraction */
#line 553 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Add_Immediate_Carrying;

    /* semantic routine */
#line 2555 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Add Immediate Carrying\n"));
#line 563 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Add_Immediate_Carrying, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2555 "./ppc-instructions"
    {
      ALU_BEGIN(*rA);
      ALU_ADD(EXTS(SI));
      ALU_END(*rT, 1/*CA*/, 0/*OE*/, 0/*Rc*/);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK, 0/*Rc*/);
    }
#line 579 "icache.c"
    return nia;
    /* extraction: 0.12,6.RT,11.RA,16.SI #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 583 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Add_Immediate_Carrying_and_Record(cpu *processor,
                                         instruction_word instruction,
                                         unsigned_word cia,
                                         idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Add_Immediate_Carrying_and_Record
#undef MY_PREFIX
#define MY_PREFIX Add_Immediate_Carrying_and_Record
#line 2565 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Add Immediate Carrying and Record\n"));
#line 602 "icache.c"

  /* validate: 0.13,6.RT,11.RA,16.SI */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x34000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.13,6.RT,11.RA,16.SI #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.D.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.D.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.D.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 65 "./ppc-instructions"
    unsigned_word const EXTS_SI UNUSED = cache_entry->crack.D.EXTS_SI = ((signed_word)(signed16)instruction); /* in icache - extraction */
#line 626 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Add_Immediate_Carrying_and_Record;

    /* semantic routine */
#line 2565 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Add Immediate Carrying and Record\n"));
#line 636 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Add_Immediate_Carrying_and_Record, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2565 "./ppc-instructions"
    {
      ALU_BEGIN(*rA);
      ALU_ADD(EXTS(SI));
      ALU_END(*rT, 1/*CA*/, 0/*OE*/, 1/*Rc*/);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK, 1/*Rc*/);
    }
#line 652 "icache.c"
    return nia;
    /* extraction: 0.13,6.RT,11.RA,16.SI #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 656 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Add_Immediate_RA0(cpu *processor,
                         instruction_word instruction,
                         unsigned_word cia,
                         idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Add_Immediate
#undef MY_PREFIX
#define MY_PREFIX Add_Immediate_RA0
#line 2513 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Add Immediate\n"));
#line 675 "icache.c"

  /* validate: 0.14,6.RT,11.RA,16.SI */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x38000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.14,6.RT,11.RA,16.SI #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.D.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.D.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.D.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = 0; /* in icache - constant field */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 1 /* RA == 0 */ /* semantic compute - constant compare */
#line 65 "./ppc-instructions"
    unsigned_word const EXTS_SI UNUSED = cache_entry->crack.D.EXTS_SI = ((signed_word)(signed16)instruction); /* in icache - extraction */
#line 699 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Add_Immediate_RA0;

    /* semantic routine */
#line 2513 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Add Immediate\n"));
#line 709 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Add_Immediate, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2513 "./ppc-instructions"
    {
      if (RA_is_0)	*rT = EXTS(SI);
      else		*rT = *rA + EXTS(SI);
      ITRACE(trace_alu, (" Result = %ld (0x%lx)\n", (long)*rT, (long)*rT));
      PPC_INSN_INT(RT_BITMASK, (RA_BITMASK & ~1), 0);
    }
#line 725 "icache.c"
    return nia;
    /* extraction: 0.14,6.RT,11.RA,16.SI #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 729 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Add_Immediate_RA(cpu *processor,
                        instruction_word instruction,
                        unsigned_word cia,
                        idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Add_Immediate
#undef MY_PREFIX
#define MY_PREFIX Add_Immediate_RA
#line 2513 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Add Immediate\n"));
#line 748 "icache.c"

  /* validate: 0.14,6.RT,11.RA,16.SI */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x38000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.14,6.RT,11.RA,16.SI #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.D.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.D.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.D.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 0 /* RA == 0 */ /* semantic compute - constant compare */
#line 65 "./ppc-instructions"
    unsigned_word const EXTS_SI UNUSED = cache_entry->crack.D.EXTS_SI = ((signed_word)(signed16)instruction); /* in icache - extraction */
#line 772 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Add_Immediate_RA;

    /* semantic routine */
#line 2513 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Add Immediate\n"));
#line 782 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Add_Immediate, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2513 "./ppc-instructions"
    {
      if (RA_is_0)	*rT = EXTS(SI);
      else		*rT = *rA + EXTS(SI);
      ITRACE(trace_alu, (" Result = %ld (0x%lx)\n", (long)*rT, (long)*rT));
      PPC_INSN_INT(RT_BITMASK, (RA_BITMASK & ~1), 0);
    }
#line 798 "icache.c"
    return nia;
    /* extraction: 0.14,6.RT,11.RA,16.SI #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 802 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Add_Immediate_Shifted_RA0(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Add_Immediate_Shifted
#undef MY_PREFIX
#define MY_PREFIX Add_Immediate_Shifted_RA0
#line 2523 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Add Immediate Shifted\n"));
#line 821 "icache.c"

  /* validate: 0.15,6.RT,11.RA,16.SI */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x3c000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.15,6.RT,11.RA,16.SI #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.D.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.D.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.D.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = 0; /* in icache - constant field */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 1 /* RA == 0 */ /* semantic compute - constant compare */
#line 65 "./ppc-instructions"
    unsigned_word const EXTS_SI UNUSED = cache_entry->crack.D.EXTS_SI = ((signed_word)(signed16)instruction); /* in icache - extraction */
#line 845 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Add_Immediate_Shifted_RA0;

    /* semantic routine */
#line 2523 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Add Immediate Shifted\n"));
#line 855 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Add_Immediate_Shifted, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2523 "./ppc-instructions"
    {
      if (RA_is_0)	*rT = EXTS(SI) << 16;
      else		*rT = *rA + (EXTS(SI) << 16);
      ITRACE(trace_alu, (" Result = %ld (0x%lx)\n", (long)*rT, (long)*rT));
      PPC_INSN_INT(RT_BITMASK, (RA_BITMASK & ~1), 0);
    }
#line 871 "icache.c"
    return nia;
    /* extraction: 0.15,6.RT,11.RA,16.SI #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 875 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Add_Immediate_Shifted_RA(cpu *processor,
                                instruction_word instruction,
                                unsigned_word cia,
                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Add_Immediate_Shifted
#undef MY_PREFIX
#define MY_PREFIX Add_Immediate_Shifted_RA
#line 2523 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Add Immediate Shifted\n"));
#line 894 "icache.c"

  /* validate: 0.15,6.RT,11.RA,16.SI */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x3c000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.15,6.RT,11.RA,16.SI #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.D.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.D.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.D.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 0 /* RA == 0 */ /* semantic compute - constant compare */
#line 65 "./ppc-instructions"
    unsigned_word const EXTS_SI UNUSED = cache_entry->crack.D.EXTS_SI = ((signed_word)(signed16)instruction); /* in icache - extraction */
#line 918 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Add_Immediate_Shifted_RA;

    /* semantic routine */
#line 2523 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Add Immediate Shifted\n"));
#line 928 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Add_Immediate_Shifted, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2523 "./ppc-instructions"
    {
      if (RA_is_0)	*rT = EXTS(SI) << 16;
      else		*rT = *rA + (EXTS(SI) << 16);
      ITRACE(trace_alu, (" Result = %ld (0x%lx)\n", (long)*rT, (long)*rT));
      PPC_INSN_INT(RT_BITMASK, (RA_BITMASK & ~1), 0);
    }
#line 944 "icache.c"
    return nia;
    /* extraction: 0.15,6.RT,11.RA,16.SI #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 948 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK0_BO0(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK0_BO0
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 967 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 0; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 987 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK0_BO0;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 997 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 1046 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 1050 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK0_BO1(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK0_BO1
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 1069 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 1; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 1089 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK0_BO1;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 1099 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 1148 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 1152 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK0_BO2(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK0_BO2
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 1171 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 2; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 1191 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK0_BO2;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 1201 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 1250 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 1254 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK0_BO3(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK0_BO3
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 1273 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 3; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 1293 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK0_BO3;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 1303 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 1352 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 1356 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK0_BO4(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK0_BO4
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 1375 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 4; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 1395 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK0_BO4;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 1405 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 1454 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 1458 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK0_BO5(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK0_BO5
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 1477 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 5; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 1497 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK0_BO5;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 1507 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 1556 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 1560 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK0_BO6(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK0_BO6
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 1579 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 6; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 1599 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK0_BO6;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 1609 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 1658 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 1662 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK0_BO7(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK0_BO7
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 1681 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 7; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 1701 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK0_BO7;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 1711 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 1760 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 1764 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK0_BO8(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK0_BO8
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 1783 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 8; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 1803 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK0_BO8;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 1813 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 1862 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 1866 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK0_BO9(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK0_BO9
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 1885 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 9; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 1905 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK0_BO9;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 1915 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 1964 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 1968 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK0_BO10(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK0_BO10
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 1987 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 10; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 2007 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK0_BO10;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 2017 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 2066 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 2070 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK0_BO11(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK0_BO11
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 2089 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 11; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 2109 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK0_BO11;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 2119 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 2168 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 2172 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK0_BO12(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK0_BO12
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 2191 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 12; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 2211 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK0_BO12;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 2221 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 2270 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 2274 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK0_BO13(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK0_BO13
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 2293 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 13; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 2313 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK0_BO13;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 2323 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 2372 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 2376 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK0_BO14(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK0_BO14
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 2395 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 14; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 2415 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK0_BO14;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 2425 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 2474 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 2478 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK0_BO15(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK0_BO15
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 2497 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 15; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 2517 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK0_BO15;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 2527 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 2576 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 2580 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK0_BO16(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK0_BO16
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 2599 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 16; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 2619 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK0_BO16;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 2629 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 2678 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 2682 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK0_BO17(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK0_BO17
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 2701 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 17; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 2721 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK0_BO17;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 2731 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 2780 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 2784 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK0_BO18(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK0_BO18
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 2803 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 18; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 2823 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK0_BO18;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 2833 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 2882 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 2886 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK0_BO19(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK0_BO19
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 2905 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 19; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 2925 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK0_BO19;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 2935 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 2984 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 2988 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK0_BO20(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK0_BO20
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 3007 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 20; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 3027 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK0_BO20;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 3037 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 3086 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 3090 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK0_BO21(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK0_BO21
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 3109 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 21; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 3129 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK0_BO21;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 3139 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 3188 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 3192 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK0_BO22(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK0_BO22
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 3211 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 22; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 3231 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK0_BO22;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 3241 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 3290 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 3294 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK0_BO23(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK0_BO23
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 3313 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 23; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 3333 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK0_BO23;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 3343 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 3392 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 3396 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK0_BO24(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK0_BO24
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 3415 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 24; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 3435 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK0_BO24;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 3445 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 3494 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 3498 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK0_BO25(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK0_BO25
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 3517 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 25; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 3537 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK0_BO25;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 3547 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 3596 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 3600 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK0_BO26(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK0_BO26
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 3619 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 26; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 3639 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK0_BO26;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 3649 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 3698 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 3702 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK0_BO27(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK0_BO27
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 3721 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 27; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 3741 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK0_BO27;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 3751 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 3800 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 3804 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK0_BO28(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK0_BO28
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 3823 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 28; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 3843 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK0_BO28;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 3853 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 3902 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 3906 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK0_BO29(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK0_BO29
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 3925 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 29; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 3945 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK0_BO29;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 3955 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 4004 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 4008 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK0_BO30(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK0_BO30
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 4027 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 30; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 4047 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK0_BO30;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 4057 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 4106 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 4110 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK0_BO31(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK0_BO31
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 4129 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 31; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 4149 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK0_BO31;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 4159 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 4208 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 4212 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK1_BO0(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK1_BO0
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 4231 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 0; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 4251 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK1_BO0;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 4261 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 4310 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 4314 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK1_BO1(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK1_BO1
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 4333 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 1; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 4353 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK1_BO1;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 4363 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 4412 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 4416 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK1_BO2(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK1_BO2
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 4435 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 2; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 4455 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK1_BO2;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 4465 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 4514 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 4518 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK1_BO3(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK1_BO3
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 4537 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 3; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 4557 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK1_BO3;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 4567 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 4616 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 4620 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK1_BO4(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK1_BO4
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 4639 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 4; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 4659 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK1_BO4;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 4669 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 4718 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 4722 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK1_BO5(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK1_BO5
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 4741 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 5; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 4761 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK1_BO5;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 4771 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 4820 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 4824 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK1_BO6(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK1_BO6
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 4843 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 6; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 4863 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK1_BO6;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 4873 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 4922 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 4926 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK1_BO7(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK1_BO7
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 4945 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 7; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 4965 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK1_BO7;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 4975 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 5024 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 5028 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK1_BO8(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK1_BO8
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 5047 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 8; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 5067 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK1_BO8;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 5077 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 5126 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 5130 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK1_BO9(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK1_BO9
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 5149 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 9; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 5169 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK1_BO9;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 5179 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 5228 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 5232 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK1_BO10(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK1_BO10
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 5251 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 10; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 5271 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK1_BO10;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 5281 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 5330 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 5334 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK1_BO11(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK1_BO11
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 5353 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 11; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 5373 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK1_BO11;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 5383 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 5432 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 5436 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK1_BO12(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK1_BO12
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 5455 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 12; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 5475 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK1_BO12;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 5485 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 5534 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 5538 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK1_BO13(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK1_BO13
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 5557 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 13; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 5577 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK1_BO13;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 5587 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 5636 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 5640 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK1_BO14(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK1_BO14
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 5659 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 14; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 5679 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK1_BO14;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 5689 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 5738 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 5742 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK1_BO15(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK1_BO15
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 5761 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 15; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 5781 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK1_BO15;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 5791 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 5840 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 5844 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK1_BO16(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK1_BO16
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 5863 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 16; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 5883 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK1_BO16;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 5893 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 5942 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 5946 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK1_BO17(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK1_BO17
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 5965 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 17; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 5985 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK1_BO17;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 5995 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 6044 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 6048 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK1_BO18(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK1_BO18
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 6067 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 18; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 6087 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK1_BO18;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 6097 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 6146 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 6150 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK1_BO19(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK1_BO19
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 6169 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 19; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 6189 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK1_BO19;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 6199 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 6248 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 6252 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK1_BO20(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK1_BO20
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 6271 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 20; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 6291 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK1_BO20;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 6301 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 6350 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 6354 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK1_BO21(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK1_BO21
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 6373 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 21; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 6393 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK1_BO21;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 6403 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 6452 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 6456 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK1_BO22(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK1_BO22
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 6475 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 22; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 6495 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK1_BO22;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 6505 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 6554 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 6558 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK1_BO23(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK1_BO23
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 6577 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 23; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 6597 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK1_BO23;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 6607 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 6656 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 6660 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK1_BO24(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK1_BO24
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 6679 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 24; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 6699 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK1_BO24;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 6709 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 6758 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 6762 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK1_BO25(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK1_BO25
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 6781 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 25; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 6801 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK1_BO25;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 6811 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 6860 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 6864 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK1_BO26(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK1_BO26
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 6883 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 26; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 6903 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK1_BO26;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 6913 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 6962 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 6966 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK1_BO27(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK1_BO27
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 6985 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 27; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 7005 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK1_BO27;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 7015 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 7064 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 7068 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK1_BO28(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK1_BO28
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 7087 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 28; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 7107 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK1_BO28;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 7117 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 7166 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 7170 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK1_BO29(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK1_BO29
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 7189 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 29; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 7209 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK1_BO29;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 7219 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 7268 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 7272 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK1_BO30(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK1_BO30
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 7291 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 30; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 7311 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK1_BO30;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 7321 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 7370 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 7374 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK1_BO31(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK1_BO31
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 7393 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 31; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 7413 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK1_BO31;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 7423 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 7472 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 7476 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK0_BO0(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK0_BO0
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 7495 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 0; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 7515 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK0_BO0;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 7525 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 7574 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 7578 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK0_BO1(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK0_BO1
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 7597 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 1; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 7617 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK0_BO1;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 7627 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 7676 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 7680 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK0_BO2(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK0_BO2
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 7699 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 2; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 7719 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK0_BO2;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 7729 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 7778 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 7782 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK0_BO3(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK0_BO3
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 7801 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 3; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 7821 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK0_BO3;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 7831 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 7880 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 7884 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK0_BO4(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK0_BO4
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 7903 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 4; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 7923 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK0_BO4;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 7933 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 7982 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 7986 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK0_BO5(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK0_BO5
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 8005 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 5; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 8025 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK0_BO5;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 8035 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 8084 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 8088 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK0_BO6(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK0_BO6
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 8107 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 6; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 8127 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK0_BO6;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 8137 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 8186 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 8190 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK0_BO7(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK0_BO7
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 8209 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 7; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 8229 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK0_BO7;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 8239 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 8288 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 8292 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK0_BO8(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK0_BO8
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 8311 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 8; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 8331 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK0_BO8;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 8341 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 8390 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 8394 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK0_BO9(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK0_BO9
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 8413 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 9; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 8433 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK0_BO9;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 8443 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 8492 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 8496 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK0_BO10(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK0_BO10
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 8515 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 10; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 8535 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK0_BO10;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 8545 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 8594 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 8598 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK0_BO11(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK0_BO11
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 8617 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 11; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 8637 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK0_BO11;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 8647 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 8696 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 8700 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK0_BO12(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK0_BO12
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 8719 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 12; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 8739 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK0_BO12;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 8749 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 8798 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 8802 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK0_BO13(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK0_BO13
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 8821 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 13; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 8841 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK0_BO13;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 8851 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 8900 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 8904 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK0_BO14(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK0_BO14
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 8923 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 14; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 8943 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK0_BO14;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 8953 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 9002 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 9006 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK0_BO15(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK0_BO15
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 9025 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 15; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 9045 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK0_BO15;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 9055 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 9104 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 9108 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK0_BO16(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK0_BO16
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 9127 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 16; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 9147 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK0_BO16;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 9157 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 9206 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 9210 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK0_BO17(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK0_BO17
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 9229 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 17; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 9249 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK0_BO17;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 9259 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 9308 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 9312 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK0_BO18(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK0_BO18
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 9331 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 18; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 9351 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK0_BO18;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 9361 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 9410 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 9414 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK0_BO19(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK0_BO19
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 9433 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 19; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 9453 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK0_BO19;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 9463 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 9512 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 9516 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK0_BO20(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK0_BO20
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 9535 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 20; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 9555 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK0_BO20;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 9565 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 9614 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 9618 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK0_BO21(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK0_BO21
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 9637 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 21; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 9657 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK0_BO21;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 9667 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 9716 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 9720 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK0_BO22(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK0_BO22
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 9739 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 22; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 9759 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK0_BO22;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 9769 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 9818 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 9822 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK0_BO23(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK0_BO23
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 9841 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 23; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 9861 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK0_BO23;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 9871 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 9920 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 9924 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK0_BO24(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK0_BO24
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 9943 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 24; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 9963 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK0_BO24;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 9973 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 10022 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 10026 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK0_BO25(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK0_BO25
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 10045 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 25; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 10065 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK0_BO25;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 10075 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 10124 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 10128 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK0_BO26(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK0_BO26
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 10147 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 26; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 10167 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK0_BO26;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 10177 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 10226 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 10230 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK0_BO27(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK0_BO27
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 10249 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 27; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 10269 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK0_BO27;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 10279 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 10328 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 10332 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK0_BO28(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK0_BO28
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 10351 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 28; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 10371 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK0_BO28;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 10381 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 10430 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 10434 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK0_BO29(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK0_BO29
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 10453 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 29; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 10473 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK0_BO29;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 10483 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 10532 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 10536 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK0_BO30(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK0_BO30
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 10555 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 30; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 10575 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK0_BO30;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 10585 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 10634 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 10638 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK0_BO31(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK0_BO31
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 10657 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 31; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 10677 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK0_BO31;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 10687 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 10736 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 10740 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK1_BO0(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK1_BO0
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 10759 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 0; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 10779 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK1_BO0;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 10789 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 10838 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 10842 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK1_BO1(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK1_BO1
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 10861 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 1; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 10881 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK1_BO1;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 10891 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 10940 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 10944 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK1_BO2(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK1_BO2
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 10963 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 2; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 10983 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK1_BO2;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 10993 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 11042 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 11046 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK1_BO3(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK1_BO3
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 11065 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 3; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 11085 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK1_BO3;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 11095 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 11144 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 11148 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK1_BO4(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK1_BO4
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 11167 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 4; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 11187 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK1_BO4;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 11197 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 11246 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 11250 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK1_BO5(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK1_BO5
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 11269 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 5; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 11289 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK1_BO5;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 11299 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 11348 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 11352 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK1_BO6(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK1_BO6
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 11371 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 6; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 11391 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK1_BO6;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 11401 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 11450 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 11454 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK1_BO7(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK1_BO7
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 11473 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 7; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 11493 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK1_BO7;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 11503 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 11552 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 11556 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK1_BO8(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK1_BO8
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 11575 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 8; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 11595 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK1_BO8;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 11605 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 11654 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 11658 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK1_BO9(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK1_BO9
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 11677 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 9; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 11697 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK1_BO9;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 11707 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 11756 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 11760 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK1_BO10(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK1_BO10
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 11779 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 10; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 11799 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK1_BO10;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 11809 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 11858 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 11862 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK1_BO11(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK1_BO11
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 11881 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 11; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 11901 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK1_BO11;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 11911 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 11960 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 11964 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK1_BO12(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK1_BO12
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 11983 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 12; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 12003 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK1_BO12;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 12013 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 12062 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 12066 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK1_BO13(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK1_BO13
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 12085 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 13; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 12105 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK1_BO13;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 12115 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 12164 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 12168 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK1_BO14(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK1_BO14
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 12187 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 14; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 12207 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK1_BO14;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 12217 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 12266 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 12270 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK1_BO15(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK1_BO15
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 12289 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 15; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 12309 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK1_BO15;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 12319 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 12368 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 12372 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK1_BO16(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK1_BO16
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 12391 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 16; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 12411 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK1_BO16;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 12421 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 12470 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 12474 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK1_BO17(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK1_BO17
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 12493 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 17; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 12513 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK1_BO17;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 12523 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 12572 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 12576 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK1_BO18(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK1_BO18
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 12595 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 18; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 12615 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK1_BO18;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 12625 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 12674 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 12678 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK1_BO19(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK1_BO19
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 12697 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 19; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 12717 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK1_BO19;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 12727 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 12776 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 12780 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK1_BO20(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK1_BO20
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 12799 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 20; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 12819 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK1_BO20;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 12829 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 12878 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 12882 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK1_BO21(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK1_BO21
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 12901 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 21; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 12921 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK1_BO21;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 12931 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 12980 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 12984 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK1_BO22(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK1_BO22
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 13003 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 22; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 13023 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK1_BO22;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 13033 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 13082 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 13086 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK1_BO23(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK1_BO23
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 13105 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 23; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 13125 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK1_BO23;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 13135 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 13184 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 13188 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK1_BO24(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK1_BO24
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 13207 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 24; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 13227 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK1_BO24;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 13237 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 13286 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 13290 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK1_BO25(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK1_BO25
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 13309 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 25; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 13329 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK1_BO25;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 13339 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 13388 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 13392 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK1_BO26(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK1_BO26
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 13411 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 26; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 13431 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK1_BO26;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 13441 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 13490 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 13494 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK1_BO27(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK1_BO27
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 13513 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 27; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 13533 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK1_BO27;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 13543 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 13592 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 13596 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK1_BO28(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK1_BO28
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 13615 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 28; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 13635 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK1_BO28;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 13645 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 13694 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 13698 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK1_BO29(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK1_BO29
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 13717 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 29; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 13737 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK1_BO29;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 13747 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 13796 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 13800 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK1_BO30(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK1_BO30
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 13819 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 30; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 13839 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK1_BO30;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 13849 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 13898 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 13902 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK1_BO31(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK1_BO31
#line 1519 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 13921 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1519 "./ppc-instructions"
    unsigned const BO UNUSED = 31; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 78 "./ppc-instructions"
    unsigned_word const EXTS_BD_0b00 UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(signed16)instruction) & ~3); /* in icache - extraction */
#line 1519 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1519 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 13941 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK1_BO31;

    /* semantic routine */
#line 1519 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 13951 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1519 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 14000 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 14004 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_System_Call(cpu *processor,
                   instruction_word instruction,
                   unsigned_word cia,
                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_System_Call
#undef MY_PREFIX
#define MY_PREFIX System_Call
#line 1631 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode System Call\n"));
#line 14023 "icache.c"

  /* validate: 0.17,6./,11./,16./,30.1,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xffffffff) != 0x44000002)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.17,6./,11./,16./,30.1,31./ #define get-values-from-icache|put-values-in-icache */
#line 14032 "icache.c"

#define insn cache_entry->crack.SC.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_System_Call;

    /* semantic routine */
#line 1631 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics System Call\n"));
#line 14042 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_System_Call, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1631 "./ppc-instructions"
    {
      if (CURRENT_MODEL_ISSUE > 0)
        model_serialize(MY_INDEX, cpu_model(processor));
      system_call_interrupt(processor, cia);
    }
#line 14057 "icache.c"
    return nia;
    /* extraction: 0.17,6./,11./,16./,30.1,31./ #undef get-values-from-icache|put-values-in-icache */
#line 14060 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_AA0_LK0(cpu *processor,
                      instruction_word instruction,
                      unsigned_word cia,
                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch
#undef MY_PREFIX
#define MY_PREFIX Branch_AA0_LK0
#line 1506 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch\n"));
#line 14079 "icache.c"

  /* validate: 0.18,6.LI,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x48000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.18,6.LI,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 79 "./ppc-instructions"
    unsigned_word const EXTS_LI_0b00 UNUSED = cache_entry->crack.I.EXTS_LI_0b00 = ((((signed_word)(signed32)(instruction << 6)) >> 6) & ~0x3); /* in icache - extraction */
#line 1506 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1506 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 14094 "icache.c"

#define insn cache_entry->crack.I.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_AA0_LK0;

    /* semantic routine */
#line 1506 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch\n"));
#line 14104 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1506 "./ppc-instructions"
    {
      /* option_mpc860c0:
      No problem here because this branch is predicted taken (unconditional). */
      if (AA) NIA = IEA(EXTS(LI_0b00));
      else    NIA = IEA(CIA + EXTS(LI_0b00));
      if (LK) LR = (spreg)CIA+4;
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), 1, -1);
    }
#line 14123 "icache.c"
    return nia;
    /* extraction: 0.18,6.LI,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#line 14126 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_AA0_LK1(cpu *processor,
                      instruction_word instruction,
                      unsigned_word cia,
                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch
#undef MY_PREFIX
#define MY_PREFIX Branch_AA0_LK1
#line 1506 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch\n"));
#line 14145 "icache.c"

  /* validate: 0.18,6.LI,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x48000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.18,6.LI,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 79 "./ppc-instructions"
    unsigned_word const EXTS_LI_0b00 UNUSED = cache_entry->crack.I.EXTS_LI_0b00 = ((((signed_word)(signed32)(instruction << 6)) >> 6) & ~0x3); /* in icache - extraction */
#line 1506 "./ppc-instructions"
    unsigned const AA UNUSED = 0; /* default in icache - constant field */
#line 1506 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 14160 "icache.c"

#define insn cache_entry->crack.I.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_AA0_LK1;

    /* semantic routine */
#line 1506 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch\n"));
#line 14170 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1506 "./ppc-instructions"
    {
      /* option_mpc860c0:
      No problem here because this branch is predicted taken (unconditional). */
      if (AA) NIA = IEA(EXTS(LI_0b00));
      else    NIA = IEA(CIA + EXTS(LI_0b00));
      if (LK) LR = (spreg)CIA+4;
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), 1, -1);
    }
#line 14189 "icache.c"
    return nia;
    /* extraction: 0.18,6.LI,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#line 14192 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_AA1_LK0(cpu *processor,
                      instruction_word instruction,
                      unsigned_word cia,
                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch
#undef MY_PREFIX
#define MY_PREFIX Branch_AA1_LK0
#line 1506 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch\n"));
#line 14211 "icache.c"

  /* validate: 0.18,6.LI,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x48000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.18,6.LI,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 79 "./ppc-instructions"
    unsigned_word const EXTS_LI_0b00 UNUSED = cache_entry->crack.I.EXTS_LI_0b00 = ((((signed_word)(signed32)(instruction << 6)) >> 6) & ~0x3); /* in icache - extraction */
#line 1506 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1506 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 14226 "icache.c"

#define insn cache_entry->crack.I.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_AA1_LK0;

    /* semantic routine */
#line 1506 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch\n"));
#line 14236 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1506 "./ppc-instructions"
    {
      /* option_mpc860c0:
      No problem here because this branch is predicted taken (unconditional). */
      if (AA) NIA = IEA(EXTS(LI_0b00));
      else    NIA = IEA(CIA + EXTS(LI_0b00));
      if (LK) LR = (spreg)CIA+4;
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), 1, -1);
    }
#line 14255 "icache.c"
    return nia;
    /* extraction: 0.18,6.LI,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#line 14258 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_AA1_LK1(cpu *processor,
                      instruction_word instruction,
                      unsigned_word cia,
                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch
#undef MY_PREFIX
#define MY_PREFIX Branch_AA1_LK1
#line 1506 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch\n"));
#line 14277 "icache.c"

  /* validate: 0.18,6.LI,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x48000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.18,6.LI,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 79 "./ppc-instructions"
    unsigned_word const EXTS_LI_0b00 UNUSED = cache_entry->crack.I.EXTS_LI_0b00 = ((((signed_word)(signed32)(instruction << 6)) >> 6) & ~0x3); /* in icache - extraction */
#line 1506 "./ppc-instructions"
    unsigned const AA UNUSED = 1; /* default in icache - constant field */
#line 1506 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 14292 "icache.c"

#define insn cache_entry->crack.I.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_AA1_LK1;

    /* semantic routine */
#line 1506 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch\n"));
#line 14302 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1506 "./ppc-instructions"
    {
      /* option_mpc860c0:
      No problem here because this branch is predicted taken (unconditional). */
      if (AA) NIA = IEA(EXTS(LI_0b00));
      else    NIA = IEA(CIA + EXTS(LI_0b00));
      if (LK) LR = (spreg)CIA+4;
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), 1, -1);
    }
#line 14321 "icache.c"
    return nia;
    /* extraction: 0.18,6.LI,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#line 14324 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Move_Condition_Register_Field(cpu *processor,
                                     instruction_word instruction,
                                     unsigned_word cia,
                                     idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Move_Condition_Register_Field
#undef MY_PREFIX
#define MY_PREFIX Move_Condition_Register_Field
#line 1710 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Move Condition Register Field\n"));
#line 14343 "icache.c"

  /* validate: 0.19,6.BF,9./,11.BFA,14./,16./,21.0,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc63ffff) != 0x4c000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.19,6.BF,9./,11.BFA,14./,16./,21.0,31./ #define get-values-from-icache|put-values-in-icache */
#line 68 "./ppc-instructions"
    unsigned const BF UNUSED = cache_entry->crack.XL.BF = EXTRACTED32(instruction, 6, 8); /* in icache - extraction */
#line 69 "./ppc-instructions"
    unsigned32 const BF_BITMASK UNUSED = cache_entry->crack.XL.BF_BITMASK = (1 << BF); /* in icache - extraction */
#line 1710 "./ppc-instructions"
    unsigned const BFA UNUSED = cache_entry->crack.XL.BFA = EXTRACTED32(instruction, 11, 13); /* default in icache - extraction */
#line 14358 "icache.c"

#define insn cache_entry->crack.XL.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Move_Condition_Register_Field;

    /* semantic routine */
#line 1710 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Move Condition Register Field\n"));
#line 14368 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Move_Condition_Register_Field, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1710 "./ppc-instructions"
    {
      MBLIT32(CR, 4*BF, 4*BF+3, EXTRACTED32(CR, 4*BFA, 4*BFA+3));
      PPC_INSN_CR(BF_BITMASK, 1 << BFA);
    }
#line 14382 "icache.c"
    return nia;
    /* extraction: 0.19,6.BF,9./,11.BFA,14./,16./,21.0,31./ #undef get-values-from-icache|put-values-in-icache */
#line 14385 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_to_Link_Register_LK0(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional_to_Link_Register
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_to_Link_Register_LK0
#line 1562 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional to Link Register\n"));
#line 14404 "icache.c"

  /* validate: 0.19,6.BO,11.BI,16./,21.16,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00fffe) != 0x4c000020)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.19,6.BO,11.BI,16./,21.16,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1562 "./ppc-instructions"
    unsigned const BO UNUSED = cache_entry->crack.XL.BO = EXTRACTED32(instruction, 6, 10); /* default in icache - extraction */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.XL.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 1562 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 14420 "icache.c"

#define insn cache_entry->crack.XL.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_to_Link_Register_LK0;

    /* semantic routine */
#line 1562 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional to Link Register\n"));
#line 14430 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional_to_Link_Register, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1562 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != BO_3_);
      cond_ok = BO_0_ || (CR_BI_ == BO_1_);
      if (ctr_ok && cond_ok) {
        NIA = IEA(LR_0b00);
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as not-taken.
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0) {
        model_branches(cpu_model(processor), succeed, BO);
        if (! BO_0_)
          model_branch_predict(cpu_model(processor), BO_4_ ? !succeed : succeed);
      }
    }
#line 14471 "icache.c"
    return nia;
    /* extraction: 0.19,6.BO,11.BI,16./,21.16,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 14475 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_to_Link_Register_LK1(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional_to_Link_Register
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_to_Link_Register_LK1
#line 1562 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional to Link Register\n"));
#line 14494 "icache.c"

  /* validate: 0.19,6.BO,11.BI,16./,21.16,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00fffe) != 0x4c000020)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.19,6.BO,11.BI,16./,21.16,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1562 "./ppc-instructions"
    unsigned const BO UNUSED = cache_entry->crack.XL.BO = EXTRACTED32(instruction, 6, 10); /* default in icache - extraction */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.XL.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 1562 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 14510 "icache.c"

#define insn cache_entry->crack.XL.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_to_Link_Register_LK1;

    /* semantic routine */
#line 1562 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional to Link Register\n"));
#line 14520 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional_to_Link_Register, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1562 "./ppc-instructions"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != BO_3_);
      cond_ok = BO_0_ || (CR_BI_ == BO_1_);
      if (ctr_ok && cond_ok) {
        NIA = IEA(LR_0b00);
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as not-taken.
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0) {
        model_branches(cpu_model(processor), succeed, BO);
        if (! BO_0_)
          model_branch_predict(cpu_model(processor), BO_4_ ? !succeed : succeed);
      }
    }
#line 14561 "icache.c"
    return nia;
    /* extraction: 0.19,6.BO,11.BI,16./,21.16,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 14565 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Condition_Register_NOR(cpu *processor,
                              instruction_word instruction,
                              unsigned_word cia,
                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Condition_Register_NOR
#undef MY_PREFIX
#define MY_PREFIX Condition_Register_NOR
#line 1675 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Condition Register NOR\n"));
#line 14584 "icache.c"

  /* validate: 0.19,6.BT,11.BA,16.BB,21.33,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x4c000042)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.19,6.BT,11.BA,16.BB,21.33,31./ #define get-values-from-icache|put-values-in-icache */
#line 76 "./ppc-instructions"
    unsigned const BT UNUSED = cache_entry->crack.XL.BT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned32 const BT_BITMASK UNUSED = cache_entry->crack.XL.BT_BITMASK = (1 << BT); /* in icache - extraction */
#define BA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 71 "./ppc-instructions"
    unsigned const BIT32_BA UNUSED = cache_entry->crack.XL.BIT32_BA = BIT32(BA); /* in icache - extraction */
#line 72 "./ppc-instructions"
    unsigned32 const BA_BITMASK UNUSED = cache_entry->crack.XL.BA_BITMASK = (1 << BA); /* in icache - extraction */
#define BB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 74 "./ppc-instructions"
    unsigned const BIT32_BB UNUSED = cache_entry->crack.XL.BIT32_BB = BIT32(BB); /* in icache - extraction */
#line 75 "./ppc-instructions"
    unsigned32 const BB_BITMASK UNUSED = cache_entry->crack.XL.BB_BITMASK = (1 << BB); /* in icache - extraction */
#line 14607 "icache.c"

#define insn cache_entry->crack.XL.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Condition_Register_NOR;

    /* semantic routine */
#line 1675 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Condition Register NOR\n"));
#line 14617 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Condition_Register_NOR, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1675 "./ppc-instructions"
    {
      BLIT32(CR, BT, !(CR_BA_ || CR_BB_));
      PPC_INSN_CR(BT_BITMASK, BA_BITMASK | BB_BITMASK);
    }
#line 14631 "icache.c"
    return nia;
    /* extraction: 0.19,6.BT,11.BA,16.BB,21.33,31./ #undef get-values-from-icache|put-values-in-icache */
#undef BA
#undef BB
#line 14636 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Return_From_Interrupt(cpu *processor,
                             instruction_word instruction,
                             unsigned_word cia,
                             idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Return_From_Interrupt
#undef MY_PREFIX
#define MY_PREFIX Return_From_Interrupt
#line 4847 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Return From Interrupt\n"));
#line 14655 "icache.c"

  /* validate: 0.19,6./,11./,16./,21.50,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xffffffff) != 0x4c000064)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.19,6./,11./,16./,21.50,31./ #define get-values-from-icache|put-values-in-icache */
#line 14664 "icache.c"

#define insn cache_entry->crack.XL.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Return_From_Interrupt;

    /* semantic routine */
#line 4847 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Return From Interrupt\n"));
#line 14674 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Return_From_Interrupt, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4847 "./ppc-instructions"
    {
      if (IS_PROBLEM_STATE(processor)) {
        program_interrupt(processor, cia,
                          privileged_instruction_program_interrupt);
      }
      else {
        MSR = (MASKED(SRR1, 0, 32)
               | MASKED(SRR1, 37, 41)
               | MASKED(SRR1, 48, 63));
        NIA = MASKED(SRR0, 0, 61);
        cpu_synchronize_context(processor, cia);
        check_masked_interrupts(processor);
      }
    }
#line 14698 "icache.c"
    return nia;
    /* extraction: 0.19,6./,11./,16./,21.50,31./ #undef get-values-from-icache|put-values-in-icache */
#line 14701 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Condition_Register_AND_with_Complement(cpu *processor,
                                              instruction_word instruction,
                                              unsigned_word cia,
                                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Condition_Register_AND_with_Complement
#undef MY_PREFIX
#define MY_PREFIX Condition_Register_AND_with_Complement
#line 1691 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Condition Register AND with Complement\n"));
#line 14720 "icache.c"

  /* validate: 0.19,6.BT,11.BA,16.BB,21.129,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x4c000102)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.19,6.BT,11.BA,16.BB,21.129,31./ #define get-values-from-icache|put-values-in-icache */
#line 76 "./ppc-instructions"
    unsigned const BT UNUSED = cache_entry->crack.XL.BT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned32 const BT_BITMASK UNUSED = cache_entry->crack.XL.BT_BITMASK = (1 << BT); /* in icache - extraction */
#define BA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 71 "./ppc-instructions"
    unsigned const BIT32_BA UNUSED = cache_entry->crack.XL.BIT32_BA = BIT32(BA); /* in icache - extraction */
#line 72 "./ppc-instructions"
    unsigned32 const BA_BITMASK UNUSED = cache_entry->crack.XL.BA_BITMASK = (1 << BA); /* in icache - extraction */
#define BB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 74 "./ppc-instructions"
    unsigned const BIT32_BB UNUSED = cache_entry->crack.XL.BIT32_BB = BIT32(BB); /* in icache - extraction */
#line 75 "./ppc-instructions"
    unsigned32 const BB_BITMASK UNUSED = cache_entry->crack.XL.BB_BITMASK = (1 << BB); /* in icache - extraction */
#line 14743 "icache.c"

#define insn cache_entry->crack.XL.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Condition_Register_AND_with_Complement;

    /* semantic routine */
#line 1691 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Condition Register AND with Complement\n"));
#line 14753 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Condition_Register_AND_with_Complement, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1691 "./ppc-instructions"
    {
      BLIT32(CR, BT, CR_BA_ && !CR_BB_);
      PPC_INSN_CR(BT_BITMASK, BA_BITMASK | BB_BITMASK);
    }
#line 14767 "icache.c"
    return nia;
    /* extraction: 0.19,6.BT,11.BA,16.BB,21.129,31./ #undef get-values-from-icache|put-values-in-icache */
#undef BA
#undef BB
#line 14772 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Instruction_Synchronize(cpu *processor,
                               instruction_word instruction,
                               unsigned_word cia,
                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Instruction_Synchronize
#undef MY_PREFIX
#define MY_PREFIX Instruction_Synchronize
#line 4760 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Instruction Synchronize\n"));
#line 14791 "icache.c"

  /* validate: 0.19,6./,11./,16./,21.150,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xffffffff) != 0x4c00012c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.19,6./,11./,16./,21.150,31./ #define get-values-from-icache|put-values-in-icache */
#line 14800 "icache.c"

#define insn cache_entry->crack.XL.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Instruction_Synchronize;

    /* semantic routine */
#line 4760 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Instruction Synchronize\n"));
#line 14810 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Instruction_Synchronize, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4760 "./ppc-instructions"
    {
      cpu_synchronize_context(processor, cia);
      PPC_INSN_INT(0, 0, 0);
    }
#line 14824 "icache.c"
    return nia;
    /* extraction: 0.19,6./,11./,16./,21.150,31./ #undef get-values-from-icache|put-values-in-icache */
#line 14827 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Condition_Register_XOR(cpu *processor,
                              instruction_word instruction,
                              unsigned_word cia,
                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Condition_Register_XOR
#undef MY_PREFIX
#define MY_PREFIX Condition_Register_XOR
#line 1659 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Condition Register XOR\n"));
#line 14846 "icache.c"

  /* validate: 0.19,6.BT,11.BA,16.BB,21.193,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x4c000182)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.19,6.BT,11.BA,16.BB,21.193,31./ #define get-values-from-icache|put-values-in-icache */
#line 76 "./ppc-instructions"
    unsigned const BT UNUSED = cache_entry->crack.XL.BT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned32 const BT_BITMASK UNUSED = cache_entry->crack.XL.BT_BITMASK = (1 << BT); /* in icache - extraction */
#define BA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 71 "./ppc-instructions"
    unsigned const BIT32_BA UNUSED = cache_entry->crack.XL.BIT32_BA = BIT32(BA); /* in icache - extraction */
#line 72 "./ppc-instructions"
    unsigned32 const BA_BITMASK UNUSED = cache_entry->crack.XL.BA_BITMASK = (1 << BA); /* in icache - extraction */
#define BB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 74 "./ppc-instructions"
    unsigned const BIT32_BB UNUSED = cache_entry->crack.XL.BIT32_BB = BIT32(BB); /* in icache - extraction */
#line 75 "./ppc-instructions"
    unsigned32 const BB_BITMASK UNUSED = cache_entry->crack.XL.BB_BITMASK = (1 << BB); /* in icache - extraction */
#line 14869 "icache.c"

#define insn cache_entry->crack.XL.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Condition_Register_XOR;

    /* semantic routine */
#line 1659 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Condition Register XOR\n"));
#line 14879 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Condition_Register_XOR, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1659 "./ppc-instructions"
    {
      BLIT32(CR, BT, CR_BA_ != CR_BB_);
      PPC_INSN_CR(BT_BITMASK, BA_BITMASK | BB_BITMASK);
    }
#line 14893 "icache.c"
    return nia;
    /* extraction: 0.19,6.BT,11.BA,16.BB,21.193,31./ #undef get-values-from-icache|put-values-in-icache */
#undef BA
#undef BB
#line 14898 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Condition_Register_NAND(cpu *processor,
                               instruction_word instruction,
                               unsigned_word cia,
                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Condition_Register_NAND
#undef MY_PREFIX
#define MY_PREFIX Condition_Register_NAND
#line 1667 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Condition Register NAND\n"));
#line 14917 "icache.c"

  /* validate: 0.19,6.BT,11.BA,16.BB,21.225,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x4c0001c2)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.19,6.BT,11.BA,16.BB,21.225,31./ #define get-values-from-icache|put-values-in-icache */
#line 76 "./ppc-instructions"
    unsigned const BT UNUSED = cache_entry->crack.XL.BT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned32 const BT_BITMASK UNUSED = cache_entry->crack.XL.BT_BITMASK = (1 << BT); /* in icache - extraction */
#define BA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 71 "./ppc-instructions"
    unsigned const BIT32_BA UNUSED = cache_entry->crack.XL.BIT32_BA = BIT32(BA); /* in icache - extraction */
#line 72 "./ppc-instructions"
    unsigned32 const BA_BITMASK UNUSED = cache_entry->crack.XL.BA_BITMASK = (1 << BA); /* in icache - extraction */
#define BB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 74 "./ppc-instructions"
    unsigned const BIT32_BB UNUSED = cache_entry->crack.XL.BIT32_BB = BIT32(BB); /* in icache - extraction */
#line 75 "./ppc-instructions"
    unsigned32 const BB_BITMASK UNUSED = cache_entry->crack.XL.BB_BITMASK = (1 << BB); /* in icache - extraction */
#line 14940 "icache.c"

#define insn cache_entry->crack.XL.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Condition_Register_NAND;

    /* semantic routine */
#line 1667 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Condition Register NAND\n"));
#line 14950 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Condition_Register_NAND, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1667 "./ppc-instructions"
    {
      BLIT32(CR, BT, !(CR_BA_ && CR_BB_));
      PPC_INSN_CR(BT_BITMASK, BA_BITMASK | BB_BITMASK);
    }
#line 14964 "icache.c"
    return nia;
    /* extraction: 0.19,6.BT,11.BA,16.BB,21.225,31./ #undef get-values-from-icache|put-values-in-icache */
#undef BA
#undef BB
#line 14969 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Condition_Register_AND(cpu *processor,
                              instruction_word instruction,
                              unsigned_word cia,
                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Condition_Register_AND
#undef MY_PREFIX
#define MY_PREFIX Condition_Register_AND
#line 1643 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Condition Register AND\n"));
#line 14988 "icache.c"

  /* validate: 0.19,6.BT,11.BA,16.BB,21.257,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x4c000202)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.19,6.BT,11.BA,16.BB,21.257,31./ #define get-values-from-icache|put-values-in-icache */
#line 76 "./ppc-instructions"
    unsigned const BT UNUSED = cache_entry->crack.XL.BT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned32 const BT_BITMASK UNUSED = cache_entry->crack.XL.BT_BITMASK = (1 << BT); /* in icache - extraction */
#define BA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 71 "./ppc-instructions"
    unsigned const BIT32_BA UNUSED = cache_entry->crack.XL.BIT32_BA = BIT32(BA); /* in icache - extraction */
#line 72 "./ppc-instructions"
    unsigned32 const BA_BITMASK UNUSED = cache_entry->crack.XL.BA_BITMASK = (1 << BA); /* in icache - extraction */
#define BB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 74 "./ppc-instructions"
    unsigned const BIT32_BB UNUSED = cache_entry->crack.XL.BIT32_BB = BIT32(BB); /* in icache - extraction */
#line 75 "./ppc-instructions"
    unsigned32 const BB_BITMASK UNUSED = cache_entry->crack.XL.BB_BITMASK = (1 << BB); /* in icache - extraction */
#line 15011 "icache.c"

#define insn cache_entry->crack.XL.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Condition_Register_AND;

    /* semantic routine */
#line 1643 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Condition Register AND\n"));
#line 15021 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Condition_Register_AND, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1643 "./ppc-instructions"
    {
      BLIT32(CR, BT, CR_BA_ && CR_BB_);
      PPC_INSN_CR(BT_BITMASK, BA_BITMASK | BB_BITMASK);
    }
#line 15035 "icache.c"
    return nia;
    /* extraction: 0.19,6.BT,11.BA,16.BB,21.257,31./ #undef get-values-from-icache|put-values-in-icache */
#undef BA
#undef BB
#line 15040 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Condition_Register_Equivalent(cpu *processor,
                                     instruction_word instruction,
                                     unsigned_word cia,
                                     idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Condition_Register_Equivalent
#undef MY_PREFIX
#define MY_PREFIX Condition_Register_Equivalent
#line 1683 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Condition Register Equivalent\n"));
#line 15059 "icache.c"

  /* validate: 0.19,6.BT,11.BA,16.BB,21.289,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x4c000242)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.19,6.BT,11.BA,16.BB,21.289,31./ #define get-values-from-icache|put-values-in-icache */
#line 76 "./ppc-instructions"
    unsigned const BT UNUSED = cache_entry->crack.XL.BT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned32 const BT_BITMASK UNUSED = cache_entry->crack.XL.BT_BITMASK = (1 << BT); /* in icache - extraction */
#define BA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 71 "./ppc-instructions"
    unsigned const BIT32_BA UNUSED = cache_entry->crack.XL.BIT32_BA = BIT32(BA); /* in icache - extraction */
#line 72 "./ppc-instructions"
    unsigned32 const BA_BITMASK UNUSED = cache_entry->crack.XL.BA_BITMASK = (1 << BA); /* in icache - extraction */
#define BB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 74 "./ppc-instructions"
    unsigned const BIT32_BB UNUSED = cache_entry->crack.XL.BIT32_BB = BIT32(BB); /* in icache - extraction */
#line 75 "./ppc-instructions"
    unsigned32 const BB_BITMASK UNUSED = cache_entry->crack.XL.BB_BITMASK = (1 << BB); /* in icache - extraction */
#line 15082 "icache.c"

#define insn cache_entry->crack.XL.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Condition_Register_Equivalent;

    /* semantic routine */
#line 1683 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Condition Register Equivalent\n"));
#line 15092 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Condition_Register_Equivalent, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1683 "./ppc-instructions"
    {
      BLIT32(CR, BT, CR_BA_ == CR_BB_);
      PPC_INSN_CR(BT_BITMASK, BA_BITMASK | BB_BITMASK);
    }
#line 15106 "icache.c"
    return nia;
    /* extraction: 0.19,6.BT,11.BA,16.BB,21.289,31./ #undef get-values-from-icache|put-values-in-icache */
#undef BA
#undef BB
#line 15111 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Condition_Register_OR_with_Complement(cpu *processor,
                                             instruction_word instruction,
                                             unsigned_word cia,
                                             idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Condition_Register_OR_with_Complement
#undef MY_PREFIX
#define MY_PREFIX Condition_Register_OR_with_Complement
#line 1699 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Condition Register OR with Complement\n"));
#line 15130 "icache.c"

  /* validate: 0.19,6.BT,11.BA,16.BB,21.417,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x4c000342)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.19,6.BT,11.BA,16.BB,21.417,31./ #define get-values-from-icache|put-values-in-icache */
#line 76 "./ppc-instructions"
    unsigned const BT UNUSED = cache_entry->crack.XL.BT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned32 const BT_BITMASK UNUSED = cache_entry->crack.XL.BT_BITMASK = (1 << BT); /* in icache - extraction */
#define BA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 71 "./ppc-instructions"
    unsigned const BIT32_BA UNUSED = cache_entry->crack.XL.BIT32_BA = BIT32(BA); /* in icache - extraction */
#line 72 "./ppc-instructions"
    unsigned32 const BA_BITMASK UNUSED = cache_entry->crack.XL.BA_BITMASK = (1 << BA); /* in icache - extraction */
#define BB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 74 "./ppc-instructions"
    unsigned const BIT32_BB UNUSED = cache_entry->crack.XL.BIT32_BB = BIT32(BB); /* in icache - extraction */
#line 75 "./ppc-instructions"
    unsigned32 const BB_BITMASK UNUSED = cache_entry->crack.XL.BB_BITMASK = (1 << BB); /* in icache - extraction */
#line 15153 "icache.c"

#define insn cache_entry->crack.XL.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Condition_Register_OR_with_Complement;

    /* semantic routine */
#line 1699 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Condition Register OR with Complement\n"));
#line 15163 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Condition_Register_OR_with_Complement, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1699 "./ppc-instructions"
    {
      BLIT32(CR, BT, CR_BA_ || !CR_BB_);
      PPC_INSN_CR(BT_BITMASK, BA_BITMASK | BB_BITMASK);
    }
#line 15177 "icache.c"
    return nia;
    /* extraction: 0.19,6.BT,11.BA,16.BB,21.417,31./ #undef get-values-from-icache|put-values-in-icache */
#undef BA
#undef BB
#line 15182 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Condition_Register_OR(cpu *processor,
                             instruction_word instruction,
                             unsigned_word cia,
                             idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Condition_Register_OR
#undef MY_PREFIX
#define MY_PREFIX Condition_Register_OR
#line 1651 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Condition Register OR\n"));
#line 15201 "icache.c"

  /* validate: 0.19,6.BT,11.BA,16.BB,21.449,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x4c000382)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.19,6.BT,11.BA,16.BB,21.449,31./ #define get-values-from-icache|put-values-in-icache */
#line 76 "./ppc-instructions"
    unsigned const BT UNUSED = cache_entry->crack.XL.BT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned32 const BT_BITMASK UNUSED = cache_entry->crack.XL.BT_BITMASK = (1 << BT); /* in icache - extraction */
#define BA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 71 "./ppc-instructions"
    unsigned const BIT32_BA UNUSED = cache_entry->crack.XL.BIT32_BA = BIT32(BA); /* in icache - extraction */
#line 72 "./ppc-instructions"
    unsigned32 const BA_BITMASK UNUSED = cache_entry->crack.XL.BA_BITMASK = (1 << BA); /* in icache - extraction */
#define BB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 74 "./ppc-instructions"
    unsigned const BIT32_BB UNUSED = cache_entry->crack.XL.BIT32_BB = BIT32(BB); /* in icache - extraction */
#line 75 "./ppc-instructions"
    unsigned32 const BB_BITMASK UNUSED = cache_entry->crack.XL.BB_BITMASK = (1 << BB); /* in icache - extraction */
#line 15224 "icache.c"

#define insn cache_entry->crack.XL.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Condition_Register_OR;

    /* semantic routine */
#line 1651 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Condition Register OR\n"));
#line 15234 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Condition_Register_OR, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1651 "./ppc-instructions"
    {
      BLIT32(CR, BT, CR_BA_ || CR_BB_);
      PPC_INSN_CR(BT_BITMASK, BA_BITMASK | BB_BITMASK);
    }
#line 15248 "icache.c"
    return nia;
    /* extraction: 0.19,6.BT,11.BA,16.BB,21.449,31./ #undef get-values-from-icache|put-values-in-icache */
#undef BA
#undef BB
#line 15253 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_to_Count_Register_LK0(cpu *processor,
                                                instruction_word instruction,
                                                unsigned_word cia,
                                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional_to_Count_Register
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_to_Count_Register_LK0
#line 1597 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional to Count Register\n"));
#line 15272 "icache.c"

  /* validate: 0.19,6.BO,11.BI,16./,21.528,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00fffe) != 0x4c000420)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.19,6.BO,11.BI,16./,21.528,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1597 "./ppc-instructions"
    unsigned const BO UNUSED = cache_entry->crack.XL.BO = EXTRACTED32(instruction, 6, 10); /* default in icache - extraction */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.XL.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 1597 "./ppc-instructions"
    unsigned const LK UNUSED = 0; /* default in icache - constant field */
#line 15288 "icache.c"

#define insn cache_entry->crack.XL.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_to_Count_Register_LK0;

    /* semantic routine */
#line 1597 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional to Count Register\n"));
#line 15298 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional_to_Count_Register, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1597 "./ppc-instructions"
    {
      int cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      cond_ok = BO_0_ || (CR_BI_ == BO_1_);
      if (cond_ok) {
        NIA = IEA(CTR_0b00);
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as not-taken.
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0) {
        model_branches(cpu_model(processor), succeed, BO);
        if (! BO_0_)
          model_branch_predict(cpu_model(processor), BO_4_ ? !succeed : succeed);
      }
    }
#line 15335 "icache.c"
    return nia;
    /* extraction: 0.19,6.BO,11.BI,16./,21.528,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 15339 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_to_Count_Register_LK1(cpu *processor,
                                                instruction_word instruction,
                                                unsigned_word cia,
                                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional_to_Count_Register
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_to_Count_Register_LK1
#line 1597 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Branch Conditional to Count Register\n"));
#line 15358 "icache.c"

  /* validate: 0.19,6.BO,11.BI,16./,21.528,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00fffe) != 0x4c000420)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.19,6.BO,11.BI,16./,21.528,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1597 "./ppc-instructions"
    unsigned const BO UNUSED = cache_entry->crack.XL.BO = EXTRACTED32(instruction, 6, 10); /* default in icache - extraction */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 67 "./ppc-instructions"
    unsigned const BIT32_BI UNUSED = cache_entry->crack.XL.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 1597 "./ppc-instructions"
    unsigned const LK UNUSED = 1; /* default in icache - constant field */
#line 15374 "icache.c"

#define insn cache_entry->crack.XL.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_to_Count_Register_LK1;

    /* semantic routine */
#line 1597 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Branch Conditional to Count Register\n"));
#line 15384 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional_to_Count_Register, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1597 "./ppc-instructions"
    {
      int cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      cond_ok = BO_0_ || (CR_BI_ == BO_1_);
      if (cond_ok) {
        NIA = IEA(CTR_0b00);
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as not-taken.
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (PAGE_SIZE - (CIA & (PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0) {
        model_branches(cpu_model(processor), succeed, BO);
        if (! BO_0_)
          model_branch_predict(cpu_model(processor), BO_4_ ? !succeed : succeed);
      }
    }
#line 15421 "icache.c"
    return nia;
    /* extraction: 0.19,6.BO,11.BI,16./,21.528,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 15425 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Rotate_Left_Word_Immediate_then_Mask_Insert_Rc0(cpu *processor,
                                                       instruction_word instruction,
                                                       unsigned_word cia,
                                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Rotate_Left_Word_Immediate_then_Mask_Insert
#undef MY_PREFIX
#define MY_PREFIX Rotate_Left_Word_Immediate_then_Mask_Insert_Rc0
#line 3232 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Rotate Left Word Immediate then Mask Insert\n"));
#line 15444 "icache.c"

  /* validate: 0.20,6.RS,11.RA,16.SH,21.MB,26.ME,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x50000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.20,6.RS,11.RA,16.SH,21.MB,26.ME,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 44 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.M.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 45 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.M.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 46 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.M.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.M.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.M.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.M.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 3232 "./ppc-instructions"
    unsigned const SH UNUSED = cache_entry->crack.M.SH = EXTRACTED32(instruction, 16, 20); /* default in icache - extraction */
#line 3232 "./ppc-instructions"
    unsigned const MB UNUSED = cache_entry->crack.M.MB = EXTRACTED32(instruction, 21, 25); /* default in icache - extraction */
#line 3232 "./ppc-instructions"
    unsigned const ME UNUSED = cache_entry->crack.M.ME = EXTRACTED32(instruction, 26, 30); /* default in icache - extraction */
#line 3232 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 15474 "icache.c"

#define insn cache_entry->crack.M.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Rotate_Left_Word_Immediate_then_Mask_Insert_Rc0;

    /* semantic routine */
#line 3232 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Rotate Left Word Immediate then Mask Insert\n"));
#line 15484 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Rotate_Left_Word_Immediate_then_Mask_Insert, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3232 "./ppc-instructions"
    {
      long n = SH;
      unsigned32 r = ROTL32(*rS, n);
      unsigned32 m = MASK(MB+32, ME+32);
      signed_word result = (r & m) | (*rA & ~m);
      *rA = result;
      ITRACE(trace_alu, (": n=%ld *rS=0x%lx r=0x%lx m=0x%lx result=0x%lx\n",
                         n, (unsigned long)*rS, (unsigned long)r, (unsigned long)m,
      		   (unsigned long)result));
      CR0_COMPARE(result, 0, Rc);
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK, Rc);
    }
#line 15506 "icache.c"
    return nia;
    /* extraction: 0.20,6.RS,11.RA,16.SH,21.MB,26.ME,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 15510 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Rotate_Left_Word_Immediate_then_Mask_Insert_Rc1(cpu *processor,
                                                       instruction_word instruction,
                                                       unsigned_word cia,
                                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Rotate_Left_Word_Immediate_then_Mask_Insert
#undef MY_PREFIX
#define MY_PREFIX Rotate_Left_Word_Immediate_then_Mask_Insert_Rc1
#line 3232 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Rotate Left Word Immediate then Mask Insert\n"));
#line 15529 "icache.c"

  /* validate: 0.20,6.RS,11.RA,16.SH,21.MB,26.ME,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x50000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.20,6.RS,11.RA,16.SH,21.MB,26.ME,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 44 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.M.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 45 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.M.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 46 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.M.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.M.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.M.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.M.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 3232 "./ppc-instructions"
    unsigned const SH UNUSED = cache_entry->crack.M.SH = EXTRACTED32(instruction, 16, 20); /* default in icache - extraction */
#line 3232 "./ppc-instructions"
    unsigned const MB UNUSED = cache_entry->crack.M.MB = EXTRACTED32(instruction, 21, 25); /* default in icache - extraction */
#line 3232 "./ppc-instructions"
    unsigned const ME UNUSED = cache_entry->crack.M.ME = EXTRACTED32(instruction, 26, 30); /* default in icache - extraction */
#line 3232 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 15559 "icache.c"

#define insn cache_entry->crack.M.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Rotate_Left_Word_Immediate_then_Mask_Insert_Rc1;

    /* semantic routine */
#line 3232 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Rotate Left Word Immediate then Mask Insert\n"));
#line 15569 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Rotate_Left_Word_Immediate_then_Mask_Insert, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3232 "./ppc-instructions"
    {
      long n = SH;
      unsigned32 r = ROTL32(*rS, n);
      unsigned32 m = MASK(MB+32, ME+32);
      signed_word result = (r & m) | (*rA & ~m);
      *rA = result;
      ITRACE(trace_alu, (": n=%ld *rS=0x%lx r=0x%lx m=0x%lx result=0x%lx\n",
                         n, (unsigned long)*rS, (unsigned long)r, (unsigned long)m,
      		   (unsigned long)result));
      CR0_COMPARE(result, 0, Rc);
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK, Rc);
    }
#line 15591 "icache.c"
    return nia;
    /* extraction: 0.20,6.RS,11.RA,16.SH,21.MB,26.ME,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 15595 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Rotate_Left_Word_Immediate_then_AND_with_Mask_Rc0(cpu *processor,
                                                         instruction_word instruction,
                                                         unsigned_word cia,
                                                         idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Rotate_Left_Word_Immediate_then_AND_with_Mask
#undef MY_PREFIX
#define MY_PREFIX Rotate_Left_Word_Immediate_then_AND_with_Mask_Rc0
#line 3179 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Rotate Left Word Immediate then AND with Mask\n"));
#line 15614 "icache.c"

  /* validate: 0.21,6.RS,11.RA,16.SH,21.MB,26.ME,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x54000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.21,6.RS,11.RA,16.SH,21.MB,26.ME,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 44 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.M.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 45 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.M.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 46 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.M.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.M.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.M.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.M.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 3179 "./ppc-instructions"
    unsigned const SH UNUSED = cache_entry->crack.M.SH = EXTRACTED32(instruction, 16, 20); /* default in icache - extraction */
#line 3179 "./ppc-instructions"
    unsigned const MB UNUSED = cache_entry->crack.M.MB = EXTRACTED32(instruction, 21, 25); /* default in icache - extraction */
#line 3179 "./ppc-instructions"
    unsigned const ME UNUSED = cache_entry->crack.M.ME = EXTRACTED32(instruction, 26, 30); /* default in icache - extraction */
#line 3179 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 15644 "icache.c"

#define insn cache_entry->crack.M.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Rotate_Left_Word_Immediate_then_AND_with_Mask_Rc0;

    /* semantic routine */
#line 3179 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Rotate Left Word Immediate then AND with Mask\n"));
#line 15654 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Rotate_Left_Word_Immediate_then_AND_with_Mask, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3179 "./ppc-instructions"
    {
      long n = SH;
      unsigned32 s = *rS;
      unsigned32 r = ROTL32(s, n);
      unsigned32 m = MASK(MB+32, ME+32);
      signed_word result = r & m;
      *rA = result;
      CR0_COMPARE(result, 0, Rc);
      ITRACE(trace_alu,
             ("n=%ld, s=0x%lx, r=0x%lx, m=0x%lx, result=0x%lx, cr=0x%lx\n",
              n, (unsigned long)s, (unsigned long)r, (unsigned long)m,
      	(unsigned long)result, (unsigned long)CR));
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK, Rc);
    }
#line 15678 "icache.c"
    return nia;
    /* extraction: 0.21,6.RS,11.RA,16.SH,21.MB,26.ME,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 15682 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Rotate_Left_Word_Immediate_then_AND_with_Mask_Rc1(cpu *processor,
                                                         instruction_word instruction,
                                                         unsigned_word cia,
                                                         idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Rotate_Left_Word_Immediate_then_AND_with_Mask
#undef MY_PREFIX
#define MY_PREFIX Rotate_Left_Word_Immediate_then_AND_with_Mask_Rc1
#line 3179 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Rotate Left Word Immediate then AND with Mask\n"));
#line 15701 "icache.c"

  /* validate: 0.21,6.RS,11.RA,16.SH,21.MB,26.ME,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x54000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.21,6.RS,11.RA,16.SH,21.MB,26.ME,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 44 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.M.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 45 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.M.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 46 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.M.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.M.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.M.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.M.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 3179 "./ppc-instructions"
    unsigned const SH UNUSED = cache_entry->crack.M.SH = EXTRACTED32(instruction, 16, 20); /* default in icache - extraction */
#line 3179 "./ppc-instructions"
    unsigned const MB UNUSED = cache_entry->crack.M.MB = EXTRACTED32(instruction, 21, 25); /* default in icache - extraction */
#line 3179 "./ppc-instructions"
    unsigned const ME UNUSED = cache_entry->crack.M.ME = EXTRACTED32(instruction, 26, 30); /* default in icache - extraction */
#line 3179 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 15731 "icache.c"

#define insn cache_entry->crack.M.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Rotate_Left_Word_Immediate_then_AND_with_Mask_Rc1;

    /* semantic routine */
#line 3179 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Rotate Left Word Immediate then AND with Mask\n"));
#line 15741 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Rotate_Left_Word_Immediate_then_AND_with_Mask, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3179 "./ppc-instructions"
    {
      long n = SH;
      unsigned32 s = *rS;
      unsigned32 r = ROTL32(s, n);
      unsigned32 m = MASK(MB+32, ME+32);
      signed_word result = r & m;
      *rA = result;
      CR0_COMPARE(result, 0, Rc);
      ITRACE(trace_alu,
             ("n=%ld, s=0x%lx, r=0x%lx, m=0x%lx, result=0x%lx, cr=0x%lx\n",
              n, (unsigned long)s, (unsigned long)r, (unsigned long)m,
      	(unsigned long)result, (unsigned long)CR));
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK, Rc);
    }
#line 15765 "icache.c"
    return nia;
    /* extraction: 0.21,6.RS,11.RA,16.SH,21.MB,26.ME,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 15769 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Rotate_Left_Word_then_AND_with_Mask_Rc0(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Rotate_Left_Word_then_AND_with_Mask
#undef MY_PREFIX
#define MY_PREFIX Rotate_Left_Word_then_AND_with_Mask_Rc0
#line 3211 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Rotate Left Word then AND with Mask\n"));
#line 15788 "icache.c"

  /* validate: 0.23,6.RS,11.RA,16.RB,21.MB,26.ME,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x5c000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.23,6.RS,11.RA,16.RB,21.MB,26.ME,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 44 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.M.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 45 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.M.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 46 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.M.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.M.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.M.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.M.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.M.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.M.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.M.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 3211 "./ppc-instructions"
    unsigned const MB UNUSED = cache_entry->crack.M.MB = EXTRACTED32(instruction, 21, 25); /* default in icache - extraction */
#line 3211 "./ppc-instructions"
    unsigned const ME UNUSED = cache_entry->crack.M.ME = EXTRACTED32(instruction, 26, 30); /* default in icache - extraction */
#line 3211 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 15822 "icache.c"

#define insn cache_entry->crack.M.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Rotate_Left_Word_then_AND_with_Mask_Rc0;

    /* semantic routine */
#line 3211 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Rotate Left Word then AND with Mask\n"));
#line 15832 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Rotate_Left_Word_then_AND_with_Mask, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3211 "./ppc-instructions"
    {
      long n = MASKED(*rB, 59, 63);
      unsigned32 r = ROTL32(*rS, n);
      unsigned32 m = MASK(MB+32, ME+32);
      signed_word result = r & m;
      *rA = result;
      CR0_COMPARE(result, 0, Rc);
    }
#line 15850 "icache.c"
    return nia;
    /* extraction: 0.23,6.RS,11.RA,16.RB,21.MB,26.ME,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 15854 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Rotate_Left_Word_then_AND_with_Mask_Rc1(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Rotate_Left_Word_then_AND_with_Mask
#undef MY_PREFIX
#define MY_PREFIX Rotate_Left_Word_then_AND_with_Mask_Rc1
#line 3211 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Rotate Left Word then AND with Mask\n"));
#line 15873 "icache.c"

  /* validate: 0.23,6.RS,11.RA,16.RB,21.MB,26.ME,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x5c000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.23,6.RS,11.RA,16.RB,21.MB,26.ME,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 44 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.M.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 45 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.M.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 46 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.M.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.M.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.M.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.M.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.M.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.M.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.M.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 3211 "./ppc-instructions"
    unsigned const MB UNUSED = cache_entry->crack.M.MB = EXTRACTED32(instruction, 21, 25); /* default in icache - extraction */
#line 3211 "./ppc-instructions"
    unsigned const ME UNUSED = cache_entry->crack.M.ME = EXTRACTED32(instruction, 26, 30); /* default in icache - extraction */
#line 3211 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 15907 "icache.c"

#define insn cache_entry->crack.M.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Rotate_Left_Word_then_AND_with_Mask_Rc1;

    /* semantic routine */
#line 3211 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Rotate Left Word then AND with Mask\n"));
#line 15917 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Rotate_Left_Word_then_AND_with_Mask, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3211 "./ppc-instructions"
    {
      long n = MASKED(*rB, 59, 63);
      unsigned32 r = ROTL32(*rS, n);
      unsigned32 m = MASK(MB+32, ME+32);
      signed_word result = r & m;
      *rA = result;
      CR0_COMPARE(result, 0, Rc);
    }
#line 15935 "icache.c"
    return nia;
    /* extraction: 0.23,6.RS,11.RA,16.RB,21.MB,26.ME,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 15939 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_OR_Immediate(cpu *processor,
                    instruction_word instruction,
                    unsigned_word cia,
                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_OR_Immediate
#undef MY_PREFIX
#define MY_PREFIX OR_Immediate
#line 2975 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode OR Immediate\n"));
#line 15958 "icache.c"

  /* validate: 0.24,6.RS,11.RA,16.UI */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x60000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.24,6.RS,11.RA,16.UI #define get-values-from-icache|put-values-in-icache */
#line 44 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.D.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 45 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.D.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 46 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.D.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 2975 "./ppc-instructions"
    unsigned const UI UNUSED = cache_entry->crack.D.UI = EXTRACTED32(instruction, 16, 31); /* default in icache - extraction */
#line 15982 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_OR_Immediate;

    /* semantic routine */
#line 2975 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics OR Immediate\n"));
#line 15992 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_OR_Immediate, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2975 "./ppc-instructions"
    {
      *rA = *rS | UI;
      ITRACE(trace_alu, (" Result = %ld (0x%lx)\n", (long)*rA, (long)*rA));
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK, 0/*Rc*/);
    }
#line 16007 "icache.c"
    return nia;
    /* extraction: 0.24,6.RS,11.RA,16.UI #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 16011 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_OR_Immediate_Shifted(cpu *processor,
                            instruction_word instruction,
                            unsigned_word cia,
                            idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_OR_Immediate_Shifted
#undef MY_PREFIX
#define MY_PREFIX OR_Immediate_Shifted
#line 2984 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode OR Immediate Shifted\n"));
#line 16030 "icache.c"

  /* validate: 0.25,6.RS,11.RA,16.UI */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x64000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.25,6.RS,11.RA,16.UI #define get-values-from-icache|put-values-in-icache */
#line 44 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.D.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 45 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.D.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 46 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.D.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 2984 "./ppc-instructions"
    unsigned const UI UNUSED = cache_entry->crack.D.UI = EXTRACTED32(instruction, 16, 31); /* default in icache - extraction */
#line 16054 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_OR_Immediate_Shifted;

    /* semantic routine */
#line 2984 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics OR Immediate Shifted\n"));
#line 16064 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_OR_Immediate_Shifted, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2984 "./ppc-instructions"
    {
      *rA = *rS | (UI << 16);
      ITRACE(trace_alu, (" Result = %ld (0x%lx)\n", (long)*rA, (long)*rA));
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK, 0/*Rc*/);
    }
#line 16079 "icache.c"
    return nia;
    /* extraction: 0.25,6.RS,11.RA,16.UI #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 16083 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_XOR_Immediate(cpu *processor,
                     instruction_word instruction,
                     unsigned_word cia,
                     idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_XOR_Immediate
#undef MY_PREFIX
#define MY_PREFIX XOR_Immediate
#line 2993 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode XOR Immediate\n"));
#line 16102 "icache.c"

  /* validate: 0.26,6.RS,11.RA,16.UI */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x68000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.26,6.RS,11.RA,16.UI #define get-values-from-icache|put-values-in-icache */
#line 44 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.D.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 45 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.D.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 46 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.D.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 2993 "./ppc-instructions"
    unsigned const UI UNUSED = cache_entry->crack.D.UI = EXTRACTED32(instruction, 16, 31); /* default in icache - extraction */
#line 16126 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_XOR_Immediate;

    /* semantic routine */
#line 2993 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics XOR Immediate\n"));
#line 16136 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_XOR_Immediate, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2993 "./ppc-instructions"
    {
      *rA = *rS ^ UI;
      ITRACE(trace_alu, (" Result = %ld (0x%lx)\n", (long)*rA, (long)*rA));
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK, 0/*Rc*/);
    }
#line 16151 "icache.c"
    return nia;
    /* extraction: 0.26,6.RS,11.RA,16.UI #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 16155 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_XOR_Immediate_Shifted(cpu *processor,
                             instruction_word instruction,
                             unsigned_word cia,
                             idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_XOR_Immediate_Shifted
#undef MY_PREFIX
#define MY_PREFIX XOR_Immediate_Shifted
#line 3002 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode XOR Immediate Shifted\n"));
#line 16174 "icache.c"

  /* validate: 0.27,6.RS,11.RA,16.UI */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x6c000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.27,6.RS,11.RA,16.UI #define get-values-from-icache|put-values-in-icache */
#line 44 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.D.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 45 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.D.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 46 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.D.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 3002 "./ppc-instructions"
    unsigned const UI UNUSED = cache_entry->crack.D.UI = EXTRACTED32(instruction, 16, 31); /* default in icache - extraction */
#line 16198 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_XOR_Immediate_Shifted;

    /* semantic routine */
#line 3002 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics XOR Immediate Shifted\n"));
#line 16208 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_XOR_Immediate_Shifted, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3002 "./ppc-instructions"
    {
      *rA = *rS ^ (UI << 16);
      ITRACE(trace_alu, (" Result = %ld (0x%lx)\n", (long)*rA, (long)*rA));
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK, 0/*Rc*/);
    }
#line 16223 "icache.c"
    return nia;
    /* extraction: 0.27,6.RS,11.RA,16.UI #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 16227 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_AND_Immediate(cpu *processor,
                     instruction_word instruction,
                     unsigned_word cia,
                     idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_AND_Immediate
#undef MY_PREFIX
#define MY_PREFIX AND_Immediate
#line 2955 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode AND Immediate\n"));
#line 16246 "icache.c"

  /* validate: 0.28,6.RS,11.RA,16.UI */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x70000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.28,6.RS,11.RA,16.UI #define get-values-from-icache|put-values-in-icache */
#line 44 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.D.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 45 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.D.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 46 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.D.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 2955 "./ppc-instructions"
    unsigned const UI UNUSED = cache_entry->crack.D.UI = EXTRACTED32(instruction, 16, 31); /* default in icache - extraction */
#line 16270 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_AND_Immediate;

    /* semantic routine */
#line 2955 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics AND Immediate\n"));
#line 16280 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_AND_Immediate, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2955 "./ppc-instructions"
    {
      *rA = *rS & UI;
      CR0_COMPARE(*rA, 0, 1/*Rc*/);
      ITRACE(trace_alu, (" Result = %ld (0x%lx)\n", (long)*rA, (long)*rA));
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK, 1/*Rc*/);
    }
#line 16296 "icache.c"
    return nia;
    /* extraction: 0.28,6.RS,11.RA,16.UI #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 16300 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_AND_Immediate_Shifted(cpu *processor,
                             instruction_word instruction,
                             unsigned_word cia,
                             idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_AND_Immediate_Shifted
#undef MY_PREFIX
#define MY_PREFIX AND_Immediate_Shifted
#line 2965 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode AND Immediate Shifted\n"));
#line 16319 "icache.c"

  /* validate: 0.29,6.RS,11.RA,16.UI */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x74000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.29,6.RS,11.RA,16.UI #define get-values-from-icache|put-values-in-icache */
#line 44 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.D.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 45 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.D.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 46 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.D.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 2965 "./ppc-instructions"
    unsigned const UI UNUSED = cache_entry->crack.D.UI = EXTRACTED32(instruction, 16, 31); /* default in icache - extraction */
#line 16343 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_AND_Immediate_Shifted;

    /* semantic routine */
#line 2965 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics AND Immediate Shifted\n"));
#line 16353 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_AND_Immediate_Shifted, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2965 "./ppc-instructions"
    {
      *rA = *rS & (UI << 16);
      CR0_COMPARE(*rA, 0, 1/*Rc*/);
      ITRACE(trace_alu, (" Result = %ld (0x%lx)\n", (long)*rA, (long)*rA));
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK, 1/*Rc*/);
    }
#line 16369 "icache.c"
    return nia;
    /* extraction: 0.29,6.RS,11.RA,16.UI #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 16373 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Compare(cpu *processor,
               instruction_word instruction,
               unsigned_word cia,
               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Compare
#undef MY_PREFIX
#define MY_PREFIX Compare
#line 2811 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Compare\n"));
#line 16392 "icache.c"

  /* validate: 0.31,6.BF,9./,10.L,11.RA,16.RB,21.0,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc4007ff) != 0x7c000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.BF,9./,10.L,11.RA,16.RB,21.0,31./ #define get-values-from-icache|put-values-in-icache */
#line 68 "./ppc-instructions"
    unsigned const BF UNUSED = cache_entry->crack.X.BF = EXTRACTED32(instruction, 6, 8); /* in icache - extraction */
#line 69 "./ppc-instructions"
    unsigned32 const BF_BITMASK UNUSED = cache_entry->crack.X.BF_BITMASK = (1 << BF); /* in icache - extraction */
#line 2811 "./ppc-instructions"
    unsigned const L UNUSED = cache_entry->crack.X.L = EXTRACTED32(instruction, 10, 10); /* default in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 16420 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Compare;

    /* semantic routine */
#line 2811 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Compare\n"));
#line 16430 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Compare, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2811 "./ppc-instructions"
    {
      if (!is_64bit_mode && L)
        program_interrupt(processor, cia,
                          illegal_instruction_program_interrupt);
      else {
        signed_word a;
        signed_word b;
        if (L == 0) {
          a = EXTENDED(*rA);
          b = EXTENDED(*rB);
        }
        else {
          a = *rA;
          b = *rB;
        }
        CR_COMPARE(BF, a, b);
      }
      PPC_INSN_INT_CR(0, RA_BITMASK | RB_BITMASK, BF_BITMASK);
    }
#line 16459 "icache.c"
    return nia;
    /* extraction: 0.31,6.BF,9./,10.L,11.RA,16.RB,21.0,31./ #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 16463 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Trap_Word(cpu *processor,
                 instruction_word instruction,
                 unsigned_word cia,
                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Trap_Word
#undef MY_PREFIX
#define MY_PREFIX Trap_Word
#line 2931 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Trap Word\n"));
#line 16482 "icache.c"

  /* validate: 0.31,6.TO,11.RA,16.RB,21.4,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c000008)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.TO,11.RA,16.RB,21.4,31./ #define get-values-from-icache|put-values-in-icache */
#line 2931 "./ppc-instructions"
    unsigned const TO UNUSED = cache_entry->crack.X.TO = EXTRACTED32(instruction, 6, 10); /* default in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 16506 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Trap_Word;

    /* semantic routine */
#line 2931 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Trap Word\n"));
#line 16516 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Trap_Word, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2931 "./ppc-instructions"
    {
      signed_word a = EXTENDED(*rA);
      signed_word b = EXTENDED(*rB);
      if (TO == 12 && rA == rB) {
        ITRACE(trace_breakpoint, ("breakpoint\n"));
        cpu_halt(processor, cia, was_trap, 0);
      }
      else if ((a < b && TO_0_)
          || (a > b && TO_1_)
          || (a == b && TO_2_)
          || ((unsigned_word)a < (unsigned_word)b && TO_3_)
          || ((unsigned_word)a > (unsigned_word)b && TO_4_)
          )
        program_interrupt(processor, cia,
                          trap_program_interrupt);
    }
#line 16542 "icache.c"
    return nia;
    /* extraction: 0.31,6.TO,11.RA,16.RB,21.4,31./ #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 16546 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Subtract_From_Carrying_OE0_Rc0(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Subtract_From_Carrying
#undef MY_PREFIX
#define MY_PREFIX Subtract_From_Carrying_OE0_Rc0
#line 2597 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Subtract From Carrying\n"));
#line 16565 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.OE,22.8,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0003fe) != 0x7c000010)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.8,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2597 "./ppc-instructions"
    unsigned const OE UNUSED = 0; /* default in icache - constant field */
#line 2597 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 16597 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Subtract_From_Carrying_OE0_Rc0;

    /* semantic routine */
#line 2597 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Subtract From Carrying\n"));
#line 16607 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Subtract_From_Carrying, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2597 "./ppc-instructions"
    {
      /* RT <- ~RA + RB + 1 === RT <- RB - RA */
      ALU_BEGIN(*rA);
      ALU_NOT;
      ALU_ADD(*rB);
      ALU_ADD(1);
      ALU_END(*rT, 1/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 16626 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.8,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 16630 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Subtract_From_Carrying_OE0_Rc1(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Subtract_From_Carrying
#undef MY_PREFIX
#define MY_PREFIX Subtract_From_Carrying_OE0_Rc1
#line 2597 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Subtract From Carrying\n"));
#line 16649 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.OE,22.8,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0003fe) != 0x7c000010)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.8,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2597 "./ppc-instructions"
    unsigned const OE UNUSED = 0; /* default in icache - constant field */
#line 2597 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 16681 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Subtract_From_Carrying_OE0_Rc1;

    /* semantic routine */
#line 2597 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Subtract From Carrying\n"));
#line 16691 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Subtract_From_Carrying, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2597 "./ppc-instructions"
    {
      /* RT <- ~RA + RB + 1 === RT <- RB - RA */
      ALU_BEGIN(*rA);
      ALU_NOT;
      ALU_ADD(*rB);
      ALU_ADD(1);
      ALU_END(*rT, 1/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 16710 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.8,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 16714 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Add_Carrying_OE0_Rc0(cpu *processor,
                            instruction_word instruction,
                            unsigned_word cia,
                            idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Add_Carrying
#undef MY_PREFIX
#define MY_PREFIX Add_Carrying_OE0_Rc0
#line 2587 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Add Carrying\n"));
#line 16733 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.OE,22.10,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0003fe) != 0x7c000014)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.10,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2587 "./ppc-instructions"
    unsigned const OE UNUSED = 0; /* default in icache - constant field */
#line 2587 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 16765 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Add_Carrying_OE0_Rc0;

    /* semantic routine */
#line 2587 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Add Carrying\n"));
#line 16775 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Add_Carrying, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2587 "./ppc-instructions"
    {
      ALU_BEGIN(*rA);
      ALU_ADD(*rB);
      ALU_END(*rT, 1/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 16791 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.10,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 16795 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Add_Carrying_OE0_Rc1(cpu *processor,
                            instruction_word instruction,
                            unsigned_word cia,
                            idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Add_Carrying
#undef MY_PREFIX
#define MY_PREFIX Add_Carrying_OE0_Rc1
#line 2587 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Add Carrying\n"));
#line 16814 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.OE,22.10,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0003fe) != 0x7c000014)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.10,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2587 "./ppc-instructions"
    unsigned const OE UNUSED = 0; /* default in icache - constant field */
#line 2587 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 16846 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Add_Carrying_OE0_Rc1;

    /* semantic routine */
#line 2587 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Add Carrying\n"));
#line 16856 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Add_Carrying, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2587 "./ppc-instructions"
    {
      ALU_BEGIN(*rA);
      ALU_ADD(*rB);
      ALU_END(*rT, 1/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 16872 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.10,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 16876 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Multiply_High_Word_Unsigned_Rc0(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Multiply_High_Word_Unsigned
#undef MY_PREFIX
#define MY_PREFIX Multiply_High_Word_Unsigned_Rc0
#line 2731 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Multiply High Word Unsigned\n"));
#line 16895 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21./,22.11,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0x7c000016)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21./,22.11,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2731 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 16925 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Multiply_High_Word_Unsigned_Rc0;

    /* semantic routine */
#line 2731 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Multiply High Word Unsigned\n"));
#line 16935 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Multiply_High_Word_Unsigned, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2731 "./ppc-instructions"
    {
      unsigned64 a = (unsigned32)(*rA);
      unsigned64 b = (unsigned32)(*rB);
      unsigned64 prod = a * b;
      signed_word t = EXTRACTED64(prod, 0, 31);
      *rT = t;
      CR0_COMPARE(t, 0, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 16954 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21./,22.11,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 16958 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Multiply_High_Word_Unsigned_Rc1(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Multiply_High_Word_Unsigned
#undef MY_PREFIX
#define MY_PREFIX Multiply_High_Word_Unsigned_Rc1
#line 2731 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Multiply High Word Unsigned\n"));
#line 16977 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21./,22.11,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0x7c000016)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21./,22.11,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2731 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 17007 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Multiply_High_Word_Unsigned_Rc1;

    /* semantic routine */
#line 2731 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Multiply High Word Unsigned\n"));
#line 17017 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Multiply_High_Word_Unsigned, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2731 "./ppc-instructions"
    {
      unsigned64 a = (unsigned32)(*rA);
      unsigned64 b = (unsigned32)(*rB);
      unsigned64 prod = a * b;
      signed_word t = EXTRACTED64(prod, 0, 31);
      *rT = t;
      CR0_COMPARE(t, 0, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 17036 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21./,22.11,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 17040 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Move_From_Condition_Register(cpu *processor,
                                    instruction_word instruction,
                                    unsigned_word cia,
                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Move_From_Condition_Register
#undef MY_PREFIX
#define MY_PREFIX Move_From_Condition_Register
#line 3454 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Move From Condition Register\n"));
#line 17059 "icache.c"

  /* validate: 0.31,6.RT,11./,16./,21.19,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc1fffff) != 0x7c000026)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11./,16./,21.19,31./ #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.X.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.X.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.X.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 17074 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Move_From_Condition_Register;

    /* semantic routine */
#line 3454 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Move From Condition Register\n"));
#line 17084 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Move_From_Condition_Register, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3454 "./ppc-instructions"
    {
      *rT = (unsigned32)CR;
      PPC_INSN_MFCR(RT_BITMASK);
    }
#line 17098 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11./,16./,21.19,31./ #undef get-values-from-icache|put-values-in-icache */
#line 17101 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Load_Word_And_Reserve_Indexed(cpu *processor,
                                     instruction_word instruction,
                                     unsigned_word cia,
                                     idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Load_Word_And_Reserve_Indexed
#undef MY_PREFIX
#define MY_PREFIX Load_Word_And_Reserve_Indexed
#line 2423 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Load Word And Reserve Indexed\n"));
#line 17120 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.20,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c000028)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.20,31./ #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.X.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.X.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.X.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 17148 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Load_Word_And_Reserve_Indexed;

    /* semantic routine */
#line 2423 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Load Word And Reserve Indexed\n"));
#line 17158 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Load_Word_And_Reserve_Indexed, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2423 "./ppc-instructions"
    {
      unsigned_word b;
      unsigned_word EA;
      if (RA_is_0) b = 0;
      else         b = *rA;
      EA = b + *rB;
      RESERVE = 1;
      RESERVE_ADDR = real_addr(EA, 1/*is-read?*/);
      RESERVE_DATA = MEM(unsigned, EA, 4);
      *rT = RESERVE_DATA;
      PPC_INSN_INT(RT_BITMASK, (RA_BITMASK & ~1) | RB_BITMASK, 0);
    }
#line 17180 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.20,31./ #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 17184 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Load_Word_and_Zero_Indexed(cpu *processor,
                                  instruction_word instruction,
                                  unsigned_word cia,
                                  idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Load_Word_and_Zero_Indexed
#undef MY_PREFIX
#define MY_PREFIX Load_Word_and_Zero_Indexed
#line 1899 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Load Word and Zero Indexed\n"));
#line 17203 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.23,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.23,31./ #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.X.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.X.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.X.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 17231 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Load_Word_and_Zero_Indexed;

    /* semantic routine */
#line 1899 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Load Word and Zero Indexed\n"));
#line 17241 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Load_Word_and_Zero_Indexed, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1899 "./ppc-instructions"
    {
      unsigned_word b;
      unsigned_word EA;
      if (RA_is_0) b = 0;
      else         b = *rA;
      EA = b + *rB;
      *rT = MEM(unsigned, EA, 4);
      PPC_INSN_INT(RT_BITMASK, (RA_BITMASK & ~1) | RB_BITMASK, 0);
    }
#line 17260 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.23,31./ #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 17264 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Shift_Left_Word_Rc0(cpu *processor,
                           instruction_word instruction,
                           unsigned_word cia,
                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Shift_Left_Word
#undef MY_PREFIX
#define MY_PREFIX Shift_Left_Word_Rc0
#line 3251 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Shift Left Word\n"));
#line 17283 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16.RB,21.24,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0x7c000030)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.24,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 44 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 45 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 46 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 3251 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 17313 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Shift_Left_Word_Rc0;

    /* semantic routine */
#line 3251 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Shift Left Word\n"));
#line 17323 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Shift_Left_Word, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3251 "./ppc-instructions"
    {
      int n = MASKED(*rB, 58, 63);
      unsigned32 source = *rS;
      signed_word shifted;
      if (n < 32)
        shifted = (source << n);
      else
        shifted = 0;
      *rA = shifted;
      CR0_COMPARE(shifted, 0, Rc);
      ITRACE(trace_alu,
             ("n=%d, source=0x%lx, shifted=0x%lx\n",
              n, (unsigned long)source, (unsigned long)shifted));
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK, Rc);
    }
#line 17348 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.24,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 17352 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Shift_Left_Word_Rc1(cpu *processor,
                           instruction_word instruction,
                           unsigned_word cia,
                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Shift_Left_Word
#undef MY_PREFIX
#define MY_PREFIX Shift_Left_Word_Rc1
#line 3251 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Shift Left Word\n"));
#line 17371 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16.RB,21.24,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0x7c000030)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.24,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 44 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 45 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 46 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 3251 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 17401 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Shift_Left_Word_Rc1;

    /* semantic routine */
#line 3251 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Shift Left Word\n"));
#line 17411 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Shift_Left_Word, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3251 "./ppc-instructions"
    {
      int n = MASKED(*rB, 58, 63);
      unsigned32 source = *rS;
      signed_word shifted;
      if (n < 32)
        shifted = (source << n);
      else
        shifted = 0;
      *rA = shifted;
      CR0_COMPARE(shifted, 0, Rc);
      ITRACE(trace_alu,
             ("n=%d, source=0x%lx, shifted=0x%lx\n",
              n, (unsigned long)source, (unsigned long)shifted));
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK, Rc);
    }
#line 17436 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.24,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 17440 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Count_Leading_Zeros_Word_Rc0(cpu *processor,
                                    instruction_word instruction,
                                    unsigned_word cia,
                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Count_Leading_Zeros_Word
#undef MY_PREFIX
#define MY_PREFIX Count_Leading_Zeros_Word_Rc0
#line 3130 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Count Leading Zeros Word\n"));
#line 17459 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16./,21.26,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00fffe) != 0x7c000034)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16./,21.26,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 44 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 45 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 46 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 3130 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 17483 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Count_Leading_Zeros_Word_Rc0;

    /* semantic routine */
#line 3130 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Count Leading Zeros Word\n"));
#line 17493 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Count_Leading_Zeros_Word, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3130 "./ppc-instructions"
    {
      int count = 0;
      unsigned32 mask = BIT32(0);
      unsigned32 source = *rS;
      while (!(source & mask) && mask != 0) {
        mask >>= 1;
        count++;
      }
      *rA = count;
      ITRACE(trace_alu, (" Result = %ld (0x%lx)\n", (long)*rA, (long)*rA));
      CR0_COMPARE(count, 0, Rc); /* FIXME - is this correct */
    }
#line 17515 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16./,21.26,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 17519 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Count_Leading_Zeros_Word_Rc1(cpu *processor,
                                    instruction_word instruction,
                                    unsigned_word cia,
                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Count_Leading_Zeros_Word
#undef MY_PREFIX
#define MY_PREFIX Count_Leading_Zeros_Word_Rc1
#line 3130 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Count Leading Zeros Word\n"));
#line 17538 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16./,21.26,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00fffe) != 0x7c000034)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16./,21.26,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 44 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 45 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 46 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 3130 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 17562 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Count_Leading_Zeros_Word_Rc1;

    /* semantic routine */
#line 3130 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Count Leading Zeros Word\n"));
#line 17572 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Count_Leading_Zeros_Word, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3130 "./ppc-instructions"
    {
      int count = 0;
      unsigned32 mask = BIT32(0);
      unsigned32 source = *rS;
      while (!(source & mask) && mask != 0) {
        mask >>= 1;
        count++;
      }
      *rA = count;
      ITRACE(trace_alu, (" Result = %ld (0x%lx)\n", (long)*rA, (long)*rA));
      CR0_COMPARE(count, 0, Rc); /* FIXME - is this correct */
    }
#line 17594 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16./,21.26,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 17598 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_AND_Rc0(cpu *processor,
               instruction_word instruction,
               unsigned_word cia,
               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_AND
#undef MY_PREFIX
#define MY_PREFIX AND_Rc0
#line 3011 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode AND\n"));
#line 17617 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16.RB,21.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0x7c000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 44 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 45 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 46 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 3011 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 17647 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_AND_Rc0;

    /* semantic routine */
#line 3011 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics AND\n"));
#line 17657 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_AND, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3011 "./ppc-instructions"
    {
      *rA = *rS & *rB;
      CR0_COMPARE(*rA, 0, Rc);
      ITRACE(trace_alu, (" Result = %ld (0x%lx)\n", (long)*rA, (long)*rA));
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK | RB_BITMASK, Rc);
    }
#line 17673 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 17677 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_AND_Rc1(cpu *processor,
               instruction_word instruction,
               unsigned_word cia,
               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_AND
#undef MY_PREFIX
#define MY_PREFIX AND_Rc1
#line 3011 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode AND\n"));
#line 17696 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16.RB,21.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0x7c000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 44 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 45 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 46 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 3011 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 17726 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_AND_Rc1;

    /* semantic routine */
#line 3011 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics AND\n"));
#line 17736 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_AND, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3011 "./ppc-instructions"
    {
      *rA = *rS & *rB;
      CR0_COMPARE(*rA, 0, Rc);
      ITRACE(trace_alu, (" Result = %ld (0x%lx)\n", (long)*rA, (long)*rA));
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK | RB_BITMASK, Rc);
    }
#line 17752 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 17756 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Compare_Logical(cpu *processor,
                       instruction_word instruction,
                       unsigned_word cia,
                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Compare_Logical
#undef MY_PREFIX
#define MY_PREFIX Compare_Logical
#line 2853 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Compare Logical\n"));
#line 17775 "icache.c"

  /* validate: 0.31,6.BF,9./,10.L,11.RA,16.RB,21.32,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc4007ff) != 0x7c000040)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.BF,9./,10.L,11.RA,16.RB,21.32,31./ #define get-values-from-icache|put-values-in-icache */
#line 68 "./ppc-instructions"
    unsigned const BF UNUSED = cache_entry->crack.X.BF = EXTRACTED32(instruction, 6, 8); /* in icache - extraction */
#line 69 "./ppc-instructions"
    unsigned32 const BF_BITMASK UNUSED = cache_entry->crack.X.BF_BITMASK = (1 << BF); /* in icache - extraction */
#line 2853 "./ppc-instructions"
    unsigned const L UNUSED = cache_entry->crack.X.L = EXTRACTED32(instruction, 10, 10); /* default in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 17803 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Compare_Logical;

    /* semantic routine */
#line 2853 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Compare Logical\n"));
#line 17813 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Compare_Logical, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2853 "./ppc-instructions"
    {
      if (!is_64bit_mode && L)
        program_interrupt(processor, cia,
                          illegal_instruction_program_interrupt);
      else {
        unsigned_word a;
        unsigned_word b;
        if (L == 0) {
          a = MASKED(*rA, 32, 63);
          b = MASKED(*rB, 32, 63);
        }
        else {
          a = *rA;
          b = *rB;
        }
        CR_COMPARE(BF, a, b);
      }
      PPC_INSN_INT_CR(0, RA_BITMASK | RB_BITMASK, BF_BITMASK);
    }
#line 17842 "icache.c"
    return nia;
    /* extraction: 0.31,6.BF,9./,10.L,11.RA,16.RB,21.32,31./ #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 17846 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Subtract_From_OE0_Rc0(cpu *processor,
                             instruction_word instruction,
                             unsigned_word cia,
                             idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Subtract_From
#undef MY_PREFIX
#define MY_PREFIX Subtract_From_OE0_Rc0
#line 2543 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Subtract From\n"));
#line 17865 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.OE,22.40,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0003fe) != 0x7c000050)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.40,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2543 "./ppc-instructions"
    unsigned const OE UNUSED = 0; /* default in icache - constant field */
#line 2543 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 17897 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Subtract_From_OE0_Rc0;

    /* semantic routine */
#line 2543 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Subtract From\n"));
#line 17907 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Subtract_From, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2543 "./ppc-instructions"
    {
      ALU_BEGIN(*rA);
      ALU_NOT;
      ALU_ADD(*rB);
      ALU_ADD(1);
      ALU_END(*rT, 0/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 17925 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.40,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 17929 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Subtract_From_OE0_Rc1(cpu *processor,
                             instruction_word instruction,
                             unsigned_word cia,
                             idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Subtract_From
#undef MY_PREFIX
#define MY_PREFIX Subtract_From_OE0_Rc1
#line 2543 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Subtract From\n"));
#line 17948 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.OE,22.40,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0003fe) != 0x7c000050)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.40,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2543 "./ppc-instructions"
    unsigned const OE UNUSED = 0; /* default in icache - constant field */
#line 2543 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 17980 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Subtract_From_OE0_Rc1;

    /* semantic routine */
#line 2543 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Subtract From\n"));
#line 17990 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Subtract_From, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2543 "./ppc-instructions"
    {
      ALU_BEGIN(*rA);
      ALU_NOT;
      ALU_ADD(*rB);
      ALU_ADD(1);
      ALU_END(*rT, 0/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 18008 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.40,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 18012 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Data_Cache_Block_Store(cpu *processor,
                              instruction_word instruction,
                              unsigned_word cia,
                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Data_Cache_Block_Store
#undef MY_PREFIX
#define MY_PREFIX Data_Cache_Block_Store
#line 4797 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Data Cache Block Store\n"));
#line 18031 "icache.c"

  /* validate: 0.31,6./,11.RA,16.RB,21.54,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xffe007ff) != 0x7c00006c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6./,11.RA,16.RB,21.54,31./ #define get-values-from-icache|put-values-in-icache */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 18053 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Data_Cache_Block_Store;

    /* semantic routine */
#line 4797 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Data Cache Block Store\n"));
#line 18063 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Data_Cache_Block_Store, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4797 "./ppc-instructions"
    {
      TRACE(trace_tbd,("Data Cache Block Store\n"));
      PPC_INSN_INT(0, (RA_BITMASK & ~1) | RB_BITMASK, 0/*Rc*/);
    }
#line 18077 "icache.c"
    return nia;
    /* extraction: 0.31,6./,11.RA,16.RB,21.54,31./ #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 18081 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Load_Word_and_Zero_with_Update_Indexed(cpu *processor,
                                              instruction_word instruction,
                                              unsigned_word cia,
                                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Load_Word_and_Zero_with_Update_Indexed
#undef MY_PREFIX
#define MY_PREFIX Load_Word_and_Zero_with_Update_Indexed
#line 1926 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Load Word and Zero with Update Indexed\n"));
#line 18100 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.55,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c00006e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.55,31./ #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.X.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.X.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.X.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 18128 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Load_Word_and_Zero_with_Update_Indexed;

    /* semantic routine */
#line 1926 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Load Word and Zero with Update Indexed\n"));
#line 18138 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Load_Word_and_Zero_with_Update_Indexed, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1926 "./ppc-instructions"
    {
      unsigned_word EA;
      if (RA_is_0 || RA == RT)
        program_interrupt(processor, cia,
                          illegal_instruction_program_interrupt);
      EA = *rA + *rB;
      *rT = MEM(unsigned, EA, 4);
      *rA = EA;
      PPC_INSN_INT(RT_BITMASK | RA_BITMASK, RA_BITMASK | RB_BITMASK, 0);
    }
#line 18158 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.55,31./ #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 18162 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_AND_with_Complement_Rc0(cpu *processor,
                               instruction_word instruction,
                               unsigned_word cia,
                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_AND_with_Complement
#undef MY_PREFIX
#define MY_PREFIX AND_with_Complement_Rc0
#line 3071 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode AND with Complement\n"));
#line 18181 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16.RB,21.60,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0x7c000078)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.60,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 44 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 45 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 46 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 3071 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 18211 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_AND_with_Complement_Rc0;

    /* semantic routine */
#line 3071 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics AND with Complement\n"));
#line 18221 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_AND_with_Complement, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3071 "./ppc-instructions"
    {
      *rA = *rS & ~*rB;
      CR0_COMPARE(*rA, 0, Rc);
      ITRACE(trace_alu, (" Result = %ld (0x%lx)\n", (long)*rA, (long)*rA));
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK | RB_BITMASK, Rc);
    }
#line 18237 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.60,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 18241 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_AND_with_Complement_Rc1(cpu *processor,
                               instruction_word instruction,
                               unsigned_word cia,
                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_AND_with_Complement
#undef MY_PREFIX
#define MY_PREFIX AND_with_Complement_Rc1
#line 3071 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode AND with Complement\n"));
#line 18260 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16.RB,21.60,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0x7c000078)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.60,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 44 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 45 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 46 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 3071 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 18290 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_AND_with_Complement_Rc1;

    /* semantic routine */
#line 3071 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics AND with Complement\n"));
#line 18300 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_AND_with_Complement, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3071 "./ppc-instructions"
    {
      *rA = *rS & ~*rB;
      CR0_COMPARE(*rA, 0, Rc);
      ITRACE(trace_alu, (" Result = %ld (0x%lx)\n", (long)*rA, (long)*rA));
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK | RB_BITMASK, Rc);
    }
#line 18316 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.60,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 18320 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Multiply_High_Word_Rc0(cpu *processor,
                              instruction_word instruction,
                              unsigned_word cia,
                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Multiply_High_Word
#undef MY_PREFIX
#define MY_PREFIX Multiply_High_Word_Rc0
#line 2716 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Multiply High Word\n"));
#line 18339 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21./,22.75,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0x7c000096)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21./,22.75,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2716 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 18369 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Multiply_High_Word_Rc0;

    /* semantic routine */
#line 2716 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Multiply High Word\n"));
#line 18379 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Multiply_High_Word, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2716 "./ppc-instructions"
    {
      signed64 a = (signed32)(*rA);
      signed64 b = (signed32)(*rB);
      signed64 prod = a * b;
      signed_word t = EXTRACTED64(prod, 0, 31);
      *rT = t;
      CR0_COMPARE(t, 0, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 18398 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21./,22.75,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 18402 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Multiply_High_Word_Rc1(cpu *processor,
                              instruction_word instruction,
                              unsigned_word cia,
                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Multiply_High_Word
#undef MY_PREFIX
#define MY_PREFIX Multiply_High_Word_Rc1
#line 2716 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Multiply High Word\n"));
#line 18421 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21./,22.75,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0x7c000096)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21./,22.75,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2716 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 18451 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Multiply_High_Word_Rc1;

    /* semantic routine */
#line 2716 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Multiply High Word\n"));
#line 18461 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Multiply_High_Word, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2716 "./ppc-instructions"
    {
      signed64 a = (signed32)(*rA);
      signed64 b = (signed32)(*rB);
      signed64 prod = a * b;
      signed_word t = EXTRACTED64(prod, 0, 31);
      *rT = t;
      CR0_COMPARE(t, 0, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 18480 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21./,22.75,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 18484 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Move_From_Machine_State_Register(cpu *processor,
                                        instruction_word instruction,
                                        unsigned_word cia,
                                        idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Move_From_Machine_State_Register
#undef MY_PREFIX
#define MY_PREFIX Move_From_Machine_State_Register
#line 4884 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Move From Machine State Register\n"));
#line 18503 "icache.c"

  /* validate: 0.31,6.RT,11./,16./,21.83,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc1fffff) != 0x7c0000a6)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11./,16./,21.83,31./ #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.X.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.X.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.X.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 18518 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Move_From_Machine_State_Register;

    /* semantic routine */
#line 4884 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Move From Machine State Register\n"));
#line 18528 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Move_From_Machine_State_Register, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4884 "./ppc-instructions"
    {
      if (IS_PROBLEM_STATE(processor))
        program_interrupt(processor, cia,
                          privileged_instruction_program_interrupt);
      else {
        *rT = MSR;
        check_masked_interrupts(processor);
      }
    }
#line 18547 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11./,16./,21.83,31./ #undef get-values-from-icache|put-values-in-icache */
#line 18550 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Data_Cache_Block_Flush(cpu *processor,
                              instruction_word instruction,
                              unsigned_word cia,
                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Data_Cache_Block_Flush
#undef MY_PREFIX
#define MY_PREFIX Data_Cache_Block_Flush
#line 4805 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Data Cache Block Flush\n"));
#line 18569 "icache.c"

  /* validate: 0.31,6./,11.RA,16.RB,21.86,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xffe007ff) != 0x7c0000ac)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6./,11.RA,16.RB,21.86,31./ #define get-values-from-icache|put-values-in-icache */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 18591 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Data_Cache_Block_Flush;

    /* semantic routine */
#line 4805 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Data Cache Block Flush\n"));
#line 18601 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Data_Cache_Block_Flush, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4805 "./ppc-instructions"
    {
      TRACE(trace_tbd,("Data Cache Block Flush\n"));
      PPC_INSN_INT(0, (RA_BITMASK & ~1) | RB_BITMASK, 0/*Rc*/);
    }
#line 18615 "icache.c"
    return nia;
    /* extraction: 0.31,6./,11.RA,16.RB,21.86,31./ #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 18619 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Load_Byte_and_Zero_Indexed(cpu *processor,
                                  instruction_word instruction,
                                  unsigned_word cia,
                                  idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Load_Byte_and_Zero_Indexed
#undef MY_PREFIX
#define MY_PREFIX Load_Byte_and_Zero_Indexed
#line 1737 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Load Byte and Zero Indexed\n"));
#line 18638 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.87,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c0000ae)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.87,31./ #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.X.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.X.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.X.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 18666 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Load_Byte_and_Zero_Indexed;

    /* semantic routine */
#line 1737 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Load Byte and Zero Indexed\n"));
#line 18676 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Load_Byte_and_Zero_Indexed, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1737 "./ppc-instructions"
    {
      unsigned_word b;
      unsigned_word EA;
      if (RA_is_0) b = 0;
      else         b = *rA;
      EA = b + *rB;
      *rT = MEM(unsigned, EA, 1);
      PPC_INSN_INT(RT_BITMASK, (RA_BITMASK & ~1) | RB_BITMASK, 0);
    }
#line 18695 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.87,31./ #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 18699 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Negate_OE0_Rc0(cpu *processor,
                      instruction_word instruction,
                      unsigned_word cia,
                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Negate
#undef MY_PREFIX
#define MY_PREFIX Negate_OE0_Rc0
#line 2677 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Negate\n"));
#line 18718 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16./,21.OE,22.104,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00fbfe) != 0x7c0000d0)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.104,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 2677 "./ppc-instructions"
    unsigned const OE UNUSED = 0; /* default in icache - constant field */
#line 2677 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 18744 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Negate_OE0_Rc0;

    /* semantic routine */
#line 2677 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Negate\n"));
#line 18754 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Negate, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2677 "./ppc-instructions"
    {
      ALU_BEGIN(*rA);
      ALU_NOT;
      ALU_ADD(1);
      ALU_END(*rT,0/*CA*/,OE,Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK, Rc);
    }
#line 18771 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.104,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 18775 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Negate_OE0_Rc1(cpu *processor,
                      instruction_word instruction,
                      unsigned_word cia,
                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Negate
#undef MY_PREFIX
#define MY_PREFIX Negate_OE0_Rc1
#line 2677 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Negate\n"));
#line 18794 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16./,21.OE,22.104,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00fbfe) != 0x7c0000d0)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.104,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 2677 "./ppc-instructions"
    unsigned const OE UNUSED = 0; /* default in icache - constant field */
#line 2677 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 18820 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Negate_OE0_Rc1;

    /* semantic routine */
#line 2677 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Negate\n"));
#line 18830 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Negate, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2677 "./ppc-instructions"
    {
      ALU_BEGIN(*rA);
      ALU_NOT;
      ALU_ADD(1);
      ALU_END(*rT,0/*CA*/,OE,Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK, Rc);
    }
#line 18847 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.104,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 18851 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Load_Byte_and_Zero_with_Update_Indexed(cpu *processor,
                                              instruction_word instruction,
                                              unsigned_word cia,
                                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Load_Byte_and_Zero_with_Update_Indexed
#undef MY_PREFIX
#define MY_PREFIX Load_Byte_and_Zero_with_Update_Indexed
#line 1764 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Load Byte and Zero with Update Indexed\n"));
#line 18870 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.119,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c0000ee)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.119,31./ #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.X.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.X.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.X.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 18898 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Load_Byte_and_Zero_with_Update_Indexed;

    /* semantic routine */
#line 1764 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Load Byte and Zero with Update Indexed\n"));
#line 18908 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Load_Byte_and_Zero_with_Update_Indexed, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1764 "./ppc-instructions"
    {
      unsigned_word EA;
      if (RA_is_0 || RA == RT)
        program_interrupt(processor, cia,
                          illegal_instruction_program_interrupt);
      EA = *rA + *rB;
      *rT = MEM(unsigned, EA, 1);
      *rA = EA;
      PPC_INSN_INT(RT_BITMASK | RA_BITMASK, RA_BITMASK | RB_BITMASK, 0);
    }
#line 18928 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.119,31./ #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 18932 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_NOR_Rc0(cpu *processor,
               instruction_word instruction,
               unsigned_word cia,
               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_NOR
#undef MY_PREFIX
#define MY_PREFIX NOR_Rc0
#line 3051 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode NOR\n"));
#line 18951 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16.RB,21.124,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0x7c0000f8)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.124,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 44 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 45 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 46 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 3051 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 18981 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_NOR_Rc0;

    /* semantic routine */
#line 3051 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics NOR\n"));
#line 18991 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_NOR, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3051 "./ppc-instructions"
    {
      *rA = ~(*rS | *rB);
      CR0_COMPARE(*rA, 0, Rc);
      ITRACE(trace_alu, (" Result = %ld (0x%lx)\n", (long)*rA, (long)*rA));
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK | RB_BITMASK, Rc);
    }
#line 19007 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.124,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 19011 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_NOR_Rc1(cpu *processor,
               instruction_word instruction,
               unsigned_word cia,
               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_NOR
#undef MY_PREFIX
#define MY_PREFIX NOR_Rc1
#line 3051 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode NOR\n"));
#line 19030 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16.RB,21.124,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0x7c0000f8)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.124,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 44 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 45 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 46 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 3051 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 19060 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_NOR_Rc1;

    /* semantic routine */
#line 3051 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics NOR\n"));
#line 19070 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_NOR, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3051 "./ppc-instructions"
    {
      *rA = ~(*rS | *rB);
      CR0_COMPARE(*rA, 0, Rc);
      ITRACE(trace_alu, (" Result = %ld (0x%lx)\n", (long)*rA, (long)*rA));
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK | RB_BITMASK, Rc);
    }
#line 19086 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.124,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 19090 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Subtract_From_Extended_OE0_Rc0(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Subtract_From_Extended
#undef MY_PREFIX
#define MY_PREFIX Subtract_From_Extended_OE0_Rc0
#line 2621 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Subtract From Extended\n"));
#line 19109 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.OE,22.136,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0003fe) != 0x7c000110)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.136,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2621 "./ppc-instructions"
    unsigned const OE UNUSED = 0; /* default in icache - constant field */
#line 2621 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 19141 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Subtract_From_Extended_OE0_Rc0;

    /* semantic routine */
#line 2621 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Subtract From Extended\n"));
#line 19151 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Subtract_From_Extended, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2621 "./ppc-instructions"
    {
      ALU_BEGIN(*rA);
      ALU_NOT;
      ALU_ADD(*rB);
      ALU_ADD_CA;
      ALU_END(*rT, 1/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 19169 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.136,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 19173 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Subtract_From_Extended_OE0_Rc1(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Subtract_From_Extended
#undef MY_PREFIX
#define MY_PREFIX Subtract_From_Extended_OE0_Rc1
#line 2621 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Subtract From Extended\n"));
#line 19192 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.OE,22.136,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0003fe) != 0x7c000110)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.136,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2621 "./ppc-instructions"
    unsigned const OE UNUSED = 0; /* default in icache - constant field */
#line 2621 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 19224 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Subtract_From_Extended_OE0_Rc1;

    /* semantic routine */
#line 2621 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Subtract From Extended\n"));
#line 19234 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Subtract_From_Extended, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2621 "./ppc-instructions"
    {
      ALU_BEGIN(*rA);
      ALU_NOT;
      ALU_ADD(*rB);
      ALU_ADD_CA;
      ALU_END(*rT, 1/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 19252 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.136,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 19256 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Add_Extended_OE0_Rc0(cpu *processor,
                            instruction_word instruction,
                            unsigned_word cia,
                            idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Add_Extended
#undef MY_PREFIX
#define MY_PREFIX Add_Extended_OE0_Rc0
#line 2610 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Add Extended\n"));
#line 19275 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.OE,22.138,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0003fe) != 0x7c000114)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.138,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2610 "./ppc-instructions"
    unsigned const OE UNUSED = 0; /* default in icache - constant field */
#line 2610 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 19307 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Add_Extended_OE0_Rc0;

    /* semantic routine */
#line 2610 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Add Extended\n"));
#line 19317 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Add_Extended, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2610 "./ppc-instructions"
    {
      ALU_BEGIN(*rA);
      ALU_ADD(*rB);
      ALU_ADD_CA;
      ALU_END(*rT, 1/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 19334 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.138,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 19338 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Add_Extended_OE0_Rc1(cpu *processor,
                            instruction_word instruction,
                            unsigned_word cia,
                            idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Add_Extended
#undef MY_PREFIX
#define MY_PREFIX Add_Extended_OE0_Rc1
#line 2610 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Add Extended\n"));
#line 19357 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.OE,22.138,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0003fe) != 0x7c000114)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.138,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2610 "./ppc-instructions"
    unsigned const OE UNUSED = 0; /* default in icache - constant field */
#line 2610 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 19389 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Add_Extended_OE0_Rc1;

    /* semantic routine */
#line 2610 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Add Extended\n"));
#line 19399 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Add_Extended, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2610 "./ppc-instructions"
    {
      ALU_BEGIN(*rA);
      ALU_ADD(*rB);
      ALU_ADD_CA;
      ALU_END(*rT, 1/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 19416 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.138,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 19420 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Move_to_Condition_Register_Fields(cpu *processor,
                                         instruction_word instruction,
                                         unsigned_word cia,
                                         idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Move_to_Condition_Register_Fields
#undef MY_PREFIX
#define MY_PREFIX Move_to_Condition_Register_Fields
#line 3431 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Move to Condition Register Fields\n"));
#line 19439 "icache.c"

  /* validate: 0.31,6.RS,11./,12.FXM,20./,21.144,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc100fff) != 0x7c000120)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11./,12.FXM,20./,21.144,31./ #define get-values-from-icache|put-values-in-icache */
#line 44 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.XFX.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 45 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.XFX.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 46 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.XFX.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 3431 "./ppc-instructions"
    unsigned const FXM UNUSED = cache_entry->crack.XFX.FXM = EXTRACTED32(instruction, 12, 19); /* default in icache - extraction */
#line 19456 "icache.c"

#define insn cache_entry->crack.XFX.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Move_to_Condition_Register_Fields;

    /* semantic routine */
#line 3431 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Move to Condition Register Fields\n"));
#line 19466 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Move_to_Condition_Register_Fields, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3431 "./ppc-instructions"
    {
      if (FXM == 0xff) {
        CR = *rS;
      }
      else {
        unsigned_word mask = 0;
        unsigned_word f;
        for (f = 0; f < 8; f++) {
          if (FXM & (0x80 >> f))
            mask |= (0xf << 4*(7-f));
        }
        CR = (MASKED(*rS, 32, 63) & mask) | (CR & ~mask);
      }
      PPC_INSN_MTCR(RS_BITMASK, FXM);
    }
#line 19491 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11./,12.FXM,20./,21.144,31./ #undef get-values-from-icache|put-values-in-icache */
#line 19494 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Move_To_Machine_State_Register(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Move_To_Machine_State_Register
#undef MY_PREFIX
#define MY_PREFIX Move_To_Machine_State_Register
#line 4871 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Move To Machine State Register\n"));
#line 19513 "icache.c"

  /* validate: 0.31,6.RS,11./,16./,21.146,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc1fffff) != 0x7c000124)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11./,16./,21.146,31./ #define get-values-from-icache|put-values-in-icache */
#line 44 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 45 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 46 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 19528 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Move_To_Machine_State_Register;

    /* semantic routine */
#line 4871 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Move To Machine State Register\n"));
#line 19538 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Move_To_Machine_State_Register, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4871 "./ppc-instructions"
    {
      if (IS_PROBLEM_STATE(processor))
        program_interrupt(processor, cia,
                          privileged_instruction_program_interrupt);
      else {
        MSR = *rS;
        check_masked_interrupts(processor);
      }
    }
#line 19557 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11./,16./,21.146,31./ #undef get-values-from-icache|put-values-in-icache */
#line 19560 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Store_Word_Conditional_Indexed(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Store_Word_Conditional_Indexed
#undef MY_PREFIX
#define MY_PREFIX Store_Word_Conditional_Indexed
#line 2451 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Store Word Conditional Indexed\n"));
#line 19579 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16.RB,21.150,31.1 */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c00012d)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.150,31.1 #define get-values-from-icache|put-values-in-icache */
#line 44 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 45 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 46 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 19607 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Store_Word_Conditional_Indexed;

    /* semantic routine */
#line 2451 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Store Word Conditional Indexed\n"));
#line 19617 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Store_Word_Conditional_Indexed, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2451 "./ppc-instructions"
    {
      unsigned_word b;
      unsigned_word EA;
      if (RA_is_0) b = 0;
      else         b = *rA;
      EA = b + *rB;
      if (RESERVE) {
        if (RESERVE_ADDR == real_addr(EA, 0/*is-read?*/)
            && /*HACK*/ RESERVE_DATA == MEM(unsigned, EA, 4)) {
          STORE(EA, 4, *rS);
          CR_SET_XER_SO(0, cr_i_zero);
        }
        else {
          /* ment to randomly to store, we never do! */	
          CR_SET_XER_SO(0, 0);
        }
        RESERVE = 0;
      }
      else {
        CR_SET_XER_SO(0, 0);
      }
      PPC_INSN_INT(0, (RA_BITMASK & ~1) | RB_BITMASK | RS_BITMASK, 1/*Rc*/);
    }
#line 19650 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.150,31.1 #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 19654 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Store_Word_Indexed(cpu *processor,
                          instruction_word instruction,
                          unsigned_word cia,
                          idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Store_Word_Indexed
#undef MY_PREFIX
#define MY_PREFIX Store_Word_Indexed
#line 2126 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Store Word Indexed\n"));
#line 19673 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16.RB,21.151,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c00012e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.151,31./ #define get-values-from-icache|put-values-in-icache */
#line 44 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 45 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 46 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 19701 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Store_Word_Indexed;

    /* semantic routine */
#line 2126 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Store Word Indexed\n"));
#line 19711 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Store_Word_Indexed, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2126 "./ppc-instructions"
    {
      unsigned_word b;
      unsigned_word EA;
      if (RA_is_0) b = 0;
      else         b = *rA;
      EA = b + *rB;
      STORE(EA, 4, *rS);
      PPC_INSN_INT(0, (RA_BITMASK & ~1) | RB_BITMASK | RS_BITMASK, 0);
    }
#line 19730 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.151,31./ #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 19734 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Store_Word_with_Update_Indexed(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Store_Word_with_Update_Indexed
#undef MY_PREFIX
#define MY_PREFIX Store_Word_with_Update_Indexed
#line 2153 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Store Word with Update Indexed\n"));
#line 19753 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16.RB,21.183,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c00016e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.183,31./ #define get-values-from-icache|put-values-in-icache */
#line 44 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 45 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 46 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 19781 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Store_Word_with_Update_Indexed;

    /* semantic routine */
#line 2153 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Store Word with Update Indexed\n"));
#line 19791 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Store_Word_with_Update_Indexed, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2153 "./ppc-instructions"
    {
      unsigned_word EA;
      if (RA_is_0)
        program_interrupt(processor, cia,
                          illegal_instruction_program_interrupt);
      EA = *rA + *rB;
      STORE(EA, 4, *rS);
      *rA = EA;
      PPC_INSN_INT(RA_BITMASK, RA_BITMASK | RB_BITMASK | RS_BITMASK, 0);
    }
#line 19811 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.183,31./ #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 19815 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Subtract_from_Zero_Extended_OE0_Rc0(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Subtract_from_Zero_Extended
#undef MY_PREFIX
#define MY_PREFIX Subtract_from_Zero_Extended_OE0_Rc0
#line 2666 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Subtract from Zero Extended\n"));
#line 19834 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16./,21.OE,22.200,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00fbfe) != 0x7c000190)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.200,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 2666 "./ppc-instructions"
    unsigned const OE UNUSED = 0; /* default in icache - constant field */
#line 2666 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 19860 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Subtract_from_Zero_Extended_OE0_Rc0;

    /* semantic routine */
#line 2666 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Subtract from Zero Extended\n"));
#line 19870 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Subtract_from_Zero_Extended, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2666 "./ppc-instructions"
    {
      ALU_BEGIN(*rA);
      ALU_NOT;
      ALU_ADD_CA;
      ALU_END(*rT, 1/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK, Rc);
    }
#line 19887 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.200,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 19891 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Subtract_from_Zero_Extended_OE0_Rc1(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Subtract_from_Zero_Extended
#undef MY_PREFIX
#define MY_PREFIX Subtract_from_Zero_Extended_OE0_Rc1
#line 2666 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Subtract from Zero Extended\n"));
#line 19910 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16./,21.OE,22.200,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00fbfe) != 0x7c000190)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.200,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 2666 "./ppc-instructions"
    unsigned const OE UNUSED = 0; /* default in icache - constant field */
#line 2666 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 19936 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Subtract_from_Zero_Extended_OE0_Rc1;

    /* semantic routine */
#line 2666 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Subtract from Zero Extended\n"));
#line 19946 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Subtract_from_Zero_Extended, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2666 "./ppc-instructions"
    {
      ALU_BEGIN(*rA);
      ALU_NOT;
      ALU_ADD_CA;
      ALU_END(*rT, 1/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK, Rc);
    }
#line 19963 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.200,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 19967 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Add_to_Zero_Extended_OE0_Rc0(cpu *processor,
                                    instruction_word instruction,
                                    unsigned_word cia,
                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Add_to_Zero_Extended
#undef MY_PREFIX
#define MY_PREFIX Add_to_Zero_Extended_OE0_Rc0
#line 2656 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Add to Zero Extended\n"));
#line 19986 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16./,21.OE,22.202,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00fbfe) != 0x7c000194)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.202,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 2656 "./ppc-instructions"
    unsigned const OE UNUSED = 0; /* default in icache - constant field */
#line 2656 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 20012 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Add_to_Zero_Extended_OE0_Rc0;

    /* semantic routine */
#line 2656 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Add to Zero Extended\n"));
#line 20022 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Add_to_Zero_Extended, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2656 "./ppc-instructions"
    {
      ALU_BEGIN(*rA);
      ALU_ADD_CA;
      ALU_END(*rT, 1/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK, Rc);
    }
#line 20038 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.202,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 20042 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Add_to_Zero_Extended_OE0_Rc1(cpu *processor,
                                    instruction_word instruction,
                                    unsigned_word cia,
                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Add_to_Zero_Extended
#undef MY_PREFIX
#define MY_PREFIX Add_to_Zero_Extended_OE0_Rc1
#line 2656 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Add to Zero Extended\n"));
#line 20061 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16./,21.OE,22.202,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00fbfe) != 0x7c000194)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.202,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 2656 "./ppc-instructions"
    unsigned const OE UNUSED = 0; /* default in icache - constant field */
#line 2656 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 20087 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Add_to_Zero_Extended_OE0_Rc1;

    /* semantic routine */
#line 2656 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Add to Zero Extended\n"));
#line 20097 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Add_to_Zero_Extended, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2656 "./ppc-instructions"
    {
      ALU_BEGIN(*rA);
      ALU_ADD_CA;
      ALU_END(*rT, 1/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK, Rc);
    }
#line 20113 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.202,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 20117 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Move_To_Segment_Register(cpu *processor,
                                instruction_word instruction,
                                unsigned_word cia,
                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Move_To_Segment_Register
#undef MY_PREFIX
#define MY_PREFIX Move_To_Segment_Register
#line 4917 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Move To Segment Register\n"));
#line 20136 "icache.c"

  /* validate: 0.31,6.RS,11./,12.SR,16./,21.210,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc10ffff) != 0x7c0001a4)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11./,12.SR,16./,21.210,31./ #define get-values-from-icache|put-values-in-icache */
#line 44 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 45 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 46 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 4917 "./ppc-instructions"
    unsigned const SR UNUSED = cache_entry->crack.X.SR = EXTRACTED32(instruction, 12, 15); /* default in icache - extraction */
#line 20153 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Move_To_Segment_Register;

    /* semantic routine */
#line 4917 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Move To Segment Register\n"));
#line 20163 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Move_To_Segment_Register, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4917 "./ppc-instructions"
    {
      if (IS_PROBLEM_STATE(processor))
        program_interrupt(processor, cia,
                          privileged_instruction_program_interrupt);
      else
        SEGREG(SR) = *rS;
    }
#line 20180 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11./,12.SR,16./,21.210,31./ #undef get-values-from-icache|put-values-in-icache */
#line 20183 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Store_Byte_Indexed(cpu *processor,
                          instruction_word instruction,
                          unsigned_word cia,
                          idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Store_Byte_Indexed
#undef MY_PREFIX
#define MY_PREFIX Store_Byte_Indexed
#line 2018 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Store Byte Indexed\n"));
#line 20202 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16.RB,21.215,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c0001ae)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.215,31./ #define get-values-from-icache|put-values-in-icache */
#line 44 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 45 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 46 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 20230 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Store_Byte_Indexed;

    /* semantic routine */
#line 2018 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Store Byte Indexed\n"));
#line 20240 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Store_Byte_Indexed, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2018 "./ppc-instructions"
    {
      unsigned_word b;
      unsigned_word EA;
      if (RA_is_0) b = 0;
      else         b = *rA;
      EA = b + *rB;
      STORE(EA, 1, *rS);
      PPC_INSN_INT(0, (RA_BITMASK & ~1) | RB_BITMASK | RS_BITMASK, 0);
    }
#line 20259 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.215,31./ #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 20263 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Subtract_From_Minus_One_Extended_OE0_Rc0(cpu *processor,
                                                instruction_word instruction,
                                                unsigned_word cia,
                                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Subtract_From_Minus_One_Extended
#undef MY_PREFIX
#define MY_PREFIX Subtract_From_Minus_One_Extended_OE0_Rc0
#line 2644 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Subtract From Minus One Extended\n"));
#line 20282 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16./,21.OE,22.232,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00fbfe) != 0x7c0001d0)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.232,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 2644 "./ppc-instructions"
    unsigned const OE UNUSED = 0; /* default in icache - constant field */
#line 2644 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 20308 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Subtract_From_Minus_One_Extended_OE0_Rc0;

    /* semantic routine */
#line 2644 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Subtract From Minus One Extended\n"));
#line 20318 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Subtract_From_Minus_One_Extended, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2644 "./ppc-instructions"
    {
      ALU_BEGIN(*rA);
      ALU_NOT;
      ALU_ADD_CA;
      ALU_ADD(-1);
      ALU_END(*rT, 1/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK, Rc);
    }
#line 20336 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.232,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 20340 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Subtract_From_Minus_One_Extended_OE0_Rc1(cpu *processor,
                                                instruction_word instruction,
                                                unsigned_word cia,
                                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Subtract_From_Minus_One_Extended
#undef MY_PREFIX
#define MY_PREFIX Subtract_From_Minus_One_Extended_OE0_Rc1
#line 2644 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Subtract From Minus One Extended\n"));
#line 20359 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16./,21.OE,22.232,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00fbfe) != 0x7c0001d0)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.232,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 2644 "./ppc-instructions"
    unsigned const OE UNUSED = 0; /* default in icache - constant field */
#line 2644 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 20385 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Subtract_From_Minus_One_Extended_OE0_Rc1;

    /* semantic routine */
#line 2644 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Subtract From Minus One Extended\n"));
#line 20395 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Subtract_From_Minus_One_Extended, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2644 "./ppc-instructions"
    {
      ALU_BEGIN(*rA);
      ALU_NOT;
      ALU_ADD_CA;
      ALU_ADD(-1);
      ALU_END(*rT, 1/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK, Rc);
    }
#line 20413 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.232,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 20417 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Add_to_Minus_One_Extended_OE0_Rc0(cpu *processor,
                                         instruction_word instruction,
                                         unsigned_word cia,
                                         idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Add_to_Minus_One_Extended
#undef MY_PREFIX
#define MY_PREFIX Add_to_Minus_One_Extended_OE0_Rc0
#line 2633 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Add to Minus One Extended\n"));
#line 20436 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16./,21.OE,22.234,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00fbfe) != 0x7c0001d4)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.234,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 2633 "./ppc-instructions"
    unsigned const OE UNUSED = 0; /* default in icache - constant field */
#line 2633 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 20462 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Add_to_Minus_One_Extended_OE0_Rc0;

    /* semantic routine */
#line 2633 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Add to Minus One Extended\n"));
#line 20472 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Add_to_Minus_One_Extended, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2633 "./ppc-instructions"
    {
      ALU_BEGIN(*rA);
      ALU_ADD_CA;
      ALU_ADD(-1);
      ALU_END(*rT, 1/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK, Rc);
    }
#line 20489 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.234,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 20493 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Add_to_Minus_One_Extended_OE0_Rc1(cpu *processor,
                                         instruction_word instruction,
                                         unsigned_word cia,
                                         idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Add_to_Minus_One_Extended
#undef MY_PREFIX
#define MY_PREFIX Add_to_Minus_One_Extended_OE0_Rc1
#line 2633 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Add to Minus One Extended\n"));
#line 20512 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16./,21.OE,22.234,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00fbfe) != 0x7c0001d4)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.234,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 2633 "./ppc-instructions"
    unsigned const OE UNUSED = 0; /* default in icache - constant field */
#line 2633 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 20538 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Add_to_Minus_One_Extended_OE0_Rc1;

    /* semantic routine */
#line 2633 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Add to Minus One Extended\n"));
#line 20548 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Add_to_Minus_One_Extended, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2633 "./ppc-instructions"
    {
      ALU_BEGIN(*rA);
      ALU_ADD_CA;
      ALU_ADD(-1);
      ALU_END(*rT, 1/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK, Rc);
    }
#line 20565 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.234,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 20569 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Multiply_Low_Word_OE0_Rc0(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Multiply_Low_Word
#undef MY_PREFIX
#define MY_PREFIX Multiply_Low_Word_OE0_Rc0
#line 2699 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Multiply Low Word\n"));
#line 20588 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.OE,22.235,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0003fe) != 0x7c0001d6)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.235,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2699 "./ppc-instructions"
    unsigned const OE UNUSED = 0; /* default in icache - constant field */
#line 2699 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 20620 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Multiply_Low_Word_OE0_Rc0;

    /* semantic routine */
#line 2699 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Multiply Low Word\n"));
#line 20630 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Multiply_Low_Word, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2699 "./ppc-instructions"
    {
      signed64 a = (signed32)(*rA);
      signed64 b = (signed32)(*rB);
      signed64 prod = a * b;
      signed_word t = prod;
      *rT = *rA * *rB;
      if (t != prod && OE)
        XER |= (xer_overflow | xer_summary_overflow);
      CR0_COMPARE(t, 0, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 20651 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.235,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 20655 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Multiply_Low_Word_OE0_Rc1(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Multiply_Low_Word
#undef MY_PREFIX
#define MY_PREFIX Multiply_Low_Word_OE0_Rc1
#line 2699 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Multiply Low Word\n"));
#line 20674 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.OE,22.235,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0003fe) != 0x7c0001d6)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.235,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2699 "./ppc-instructions"
    unsigned const OE UNUSED = 0; /* default in icache - constant field */
#line 2699 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 20706 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Multiply_Low_Word_OE0_Rc1;

    /* semantic routine */
#line 2699 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Multiply Low Word\n"));
#line 20716 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Multiply_Low_Word, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2699 "./ppc-instructions"
    {
      signed64 a = (signed32)(*rA);
      signed64 b = (signed32)(*rB);
      signed64 prod = a * b;
      signed_word t = prod;
      *rT = *rA * *rB;
      if (t != prod && OE)
        XER |= (xer_overflow | xer_summary_overflow);
      CR0_COMPARE(t, 0, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 20737 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.235,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 20741 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Move_To_Segment_Register_Indirect(cpu *processor,
                                         instruction_word instruction,
                                         unsigned_word cia,
                                         idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Move_To_Segment_Register_Indirect
#undef MY_PREFIX
#define MY_PREFIX Move_To_Segment_Register_Indirect
#line 4928 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Move To Segment Register Indirect\n"));
#line 20760 "icache.c"

  /* validate: 0.31,6.RS,11./,16.RB,21.242,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc1f07ff) != 0x7c0001e4)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11./,16.RB,21.242,31./ #define get-values-from-icache|put-values-in-icache */
#line 44 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 45 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 46 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 20781 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Move_To_Segment_Register_Indirect;

    /* semantic routine */
#line 4928 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Move To Segment Register Indirect\n"));
#line 20791 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Move_To_Segment_Register_Indirect, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4928 "./ppc-instructions"
    {
      if (IS_PROBLEM_STATE(processor))
        program_interrupt(processor, cia,
                          privileged_instruction_program_interrupt);
      else
        SEGREG(EXTRACTED32(*rB, 0, 3)) = *rS;
    }
#line 20808 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11./,16.RB,21.242,31./ #undef get-values-from-icache|put-values-in-icache */
#line 20811 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Data_Cache_Block_Touch_for_Store(cpu *processor,
                                        instruction_word instruction,
                                        unsigned_word cia,
                                        idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Data_Cache_Block_Touch_for_Store
#undef MY_PREFIX
#define MY_PREFIX Data_Cache_Block_Touch_for_Store
#line 4781 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Data Cache Block Touch for Store\n"));
#line 20830 "icache.c"

  /* validate: 0.31,6./,11.RA,16.RB,21.246,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xffe007ff) != 0x7c0001ec)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6./,11.RA,16.RB,21.246,31./ #define get-values-from-icache|put-values-in-icache */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 20852 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Data_Cache_Block_Touch_for_Store;

    /* semantic routine */
#line 4781 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Data Cache Block Touch for Store\n"));
#line 20862 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Data_Cache_Block_Touch_for_Store, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4781 "./ppc-instructions"
    {
      TRACE(trace_tbd,("Data Cache Block Touch for Store\n"));
      PPC_INSN_INT(0, (RA_BITMASK & ~1) | RB_BITMASK, 0/*Rc*/);
    }
#line 20876 "icache.c"
    return nia;
    /* extraction: 0.31,6./,11.RA,16.RB,21.246,31./ #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 20880 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Store_Byte_with_Update_Indexed(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Store_Byte_with_Update_Indexed
#undef MY_PREFIX
#define MY_PREFIX Store_Byte_with_Update_Indexed
#line 2045 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Store Byte with Update Indexed\n"));
#line 20899 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16.RB,21.247,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c0001ee)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.247,31./ #define get-values-from-icache|put-values-in-icache */
#line 44 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 45 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 46 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 20927 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Store_Byte_with_Update_Indexed;

    /* semantic routine */
#line 2045 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Store Byte with Update Indexed\n"));
#line 20937 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Store_Byte_with_Update_Indexed, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2045 "./ppc-instructions"
    {
      unsigned_word EA;
      if (RA_is_0)
        program_interrupt(processor, cia,
                          illegal_instruction_program_interrupt);
      EA = *rA + *rB;
      STORE(EA, 1, *rS);
      *rA = EA;
      PPC_INSN_INT(RA_BITMASK, RA_BITMASK | RB_BITMASK | RS_BITMASK, 0);
    }
#line 20957 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.247,31./ #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 20961 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Add_OE0_Rc0(cpu *processor,
                   instruction_word instruction,
                   unsigned_word cia,
                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Add
#undef MY_PREFIX
#define MY_PREFIX Add_OE0_Rc0
#line 2533 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Add\n"));
#line 20980 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.OE,22.266,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0003fe) != 0x7c000214)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.266,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2533 "./ppc-instructions"
    unsigned const OE UNUSED = 0; /* default in icache - constant field */
#line 2533 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 21012 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Add_OE0_Rc0;

    /* semantic routine */
#line 2533 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Add\n"));
#line 21022 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Add, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2533 "./ppc-instructions"
    {
      ALU_BEGIN(*rA);
      ALU_ADD(*rB);
      ALU_END(*rT, 0/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 21038 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.266,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 21042 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Add_OE0_Rc1(cpu *processor,
                   instruction_word instruction,
                   unsigned_word cia,
                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Add
#undef MY_PREFIX
#define MY_PREFIX Add_OE0_Rc1
#line 2533 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Add\n"));
#line 21061 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.OE,22.266,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0003fe) != 0x7c000214)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.266,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2533 "./ppc-instructions"
    unsigned const OE UNUSED = 0; /* default in icache - constant field */
#line 2533 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 21093 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Add_OE0_Rc1;

    /* semantic routine */
#line 2533 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Add\n"));
#line 21103 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Add, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2533 "./ppc-instructions"
    {
      ALU_BEGIN(*rA);
      ALU_ADD(*rB);
      ALU_END(*rT, 0/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 21119 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.266,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 21123 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Data_Cache_Block_Touch(cpu *processor,
                              instruction_word instruction,
                              unsigned_word cia,
                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Data_Cache_Block_Touch
#undef MY_PREFIX
#define MY_PREFIX Data_Cache_Block_Touch
#line 4773 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Data Cache Block Touch\n"));
#line 21142 "icache.c"

  /* validate: 0.31,6./,11.RA,16.RB,21.278,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xffe007ff) != 0x7c00022c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6./,11.RA,16.RB,21.278,31./ #define get-values-from-icache|put-values-in-icache */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 21164 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Data_Cache_Block_Touch;

    /* semantic routine */
#line 4773 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Data Cache Block Touch\n"));
#line 21174 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Data_Cache_Block_Touch, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4773 "./ppc-instructions"
    {
      TRACE(trace_tbd,("Data Cache Block Touch\n"));
      PPC_INSN_INT(0, (RA_BITMASK & ~1) | RB_BITMASK, 0/*Rc*/);
    }
#line 21188 "icache.c"
    return nia;
    /* extraction: 0.31,6./,11.RA,16.RB,21.278,31./ #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 21192 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Load_Halfword_and_Zero_Indexed(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Load_Halfword_and_Zero_Indexed
#undef MY_PREFIX
#define MY_PREFIX Load_Halfword_and_Zero_Indexed
#line 1791 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Load Halfword and Zero Indexed\n"));
#line 21211 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.279,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c00022e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.279,31./ #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.X.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.X.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.X.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 21239 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Load_Halfword_and_Zero_Indexed;

    /* semantic routine */
#line 1791 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Load Halfword and Zero Indexed\n"));
#line 21249 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Load_Halfword_and_Zero_Indexed, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1791 "./ppc-instructions"
    {
      unsigned_word b;
      unsigned_word EA;
      if (RA_is_0) b = 0;
      else         b = *rA;
      EA = b + *rB;
      *rT = MEM(unsigned, EA, 2);
      PPC_INSN_INT(RT_BITMASK, (RA_BITMASK & ~1) | RB_BITMASK, 0);
    }
#line 21268 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.279,31./ #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 21272 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Equivalent_Rc0(cpu *processor,
                      instruction_word instruction,
                      unsigned_word cia,
                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Equivalent
#undef MY_PREFIX
#define MY_PREFIX Equivalent_Rc0
#line 3061 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Equivalent\n"));
#line 21291 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16.RB,21.284,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0x7c000238)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.284,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 44 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 45 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 46 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 3061 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 21321 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Equivalent_Rc0;

    /* semantic routine */
#line 3061 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Equivalent\n"));
#line 21331 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Equivalent, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3061 "./ppc-instructions"
    {
      *rA = ~(*rS ^ *rB); /* A === B */
      CR0_COMPARE(*rA, 0, Rc);
      ITRACE(trace_alu, (" Result = %ld (0x%lx)\n", (long)*rA, (long)*rA));
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK | RB_BITMASK, Rc);
    }
#line 21347 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.284,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 21351 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Equivalent_Rc1(cpu *processor,
                      instruction_word instruction,
                      unsigned_word cia,
                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Equivalent
#undef MY_PREFIX
#define MY_PREFIX Equivalent_Rc1
#line 3061 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Equivalent\n"));
#line 21370 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16.RB,21.284,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0x7c000238)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.284,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 44 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 45 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 46 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 3061 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 21400 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Equivalent_Rc1;

    /* semantic routine */
#line 3061 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Equivalent\n"));
#line 21410 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Equivalent, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3061 "./ppc-instructions"
    {
      *rA = ~(*rS ^ *rB); /* A === B */
      CR0_COMPARE(*rA, 0, Rc);
      ITRACE(trace_alu, (" Result = %ld (0x%lx)\n", (long)*rA, (long)*rA));
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK | RB_BITMASK, Rc);
    }
#line 21426 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.284,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 21430 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_TLB_Invalidate_Entry(cpu *processor,
                            instruction_word instruction,
                            unsigned_word cia,
                            idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_TLB_Invalidate_Entry
#undef MY_PREFIX
#define MY_PREFIX TLB_Invalidate_Entry
#line 4966 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode TLB Invalidate Entry\n"));
#line 21449 "icache.c"

  /* validate: 0.31,6./,11./,16.RB,21.306,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xffff07ff) != 0x7c000264)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6./,11./,16.RB,21.306,31./ #define get-values-from-icache|put-values-in-icache */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 21464 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_TLB_Invalidate_Entry;

    /* semantic routine */
#line 4966 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics TLB Invalidate Entry\n"));
#line 21474 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_TLB_Invalidate_Entry, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4966 "./ppc-instructions"
    {
      if (IS_PROBLEM_STATE(processor))
        program_interrupt(processor, cia,
                          privileged_instruction_program_interrupt);
      else {
        int nr = 0;
        cpu *proc;
        while (1) {
          proc = psim_cpu(cpu_system(processor), nr);
          if (proc == NULL) break;
          cpu_page_tlb_invalidate_entry(proc, *rB);
          nr++;
        }
      }
    }
#line 21499 "icache.c"
    return nia;
    /* extraction: 0.31,6./,11./,16.RB,21.306,31./ #undef get-values-from-icache|put-values-in-icache */
#line 21502 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Load_Halfword_and_Zero_with_Update_Indexed(cpu *processor,
                                                  instruction_word instruction,
                                                  unsigned_word cia,
                                                  idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Load_Halfword_and_Zero_with_Update_Indexed
#undef MY_PREFIX
#define MY_PREFIX Load_Halfword_and_Zero_with_Update_Indexed
#line 1818 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Load Halfword and Zero with Update Indexed\n"));
#line 21521 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.311,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c00026e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.311,31./ #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.X.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.X.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.X.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 21549 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Load_Halfword_and_Zero_with_Update_Indexed;

    /* semantic routine */
#line 1818 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Load Halfword and Zero with Update Indexed\n"));
#line 21559 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Load_Halfword_and_Zero_with_Update_Indexed, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1818 "./ppc-instructions"
    {
      unsigned_word EA;
      if (RA_is_0 || RA == RT)
        program_interrupt(processor, cia,
                          illegal_instruction_program_interrupt);
      EA = *rA + *rB;
      *rT = MEM(unsigned, EA, 2);
      *rA = EA;
      PPC_INSN_INT(RT_BITMASK | RA_BITMASK, RA_BITMASK | RB_BITMASK, 0);
    }
#line 21579 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.311,31./ #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 21583 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_XOR_Rc0(cpu *processor,
               instruction_word instruction,
               unsigned_word cia,
               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_XOR
#undef MY_PREFIX
#define MY_PREFIX XOR_Rc0
#line 3031 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode XOR\n"));
#line 21602 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16.RB,21.316,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0x7c000278)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.316,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 44 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 45 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 46 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 3031 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 21632 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_XOR_Rc0;

    /* semantic routine */
#line 3031 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics XOR\n"));
#line 21642 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_XOR, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3031 "./ppc-instructions"
    {
      *rA = *rS ^ *rB;
      CR0_COMPARE(*rA, 0, Rc);
      ITRACE(trace_alu, (" Result = %ld (0x%lx)\n", (long)*rA, (long)*rA));
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK | RB_BITMASK, Rc);
    }
#line 21658 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.316,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 21662 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_XOR_Rc1(cpu *processor,
               instruction_word instruction,
               unsigned_word cia,
               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_XOR
#undef MY_PREFIX
#define MY_PREFIX XOR_Rc1
#line 3031 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode XOR\n"));
#line 21681 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16.RB,21.316,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0x7c000278)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.316,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 44 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 45 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 46 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 3031 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 21711 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_XOR_Rc1;

    /* semantic routine */
#line 3031 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics XOR\n"));
#line 21721 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_XOR, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3031 "./ppc-instructions"
    {
      *rA = *rS ^ *rB;
      CR0_COMPARE(*rA, 0, Rc);
      ITRACE(trace_alu, (" Result = %ld (0x%lx)\n", (long)*rA, (long)*rA));
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK | RB_BITMASK, Rc);
    }
#line 21737 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.316,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 21741 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Move_from_Special_Purpose_Register_SPR256(cpu *processor,
                                                 instruction_word instruction,
                                                 unsigned_word cia,
                                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Move_from_Special_Purpose_Register
#undef MY_PREFIX
#define MY_PREFIX Move_from_Special_Purpose_Register_SPR256
#line 3390 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Move from Special Purpose Register\n"));
#line 21760 "icache.c"

  /* validate: 0.31,6.RT,11.SPR,21.339,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c0002a6)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.SPR,21.339,31./ #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XFX.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XFX.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XFX.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 3390 "./ppc-instructions"
    unsigned const SPR UNUSED = 256; /* default in icache - constant field */
#line 21777 "icache.c"

#define insn cache_entry->crack.XFX.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Move_from_Special_Purpose_Register_SPR256;

    /* semantic routine */
#line 3390 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Move from Special Purpose Register\n"));
#line 21787 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Move_from_Special_Purpose_Register, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3390 "./ppc-instructions"
    {
      int n = (SPR_5_9_ << 5) | SPR_0_4_;
      if (SPR_0_ && IS_PROBLEM_STATE(processor))
        program_interrupt(processor, cia,
                          privileged_instruction_program_interrupt);
      else if (!spr_is_valid(n))
        program_interrupt(processor, cia,
                          illegal_instruction_program_interrupt);
      else {
        /* HACK - time base registers need to be calculated */
        if (WITH_TIME_BASE) {
          switch (n) {
          case spr_dec:
            *rT = cpu_get_decrementer(processor);
            break;
      	case spr_tbrl:
        	  if (is_64bit_implementation) *rT = TB;
        	  else                         *rT = EXTRACTED64(TB, 32, 63);
      	break;
      	case spr_tbru:
        	  if (is_64bit_implementation) *rT = EXTRACTED64(TB, 0, 31);
      	  else                         *rT = EXTRACTED64(TB, 0, 31);
      	break;
          case spr_tbu:
          case spr_tbl:
            /* NOTE - these SPR's are not readable. Use mftb[ul] */
          default:
            *rT = SPREG(n);
            break;
          }
        }
        else {
          *rT = SPREG(n);
        }
      }
      PPC_INSN_FROM_SPR(RT_BITMASK, n);
    }
#line 21834 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.SPR,21.339,31./ #undef get-values-from-icache|put-values-in-icache */
#line 21837 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Move_from_Special_Purpose_Register_SPR(cpu *processor,
                                              instruction_word instruction,
                                              unsigned_word cia,
                                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Move_from_Special_Purpose_Register
#undef MY_PREFIX
#define MY_PREFIX Move_from_Special_Purpose_Register_SPR
#line 3390 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Move from Special Purpose Register\n"));
#line 21856 "icache.c"

  /* validate: 0.31,6.RT,11.SPR,21.339,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c0002a6)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.SPR,21.339,31./ #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XFX.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XFX.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XFX.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 3390 "./ppc-instructions"
    unsigned const SPR UNUSED = cache_entry->crack.XFX.SPR = EXTRACTED32(instruction, 11, 20); /* default in icache - extraction */
#line 21873 "icache.c"

#define insn cache_entry->crack.XFX.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Move_from_Special_Purpose_Register_SPR;

    /* semantic routine */
#line 3390 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Move from Special Purpose Register\n"));
#line 21883 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Move_from_Special_Purpose_Register, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3390 "./ppc-instructions"
    {
      int n = (SPR_5_9_ << 5) | SPR_0_4_;
      if (SPR_0_ && IS_PROBLEM_STATE(processor))
        program_interrupt(processor, cia,
                          privileged_instruction_program_interrupt);
      else if (!spr_is_valid(n))
        program_interrupt(processor, cia,
                          illegal_instruction_program_interrupt);
      else {
        /* HACK - time base registers need to be calculated */
        if (WITH_TIME_BASE) {
          switch (n) {
          case spr_dec:
            *rT = cpu_get_decrementer(processor);
            break;
      	case spr_tbrl:
        	  if (is_64bit_implementation) *rT = TB;
        	  else                         *rT = EXTRACTED64(TB, 32, 63);
      	break;
      	case spr_tbru:
        	  if (is_64bit_implementation) *rT = EXTRACTED64(TB, 0, 31);
      	  else                         *rT = EXTRACTED64(TB, 0, 31);
      	break;
          case spr_tbu:
          case spr_tbl:
            /* NOTE - these SPR's are not readable. Use mftb[ul] */
          default:
            *rT = SPREG(n);
            break;
          }
        }
        else {
          *rT = SPREG(n);
        }
      }
      PPC_INSN_FROM_SPR(RT_BITMASK, n);
    }
#line 21930 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.SPR,21.339,31./ #undef get-values-from-icache|put-values-in-icache */
#line 21933 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Load_Halfword_Algebraic_Indexed(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Load_Halfword_Algebraic_Indexed
#undef MY_PREFIX
#define MY_PREFIX Load_Halfword_Algebraic_Indexed
#line 1845 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Load Halfword Algebraic Indexed\n"));
#line 21952 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.343,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c0002ae)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.343,31./ #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.X.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.X.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.X.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 21980 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Load_Halfword_Algebraic_Indexed;

    /* semantic routine */
#line 1845 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Load Halfword Algebraic Indexed\n"));
#line 21990 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Load_Halfword_Algebraic_Indexed, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1845 "./ppc-instructions"
    {
      unsigned_word b;
      unsigned_word EA;
      if (RA_is_0) b = 0;
      else         b = *rA;
      EA = b + *rB;
      *rT = MEM(signed, EA, 2);
      PPC_INSN_INT(RT_BITMASK, (RA_BITMASK & ~1) | RB_BITMASK, 0);
    }
#line 22009 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.343,31./ #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 22013 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_TLB_Invalidate_All(cpu *processor,
                          instruction_word instruction,
                          unsigned_word cia,
                          idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_TLB_Invalidate_All
#undef MY_PREFIX
#define MY_PREFIX TLB_Invalidate_All
#line 4981 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode TLB Invalidate All\n"));
#line 22032 "icache.c"

  /* validate: 0.31,6./,11./,16./,21.370,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xffffffff) != 0x7c0002e4)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6./,11./,16./,21.370,31./ #define get-values-from-icache|put-values-in-icache */
#line 22041 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_TLB_Invalidate_All;

    /* semantic routine */
#line 4981 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics TLB Invalidate All\n"));
#line 22051 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_TLB_Invalidate_All, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4981 "./ppc-instructions"
    {
      if (IS_PROBLEM_STATE(processor))
        program_interrupt(processor, cia,
                          privileged_instruction_program_interrupt);
      else {
        int nr = 0;
        cpu *proc;
        while (1) {
          proc = psim_cpu(cpu_system(processor), nr);
          if (proc == NULL) break;
          cpu_page_tlb_invalidate_all(proc);
          nr++;
        }
      }
    }
#line 22076 "icache.c"
    return nia;
    /* extraction: 0.31,6./,11./,16./,21.370,31./ #undef get-values-from-icache|put-values-in-icache */
#line 22079 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Move_From_Time_Base(cpu *processor,
                           instruction_word instruction,
                           unsigned_word cia,
                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Move_From_Time_Base
#undef MY_PREFIX
#define MY_PREFIX Move_From_Time_Base
#line 4824 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Move From Time Base\n"));
#line 22098 "icache.c"

  /* validate: 0.31,6.RT,11.tbr,21.371,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c0002e6)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.tbr,21.371,31./ #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XFX.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XFX.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XFX.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 4824 "./ppc-instructions"
    unsigned const tbr UNUSED = cache_entry->crack.XFX.tbr = EXTRACTED32(instruction, 11, 20); /* default in icache - extraction */
#line 22115 "icache.c"

#define insn cache_entry->crack.XFX.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Move_From_Time_Base;

    /* semantic routine */
#line 4824 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Move From Time Base\n"));
#line 22125 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Move_From_Time_Base, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4824 "./ppc-instructions"
    {
      int n = (tbr_5_9_ << 5) | tbr_0_4_;
      if (n == 268) {
        if (is_64bit_implementation) *rT = TB;
        else                         *rT = EXTRACTED64(TB, 32, 63);
      }
      else if (n == 269) {
        if (is_64bit_implementation) *rT = EXTRACTED64(TB, 0, 31);
        else                         *rT = EXTRACTED64(TB, 0, 31);
      }
      else
        program_interrupt(processor, cia,
                          illegal_instruction_program_interrupt);
    }
#line 22149 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.tbr,21.371,31./ #undef get-values-from-icache|put-values-in-icache */
#line 22152 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Load_Halfword_Algebraic_with_Update_Indexed(cpu *processor,
                                                   instruction_word instruction,
                                                   unsigned_word cia,
                                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Load_Halfword_Algebraic_with_Update_Indexed
#undef MY_PREFIX
#define MY_PREFIX Load_Halfword_Algebraic_with_Update_Indexed
#line 1872 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Load Halfword Algebraic with Update Indexed\n"));
#line 22171 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.375,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c0002ee)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.375,31./ #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.X.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.X.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.X.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 22199 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Load_Halfword_Algebraic_with_Update_Indexed;

    /* semantic routine */
#line 1872 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Load Halfword Algebraic with Update Indexed\n"));
#line 22209 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Load_Halfword_Algebraic_with_Update_Indexed, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1872 "./ppc-instructions"
    {
      unsigned_word EA;
      if (RA_is_0 || RA == RT)
        program_interrupt(processor, cia,
                          illegal_instruction_program_interrupt);
      EA = *rA + *rB;
      *rT = MEM(signed, EA, 2);
      *rA = EA;
      PPC_INSN_INT(RT_BITMASK | RA_BITMASK, RA_BITMASK | RB_BITMASK, 0);
    }
#line 22229 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.375,31./ #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 22233 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Store_Half_Word_Indexed(cpu *processor,
                               instruction_word instruction,
                               unsigned_word cia,
                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Store_Half_Word_Indexed
#undef MY_PREFIX
#define MY_PREFIX Store_Half_Word_Indexed
#line 2072 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Store Half Word Indexed\n"));
#line 22252 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16.RB,21.407,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c00032e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.407,31./ #define get-values-from-icache|put-values-in-icache */
#line 44 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 45 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 46 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 22280 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Store_Half_Word_Indexed;

    /* semantic routine */
#line 2072 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Store Half Word Indexed\n"));
#line 22290 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Store_Half_Word_Indexed, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2072 "./ppc-instructions"
    {
      unsigned_word b;
      unsigned_word EA;
      if (RA_is_0) b = 0;
      else         b = *rA;
      EA = b + *rB;
      STORE(EA, 2, *rS);
      PPC_INSN_INT(0, (RA_BITMASK & ~1) | RB_BITMASK | RS_BITMASK, 0);
    }
#line 22309 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.407,31./ #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 22313 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_OR_with_Complement_Rc0(cpu *processor,
                              instruction_word instruction,
                              unsigned_word cia,
                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_OR_with_Complement
#undef MY_PREFIX
#define MY_PREFIX OR_with_Complement_Rc0
#line 3081 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode OR with Complement\n"));
#line 22332 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16.RB,21.412,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0x7c000338)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.412,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 44 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 45 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 46 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 3081 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 22362 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_OR_with_Complement_Rc0;

    /* semantic routine */
#line 3081 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics OR with Complement\n"));
#line 22372 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_OR_with_Complement, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3081 "./ppc-instructions"
    {
      *rA = *rS | ~*rB;
      CR0_COMPARE(*rA, 0, Rc);
      ITRACE(trace_alu, (" Result = %ld (0x%lx)\n", (long)*rA, (long)*rA));
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK | RB_BITMASK, Rc);
    }
#line 22388 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.412,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 22392 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_OR_with_Complement_Rc1(cpu *processor,
                              instruction_word instruction,
                              unsigned_word cia,
                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_OR_with_Complement
#undef MY_PREFIX
#define MY_PREFIX OR_with_Complement_Rc1
#line 3081 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode OR with Complement\n"));
#line 22411 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16.RB,21.412,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0x7c000338)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.412,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 44 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 45 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 46 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 3081 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 22441 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_OR_with_Complement_Rc1;

    /* semantic routine */
#line 3081 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics OR with Complement\n"));
#line 22451 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_OR_with_Complement, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3081 "./ppc-instructions"
    {
      *rA = *rS | ~*rB;
      CR0_COMPARE(*rA, 0, Rc);
      ITRACE(trace_alu, (" Result = %ld (0x%lx)\n", (long)*rA, (long)*rA));
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK | RB_BITMASK, Rc);
    }
#line 22467 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.412,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 22471 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Store_Half_Word_with_Update_Indexed(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Store_Half_Word_with_Update_Indexed
#undef MY_PREFIX
#define MY_PREFIX Store_Half_Word_with_Update_Indexed
#line 2099 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Store Half Word with Update Indexed\n"));
#line 22490 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16.RB,21.439,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c00036e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.439,31./ #define get-values-from-icache|put-values-in-icache */
#line 44 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 45 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 46 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 22518 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Store_Half_Word_with_Update_Indexed;

    /* semantic routine */
#line 2099 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Store Half Word with Update Indexed\n"));
#line 22528 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Store_Half_Word_with_Update_Indexed, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2099 "./ppc-instructions"
    {
      unsigned_word EA;
      if (RA_is_0)
        program_interrupt(processor, cia,
                          illegal_instruction_program_interrupt);
      EA = *rA + *rB;
      STORE(EA, 2, *rS);
      *rA = EA;
      PPC_INSN_INT(RA_BITMASK, RA_BITMASK | RB_BITMASK | RS_BITMASK, 0);
    }
#line 22548 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.439,31./ #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 22552 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_OR_Rc0(cpu *processor,
              instruction_word instruction,
              unsigned_word cia,
              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_OR
#undef MY_PREFIX
#define MY_PREFIX OR_Rc0
#line 3021 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode OR\n"));
#line 22571 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16.RB,21.444,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0x7c000378)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.444,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 44 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 45 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 46 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 3021 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 22601 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_OR_Rc0;

    /* semantic routine */
#line 3021 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics OR\n"));
#line 22611 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_OR, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3021 "./ppc-instructions"
    {
      *rA = *rS | *rB;
      CR0_COMPARE(*rA, 0, Rc);
      ITRACE(trace_alu, (" Result = %ld (0x%lx)\n", (long)*rA, (long)*rA));
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK | RB_BITMASK, Rc);
    }
#line 22627 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.444,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 22631 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_OR_Rc1(cpu *processor,
              instruction_word instruction,
              unsigned_word cia,
              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_OR
#undef MY_PREFIX
#define MY_PREFIX OR_Rc1
#line 3021 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode OR\n"));
#line 22650 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16.RB,21.444,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0x7c000378)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.444,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 44 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 45 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 46 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 3021 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 22680 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_OR_Rc1;

    /* semantic routine */
#line 3021 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics OR\n"));
#line 22690 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_OR, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3021 "./ppc-instructions"
    {
      *rA = *rS | *rB;
      CR0_COMPARE(*rA, 0, Rc);
      ITRACE(trace_alu, (" Result = %ld (0x%lx)\n", (long)*rA, (long)*rA));
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK | RB_BITMASK, Rc);
    }
#line 22706 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.444,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 22710 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Divide_Word_Unsigned_OE0_Rc0(cpu *processor,
                                    instruction_word instruction,
                                    unsigned_word cia,
                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Divide_Word_Unsigned
#undef MY_PREFIX
#define MY_PREFIX Divide_Word_Unsigned_OE0_Rc0
#line 2768 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Divide Word Unsigned\n"));
#line 22729 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.OE,22.459,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0003fe) != 0x7c000396)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.459,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2768 "./ppc-instructions"
    unsigned const OE UNUSED = 0; /* default in icache - constant field */
#line 2768 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 22761 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Divide_Word_Unsigned_OE0_Rc0;

    /* semantic routine */
#line 2768 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Divide Word Unsigned\n"));
#line 22771 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Divide_Word_Unsigned, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2768 "./ppc-instructions"
    {
      unsigned64 dividend = (unsigned32)(*rA);
      unsigned64 divisor = (unsigned32)(*rB);
      if (divisor == 0) {
        if (OE)
          XER |= (xer_overflow | xer_summary_overflow);
        CR0_COMPARE(0, 0, Rc);
      }
      else {
        unsigned64 quotent = dividend / divisor;
        *rT = quotent;
        CR0_COMPARE((signed_word)quotent, 0, Rc);
      }
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 22796 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.459,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 22800 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Divide_Word_Unsigned_OE0_Rc1(cpu *processor,
                                    instruction_word instruction,
                                    unsigned_word cia,
                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Divide_Word_Unsigned
#undef MY_PREFIX
#define MY_PREFIX Divide_Word_Unsigned_OE0_Rc1
#line 2768 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Divide Word Unsigned\n"));
#line 22819 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.OE,22.459,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0003fe) != 0x7c000396)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.459,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2768 "./ppc-instructions"
    unsigned const OE UNUSED = 0; /* default in icache - constant field */
#line 2768 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 22851 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Divide_Word_Unsigned_OE0_Rc1;

    /* semantic routine */
#line 2768 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Divide Word Unsigned\n"));
#line 22861 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Divide_Word_Unsigned, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2768 "./ppc-instructions"
    {
      unsigned64 dividend = (unsigned32)(*rA);
      unsigned64 divisor = (unsigned32)(*rB);
      if (divisor == 0) {
        if (OE)
          XER |= (xer_overflow | xer_summary_overflow);
        CR0_COMPARE(0, 0, Rc);
      }
      else {
        unsigned64 quotent = dividend / divisor;
        *rT = quotent;
        CR0_COMPARE((signed_word)quotent, 0, Rc);
      }
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 22886 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.459,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 22890 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Move_to_Special_Purpose_Register_SPR256(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Move_to_Special_Purpose_Register
#undef MY_PREFIX
#define MY_PREFIX Move_to_Special_Purpose_Register_SPR256
#line 3346 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Move to Special Purpose Register\n"));
#line 22909 "icache.c"

  /* validate: 0.31,6.RS,11.SPR,21.467,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c0003a6)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.SPR,21.467,31./ #define get-values-from-icache|put-values-in-icache */
#line 44 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.XFX.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 45 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.XFX.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 46 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.XFX.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 3346 "./ppc-instructions"
    unsigned const SPR UNUSED = 256; /* default in icache - constant field */
#line 22926 "icache.c"

#define insn cache_entry->crack.XFX.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Move_to_Special_Purpose_Register_SPR256;

    /* semantic routine */
#line 3346 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Move to Special Purpose Register\n"));
#line 22936 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Move_to_Special_Purpose_Register, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3346 "./ppc-instructions"
    {
      int n = (SPR_5_9_ << 5) | SPR_0_4_;
      if (SPR_0_ && IS_PROBLEM_STATE(processor))
        program_interrupt(processor, cia,
                          privileged_instruction_program_interrupt);
      else if (!spr_is_valid(n)
               || spr_is_readonly(n))
        program_interrupt(processor, cia,
                          illegal_instruction_program_interrupt);
      else {
        spreg new_val = (spr_length(n) == 64
      		   ? *rS
      		   : MASKED(*rS, 32, 63));
        /* HACK - time base registers need to be updated immediatly */
        if (WITH_TIME_BASE) {
          switch (n) {
          case spr_tbu:
            cpu_set_time_base(processor,
                              (MASKED64(cpu_get_time_base(processor), 32, 63)
                               | INSERTED64(new_val, 0, 31)));
            break;
          case spr_tbl:
            cpu_set_time_base(processor,
                              (MASKED64(cpu_get_time_base(processor), 0, 31)
                               | INSERTED64(new_val, 32, 63)));
            break;
          case spr_dec:
            cpu_set_decrementer(processor, new_val);
            break;
          default:
            SPREG(n) = new_val;
            break;
          }
        }
        else {
          SPREG(n) = new_val;
        }
      }
      PPC_INSN_TO_SPR(RS_BITMASK, n);
    }
#line 22986 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.SPR,21.467,31./ #undef get-values-from-icache|put-values-in-icache */
#line 22989 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Move_to_Special_Purpose_Register_SPR(cpu *processor,
                                            instruction_word instruction,
                                            unsigned_word cia,
                                            idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Move_to_Special_Purpose_Register
#undef MY_PREFIX
#define MY_PREFIX Move_to_Special_Purpose_Register_SPR
#line 3346 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Move to Special Purpose Register\n"));
#line 23008 "icache.c"

  /* validate: 0.31,6.RS,11.SPR,21.467,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c0003a6)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.SPR,21.467,31./ #define get-values-from-icache|put-values-in-icache */
#line 44 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.XFX.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 45 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.XFX.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 46 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.XFX.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 3346 "./ppc-instructions"
    unsigned const SPR UNUSED = cache_entry->crack.XFX.SPR = EXTRACTED32(instruction, 11, 20); /* default in icache - extraction */
#line 23025 "icache.c"

#define insn cache_entry->crack.XFX.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Move_to_Special_Purpose_Register_SPR;

    /* semantic routine */
#line 3346 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Move to Special Purpose Register\n"));
#line 23035 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Move_to_Special_Purpose_Register, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3346 "./ppc-instructions"
    {
      int n = (SPR_5_9_ << 5) | SPR_0_4_;
      if (SPR_0_ && IS_PROBLEM_STATE(processor))
        program_interrupt(processor, cia,
                          privileged_instruction_program_interrupt);
      else if (!spr_is_valid(n)
               || spr_is_readonly(n))
        program_interrupt(processor, cia,
                          illegal_instruction_program_interrupt);
      else {
        spreg new_val = (spr_length(n) == 64
      		   ? *rS
      		   : MASKED(*rS, 32, 63));
        /* HACK - time base registers need to be updated immediatly */
        if (WITH_TIME_BASE) {
          switch (n) {
          case spr_tbu:
            cpu_set_time_base(processor,
                              (MASKED64(cpu_get_time_base(processor), 32, 63)
                               | INSERTED64(new_val, 0, 31)));
            break;
          case spr_tbl:
            cpu_set_time_base(processor,
                              (MASKED64(cpu_get_time_base(processor), 0, 31)
                               | INSERTED64(new_val, 32, 63)));
            break;
          case spr_dec:
            cpu_set_decrementer(processor, new_val);
            break;
          default:
            SPREG(n) = new_val;
            break;
          }
        }
        else {
          SPREG(n) = new_val;
        }
      }
      PPC_INSN_TO_SPR(RS_BITMASK, n);
    }
#line 23085 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.SPR,21.467,31./ #undef get-values-from-icache|put-values-in-icache */
#line 23088 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Data_Cache_Block_Invalidate(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Data_Cache_Block_Invalidate
#undef MY_PREFIX
#define MY_PREFIX Data_Cache_Block_Invalidate
#line 4902 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Data Cache Block Invalidate\n"));
#line 23107 "icache.c"

  /* validate: 0.31,6./,11.RA,16.RB,21.470,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xffe007ff) != 0x7c0003ac)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6./,11.RA,16.RB,21.470,31./ #define get-values-from-icache|put-values-in-icache */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 23129 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Data_Cache_Block_Invalidate;

    /* semantic routine */
#line 4902 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Data Cache Block Invalidate\n"));
#line 23139 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Data_Cache_Block_Invalidate, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4902 "./ppc-instructions"
    {
      if (IS_PROBLEM_STATE(processor))
        program_interrupt(processor, cia,
                          privileged_instruction_program_interrupt);
      else
        TRACE(trace_tbd,("Data Cache Block Invalidate\n"));
    }
#line 23156 "icache.c"
    return nia;
    /* extraction: 0.31,6./,11.RA,16.RB,21.470,31./ #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 23160 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_NAND_Rc0(cpu *processor,
                instruction_word instruction,
                unsigned_word cia,
                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_NAND
#undef MY_PREFIX
#define MY_PREFIX NAND_Rc0
#line 3041 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode NAND\n"));
#line 23179 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16.RB,21.476,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0x7c0003b8)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.476,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 44 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 45 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 46 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 3041 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 23209 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_NAND_Rc0;

    /* semantic routine */
#line 3041 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics NAND\n"));
#line 23219 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_NAND, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3041 "./ppc-instructions"
    {
      *rA = ~(*rS & *rB);
      CR0_COMPARE(*rA, 0, Rc);
      ITRACE(trace_alu, (" Result = %ld (0x%lx)\n", (long)*rA, (long)*rA));
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK | RB_BITMASK, Rc);
    }
#line 23235 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.476,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 23239 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_NAND_Rc1(cpu *processor,
                instruction_word instruction,
                unsigned_word cia,
                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_NAND
#undef MY_PREFIX
#define MY_PREFIX NAND_Rc1
#line 3041 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode NAND\n"));
#line 23258 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16.RB,21.476,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0x7c0003b8)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.476,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 44 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 45 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 46 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 3041 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 23288 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_NAND_Rc1;

    /* semantic routine */
#line 3041 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics NAND\n"));
#line 23298 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_NAND, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3041 "./ppc-instructions"
    {
      *rA = ~(*rS & *rB);
      CR0_COMPARE(*rA, 0, Rc);
      ITRACE(trace_alu, (" Result = %ld (0x%lx)\n", (long)*rA, (long)*rA));
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK | RB_BITMASK, Rc);
    }
#line 23314 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.476,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 23318 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Divide_Word_OE0_Rc0(cpu *processor,
                           instruction_word instruction,
                           unsigned_word cia,
                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Divide_Word
#undef MY_PREFIX
#define MY_PREFIX Divide_Word_OE0_Rc0
#line 2746 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Divide Word\n"));
#line 23337 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.OE,22.491,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0003fe) != 0x7c0003d6)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.491,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2746 "./ppc-instructions"
    unsigned const OE UNUSED = 0; /* default in icache - constant field */
#line 2746 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 23369 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Divide_Word_OE0_Rc0;

    /* semantic routine */
#line 2746 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Divide Word\n"));
#line 23379 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Divide_Word, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2746 "./ppc-instructions"
    {
      signed64 dividend = (signed32)(*rA);
      signed64 divisor = (signed32)(*rB);
      if (divisor == 0 /* nb 0x8000..0 is sign extended */
          || (dividend == 0x80000000 && divisor == -1)) {
        if (OE)
          XER |= (xer_overflow | xer_summary_overflow);
        CR0_COMPARE(0, 0, Rc);
      }
      else {
        signed64 quotent = dividend / divisor;
        *rT = quotent;
        CR0_COMPARE((signed_word)quotent, 0, Rc);
      }
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 23405 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.491,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 23409 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Divide_Word_OE0_Rc1(cpu *processor,
                           instruction_word instruction,
                           unsigned_word cia,
                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Divide_Word
#undef MY_PREFIX
#define MY_PREFIX Divide_Word_OE0_Rc1
#line 2746 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Divide Word\n"));
#line 23428 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.OE,22.491,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0003fe) != 0x7c0003d6)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.491,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2746 "./ppc-instructions"
    unsigned const OE UNUSED = 0; /* default in icache - constant field */
#line 2746 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 23460 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Divide_Word_OE0_Rc1;

    /* semantic routine */
#line 2746 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Divide Word\n"));
#line 23470 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Divide_Word, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2746 "./ppc-instructions"
    {
      signed64 dividend = (signed32)(*rA);
      signed64 divisor = (signed32)(*rB);
      if (divisor == 0 /* nb 0x8000..0 is sign extended */
          || (dividend == 0x80000000 && divisor == -1)) {
        if (OE)
          XER |= (xer_overflow | xer_summary_overflow);
        CR0_COMPARE(0, 0, Rc);
      }
      else {
        signed64 quotent = dividend / divisor;
        *rT = quotent;
        CR0_COMPARE((signed_word)quotent, 0, Rc);
      }
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 23496 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.491,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 23500 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Move_to_Condition_Register_from_XER(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Move_to_Condition_Register_from_XER
#undef MY_PREFIX
#define MY_PREFIX Move_to_Condition_Register_from_XER
#line 3446 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Move to Condition Register from XER\n"));
#line 23519 "icache.c"

  /* validate: 0.31,6.BF,9./,11./,16./,21.512,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc7fffff) != 0x7c000400)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.BF,9./,11./,16./,21.512,31./ #define get-values-from-icache|put-values-in-icache */
#line 68 "./ppc-instructions"
    unsigned const BF UNUSED = cache_entry->crack.X.BF = EXTRACTED32(instruction, 6, 8); /* in icache - extraction */
#line 69 "./ppc-instructions"
    unsigned32 const BF_BITMASK UNUSED = cache_entry->crack.X.BF_BITMASK = (1 << BF); /* in icache - extraction */
#line 23532 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Move_to_Condition_Register_from_XER;

    /* semantic routine */
#line 3446 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Move to Condition Register from XER\n"));
#line 23542 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Move_to_Condition_Register_from_XER, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3446 "./ppc-instructions"
    error("%s:%d:0x%08lx:%s unimplemented\n",
          itable[MY_INDEX].file, itable[MY_INDEX].line_nr, (long)cia, itable[MY_INDEX].name);
#line 23554 "icache.c"
    return nia;
    /* extraction: 0.31,6.BF,9./,11./,16./,21.512,31./ #undef get-values-from-icache|put-values-in-icache */
#line 23557 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Subtract_From_Carrying_OE1_Rc0(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Subtract_From_Carrying
#undef MY_PREFIX
#define MY_PREFIX Subtract_From_Carrying_OE1_Rc0
#line 2597 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Subtract From Carrying\n"));
#line 23576 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.OE,22.8,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0003fe) != 0x7c000010)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.8,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2597 "./ppc-instructions"
    unsigned const OE UNUSED = 1; /* default in icache - constant field */
#line 2597 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 23608 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Subtract_From_Carrying_OE1_Rc0;

    /* semantic routine */
#line 2597 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Subtract From Carrying\n"));
#line 23618 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Subtract_From_Carrying, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2597 "./ppc-instructions"
    {
      /* RT <- ~RA + RB + 1 === RT <- RB - RA */
      ALU_BEGIN(*rA);
      ALU_NOT;
      ALU_ADD(*rB);
      ALU_ADD(1);
      ALU_END(*rT, 1/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 23637 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.8,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 23641 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Subtract_From_Carrying_OE1_Rc1(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Subtract_From_Carrying
#undef MY_PREFIX
#define MY_PREFIX Subtract_From_Carrying_OE1_Rc1
#line 2597 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Subtract From Carrying\n"));
#line 23660 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.OE,22.8,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0003fe) != 0x7c000010)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.8,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2597 "./ppc-instructions"
    unsigned const OE UNUSED = 1; /* default in icache - constant field */
#line 2597 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 23692 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Subtract_From_Carrying_OE1_Rc1;

    /* semantic routine */
#line 2597 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Subtract From Carrying\n"));
#line 23702 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Subtract_From_Carrying, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2597 "./ppc-instructions"
    {
      /* RT <- ~RA + RB + 1 === RT <- RB - RA */
      ALU_BEGIN(*rA);
      ALU_NOT;
      ALU_ADD(*rB);
      ALU_ADD(1);
      ALU_END(*rT, 1/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 23721 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.8,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 23725 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Add_Carrying_OE1_Rc0(cpu *processor,
                            instruction_word instruction,
                            unsigned_word cia,
                            idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Add_Carrying
#undef MY_PREFIX
#define MY_PREFIX Add_Carrying_OE1_Rc0
#line 2587 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Add Carrying\n"));
#line 23744 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.OE,22.10,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0003fe) != 0x7c000014)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.10,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2587 "./ppc-instructions"
    unsigned const OE UNUSED = 1; /* default in icache - constant field */
#line 2587 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 23776 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Add_Carrying_OE1_Rc0;

    /* semantic routine */
#line 2587 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Add Carrying\n"));
#line 23786 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Add_Carrying, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2587 "./ppc-instructions"
    {
      ALU_BEGIN(*rA);
      ALU_ADD(*rB);
      ALU_END(*rT, 1/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 23802 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.10,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 23806 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Add_Carrying_OE1_Rc1(cpu *processor,
                            instruction_word instruction,
                            unsigned_word cia,
                            idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Add_Carrying
#undef MY_PREFIX
#define MY_PREFIX Add_Carrying_OE1_Rc1
#line 2587 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Add Carrying\n"));
#line 23825 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.OE,22.10,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0003fe) != 0x7c000014)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.10,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2587 "./ppc-instructions"
    unsigned const OE UNUSED = 1; /* default in icache - constant field */
#line 2587 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 23857 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Add_Carrying_OE1_Rc1;

    /* semantic routine */
#line 2587 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Add Carrying\n"));
#line 23867 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Add_Carrying, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2587 "./ppc-instructions"
    {
      ALU_BEGIN(*rA);
      ALU_ADD(*rB);
      ALU_END(*rT, 1/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 23883 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.10,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 23887 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Load_String_Word_Indexed(cpu *processor,
                                instruction_word instruction,
                                unsigned_word cia,
                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Load_String_Word_Indexed
#undef MY_PREFIX
#define MY_PREFIX Load_String_Word_Indexed
#line 2329 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Load String Word Indexed\n"));
#line 23906 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.533,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c00042a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.533,31./ #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.X.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.X.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.X.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 23934 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Load_String_Word_Indexed;

    /* semantic routine */
#line 2329 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Load String Word Indexed\n"));
#line 23944 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Load_String_Word_Indexed, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2329 "./ppc-instructions"
    {
      unsigned_word EA;
      unsigned_word b;
      int n;
      int r;
      int i;
      int nr;
      if (RA_is_0) b = 0;
      else         b = *rA;
      EA = b + *rB;
      n = EXTRACTED32(XER, 25, 31);
      r = RT - 1;
      i = 32;
      nr = (n + 3) / 4;
      if (((RT + nr >= 32)
           ? ((RA >= RT || RA < (RT + nr) % 32)
              || (RB >= RT || RB < (RT + nr) % 32))
           : ((RA >= RT && RA < RT + nr)
              || (RB >= RT && RB < RT + nr)))
          || (RT == RA || RT == RB))
        program_interrupt(processor, cia,
                        illegal_instruction_program_interrupt);
      if (CURRENT_ALIGNMENT == STRICT_ALIGNMENT)
        alignment_interrupt(processor, cia, EA);
      while (n > 0) {
        if (i == 32) {
          r = (r + 1) % 32;
          GPR(r) = 0;
        }
        GPR(r) |= INSERTED(MEM(unsigned, EA, 1), i, i+7);
        i = i + 8;
        if (i == 64) i = 32;
        EA = EA + 1;
        n = n - 1;
      }
    }
#line 23990 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.533,31./ #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 23994 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Load_Word_ByteReverse_Indexed(cpu *processor,
                                     instruction_word instruction,
                                     unsigned_word cia,
                                     idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Load_Word_ByteReverse_Indexed
#undef MY_PREFIX
#define MY_PREFIX Load_Word_ByteReverse_Indexed
#line 2216 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Load Word Byte-Reverse Indexed\n"));
#line 24013 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.534,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c00042c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.534,31./ #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.X.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.X.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.X.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 24041 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Load_Word_ByteReverse_Indexed;

    /* semantic routine */
#line 2216 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Load Word Byte-Reverse Indexed\n"));
#line 24051 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Load_Word_ByteReverse_Indexed, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2216 "./ppc-instructions"
    {
      unsigned_word b;
      unsigned_word EA;
      if (RA_is_0) b = 0;
      else         b = *rA;
      EA = b + *rB;
      *rT = SWAP_4(MEM(unsigned, EA, 4));
      PPC_INSN_INT(RT_BITMASK, (RA_BITMASK & ~1) | RB_BITMASK, 0);
    }
#line 24070 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.534,31./ #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 24074 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Load_FloatingPoint_Single_Indexed(cpu *processor,
                                         instruction_word instruction,
                                         unsigned_word cia,
                                         idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Load_FloatingPoint_Single_Indexed
#undef MY_PREFIX
#define MY_PREFIX Load_FloatingPoint_Single_Indexed
#line 3479 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Load Floating-Point Single Indexed\n"));
#line 24093 "icache.c"

  /* validate: 0.31,6.FRT,11.RA,16.RB,21.535,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c00042e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.FRT,11.RA,16.RB,21.535,31./ #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.X.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.X.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 24128 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Load_FloatingPoint_Single_Indexed;

    /* semantic routine */
#line 3479 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Load Floating-Point Single Indexed\n"));
#line 24138 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Load_FloatingPoint_Single_Indexed, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3479 "./ppc-instructions"
    {
      unsigned_word b;
      unsigned_word EA;
      if (RA_is_0) b = 0;
      else         b = *rA;
      EA = b + *rB;
      *frT = DOUBLE(MEM(unsigned, EA, 4));
      PPC_INSN_INT_FLOAT(0, FRT_BITMASK, (RA_BITMASK & ~1) | RB_BITMASK, 0);
    }
#line 24157 "icache.c"
    return nia;
    /* extraction: 0.31,6.FRT,11.RA,16.RB,21.535,31./ #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef RA_is_0
#line 24162 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Shift_Right_Word_Rc0(cpu *processor,
                            instruction_word instruction,
                            unsigned_word cia,
                            idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Shift_Right_Word
#undef MY_PREFIX
#define MY_PREFIX Shift_Right_Word_Rc0
#line 3272 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Shift Right Word\n"));
#line 24181 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16.RB,21.536,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0x7c000430)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.536,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 44 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 45 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 46 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 3272 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 24211 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Shift_Right_Word_Rc0;

    /* semantic routine */
#line 3272 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Shift Right Word\n"));
#line 24221 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Shift_Right_Word, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3272 "./ppc-instructions"
    {
      int n = MASKED(*rB, 58, 63);
      unsigned32 source = *rS;
      signed_word shifted;
      if (n < 32)
        shifted = (source >> n);
      else
        shifted = 0;
      *rA = shifted;
      CR0_COMPARE(shifted, 0, Rc);
      ITRACE(trace_alu, \
             ("n=%d, source=0x%lx, shifted=0x%lx\n",
              n, (unsigned long)source, (unsigned long)shifted));
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK, Rc);
    }
#line 24246 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.536,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 24250 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Shift_Right_Word_Rc1(cpu *processor,
                            instruction_word instruction,
                            unsigned_word cia,
                            idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Shift_Right_Word
#undef MY_PREFIX
#define MY_PREFIX Shift_Right_Word_Rc1
#line 3272 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Shift Right Word\n"));
#line 24269 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16.RB,21.536,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0x7c000430)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.536,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 44 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 45 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 46 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 3272 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 24299 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Shift_Right_Word_Rc1;

    /* semantic routine */
#line 3272 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Shift Right Word\n"));
#line 24309 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Shift_Right_Word, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3272 "./ppc-instructions"
    {
      int n = MASKED(*rB, 58, 63);
      unsigned32 source = *rS;
      signed_word shifted;
      if (n < 32)
        shifted = (source >> n);
      else
        shifted = 0;
      *rA = shifted;
      CR0_COMPARE(shifted, 0, Rc);
      ITRACE(trace_alu, \
             ("n=%d, source=0x%lx, shifted=0x%lx\n",
              n, (unsigned long)source, (unsigned long)shifted));
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK, Rc);
    }
#line 24334 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.536,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 24338 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Subtract_From_OE1_Rc0(cpu *processor,
                             instruction_word instruction,
                             unsigned_word cia,
                             idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Subtract_From
#undef MY_PREFIX
#define MY_PREFIX Subtract_From_OE1_Rc0
#line 2543 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Subtract From\n"));
#line 24357 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.OE,22.40,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0003fe) != 0x7c000050)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.40,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2543 "./ppc-instructions"
    unsigned const OE UNUSED = 1; /* default in icache - constant field */
#line 2543 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 24389 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Subtract_From_OE1_Rc0;

    /* semantic routine */
#line 2543 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Subtract From\n"));
#line 24399 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Subtract_From, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2543 "./ppc-instructions"
    {
      ALU_BEGIN(*rA);
      ALU_NOT;
      ALU_ADD(*rB);
      ALU_ADD(1);
      ALU_END(*rT, 0/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 24417 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.40,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 24421 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Subtract_From_OE1_Rc1(cpu *processor,
                             instruction_word instruction,
                             unsigned_word cia,
                             idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Subtract_From
#undef MY_PREFIX
#define MY_PREFIX Subtract_From_OE1_Rc1
#line 2543 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Subtract From\n"));
#line 24440 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.OE,22.40,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0003fe) != 0x7c000050)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.40,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2543 "./ppc-instructions"
    unsigned const OE UNUSED = 1; /* default in icache - constant field */
#line 2543 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 24472 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Subtract_From_OE1_Rc1;

    /* semantic routine */
#line 2543 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Subtract From\n"));
#line 24482 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Subtract_From, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2543 "./ppc-instructions"
    {
      ALU_BEGIN(*rA);
      ALU_NOT;
      ALU_ADD(*rB);
      ALU_ADD(1);
      ALU_END(*rT, 0/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 24500 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.40,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 24504 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_TLB_Synchronize(cpu *processor,
                       instruction_word instruction,
                       unsigned_word cia,
                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_TLB_Synchronize
#undef MY_PREFIX
#define MY_PREFIX TLB_Synchronize
#line 4996 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode TLB Synchronize\n"));
#line 24523 "icache.c"

  /* validate: 0.31,6./,11./,16./,21.566,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xffffffff) != 0x7c00046c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6./,11./,16./,21.566,31./ #define get-values-from-icache|put-values-in-icache */
#line 24532 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_TLB_Synchronize;

    /* semantic routine */
#line 4996 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics TLB Synchronize\n"));
#line 24542 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_TLB_Synchronize, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4996 "./ppc-instructions"
    {
      /* nothing happens here - always in sync */
    }
#line 24555 "icache.c"
    return nia;
    /* extraction: 0.31,6./,11./,16./,21.566,31./ #undef get-values-from-icache|put-values-in-icache */
#line 24558 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Load_FloatingPoint_Single_with_Update_Indexed(cpu *processor,
                                                     instruction_word instruction,
                                                     unsigned_word cia,
                                                     idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Load_FloatingPoint_Single_with_Update_Indexed
#undef MY_PREFIX
#define MY_PREFIX Load_FloatingPoint_Single_with_Update_Indexed
#line 3506 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Load Floating-Point Single with Update Indexed\n"));
#line 24577 "icache.c"

  /* validate: 0.31,6.FRT,11.RA,16.RB,21.567,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c00046e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.FRT,11.RA,16.RB,21.567,31./ #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.X.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.X.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 24612 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Load_FloatingPoint_Single_with_Update_Indexed;

    /* semantic routine */
#line 3506 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Load Floating-Point Single with Update Indexed\n"));
#line 24622 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Load_FloatingPoint_Single_with_Update_Indexed, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3506 "./ppc-instructions"
    {
      unsigned_word EA;
      if (RA_is_0)
        program_interrupt(processor, cia,
                          illegal_instruction_program_interrupt);
      EA = *rA + *rB;
      *frT = DOUBLE(MEM(unsigned, EA, 4));
      *rA = EA;
      PPC_INSN_INT_FLOAT(RA_BITMASK, FRT_BITMASK, (RA_BITMASK & ~1) | RB_BITMASK, 0);
    }
#line 24642 "icache.c"
    return nia;
    /* extraction: 0.31,6.FRT,11.RA,16.RB,21.567,31./ #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef RA_is_0
#line 24647 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Move_From_Segment_Register(cpu *processor,
                                  instruction_word instruction,
                                  unsigned_word cia,
                                  idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Move_From_Segment_Register
#undef MY_PREFIX
#define MY_PREFIX Move_From_Segment_Register
#line 4939 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Move From Segment Register\n"));
#line 24666 "icache.c"

  /* validate: 0.31,6.RT,11./,12.SR,16./,21.595,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc10ffff) != 0x7c0004a6)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11./,12.SR,16./,21.595,31./ #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.X.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.X.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.X.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 4939 "./ppc-instructions"
    unsigned const SR UNUSED = cache_entry->crack.X.SR = EXTRACTED32(instruction, 12, 15); /* default in icache - extraction */
#line 24683 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Move_From_Segment_Register;

    /* semantic routine */
#line 4939 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Move From Segment Register\n"));
#line 24693 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Move_From_Segment_Register, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4939 "./ppc-instructions"
    {
      if (IS_PROBLEM_STATE(processor))
        program_interrupt(processor, cia,
                          privileged_instruction_program_interrupt);
      else
        *rT = SEGREG(SR);
    }
#line 24710 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11./,12.SR,16./,21.595,31./ #undef get-values-from-icache|put-values-in-icache */
#line 24713 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Load_String_Word_Immediate(cpu *processor,
                                  instruction_word instruction,
                                  unsigned_word cia,
                                  idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Load_String_Word_Immediate
#undef MY_PREFIX
#define MY_PREFIX Load_String_Word_Immediate
#line 2297 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Load String Word Immediate\n"));
#line 24732 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.NB,21.597,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c0004aa)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.NB,21.597,31./ #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.X.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.X.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.X.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 2297 "./ppc-instructions"
    unsigned const NB UNUSED = cache_entry->crack.X.NB = EXTRACTED32(instruction, 16, 20); /* default in icache - extraction */
#line 24756 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Load_String_Word_Immediate;

    /* semantic routine */
#line 2297 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Load String Word Immediate\n"));
#line 24766 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Load_String_Word_Immediate, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2297 "./ppc-instructions"
    {
      unsigned_word EA;
      int n;
      int r;
      int i;
      int nr;
      if (RA_is_0) EA = 0;
      else         EA = *rA;
      if (NB == 0) n = 32;
      else         n = NB;
      r = RT - 1;
      i = 32;
      nr = (n + 3) / 4;
      if ((RT + nr >= 32)
          ? (RA >= RT || RA < (RT + nr) % 32)
          : (RA >= RT && RA < RT + nr))
        program_interrupt(processor, cia,
                          illegal_instruction_program_interrupt);
      if (CURRENT_ALIGNMENT == STRICT_ALIGNMENT)
        alignment_interrupt(processor, cia, EA);
      while (n > 0) {
        if (i == 32) {
          r = (r + 1) % 32;
          GPR(r) = 0;
        }
        GPR(r) |= INSERTED(MEM(unsigned, EA, 1), i, i+7);
        i = i + 8;
        if (i == 64) i = 32;
        EA = EA + 1;
        n = n - 1;
      }
    }
#line 24808 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.NB,21.597,31./ #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 24812 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Synchronize(cpu *processor,
                   instruction_word instruction,
                   unsigned_word cia,
                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Synchronize
#undef MY_PREFIX
#define MY_PREFIX Synchronize
#line 2501 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Synchronize\n"));
#line 24831 "icache.c"

  /* validate: 0.31,6./,9.L,11./,16./,21.598,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xff9fffff) != 0x7c0004ac)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6./,9.L,11./,16./,21.598,31./ #define get-values-from-icache|put-values-in-icache */
#line 2501 "./ppc-instructions"
    unsigned const L UNUSED = cache_entry->crack.X.L = EXTRACTED32(instruction, 9, 10); /* default in icache - extraction */
#line 24842 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Synchronize;

    /* semantic routine */
#line 2501 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Synchronize\n"));
#line 24852 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Synchronize, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2501 "./ppc-instructions"
    {
      /* do nothing */
    }
#line 24865 "icache.c"
    return nia;
    /* extraction: 0.31,6./,9.L,11./,16./,21.598,31./ #undef get-values-from-icache|put-values-in-icache */
#line 24868 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Load_FloatingPoint_Double_Indexed(cpu *processor,
                                         instruction_word instruction,
                                         unsigned_word cia,
                                         idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Load_FloatingPoint_Double_Indexed
#undef MY_PREFIX
#define MY_PREFIX Load_FloatingPoint_Double_Indexed
#line 3533 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Load Floating-Point Double Indexed\n"));
#line 24887 "icache.c"

  /* validate: 0.31,6.FRT,11.RA,16.RB,21.599,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c0004ae)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.FRT,11.RA,16.RB,21.599,31./ #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.X.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.X.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 24922 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Load_FloatingPoint_Double_Indexed;

    /* semantic routine */
#line 3533 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Load Floating-Point Double Indexed\n"));
#line 24932 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Load_FloatingPoint_Double_Indexed, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3533 "./ppc-instructions"
    {
      unsigned_word b;
      unsigned_word EA;
      if (RA_is_0) b = 0;
      else         b = *rA;
      EA = b + *rB;
      *frT = MEM(unsigned, EA, 8);
      PPC_INSN_INT_FLOAT(0, FRT_BITMASK, (RA_BITMASK & ~1) | RB_BITMASK, 0);
    }
#line 24951 "icache.c"
    return nia;
    /* extraction: 0.31,6.FRT,11.RA,16.RB,21.599,31./ #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef RA_is_0
#line 24956 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Negate_OE1_Rc0(cpu *processor,
                      instruction_word instruction,
                      unsigned_word cia,
                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Negate
#undef MY_PREFIX
#define MY_PREFIX Negate_OE1_Rc0
#line 2677 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Negate\n"));
#line 24975 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16./,21.OE,22.104,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00fbfe) != 0x7c0000d0)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.104,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 2677 "./ppc-instructions"
    unsigned const OE UNUSED = 1; /* default in icache - constant field */
#line 2677 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 25001 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Negate_OE1_Rc0;

    /* semantic routine */
#line 2677 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Negate\n"));
#line 25011 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Negate, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2677 "./ppc-instructions"
    {
      ALU_BEGIN(*rA);
      ALU_NOT;
      ALU_ADD(1);
      ALU_END(*rT,0/*CA*/,OE,Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK, Rc);
    }
#line 25028 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.104,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 25032 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Negate_OE1_Rc1(cpu *processor,
                      instruction_word instruction,
                      unsigned_word cia,
                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Negate
#undef MY_PREFIX
#define MY_PREFIX Negate_OE1_Rc1
#line 2677 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Negate\n"));
#line 25051 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16./,21.OE,22.104,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00fbfe) != 0x7c0000d0)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.104,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 2677 "./ppc-instructions"
    unsigned const OE UNUSED = 1; /* default in icache - constant field */
#line 2677 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 25077 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Negate_OE1_Rc1;

    /* semantic routine */
#line 2677 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Negate\n"));
#line 25087 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Negate, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2677 "./ppc-instructions"
    {
      ALU_BEGIN(*rA);
      ALU_NOT;
      ALU_ADD(1);
      ALU_END(*rT,0/*CA*/,OE,Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK, Rc);
    }
#line 25104 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.104,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 25108 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Load_FloatingPoint_Double_with_Update_Indexed(cpu *processor,
                                                     instruction_word instruction,
                                                     unsigned_word cia,
                                                     idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Load_FloatingPoint_Double_with_Update_Indexed
#undef MY_PREFIX
#define MY_PREFIX Load_FloatingPoint_Double_with_Update_Indexed
#line 3560 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Load Floating-Point Double with Update Indexed\n"));
#line 25127 "icache.c"

  /* validate: 0.31,6.FRT,11.RA,16.RB,21.631,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c0004ee)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.FRT,11.RA,16.RB,21.631,31./ #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.X.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.X.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 25162 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Load_FloatingPoint_Double_with_Update_Indexed;

    /* semantic routine */
#line 3560 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Load Floating-Point Double with Update Indexed\n"));
#line 25172 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Load_FloatingPoint_Double_with_Update_Indexed, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3560 "./ppc-instructions"
    {
      unsigned_word EA;
      if (RA_is_0)
        program_interrupt(processor, cia,
                          illegal_instruction_program_interrupt);
      EA = *rA + *rB;
      *frT = MEM(unsigned, EA, 8);
      *rA = EA;
      PPC_INSN_INT_FLOAT(RA_BITMASK, FRT_BITMASK, (RA_BITMASK & ~1) | RB_BITMASK, 0);
    }
#line 25192 "icache.c"
    return nia;
    /* extraction: 0.31,6.FRT,11.RA,16.RB,21.631,31./ #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef RA_is_0
#line 25197 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Subtract_From_Extended_OE1_Rc0(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Subtract_From_Extended
#undef MY_PREFIX
#define MY_PREFIX Subtract_From_Extended_OE1_Rc0
#line 2621 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Subtract From Extended\n"));
#line 25216 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.OE,22.136,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0003fe) != 0x7c000110)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.136,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2621 "./ppc-instructions"
    unsigned const OE UNUSED = 1; /* default in icache - constant field */
#line 2621 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 25248 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Subtract_From_Extended_OE1_Rc0;

    /* semantic routine */
#line 2621 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Subtract From Extended\n"));
#line 25258 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Subtract_From_Extended, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2621 "./ppc-instructions"
    {
      ALU_BEGIN(*rA);
      ALU_NOT;
      ALU_ADD(*rB);
      ALU_ADD_CA;
      ALU_END(*rT, 1/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 25276 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.136,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 25280 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Subtract_From_Extended_OE1_Rc1(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Subtract_From_Extended
#undef MY_PREFIX
#define MY_PREFIX Subtract_From_Extended_OE1_Rc1
#line 2621 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Subtract From Extended\n"));
#line 25299 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.OE,22.136,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0003fe) != 0x7c000110)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.136,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2621 "./ppc-instructions"
    unsigned const OE UNUSED = 1; /* default in icache - constant field */
#line 2621 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 25331 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Subtract_From_Extended_OE1_Rc1;

    /* semantic routine */
#line 2621 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Subtract From Extended\n"));
#line 25341 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Subtract_From_Extended, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2621 "./ppc-instructions"
    {
      ALU_BEGIN(*rA);
      ALU_NOT;
      ALU_ADD(*rB);
      ALU_ADD_CA;
      ALU_END(*rT, 1/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 25359 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.136,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 25363 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Add_Extended_OE1_Rc0(cpu *processor,
                            instruction_word instruction,
                            unsigned_word cia,
                            idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Add_Extended
#undef MY_PREFIX
#define MY_PREFIX Add_Extended_OE1_Rc0
#line 2610 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Add Extended\n"));
#line 25382 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.OE,22.138,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0003fe) != 0x7c000114)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.138,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2610 "./ppc-instructions"
    unsigned const OE UNUSED = 1; /* default in icache - constant field */
#line 2610 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 25414 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Add_Extended_OE1_Rc0;

    /* semantic routine */
#line 2610 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Add Extended\n"));
#line 25424 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Add_Extended, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2610 "./ppc-instructions"
    {
      ALU_BEGIN(*rA);
      ALU_ADD(*rB);
      ALU_ADD_CA;
      ALU_END(*rT, 1/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 25441 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.138,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 25445 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Add_Extended_OE1_Rc1(cpu *processor,
                            instruction_word instruction,
                            unsigned_word cia,
                            idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Add_Extended
#undef MY_PREFIX
#define MY_PREFIX Add_Extended_OE1_Rc1
#line 2610 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Add Extended\n"));
#line 25464 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.OE,22.138,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0003fe) != 0x7c000114)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.138,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2610 "./ppc-instructions"
    unsigned const OE UNUSED = 1; /* default in icache - constant field */
#line 2610 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 25496 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Add_Extended_OE1_Rc1;

    /* semantic routine */
#line 2610 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Add Extended\n"));
#line 25506 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Add_Extended, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2610 "./ppc-instructions"
    {
      ALU_BEGIN(*rA);
      ALU_ADD(*rB);
      ALU_ADD_CA;
      ALU_END(*rT, 1/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 25523 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.138,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 25527 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Move_From_Segment_Register_Indirect(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Move_From_Segment_Register_Indirect
#undef MY_PREFIX
#define MY_PREFIX Move_From_Segment_Register_Indirect
#line 4950 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Move From Segment Register Indirect\n"));
#line 25546 "icache.c"

  /* validate: 0.31,6.RT,11./,16.RB,21.659,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc1f07ff) != 0x7c000526)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11./,16.RB,21.659,31./ #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.X.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.X.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.X.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 25567 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Move_From_Segment_Register_Indirect;

    /* semantic routine */
#line 4950 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Move From Segment Register Indirect\n"));
#line 25577 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Move_From_Segment_Register_Indirect, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4950 "./ppc-instructions"
    {
      if (IS_PROBLEM_STATE(processor))
        program_interrupt(processor, cia,
                          privileged_instruction_program_interrupt);
      else
        *rT = SEGREG(EXTRACTED32(*rB, 0, 3));
    }
#line 25594 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11./,16.RB,21.659,31./ #undef get-values-from-icache|put-values-in-icache */
#line 25597 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Store_String_Word_Indexed(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Store_String_Word_Indexed
#undef MY_PREFIX
#define MY_PREFIX Store_String_Word_Indexed
#line 2387 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Store String Word Indexed\n"));
#line 25616 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16.RB,21.661,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c00052a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.661,31./ #define get-values-from-icache|put-values-in-icache */
#line 44 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 45 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 46 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 25644 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Store_String_Word_Indexed;

    /* semantic routine */
#line 2387 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Store String Word Indexed\n"));
#line 25654 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Store_String_Word_Indexed, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2387 "./ppc-instructions"
    {
      unsigned_word EA;
      unsigned_word b;
      int n;
      int r;
      int i;
      if (RA_is_0) b = 0;
      else         b = *rA;
      EA = b + *rB;
      if (CURRENT_ALIGNMENT == STRICT_ALIGNMENT)
        alignment_interrupt(processor, cia, EA);
      n = EXTRACTED32(XER, 25, 31);
      r = RS - 1;
      i = 32;
      while (n > 0) {
        if (i == 32) r = (r + 1) % 32;
        STORE(EA, 1, EXTRACTED(GPR(r), i, i+7));
        i = i + 8;
        if (i == 64) i = 32;
        EA = EA + 1;
        n = n - 1;
      }
    }
#line 25687 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.661,31./ #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 25691 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Store_Word_ByteReversed_Indexed(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Store_Word_ByteReversed_Indexed
#undef MY_PREFIX
#define MY_PREFIX Store_Word_ByteReversed_Indexed
#line 2242 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Store Word Byte-Reversed Indexed\n"));
#line 25710 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16.RB,21.662,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c00052c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.662,31./ #define get-values-from-icache|put-values-in-icache */
#line 44 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 45 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 46 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 25738 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Store_Word_ByteReversed_Indexed;

    /* semantic routine */
#line 2242 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Store Word Byte-Reversed Indexed\n"));
#line 25748 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Store_Word_ByteReversed_Indexed, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2242 "./ppc-instructions"
    {
      unsigned_word b;
      unsigned_word EA;
      if (RA_is_0) b = 0;
      else         b = *rA;
      EA = b + *rB;
      STORE(EA, 4, SWAP_4(*rS));
      PPC_INSN_INT(0, (RA_BITMASK & ~1) | RB_BITMASK | RS_BITMASK, 0);
    }
#line 25767 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.662,31./ #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 25771 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Store_FloatingPoint_Single_Indexed(cpu *processor,
                                          instruction_word instruction,
                                          unsigned_word cia,
                                          idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Store_FloatingPoint_Single_Indexed
#undef MY_PREFIX
#define MY_PREFIX Store_FloatingPoint_Single_Indexed
#line 3592 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Store Floating-Point Single Indexed\n"));
#line 25790 "icache.c"

  /* validate: 0.31,6.FRS,11.RA,16.RB,21.663,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c00052e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.FRS,11.RA,16.RB,21.663,31./ #define get-values-from-icache|put-values-in-icache */
#define FRS EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 60 "./ppc-instructions"
    unsigned64 * const frS UNUSED = cache_entry->crack.X.frS = (cpu_registers(processor)->fpr + FRS); /* in icache - extraction */
#line 61 "./ppc-instructions"
    unsigned32 const FRS_BITMASK UNUSED = cache_entry->crack.X.FRS_BITMASK = (1 << FRS); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 25825 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Store_FloatingPoint_Single_Indexed;

    /* semantic routine */
#line 3592 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Store Floating-Point Single Indexed\n"));
#line 25835 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Store_FloatingPoint_Single_Indexed, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3592 "./ppc-instructions"
    {
      unsigned_word b;
      unsigned_word EA;
      if (RA_is_0) b = 0;
      else         b = *rA;
      EA = b + *rB;
      STORE(EA, 4, SINGLE(*frS));
      PPC_INSN_INT_FLOAT(0, 0, (RA_BITMASK & ~1) | RB_BITMASK, FRS_BITMASK);
    }
#line 25854 "icache.c"
    return nia;
    /* extraction: 0.31,6.FRS,11.RA,16.RB,21.663,31./ #undef get-values-from-icache|put-values-in-icache */
#undef FRS
#undef RA_is_0
#line 25859 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Store_FloatingPoint_Single_with_Update_Indexed(cpu *processor,
                                                      instruction_word instruction,
                                                      unsigned_word cia,
                                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Store_FloatingPoint_Single_with_Update_Indexed
#undef MY_PREFIX
#define MY_PREFIX Store_FloatingPoint_Single_with_Update_Indexed
#line 3619 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Store Floating-Point Single with Update Indexed\n"));
#line 25878 "icache.c"

  /* validate: 0.31,6.FRS,11.RA,16.RB,21.695,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c00056e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.FRS,11.RA,16.RB,21.695,31./ #define get-values-from-icache|put-values-in-icache */
#define FRS EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 60 "./ppc-instructions"
    unsigned64 * const frS UNUSED = cache_entry->crack.X.frS = (cpu_registers(processor)->fpr + FRS); /* in icache - extraction */
#line 61 "./ppc-instructions"
    unsigned32 const FRS_BITMASK UNUSED = cache_entry->crack.X.FRS_BITMASK = (1 << FRS); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 25913 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Store_FloatingPoint_Single_with_Update_Indexed;

    /* semantic routine */
#line 3619 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Store Floating-Point Single with Update Indexed\n"));
#line 25923 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Store_FloatingPoint_Single_with_Update_Indexed, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3619 "./ppc-instructions"
    {
      unsigned_word EA;
      if (RA_is_0)
        program_interrupt(processor, cia,
                          illegal_instruction_program_interrupt);
      EA = *rA + *rB;
      STORE(EA, 4, SINGLE(*frS));
      *rA = EA;
      PPC_INSN_INT_FLOAT(RA_BITMASK, 0, (RA_BITMASK & ~1) | RB_BITMASK, FRS_BITMASK);
    }
#line 25943 "icache.c"
    return nia;
    /* extraction: 0.31,6.FRS,11.RA,16.RB,21.695,31./ #undef get-values-from-icache|put-values-in-icache */
#undef FRS
#undef RA_is_0
#line 25948 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Subtract_from_Zero_Extended_OE1_Rc0(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Subtract_from_Zero_Extended
#undef MY_PREFIX
#define MY_PREFIX Subtract_from_Zero_Extended_OE1_Rc0
#line 2666 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Subtract from Zero Extended\n"));
#line 25967 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16./,21.OE,22.200,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00fbfe) != 0x7c000190)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.200,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 2666 "./ppc-instructions"
    unsigned const OE UNUSED = 1; /* default in icache - constant field */
#line 2666 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 25993 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Subtract_from_Zero_Extended_OE1_Rc0;

    /* semantic routine */
#line 2666 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Subtract from Zero Extended\n"));
#line 26003 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Subtract_from_Zero_Extended, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2666 "./ppc-instructions"
    {
      ALU_BEGIN(*rA);
      ALU_NOT;
      ALU_ADD_CA;
      ALU_END(*rT, 1/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK, Rc);
    }
#line 26020 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.200,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 26024 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Subtract_from_Zero_Extended_OE1_Rc1(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Subtract_from_Zero_Extended
#undef MY_PREFIX
#define MY_PREFIX Subtract_from_Zero_Extended_OE1_Rc1
#line 2666 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Subtract from Zero Extended\n"));
#line 26043 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16./,21.OE,22.200,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00fbfe) != 0x7c000190)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.200,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 2666 "./ppc-instructions"
    unsigned const OE UNUSED = 1; /* default in icache - constant field */
#line 2666 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 26069 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Subtract_from_Zero_Extended_OE1_Rc1;

    /* semantic routine */
#line 2666 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Subtract from Zero Extended\n"));
#line 26079 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Subtract_from_Zero_Extended, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2666 "./ppc-instructions"
    {
      ALU_BEGIN(*rA);
      ALU_NOT;
      ALU_ADD_CA;
      ALU_END(*rT, 1/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK, Rc);
    }
#line 26096 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.200,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 26100 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Add_to_Zero_Extended_OE1_Rc0(cpu *processor,
                                    instruction_word instruction,
                                    unsigned_word cia,
                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Add_to_Zero_Extended
#undef MY_PREFIX
#define MY_PREFIX Add_to_Zero_Extended_OE1_Rc0
#line 2656 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Add to Zero Extended\n"));
#line 26119 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16./,21.OE,22.202,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00fbfe) != 0x7c000194)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.202,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 2656 "./ppc-instructions"
    unsigned const OE UNUSED = 1; /* default in icache - constant field */
#line 2656 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 26145 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Add_to_Zero_Extended_OE1_Rc0;

    /* semantic routine */
#line 2656 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Add to Zero Extended\n"));
#line 26155 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Add_to_Zero_Extended, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2656 "./ppc-instructions"
    {
      ALU_BEGIN(*rA);
      ALU_ADD_CA;
      ALU_END(*rT, 1/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK, Rc);
    }
#line 26171 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.202,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 26175 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Add_to_Zero_Extended_OE1_Rc1(cpu *processor,
                                    instruction_word instruction,
                                    unsigned_word cia,
                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Add_to_Zero_Extended
#undef MY_PREFIX
#define MY_PREFIX Add_to_Zero_Extended_OE1_Rc1
#line 2656 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Add to Zero Extended\n"));
#line 26194 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16./,21.OE,22.202,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00fbfe) != 0x7c000194)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.202,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 2656 "./ppc-instructions"
    unsigned const OE UNUSED = 1; /* default in icache - constant field */
#line 2656 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 26220 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Add_to_Zero_Extended_OE1_Rc1;

    /* semantic routine */
#line 2656 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Add to Zero Extended\n"));
#line 26230 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Add_to_Zero_Extended, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2656 "./ppc-instructions"
    {
      ALU_BEGIN(*rA);
      ALU_ADD_CA;
      ALU_END(*rT, 1/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK, Rc);
    }
#line 26246 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.202,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 26250 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Store_String_Word_Immedate(cpu *processor,
                                  instruction_word instruction,
                                  unsigned_word cia,
                                  idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Store_String_Word_Immedate
#undef MY_PREFIX
#define MY_PREFIX Store_String_Word_Immedate
#line 2365 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Store String Word Immedate\n"));
#line 26269 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16.NB,21.725,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c0005aa)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16.NB,21.725,31./ #define get-values-from-icache|put-values-in-icache */
#line 44 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 45 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 46 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 2365 "./ppc-instructions"
    unsigned const NB UNUSED = cache_entry->crack.X.NB = EXTRACTED32(instruction, 16, 20); /* default in icache - extraction */
#line 26293 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Store_String_Word_Immedate;

    /* semantic routine */
#line 2365 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Store String Word Immedate\n"));
#line 26303 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Store_String_Word_Immedate, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2365 "./ppc-instructions"
    {
      unsigned_word EA;
      int n;
      int r;
      int i;
      if (RA_is_0) EA = 0;
      else         EA = *rA;
      if (NB == 0) n = 32;
      else         n = NB;
      r = RS - 1;
      i = 32;
      if (CURRENT_ALIGNMENT == STRICT_ALIGNMENT)
        alignment_interrupt(processor, cia, EA);
      while (n > 0) {
        if (i == 32) r = (r + 1) % 32;
        STORE(EA, 1, EXTRACTED(GPR(r), i, i+7));
        i = i + 8;
        if (i == 64) i = 32;
        EA = EA + 1;
        n = n - 1;
      }
    }
#line 26335 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16.NB,21.725,31./ #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 26339 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Store_FloatingPoint_Double_Indexed(cpu *processor,
                                          instruction_word instruction,
                                          unsigned_word cia,
                                          idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Store_FloatingPoint_Double_Indexed
#undef MY_PREFIX
#define MY_PREFIX Store_FloatingPoint_Double_Indexed
#line 3646 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Store Floating-Point Double Indexed\n"));
#line 26358 "icache.c"

  /* validate: 0.31,6.FRS,11.RA,16.RB,21.727,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c0005ae)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.FRS,11.RA,16.RB,21.727,31./ #define get-values-from-icache|put-values-in-icache */
#define FRS EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 60 "./ppc-instructions"
    unsigned64 * const frS UNUSED = cache_entry->crack.X.frS = (cpu_registers(processor)->fpr + FRS); /* in icache - extraction */
#line 61 "./ppc-instructions"
    unsigned32 const FRS_BITMASK UNUSED = cache_entry->crack.X.FRS_BITMASK = (1 << FRS); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 26393 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Store_FloatingPoint_Double_Indexed;

    /* semantic routine */
#line 3646 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Store Floating-Point Double Indexed\n"));
#line 26403 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Store_FloatingPoint_Double_Indexed, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3646 "./ppc-instructions"
    {
      unsigned_word b;
      unsigned_word EA;
      if (RA_is_0) b = 0;
      else         b = *rA;
      EA = b + *rB;
      STORE(EA, 8, *frS);
      PPC_INSN_INT_FLOAT(0, 0, (RA_BITMASK & ~1) | RB_BITMASK, FRS_BITMASK);
    }
#line 26422 "icache.c"
    return nia;
    /* extraction: 0.31,6.FRS,11.RA,16.RB,21.727,31./ #undef get-values-from-icache|put-values-in-icache */
#undef FRS
#undef RA_is_0
#line 26427 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Subtract_From_Minus_One_Extended_OE1_Rc0(cpu *processor,
                                                instruction_word instruction,
                                                unsigned_word cia,
                                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Subtract_From_Minus_One_Extended
#undef MY_PREFIX
#define MY_PREFIX Subtract_From_Minus_One_Extended_OE1_Rc0
#line 2644 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Subtract From Minus One Extended\n"));
#line 26446 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16./,21.OE,22.232,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00fbfe) != 0x7c0001d0)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.232,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 2644 "./ppc-instructions"
    unsigned const OE UNUSED = 1; /* default in icache - constant field */
#line 2644 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 26472 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Subtract_From_Minus_One_Extended_OE1_Rc0;

    /* semantic routine */
#line 2644 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Subtract From Minus One Extended\n"));
#line 26482 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Subtract_From_Minus_One_Extended, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2644 "./ppc-instructions"
    {
      ALU_BEGIN(*rA);
      ALU_NOT;
      ALU_ADD_CA;
      ALU_ADD(-1);
      ALU_END(*rT, 1/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK, Rc);
    }
#line 26500 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.232,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 26504 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Subtract_From_Minus_One_Extended_OE1_Rc1(cpu *processor,
                                                instruction_word instruction,
                                                unsigned_word cia,
                                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Subtract_From_Minus_One_Extended
#undef MY_PREFIX
#define MY_PREFIX Subtract_From_Minus_One_Extended_OE1_Rc1
#line 2644 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Subtract From Minus One Extended\n"));
#line 26523 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16./,21.OE,22.232,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00fbfe) != 0x7c0001d0)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.232,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 2644 "./ppc-instructions"
    unsigned const OE UNUSED = 1; /* default in icache - constant field */
#line 2644 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 26549 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Subtract_From_Minus_One_Extended_OE1_Rc1;

    /* semantic routine */
#line 2644 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Subtract From Minus One Extended\n"));
#line 26559 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Subtract_From_Minus_One_Extended, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2644 "./ppc-instructions"
    {
      ALU_BEGIN(*rA);
      ALU_NOT;
      ALU_ADD_CA;
      ALU_ADD(-1);
      ALU_END(*rT, 1/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK, Rc);
    }
#line 26577 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.232,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 26581 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Add_to_Minus_One_Extended_OE1_Rc0(cpu *processor,
                                         instruction_word instruction,
                                         unsigned_word cia,
                                         idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Add_to_Minus_One_Extended
#undef MY_PREFIX
#define MY_PREFIX Add_to_Minus_One_Extended_OE1_Rc0
#line 2633 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Add to Minus One Extended\n"));
#line 26600 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16./,21.OE,22.234,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00fbfe) != 0x7c0001d4)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.234,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 2633 "./ppc-instructions"
    unsigned const OE UNUSED = 1; /* default in icache - constant field */
#line 2633 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 26626 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Add_to_Minus_One_Extended_OE1_Rc0;

    /* semantic routine */
#line 2633 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Add to Minus One Extended\n"));
#line 26636 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Add_to_Minus_One_Extended, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2633 "./ppc-instructions"
    {
      ALU_BEGIN(*rA);
      ALU_ADD_CA;
      ALU_ADD(-1);
      ALU_END(*rT, 1/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK, Rc);
    }
#line 26653 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.234,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 26657 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Add_to_Minus_One_Extended_OE1_Rc1(cpu *processor,
                                         instruction_word instruction,
                                         unsigned_word cia,
                                         idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Add_to_Minus_One_Extended
#undef MY_PREFIX
#define MY_PREFIX Add_to_Minus_One_Extended_OE1_Rc1
#line 2633 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Add to Minus One Extended\n"));
#line 26676 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16./,21.OE,22.234,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00fbfe) != 0x7c0001d4)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.234,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 2633 "./ppc-instructions"
    unsigned const OE UNUSED = 1; /* default in icache - constant field */
#line 2633 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 26702 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Add_to_Minus_One_Extended_OE1_Rc1;

    /* semantic routine */
#line 2633 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Add to Minus One Extended\n"));
#line 26712 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Add_to_Minus_One_Extended, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2633 "./ppc-instructions"
    {
      ALU_BEGIN(*rA);
      ALU_ADD_CA;
      ALU_ADD(-1);
      ALU_END(*rT, 1/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK, Rc);
    }
#line 26729 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.234,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 26733 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Multiply_Low_Word_OE1_Rc0(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Multiply_Low_Word
#undef MY_PREFIX
#define MY_PREFIX Multiply_Low_Word_OE1_Rc0
#line 2699 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Multiply Low Word\n"));
#line 26752 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.OE,22.235,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0003fe) != 0x7c0001d6)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.235,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2699 "./ppc-instructions"
    unsigned const OE UNUSED = 1; /* default in icache - constant field */
#line 2699 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 26784 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Multiply_Low_Word_OE1_Rc0;

    /* semantic routine */
#line 2699 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Multiply Low Word\n"));
#line 26794 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Multiply_Low_Word, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2699 "./ppc-instructions"
    {
      signed64 a = (signed32)(*rA);
      signed64 b = (signed32)(*rB);
      signed64 prod = a * b;
      signed_word t = prod;
      *rT = *rA * *rB;
      if (t != prod && OE)
        XER |= (xer_overflow | xer_summary_overflow);
      CR0_COMPARE(t, 0, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 26815 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.235,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 26819 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Multiply_Low_Word_OE1_Rc1(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Multiply_Low_Word
#undef MY_PREFIX
#define MY_PREFIX Multiply_Low_Word_OE1_Rc1
#line 2699 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Multiply Low Word\n"));
#line 26838 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.OE,22.235,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0003fe) != 0x7c0001d6)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.235,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2699 "./ppc-instructions"
    unsigned const OE UNUSED = 1; /* default in icache - constant field */
#line 2699 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 26870 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Multiply_Low_Word_OE1_Rc1;

    /* semantic routine */
#line 2699 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Multiply Low Word\n"));
#line 26880 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Multiply_Low_Word, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2699 "./ppc-instructions"
    {
      signed64 a = (signed32)(*rA);
      signed64 b = (signed32)(*rB);
      signed64 prod = a * b;
      signed_word t = prod;
      *rT = *rA * *rB;
      if (t != prod && OE)
        XER |= (xer_overflow | xer_summary_overflow);
      CR0_COMPARE(t, 0, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 26901 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.235,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 26905 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Store_FloatingPoint_Double_with_Update_Indexed(cpu *processor,
                                                      instruction_word instruction,
                                                      unsigned_word cia,
                                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Store_FloatingPoint_Double_with_Update_Indexed
#undef MY_PREFIX
#define MY_PREFIX Store_FloatingPoint_Double_with_Update_Indexed
#line 3685 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Store Floating-Point Double with Update Indexed\n"));
#line 26924 "icache.c"

  /* validate: 0.31,6.FRS,11.RA,16.RB,21.759,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c0005ee)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.FRS,11.RA,16.RB,21.759,31./ #define get-values-from-icache|put-values-in-icache */
#define FRS EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 60 "./ppc-instructions"
    unsigned64 * const frS UNUSED = cache_entry->crack.X.frS = (cpu_registers(processor)->fpr + FRS); /* in icache - extraction */
#line 61 "./ppc-instructions"
    unsigned32 const FRS_BITMASK UNUSED = cache_entry->crack.X.FRS_BITMASK = (1 << FRS); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 26959 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Store_FloatingPoint_Double_with_Update_Indexed;

    /* semantic routine */
#line 3685 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Store Floating-Point Double with Update Indexed\n"));
#line 26969 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Store_FloatingPoint_Double_with_Update_Indexed, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3685 "./ppc-instructions"
    {
      unsigned_word EA;
      if (RA_is_0)
        program_interrupt(processor, cia,
                          illegal_instruction_program_interrupt);
      EA = *rA + *rB;
      STORE(EA, 8, *frS);
      *rA = EA;
      PPC_INSN_INT_FLOAT(RA_BITMASK, 0, (RA_BITMASK & ~1) | RB_BITMASK, FRS_BITMASK);
    }
#line 26989 "icache.c"
    return nia;
    /* extraction: 0.31,6.FRS,11.RA,16.RB,21.759,31./ #undef get-values-from-icache|put-values-in-icache */
#undef FRS
#undef RA_is_0
#line 26994 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Add_OE1_Rc0(cpu *processor,
                   instruction_word instruction,
                   unsigned_word cia,
                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Add
#undef MY_PREFIX
#define MY_PREFIX Add_OE1_Rc0
#line 2533 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Add\n"));
#line 27013 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.OE,22.266,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0003fe) != 0x7c000214)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.266,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2533 "./ppc-instructions"
    unsigned const OE UNUSED = 1; /* default in icache - constant field */
#line 2533 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 27045 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Add_OE1_Rc0;

    /* semantic routine */
#line 2533 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Add\n"));
#line 27055 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Add, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2533 "./ppc-instructions"
    {
      ALU_BEGIN(*rA);
      ALU_ADD(*rB);
      ALU_END(*rT, 0/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 27071 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.266,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 27075 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Add_OE1_Rc1(cpu *processor,
                   instruction_word instruction,
                   unsigned_word cia,
                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Add
#undef MY_PREFIX
#define MY_PREFIX Add_OE1_Rc1
#line 2533 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Add\n"));
#line 27094 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.OE,22.266,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0003fe) != 0x7c000214)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.266,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2533 "./ppc-instructions"
    unsigned const OE UNUSED = 1; /* default in icache - constant field */
#line 2533 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 27126 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Add_OE1_Rc1;

    /* semantic routine */
#line 2533 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Add\n"));
#line 27136 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Add, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2533 "./ppc-instructions"
    {
      ALU_BEGIN(*rA);
      ALU_ADD(*rB);
      ALU_END(*rT, 0/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 27152 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.266,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 27156 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Load_Halfword_ByteReverse_Indexed(cpu *processor,
                                         instruction_word instruction,
                                         unsigned_word cia,
                                         idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Load_Halfword_ByteReverse_Indexed
#undef MY_PREFIX
#define MY_PREFIX Load_Halfword_ByteReverse_Indexed
#line 2203 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Load Halfword Byte-Reverse Indexed\n"));
#line 27175 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.790,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c00062c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.790,31./ #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.X.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.X.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.X.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 27203 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Load_Halfword_ByteReverse_Indexed;

    /* semantic routine */
#line 2203 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Load Halfword Byte-Reverse Indexed\n"));
#line 27213 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Load_Halfword_ByteReverse_Indexed, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2203 "./ppc-instructions"
    {
      unsigned_word b;
      unsigned_word EA;
      if (RA_is_0) b = 0;
      else         b = *rA;
      EA = b + *rB;
      *rT = SWAP_2(MEM(unsigned, EA, 2));
      PPC_INSN_INT(RT_BITMASK, (RA_BITMASK & ~1) | RB_BITMASK, 0);
    }
#line 27232 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.790,31./ #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 27236 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Shift_Right_Algebraic_Word_Rc0(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Shift_Right_Algebraic_Word
#undef MY_PREFIX
#define MY_PREFIX Shift_Right_Algebraic_Word_Rc0
#line 3315 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Shift Right Algebraic Word\n"));
#line 27255 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16.RB,21.792,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0x7c000630)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.792,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 44 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 45 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 46 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 3315 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 27285 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Shift_Right_Algebraic_Word_Rc0;

    /* semantic routine */
#line 3315 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Shift Right Algebraic Word\n"));
#line 27295 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Shift_Right_Algebraic_Word, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3315 "./ppc-instructions"
    {
      unsigned64 mask;
      int n = MASKED(*rB, 59, 63);
      signed32 source = (signed32)*rS; /* signed to keep sign bit */
      signed32 shifted = source >> n;
      int S = (MASKED(*rS,32,32) != 0);
      signed64 r = ((unsigned64) source);
      r = ((unsigned64) source) << 32 | (unsigned32) source;
      r = ROTL64(r,64-n);
      if (MASKED(*rB,58,58) == 0)
      	mask = (unsigned64) MASK64(n+32,63);
      else
      	mask = (unsigned64) 0;
      *rA = (signed_word) (r & mask | ((signed64) -1*S) & ~mask); /* if 64bit will sign extend */
      if (S && (MASKED(r & ~mask,32,63)!=0))
        XER |= xer_carry;
      else
        XER &= ~xer_carry;
      CR0_COMPARE(*rA, 0, Rc);
      ITRACE(trace_alu, (" Result = %ld (0x%lx), XER = %ld\n",
      		   (long)*rA, (long)*rA, (long)XER));
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK, Rc);
    }
#line 27328 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.792,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 27332 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Shift_Right_Algebraic_Word_Rc1(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Shift_Right_Algebraic_Word
#undef MY_PREFIX
#define MY_PREFIX Shift_Right_Algebraic_Word_Rc1
#line 3315 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Shift Right Algebraic Word\n"));
#line 27351 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16.RB,21.792,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0x7c000630)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.792,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 44 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 45 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 46 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 3315 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 27381 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Shift_Right_Algebraic_Word_Rc1;

    /* semantic routine */
#line 3315 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Shift Right Algebraic Word\n"));
#line 27391 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Shift_Right_Algebraic_Word, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3315 "./ppc-instructions"
    {
      unsigned64 mask;
      int n = MASKED(*rB, 59, 63);
      signed32 source = (signed32)*rS; /* signed to keep sign bit */
      signed32 shifted = source >> n;
      int S = (MASKED(*rS,32,32) != 0);
      signed64 r = ((unsigned64) source);
      r = ((unsigned64) source) << 32 | (unsigned32) source;
      r = ROTL64(r,64-n);
      if (MASKED(*rB,58,58) == 0)
      	mask = (unsigned64) MASK64(n+32,63);
      else
      	mask = (unsigned64) 0;
      *rA = (signed_word) (r & mask | ((signed64) -1*S) & ~mask); /* if 64bit will sign extend */
      if (S && (MASKED(r & ~mask,32,63)!=0))
        XER |= xer_carry;
      else
        XER &= ~xer_carry;
      CR0_COMPARE(*rA, 0, Rc);
      ITRACE(trace_alu, (" Result = %ld (0x%lx), XER = %ld\n",
      		   (long)*rA, (long)*rA, (long)XER));
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK, Rc);
    }
#line 27424 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.792,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 27428 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Shift_Right_Algebraic_Word_Immediate_Rc0(cpu *processor,
                                                instruction_word instruction,
                                                unsigned_word cia,
                                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Shift_Right_Algebraic_Word_Immediate
#undef MY_PREFIX
#define MY_PREFIX Shift_Right_Algebraic_Word_Immediate_Rc0
#line 3293 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Shift Right Algebraic Word Immediate\n"));
#line 27447 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16.SH,21.824,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0x7c000670)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16.SH,21.824,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 44 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 45 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 46 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 3293 "./ppc-instructions"
    unsigned const SH UNUSED = cache_entry->crack.X.SH = EXTRACTED32(instruction, 16, 20); /* default in icache - extraction */
#line 3293 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 27473 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Shift_Right_Algebraic_Word_Immediate_Rc0;

    /* semantic routine */
#line 3293 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Shift Right Algebraic Word Immediate\n"));
#line 27483 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Shift_Right_Algebraic_Word_Immediate, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3293 "./ppc-instructions"
    {
      int n = SH;
      signed_word r = ROTL32(*rS, /*64*/32-n);
      signed_word m = MASK(n+32, 63);
      int S = MASKED(*rS, 32, 32);
      signed_word shifted = (r & m) | (S ? ~m : 0);
      *rA = shifted;
      if (S && ((r & ~m) & MASK(32, 63)) != 0)
        XER |= xer_carry;
      else
        XER &= ~xer_carry;
      CR0_COMPARE(shifted, 0, Rc);
      ITRACE(trace_alu, (" Result = %ld (0x%lx), XER = %ld\n",
      		   (long)*rA, (long)*rA, (long)XER));
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK, Rc);
    }
#line 27509 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16.SH,21.824,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 27513 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Shift_Right_Algebraic_Word_Immediate_Rc1(cpu *processor,
                                                instruction_word instruction,
                                                unsigned_word cia,
                                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Shift_Right_Algebraic_Word_Immediate
#undef MY_PREFIX
#define MY_PREFIX Shift_Right_Algebraic_Word_Immediate_Rc1
#line 3293 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Shift Right Algebraic Word Immediate\n"));
#line 27532 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16.SH,21.824,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0x7c000670)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16.SH,21.824,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 44 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 45 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 46 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 3293 "./ppc-instructions"
    unsigned const SH UNUSED = cache_entry->crack.X.SH = EXTRACTED32(instruction, 16, 20); /* default in icache - extraction */
#line 3293 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 27558 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Shift_Right_Algebraic_Word_Immediate_Rc1;

    /* semantic routine */
#line 3293 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Shift Right Algebraic Word Immediate\n"));
#line 27568 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Shift_Right_Algebraic_Word_Immediate, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3293 "./ppc-instructions"
    {
      int n = SH;
      signed_word r = ROTL32(*rS, /*64*/32-n);
      signed_word m = MASK(n+32, 63);
      int S = MASKED(*rS, 32, 32);
      signed_word shifted = (r & m) | (S ? ~m : 0);
      *rA = shifted;
      if (S && ((r & ~m) & MASK(32, 63)) != 0)
        XER |= xer_carry;
      else
        XER &= ~xer_carry;
      CR0_COMPARE(shifted, 0, Rc);
      ITRACE(trace_alu, (" Result = %ld (0x%lx), XER = %ld\n",
      		   (long)*rA, (long)*rA, (long)XER));
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK, Rc);
    }
#line 27594 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16.SH,21.824,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 27598 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Enforce_Inorder_Execution_of_IO(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Enforce_Inorder_Execution_of_IO
#undef MY_PREFIX
#define MY_PREFIX Enforce_Inorder_Execution_of_IO
#line 4813 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Enforce In-order Execution of I/O\n"));
#line 27617 "icache.c"

  /* validate: 0.31,6./,11./,16./,21.854,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xffffffff) != 0x7c0006ac)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6./,11./,16./,21.854,31./ #define get-values-from-icache|put-values-in-icache */
#line 27626 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Enforce_Inorder_Execution_of_IO;

    /* semantic routine */
#line 4813 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Enforce In-order Execution of I/O\n"));
#line 27636 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Enforce_Inorder_Execution_of_IO, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4813 "./ppc-instructions"
    {
      /* Since this model has no instruction overlap
         this instruction need do nothing */
    }
#line 27650 "icache.c"
    return nia;
    /* extraction: 0.31,6./,11./,16./,21.854,31./ #undef get-values-from-icache|put-values-in-icache */
#line 27653 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Store_Half_Word_ByteReversed_Indexed(cpu *processor,
                                            instruction_word instruction,
                                            unsigned_word cia,
                                            idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Store_Half_Word_ByteReversed_Indexed
#undef MY_PREFIX
#define MY_PREFIX Store_Half_Word_ByteReversed_Indexed
#line 2229 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Store Half Word Byte-Reversed Indexed\n"));
#line 27672 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16.RB,21.918,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c00072c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.918,31./ #define get-values-from-icache|put-values-in-icache */
#line 44 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 45 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 46 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 27700 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Store_Half_Word_ByteReversed_Indexed;

    /* semantic routine */
#line 2229 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Store Half Word Byte-Reversed Indexed\n"));
#line 27710 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Store_Half_Word_ByteReversed_Indexed, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2229 "./ppc-instructions"
    {
      unsigned_word b;
      unsigned_word EA;
      if (RA_is_0) b = 0;
      else         b = *rA;
      EA = b + *rB;
      STORE(EA, 2, SWAP_2(*rS));
      PPC_INSN_INT(0, (RA_BITMASK & ~1) | RB_BITMASK | RS_BITMASK, 0);
    }
#line 27729 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.918,31./ #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 27733 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Extend_Sign_Half_Word_Rc0(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Extend_Sign_Half_Word
#undef MY_PREFIX
#define MY_PREFIX Extend_Sign_Half_Word_Rc0
#line 3101 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Extend Sign Half Word\n"));
#line 27752 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16./,21.922,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00fffe) != 0x7c000734)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16./,21.922,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 44 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 45 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 46 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 3101 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 27776 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Extend_Sign_Half_Word_Rc0;

    /* semantic routine */
#line 3101 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Extend Sign Half Word\n"));
#line 27786 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Extend_Sign_Half_Word, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3101 "./ppc-instructions"
    {
      *rA = (signed_word)(signed16)*rS;
      CR0_COMPARE(*rA, 0, Rc);
      ITRACE(trace_alu, (" Result = %ld (0x%lx)\n", (long)*rA, (long)*rA));
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK, Rc);
    }
#line 27802 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16./,21.922,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 27806 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Extend_Sign_Half_Word_Rc1(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Extend_Sign_Half_Word
#undef MY_PREFIX
#define MY_PREFIX Extend_Sign_Half_Word_Rc1
#line 3101 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Extend Sign Half Word\n"));
#line 27825 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16./,21.922,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00fffe) != 0x7c000734)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16./,21.922,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 44 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 45 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 46 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 3101 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 27849 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Extend_Sign_Half_Word_Rc1;

    /* semantic routine */
#line 3101 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Extend Sign Half Word\n"));
#line 27859 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Extend_Sign_Half_Word, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3101 "./ppc-instructions"
    {
      *rA = (signed_word)(signed16)*rS;
      CR0_COMPARE(*rA, 0, Rc);
      ITRACE(trace_alu, (" Result = %ld (0x%lx)\n", (long)*rA, (long)*rA));
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK, Rc);
    }
#line 27875 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16./,21.922,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 27879 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Extend_Sign_Byte_Rc0(cpu *processor,
                            instruction_word instruction,
                            unsigned_word cia,
                            idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Extend_Sign_Byte
#undef MY_PREFIX
#define MY_PREFIX Extend_Sign_Byte_Rc0
#line 3091 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Extend Sign Byte\n"));
#line 27898 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16./,21.954,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00fffe) != 0x7c000774)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16./,21.954,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 44 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 45 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 46 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 3091 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 27922 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Extend_Sign_Byte_Rc0;

    /* semantic routine */
#line 3091 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Extend Sign Byte\n"));
#line 27932 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Extend_Sign_Byte, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3091 "./ppc-instructions"
    {
      *rA = (signed_word)(signed8)*rS;
      CR0_COMPARE(*rA, 0, Rc);
      ITRACE(trace_alu, (" Result = %ld (0x%lx)\n", (long)*rA, (long)*rA));
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK, Rc);
    }
#line 27948 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16./,21.954,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 27952 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Extend_Sign_Byte_Rc1(cpu *processor,
                            instruction_word instruction,
                            unsigned_word cia,
                            idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Extend_Sign_Byte
#undef MY_PREFIX
#define MY_PREFIX Extend_Sign_Byte_Rc1
#line 3091 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Extend Sign Byte\n"));
#line 27971 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16./,21.954,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00fffe) != 0x7c000774)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16./,21.954,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 44 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 45 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 46 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 3091 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 27995 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Extend_Sign_Byte_Rc1;

    /* semantic routine */
#line 3091 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Extend Sign Byte\n"));
#line 28005 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Extend_Sign_Byte, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3091 "./ppc-instructions"
    {
      *rA = (signed_word)(signed8)*rS;
      CR0_COMPARE(*rA, 0, Rc);
      ITRACE(trace_alu, (" Result = %ld (0x%lx)\n", (long)*rA, (long)*rA));
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK, Rc);
    }
#line 28021 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16./,21.954,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 28025 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Divide_Word_Unsigned_OE1_Rc0(cpu *processor,
                                    instruction_word instruction,
                                    unsigned_word cia,
                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Divide_Word_Unsigned
#undef MY_PREFIX
#define MY_PREFIX Divide_Word_Unsigned_OE1_Rc0
#line 2768 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Divide Word Unsigned\n"));
#line 28044 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.OE,22.459,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0003fe) != 0x7c000396)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.459,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2768 "./ppc-instructions"
    unsigned const OE UNUSED = 1; /* default in icache - constant field */
#line 2768 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 28076 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Divide_Word_Unsigned_OE1_Rc0;

    /* semantic routine */
#line 2768 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Divide Word Unsigned\n"));
#line 28086 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Divide_Word_Unsigned, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2768 "./ppc-instructions"
    {
      unsigned64 dividend = (unsigned32)(*rA);
      unsigned64 divisor = (unsigned32)(*rB);
      if (divisor == 0) {
        if (OE)
          XER |= (xer_overflow | xer_summary_overflow);
        CR0_COMPARE(0, 0, Rc);
      }
      else {
        unsigned64 quotent = dividend / divisor;
        *rT = quotent;
        CR0_COMPARE((signed_word)quotent, 0, Rc);
      }
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 28111 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.459,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 28115 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Divide_Word_Unsigned_OE1_Rc1(cpu *processor,
                                    instruction_word instruction,
                                    unsigned_word cia,
                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Divide_Word_Unsigned
#undef MY_PREFIX
#define MY_PREFIX Divide_Word_Unsigned_OE1_Rc1
#line 2768 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Divide Word Unsigned\n"));
#line 28134 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.OE,22.459,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0003fe) != 0x7c000396)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.459,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2768 "./ppc-instructions"
    unsigned const OE UNUSED = 1; /* default in icache - constant field */
#line 2768 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 28166 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Divide_Word_Unsigned_OE1_Rc1;

    /* semantic routine */
#line 2768 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Divide Word Unsigned\n"));
#line 28176 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Divide_Word_Unsigned, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2768 "./ppc-instructions"
    {
      unsigned64 dividend = (unsigned32)(*rA);
      unsigned64 divisor = (unsigned32)(*rB);
      if (divisor == 0) {
        if (OE)
          XER |= (xer_overflow | xer_summary_overflow);
        CR0_COMPARE(0, 0, Rc);
      }
      else {
        unsigned64 quotent = dividend / divisor;
        *rT = quotent;
        CR0_COMPARE((signed_word)quotent, 0, Rc);
      }
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 28201 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.459,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 28205 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Instruction_Cache_Block_Invalidate(cpu *processor,
                                          instruction_word instruction,
                                          unsigned_word cia,
                                          idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Instruction_Cache_Block_Invalidate
#undef MY_PREFIX
#define MY_PREFIX Instruction_Cache_Block_Invalidate
#line 4749 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Instruction Cache Block Invalidate\n"));
#line 28224 "icache.c"

  /* validate: 0.31,6./,11.RA,16.RB,21.982,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xffe007ff) != 0x7c0007ac)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6./,11.RA,16.RB,21.982,31./ #define get-values-from-icache|put-values-in-icache */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 28246 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Instruction_Cache_Block_Invalidate;

    /* semantic routine */
#line 4749 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Instruction Cache Block Invalidate\n"));
#line 28256 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Instruction_Cache_Block_Invalidate, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4749 "./ppc-instructions"
    {
      /* blindly flush all instruction cache entries */
#if WITH_IDECODE_CACHE_SIZE
      cpu_flush_icache(processor);
#endif
      PPC_INSN_INT(0, (RA_BITMASK & ~1) | RB_BITMASK, 0);
    }
#line 28273 "icache.c"
    return nia;
    /* extraction: 0.31,6./,11.RA,16.RB,21.982,31./ #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 28277 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Store_FloatingPoint_Integer_Word_Indexed(cpu *processor,
                                                instruction_word instruction,
                                                unsigned_word cia,
                                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Store_FloatingPoint_Integer_Word_Indexed
#undef MY_PREFIX
#define MY_PREFIX Store_FloatingPoint_Integer_Word_Indexed
#line 3658 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Store Floating-Point Integer Word Indexed\n"));
#line 28296 "icache.c"

  /* validate: 0.31,6.FRS,11.RA,16.RB,21.983,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c0007ae)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.FRS,11.RA,16.RB,21.983,31./ #define get-values-from-icache|put-values-in-icache */
#define FRS EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 60 "./ppc-instructions"
    unsigned64 * const frS UNUSED = cache_entry->crack.X.frS = (cpu_registers(processor)->fpr + FRS); /* in icache - extraction */
#line 61 "./ppc-instructions"
    unsigned32 const FRS_BITMASK UNUSED = cache_entry->crack.X.FRS_BITMASK = (1 << FRS); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 28331 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Store_FloatingPoint_Integer_Word_Indexed;

    /* semantic routine */
#line 3658 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Store Floating-Point Integer Word Indexed\n"));
#line 28341 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Store_FloatingPoint_Integer_Word_Indexed, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3658 "./ppc-instructions"
    {
      unsigned_word b;
      unsigned_word EA;
      if (RA_is_0) b = 0;
      else         b = *rA;
      EA = b + *rB;
      STORE(EA, 4, *frS);
      PPC_INSN_INT_FLOAT(0, 0, (RA_BITMASK & ~1) | RB_BITMASK, FRS_BITMASK);
    }
#line 28360 "icache.c"
    return nia;
    /* extraction: 0.31,6.FRS,11.RA,16.RB,21.983,31./ #undef get-values-from-icache|put-values-in-icache */
#undef FRS
#undef RA_is_0
#line 28365 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Divide_Word_OE1_Rc0(cpu *processor,
                           instruction_word instruction,
                           unsigned_word cia,
                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Divide_Word
#undef MY_PREFIX
#define MY_PREFIX Divide_Word_OE1_Rc0
#line 2746 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Divide Word\n"));
#line 28384 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.OE,22.491,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0003fe) != 0x7c0003d6)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.491,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2746 "./ppc-instructions"
    unsigned const OE UNUSED = 1; /* default in icache - constant field */
#line 2746 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 28416 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Divide_Word_OE1_Rc0;

    /* semantic routine */
#line 2746 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Divide Word\n"));
#line 28426 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Divide_Word, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2746 "./ppc-instructions"
    {
      signed64 dividend = (signed32)(*rA);
      signed64 divisor = (signed32)(*rB);
      if (divisor == 0 /* nb 0x8000..0 is sign extended */
          || (dividend == 0x80000000 && divisor == -1)) {
        if (OE)
          XER |= (xer_overflow | xer_summary_overflow);
        CR0_COMPARE(0, 0, Rc);
      }
      else {
        signed64 quotent = dividend / divisor;
        *rT = quotent;
        CR0_COMPARE((signed_word)quotent, 0, Rc);
      }
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 28452 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.491,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 28456 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Divide_Word_OE1_Rc1(cpu *processor,
                           instruction_word instruction,
                           unsigned_word cia,
                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Divide_Word
#undef MY_PREFIX
#define MY_PREFIX Divide_Word_OE1_Rc1
#line 2746 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Divide Word\n"));
#line 28475 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.OE,22.491,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0003fe) != 0x7c0003d6)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.491,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2746 "./ppc-instructions"
    unsigned const OE UNUSED = 1; /* default in icache - constant field */
#line 2746 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 28507 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Divide_Word_OE1_Rc1;

    /* semantic routine */
#line 2746 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Divide Word\n"));
#line 28517 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Divide_Word, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2746 "./ppc-instructions"
    {
      signed64 dividend = (signed32)(*rA);
      signed64 divisor = (signed32)(*rB);
      if (divisor == 0 /* nb 0x8000..0 is sign extended */
          || (dividend == 0x80000000 && divisor == -1)) {
        if (OE)
          XER |= (xer_overflow | xer_summary_overflow);
        CR0_COMPARE(0, 0, Rc);
      }
      else {
        signed64 quotent = dividend / divisor;
        *rT = quotent;
        CR0_COMPARE((signed_word)quotent, 0, Rc);
      }
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 28543 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.491,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 28547 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Data_Cache_Block_set_to_Zero(cpu *processor,
                                    instruction_word instruction,
                                    unsigned_word cia,
                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Data_Cache_Block_set_to_Zero
#undef MY_PREFIX
#define MY_PREFIX Data_Cache_Block_set_to_Zero
#line 4789 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Data Cache Block set to Zero\n"));
#line 28566 "icache.c"

  /* validate: 0.31,6./,11.RA,16.RB,21.1014,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xffe007ff) != 0x7c0007ec)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6./,11.RA,16.RB,21.1014,31./ #define get-values-from-icache|put-values-in-icache */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 47 "./ppc-instructions"
    unsigned const RB UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 48 "./ppc-instructions"
    signed_word * const rB UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 49 "./ppc-instructions"
    unsigned32 const RB_BITMASK UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 28588 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Data_Cache_Block_set_to_Zero;

    /* semantic routine */
#line 4789 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Data Cache Block set to Zero\n"));
#line 28598 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Data_Cache_Block_set_to_Zero, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4789 "./ppc-instructions"
    {
      TRACE(trace_tbd,("Data Cache Block set to Zero\n"));
      PPC_INSN_INT(0, (RA_BITMASK & ~1) | RB_BITMASK, 0/*Rc*/);
    }
#line 28612 "icache.c"
    return nia;
    /* extraction: 0.31,6./,11.RA,16.RB,21.1014,31./ #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 28616 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Load_Word_and_Zero(cpu *processor,
                          instruction_word instruction,
                          unsigned_word cia,
                          idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Load_Word_and_Zero
#undef MY_PREFIX
#define MY_PREFIX Load_Word_and_Zero
#line 1886 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Load Word and Zero\n"));
#line 28635 "icache.c"

  /* validate: 0.32,6.RT,11.RA,16.D */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x80000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.32,6.RT,11.RA,16.D #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.D.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.D.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.D.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 80 "./ppc-instructions"
    unsigned_word const EXTS_D UNUSED = cache_entry->crack.D.EXTS_D = ((signed_word)(signed16)(instruction)); /* in icache - extraction */
#line 28659 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Load_Word_and_Zero;

    /* semantic routine */
#line 1886 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Load Word and Zero\n"));
#line 28669 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Load_Word_and_Zero, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1886 "./ppc-instructions"
    {
      unsigned_word b;
      unsigned_word EA;
      if (RA_is_0) b = 0;
      else         b = *rA;
      EA = b + EXTS(D);
      *rT = MEM(unsigned, EA, 4);
      PPC_INSN_INT(RT_BITMASK, (RA_BITMASK & ~1), 0);
    }
#line 28688 "icache.c"
    return nia;
    /* extraction: 0.32,6.RT,11.RA,16.D #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 28692 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Load_Word_and_Zero_with_Update(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Load_Word_and_Zero_with_Update
#undef MY_PREFIX
#define MY_PREFIX Load_Word_and_Zero_with_Update
#line 1912 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Load Word and Zero with Update\n"));
#line 28711 "icache.c"

  /* validate: 0.33,6.RT,11.RA,16.D */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x84000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.33,6.RT,11.RA,16.D #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.D.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.D.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.D.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 80 "./ppc-instructions"
    unsigned_word const EXTS_D UNUSED = cache_entry->crack.D.EXTS_D = ((signed_word)(signed16)(instruction)); /* in icache - extraction */
#line 28735 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Load_Word_and_Zero_with_Update;

    /* semantic routine */
#line 1912 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Load Word and Zero with Update\n"));
#line 28745 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Load_Word_and_Zero_with_Update, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1912 "./ppc-instructions"
    {
      unsigned_word EA;
      if (RA_is_0 || RA == RT)
        program_interrupt(processor, cia,
                          illegal_instruction_program_interrupt);
      EA = *rA + EXTS(D);
      *rT = MEM(unsigned, EA, 4);
      *rA = EA;
      PPC_INSN_INT(RT_BITMASK | RA_BITMASK, RA_BITMASK, 0);
    }
#line 28765 "icache.c"
    return nia;
    /* extraction: 0.33,6.RT,11.RA,16.D #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 28769 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Load_Byte_and_Zero(cpu *processor,
                          instruction_word instruction,
                          unsigned_word cia,
                          idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Load_Byte_and_Zero
#undef MY_PREFIX
#define MY_PREFIX Load_Byte_and_Zero
#line 1723 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Load Byte and Zero\n"));
#line 28788 "icache.c"

  /* validate: 0.34,6.RT,11.RA,16.D */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x88000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.34,6.RT,11.RA,16.D #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.D.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.D.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.D.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 80 "./ppc-instructions"
    unsigned_word const EXTS_D UNUSED = cache_entry->crack.D.EXTS_D = ((signed_word)(signed16)(instruction)); /* in icache - extraction */
#line 28812 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Load_Byte_and_Zero;

    /* semantic routine */
#line 1723 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Load Byte and Zero\n"));
#line 28822 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Load_Byte_and_Zero, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1723 "./ppc-instructions"
    {
      unsigned_word b;
      unsigned_word EA;
      if (RA_is_0) b = 0;
      else         b = *rA;
      EA = b + EXTS(D);
      *rT = MEM(unsigned, EA, 1);
      PPC_INSN_INT(RT_BITMASK, (RA_BITMASK & ~1), 0);
    }
#line 28841 "icache.c"
    return nia;
    /* extraction: 0.34,6.RT,11.RA,16.D #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 28845 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Load_Byte_and_Zero_with_Update(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Load_Byte_and_Zero_with_Update
#undef MY_PREFIX
#define MY_PREFIX Load_Byte_and_Zero_with_Update
#line 1750 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Load Byte and Zero with Update\n"));
#line 28864 "icache.c"

  /* validate: 0.35,6.RT,11.RA,16.D */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x8c000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.35,6.RT,11.RA,16.D #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.D.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.D.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.D.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 80 "./ppc-instructions"
    unsigned_word const EXTS_D UNUSED = cache_entry->crack.D.EXTS_D = ((signed_word)(signed16)(instruction)); /* in icache - extraction */
#line 28888 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Load_Byte_and_Zero_with_Update;

    /* semantic routine */
#line 1750 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Load Byte and Zero with Update\n"));
#line 28898 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Load_Byte_and_Zero_with_Update, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1750 "./ppc-instructions"
    {
      unsigned_word EA;
      if (RA_is_0 || RA == RT)
        program_interrupt(processor, cia,
                          illegal_instruction_program_interrupt);
      EA = *rA + EXTS(D);
      *rT = MEM(unsigned, EA, 1);
      *rA = EA;
      PPC_INSN_INT(RT_BITMASK | RA_BITMASK, RA_BITMASK, 0);
    }
#line 28918 "icache.c"
    return nia;
    /* extraction: 0.35,6.RT,11.RA,16.D #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 28922 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Store_Word(cpu *processor,
                  instruction_word instruction,
                  unsigned_word cia,
                  idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Store_Word
#undef MY_PREFIX
#define MY_PREFIX Store_Word
#line 2113 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Store Word\n"));
#line 28941 "icache.c"

  /* validate: 0.36,6.RS,11.RA,16.D */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x90000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.36,6.RS,11.RA,16.D #define get-values-from-icache|put-values-in-icache */
#line 44 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.D.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 45 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.D.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 46 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.D.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 80 "./ppc-instructions"
    unsigned_word const EXTS_D UNUSED = cache_entry->crack.D.EXTS_D = ((signed_word)(signed16)(instruction)); /* in icache - extraction */
#line 28965 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Store_Word;

    /* semantic routine */
#line 2113 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Store Word\n"));
#line 28975 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Store_Word, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2113 "./ppc-instructions"
    {
      unsigned_word b;
      unsigned_word EA;
      if (RA_is_0) b = 0;
      else         b = *rA;
      EA = b + EXTS(D);
      STORE(EA, 4, *rS);
      PPC_INSN_INT(0, (RA_BITMASK & ~1) | RS_BITMASK, 0);
    }
#line 28994 "icache.c"
    return nia;
    /* extraction: 0.36,6.RS,11.RA,16.D #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 28998 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Store_Word_with_Update(cpu *processor,
                              instruction_word instruction,
                              unsigned_word cia,
                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Store_Word_with_Update
#undef MY_PREFIX
#define MY_PREFIX Store_Word_with_Update
#line 2139 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Store Word with Update\n"));
#line 29017 "icache.c"

  /* validate: 0.37,6.RS,11.RA,16.D */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x94000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.37,6.RS,11.RA,16.D #define get-values-from-icache|put-values-in-icache */
#line 44 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.D.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 45 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.D.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 46 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.D.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 80 "./ppc-instructions"
    unsigned_word const EXTS_D UNUSED = cache_entry->crack.D.EXTS_D = ((signed_word)(signed16)(instruction)); /* in icache - extraction */
#line 29041 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Store_Word_with_Update;

    /* semantic routine */
#line 2139 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Store Word with Update\n"));
#line 29051 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Store_Word_with_Update, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2139 "./ppc-instructions"
    {
      unsigned_word EA;
      if (RA_is_0)
        program_interrupt(processor, cia,
                          illegal_instruction_program_interrupt);
      EA = *rA + EXTS(D);
      STORE(EA, 4, *rS);
      *rA = EA;
      PPC_INSN_INT(RA_BITMASK, RA_BITMASK | RS_BITMASK, 0);
    }
#line 29071 "icache.c"
    return nia;
    /* extraction: 0.37,6.RS,11.RA,16.D #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 29075 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Store_Byte(cpu *processor,
                  instruction_word instruction,
                  unsigned_word cia,
                  idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Store_Byte
#undef MY_PREFIX
#define MY_PREFIX Store_Byte
#line 2005 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Store Byte\n"));
#line 29094 "icache.c"

  /* validate: 0.38,6.RS,11.RA,16.D */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x98000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.38,6.RS,11.RA,16.D #define get-values-from-icache|put-values-in-icache */
#line 44 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.D.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 45 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.D.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 46 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.D.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 80 "./ppc-instructions"
    unsigned_word const EXTS_D UNUSED = cache_entry->crack.D.EXTS_D = ((signed_word)(signed16)(instruction)); /* in icache - extraction */
#line 29118 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Store_Byte;

    /* semantic routine */
#line 2005 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Store Byte\n"));
#line 29128 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Store_Byte, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2005 "./ppc-instructions"
    {
      unsigned_word b;
      unsigned_word EA;
      if (RA_is_0) b = 0;
      else         b = *rA;
      EA = b + EXTS(D);
      STORE(EA, 1, *rS);
      PPC_INSN_INT(0, (RA_BITMASK & ~1) | RS_BITMASK, 0);
    }
#line 29147 "icache.c"
    return nia;
    /* extraction: 0.38,6.RS,11.RA,16.D #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 29151 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Store_Byte_with_Update(cpu *processor,
                              instruction_word instruction,
                              unsigned_word cia,
                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Store_Byte_with_Update
#undef MY_PREFIX
#define MY_PREFIX Store_Byte_with_Update
#line 2031 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Store Byte with Update\n"));
#line 29170 "icache.c"

  /* validate: 0.39,6.RS,11.RA,16.D */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x9c000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.39,6.RS,11.RA,16.D #define get-values-from-icache|put-values-in-icache */
#line 44 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.D.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 45 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.D.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 46 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.D.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 80 "./ppc-instructions"
    unsigned_word const EXTS_D UNUSED = cache_entry->crack.D.EXTS_D = ((signed_word)(signed16)(instruction)); /* in icache - extraction */
#line 29194 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Store_Byte_with_Update;

    /* semantic routine */
#line 2031 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Store Byte with Update\n"));
#line 29204 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Store_Byte_with_Update, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2031 "./ppc-instructions"
    {
      unsigned_word EA;
      if (RA_is_0)
        program_interrupt(processor, cia,
                          illegal_instruction_program_interrupt);
      EA = *rA + EXTS(D);
      STORE(EA, 1, *rS);
      *rA = EA;
      PPC_INSN_INT(RA_BITMASK, RA_BITMASK | RS_BITMASK, 0);
    }
#line 29224 "icache.c"
    return nia;
    /* extraction: 0.39,6.RS,11.RA,16.D #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 29228 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Load_Halfword_and_Zero(cpu *processor,
                              instruction_word instruction,
                              unsigned_word cia,
                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Load_Halfword_and_Zero
#undef MY_PREFIX
#define MY_PREFIX Load_Halfword_and_Zero
#line 1778 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Load Halfword and Zero\n"));
#line 29247 "icache.c"

  /* validate: 0.40,6.RT,11.RA,16.D */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0xa0000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.40,6.RT,11.RA,16.D #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.D.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.D.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.D.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 80 "./ppc-instructions"
    unsigned_word const EXTS_D UNUSED = cache_entry->crack.D.EXTS_D = ((signed_word)(signed16)(instruction)); /* in icache - extraction */
#line 29271 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Load_Halfword_and_Zero;

    /* semantic routine */
#line 1778 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Load Halfword and Zero\n"));
#line 29281 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Load_Halfword_and_Zero, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1778 "./ppc-instructions"
    {
      unsigned_word b;
      unsigned_word EA;
      if (RA_is_0) b = 0;
      else         b = *rA;
      EA = b + EXTS(D);
      *rT = MEM(unsigned, EA, 2);
      PPC_INSN_INT(RT_BITMASK, (RA_BITMASK & ~1), 0);
    }
#line 29300 "icache.c"
    return nia;
    /* extraction: 0.40,6.RT,11.RA,16.D #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 29304 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Load_Halfword_and_Zero_with_Update(cpu *processor,
                                          instruction_word instruction,
                                          unsigned_word cia,
                                          idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Load_Halfword_and_Zero_with_Update
#undef MY_PREFIX
#define MY_PREFIX Load_Halfword_and_Zero_with_Update
#line 1804 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Load Halfword and Zero with Update\n"));
#line 29323 "icache.c"

  /* validate: 0.41,6.RT,11.RA,16.D */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0xa4000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.41,6.RT,11.RA,16.D #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.D.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.D.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.D.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 80 "./ppc-instructions"
    unsigned_word const EXTS_D UNUSED = cache_entry->crack.D.EXTS_D = ((signed_word)(signed16)(instruction)); /* in icache - extraction */
#line 29347 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Load_Halfword_and_Zero_with_Update;

    /* semantic routine */
#line 1804 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Load Halfword and Zero with Update\n"));
#line 29357 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Load_Halfword_and_Zero_with_Update, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1804 "./ppc-instructions"
    {
      unsigned_word EA;
      if (RA_is_0 || RA == RT)
        program_interrupt(processor, cia,
                          illegal_instruction_program_interrupt);
      EA = *rA + EXTS(D);
      *rT = MEM(unsigned, EA, 2);
      *rA = EA;
      PPC_INSN_INT(RT_BITMASK | RA_BITMASK, RA_BITMASK, 0);
    }
#line 29377 "icache.c"
    return nia;
    /* extraction: 0.41,6.RT,11.RA,16.D #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 29381 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Load_Halfword_Algebraic(cpu *processor,
                               instruction_word instruction,
                               unsigned_word cia,
                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Load_Halfword_Algebraic
#undef MY_PREFIX
#define MY_PREFIX Load_Halfword_Algebraic
#line 1832 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Load Halfword Algebraic\n"));
#line 29400 "icache.c"

  /* validate: 0.42,6.RT,11.RA,16.D */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0xa8000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.42,6.RT,11.RA,16.D #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.D.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.D.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.D.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 80 "./ppc-instructions"
    unsigned_word const EXTS_D UNUSED = cache_entry->crack.D.EXTS_D = ((signed_word)(signed16)(instruction)); /* in icache - extraction */
#line 29424 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Load_Halfword_Algebraic;

    /* semantic routine */
#line 1832 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Load Halfword Algebraic\n"));
#line 29434 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Load_Halfword_Algebraic, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1832 "./ppc-instructions"
    {
      unsigned_word b;
      unsigned_word EA;
      if (RA_is_0) b = 0;
      else         b = *rA;
      EA = b + EXTS(D);
      *rT = MEM(signed, EA, 2);
      PPC_INSN_INT(RT_BITMASK, (RA_BITMASK & ~1), 0);
    }
#line 29453 "icache.c"
    return nia;
    /* extraction: 0.42,6.RT,11.RA,16.D #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 29457 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Load_Halfword_Algebraic_with_Update(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Load_Halfword_Algebraic_with_Update
#undef MY_PREFIX
#define MY_PREFIX Load_Halfword_Algebraic_with_Update
#line 1858 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Load Halfword Algebraic with Update\n"));
#line 29476 "icache.c"

  /* validate: 0.43,6.RT,11.RA,16.D */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0xac000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.43,6.RT,11.RA,16.D #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.D.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.D.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.D.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 80 "./ppc-instructions"
    unsigned_word const EXTS_D UNUSED = cache_entry->crack.D.EXTS_D = ((signed_word)(signed16)(instruction)); /* in icache - extraction */
#line 29500 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Load_Halfword_Algebraic_with_Update;

    /* semantic routine */
#line 1858 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Load Halfword Algebraic with Update\n"));
#line 29510 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Load_Halfword_Algebraic_with_Update, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1858 "./ppc-instructions"
    {
      unsigned_word EA;
      if (RA_is_0 || RA == RT)
        program_interrupt(processor, cia,
                          illegal_instruction_program_interrupt);
      EA = *rA + EXTS(D);
      *rT = MEM(signed, EA, 2);
      *rA = EA;
      PPC_INSN_INT(RT_BITMASK | RA_BITMASK, RA_BITMASK, 0);
    }
#line 29530 "icache.c"
    return nia;
    /* extraction: 0.43,6.RT,11.RA,16.D #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 29534 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Store_Half_Word(cpu *processor,
                       instruction_word instruction,
                       unsigned_word cia,
                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Store_Half_Word
#undef MY_PREFIX
#define MY_PREFIX Store_Half_Word
#line 2059 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Store Half Word\n"));
#line 29553 "icache.c"

  /* validate: 0.44,6.RS,11.RA,16.D */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0xb0000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.44,6.RS,11.RA,16.D #define get-values-from-icache|put-values-in-icache */
#line 44 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.D.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 45 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.D.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 46 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.D.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 80 "./ppc-instructions"
    unsigned_word const EXTS_D UNUSED = cache_entry->crack.D.EXTS_D = ((signed_word)(signed16)(instruction)); /* in icache - extraction */
#line 29577 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Store_Half_Word;

    /* semantic routine */
#line 2059 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Store Half Word\n"));
#line 29587 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Store_Half_Word, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2059 "./ppc-instructions"
    {
      unsigned_word b;
      unsigned_word EA;
      if (RA_is_0) b = 0;
      else         b = *rA;
      EA = b + EXTS(D);
      STORE(EA, 2, *rS);
      PPC_INSN_INT(0, (RA_BITMASK & ~1) | RS_BITMASK, 0);
    }
#line 29606 "icache.c"
    return nia;
    /* extraction: 0.44,6.RS,11.RA,16.D #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 29610 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Store_Half_Word_with_Update(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Store_Half_Word_with_Update
#undef MY_PREFIX
#define MY_PREFIX Store_Half_Word_with_Update
#line 2085 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Store Half Word with Update\n"));
#line 29629 "icache.c"

  /* validate: 0.45,6.RS,11.RA,16.D */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0xb4000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.45,6.RS,11.RA,16.D #define get-values-from-icache|put-values-in-icache */
#line 44 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.D.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 45 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.D.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 46 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.D.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 80 "./ppc-instructions"
    unsigned_word const EXTS_D UNUSED = cache_entry->crack.D.EXTS_D = ((signed_word)(signed16)(instruction)); /* in icache - extraction */
#line 29653 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Store_Half_Word_with_Update;

    /* semantic routine */
#line 2085 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Store Half Word with Update\n"));
#line 29663 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Store_Half_Word_with_Update, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2085 "./ppc-instructions"
    {
      unsigned_word EA;
      if (RA_is_0)
        program_interrupt(processor, cia,
                          illegal_instruction_program_interrupt);
      EA = *rA + EXTS(D);
      STORE(EA, 2, *rS);
      *rA = EA;
      PPC_INSN_INT(RA_BITMASK, RA_BITMASK | RS_BITMASK, 0);
    }
#line 29683 "icache.c"
    return nia;
    /* extraction: 0.45,6.RS,11.RA,16.D #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 29687 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Load_Multiple_Word(cpu *processor,
                          instruction_word instruction,
                          unsigned_word cia,
                          idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Load_Multiple_Word
#undef MY_PREFIX
#define MY_PREFIX Load_Multiple_Word
#line 2256 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Load Multiple Word\n"));
#line 29706 "icache.c"

  /* validate: 0.46,6.RT,11.RA,16.D */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0xb8000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.46,6.RT,11.RA,16.D #define get-values-from-icache|put-values-in-icache */
#line 41 "./ppc-instructions"
    unsigned const RT UNUSED = cache_entry->crack.D.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 42 "./ppc-instructions"
    signed_word * const rT UNUSED = cache_entry->crack.D.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 43 "./ppc-instructions"
    unsigned32 const RT_BITMASK UNUSED = cache_entry->crack.D.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 80 "./ppc-instructions"
    unsigned_word const EXTS_D UNUSED = cache_entry->crack.D.EXTS_D = ((signed_word)(signed16)(instruction)); /* in icache - extraction */
#line 29730 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Load_Multiple_Word;

    /* semantic routine */
#line 2256 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Load Multiple Word\n"));
#line 29740 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Load_Multiple_Word, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2256 "./ppc-instructions"
    {
      unsigned_word EA;
      unsigned_word b;
      int r;
      if (RA_is_0) b = 0;
      else         b = *rA;
      EA = b + EXTS(D);
      r = RT;
      if (RA >= r)
        program_interrupt(processor, cia,
                        illegal_instruction_program_interrupt);
      if (CURRENT_ALIGNMENT == STRICT_ALIGNMENT || (EA % 4 != 0))
        alignment_interrupt(processor, cia, EA);
      while (r <= 31) {
        GPR(r) = MEM(unsigned, EA, 4);
        r = r + 1;
        EA = EA + 4;
      }
    }
#line 29769 "icache.c"
    return nia;
    /* extraction: 0.46,6.RT,11.RA,16.D #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 29773 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Store_Multiple_Word(cpu *processor,
                           instruction_word instruction,
                           unsigned_word cia,
                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Store_Multiple_Word
#undef MY_PREFIX
#define MY_PREFIX Store_Multiple_Word
#line 2275 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Store Multiple Word\n"));
#line 29792 "icache.c"

  /* validate: 0.47,6.RS,11.RA,16.D */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0xbc000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.47,6.RS,11.RA,16.D #define get-values-from-icache|put-values-in-icache */
#line 44 "./ppc-instructions"
    unsigned const RS UNUSED = cache_entry->crack.D.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 45 "./ppc-instructions"
    signed_word * const rS UNUSED = cache_entry->crack.D.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 46 "./ppc-instructions"
    unsigned32 const RS_BITMASK UNUSED = cache_entry->crack.D.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 80 "./ppc-instructions"
    unsigned_word const EXTS_D UNUSED = cache_entry->crack.D.EXTS_D = ((signed_word)(signed16)(instruction)); /* in icache - extraction */
#line 29816 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Store_Multiple_Word;

    /* semantic routine */
#line 2275 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Store Multiple Word\n"));
#line 29826 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Store_Multiple_Word, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2275 "./ppc-instructions"
    {
      unsigned_word EA;
      unsigned_word b;
      int r;
      if (RA_is_0) b = 0;
      else         b = *rA;
      EA = b + EXTS(D);
      if (CURRENT_ALIGNMENT == STRICT_ALIGNMENT
          || (EA % 4 != 0))
        alignment_interrupt(processor, cia, EA);
      r = RS;
      while (r <= 31) {
        STORE(EA, 4, GPR(r));
        r = r + 1;
        EA = EA + 4;
      }
    }
#line 29853 "icache.c"
    return nia;
    /* extraction: 0.47,6.RS,11.RA,16.D #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 29857 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Load_FloatingPoint_Single(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Load_FloatingPoint_Single
#undef MY_PREFIX
#define MY_PREFIX Load_FloatingPoint_Single
#line 3466 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Load Floating-Point Single\n"));
#line 29876 "icache.c"

  /* validate: 0.48,6.FRT,11.RA,16.D */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0xc0000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.48,6.FRT,11.RA,16.D #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.D.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.D.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 80 "./ppc-instructions"
    unsigned_word const EXTS_D UNUSED = cache_entry->crack.D.EXTS_D = ((signed_word)(signed16)(instruction)); /* in icache - extraction */
#line 29907 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Load_FloatingPoint_Single;

    /* semantic routine */
#line 3466 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Load Floating-Point Single\n"));
#line 29917 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Load_FloatingPoint_Single, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3466 "./ppc-instructions"
    {
      unsigned_word b;
      unsigned_word EA;
      if (RA_is_0) b = 0;
      else         b = *rA;
      EA = b + EXTS(D);
      *frT = DOUBLE(MEM(unsigned, EA, 4));
      PPC_INSN_INT_FLOAT(0, FRT_BITMASK, (RA_BITMASK & ~1), 0);
    }
#line 29936 "icache.c"
    return nia;
    /* extraction: 0.48,6.FRT,11.RA,16.D #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef RA_is_0
#line 29941 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Load_FloatingPoint_Single_with_Update(cpu *processor,
                                             instruction_word instruction,
                                             unsigned_word cia,
                                             idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Load_FloatingPoint_Single_with_Update
#undef MY_PREFIX
#define MY_PREFIX Load_FloatingPoint_Single_with_Update
#line 3492 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Load Floating-Point Single with Update\n"));
#line 29960 "icache.c"

  /* validate: 0.49,6.FRT,11.RA,16.D */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0xc4000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.49,6.FRT,11.RA,16.D #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.D.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.D.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 80 "./ppc-instructions"
    unsigned_word const EXTS_D UNUSED = cache_entry->crack.D.EXTS_D = ((signed_word)(signed16)(instruction)); /* in icache - extraction */
#line 29991 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Load_FloatingPoint_Single_with_Update;

    /* semantic routine */
#line 3492 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Load Floating-Point Single with Update\n"));
#line 30001 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Load_FloatingPoint_Single_with_Update, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3492 "./ppc-instructions"
    {
      unsigned_word EA;
      if (RA_is_0)
        program_interrupt(processor, cia,
                          illegal_instruction_program_interrupt);
      EA = *rA + EXTS(D);
      *frT = DOUBLE(MEM(unsigned, EA, 4));
      *rA = EA;
      PPC_INSN_INT_FLOAT(RA_BITMASK, FRT_BITMASK, (RA_BITMASK & ~1), 0);
    }
#line 30021 "icache.c"
    return nia;
    /* extraction: 0.49,6.FRT,11.RA,16.D #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef RA_is_0
#line 30026 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Load_FloatingPoint_Double(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Load_FloatingPoint_Double
#undef MY_PREFIX
#define MY_PREFIX Load_FloatingPoint_Double
#line 3520 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Load Floating-Point Double\n"));
#line 30045 "icache.c"

  /* validate: 0.50,6.FRT,11.RA,16.D */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0xc8000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.50,6.FRT,11.RA,16.D #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.D.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.D.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 80 "./ppc-instructions"
    unsigned_word const EXTS_D UNUSED = cache_entry->crack.D.EXTS_D = ((signed_word)(signed16)(instruction)); /* in icache - extraction */
#line 30076 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Load_FloatingPoint_Double;

    /* semantic routine */
#line 3520 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Load Floating-Point Double\n"));
#line 30086 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Load_FloatingPoint_Double, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3520 "./ppc-instructions"
    {
      unsigned_word b;
      unsigned_word EA;
      if (RA_is_0) b = 0;
      else         b = *rA;
      EA = b + EXTS(D);
      *frT = MEM(unsigned, EA, 8);
      PPC_INSN_INT_FLOAT(0, FRT_BITMASK, (RA_BITMASK & ~1), 0);
    }
#line 30105 "icache.c"
    return nia;
    /* extraction: 0.50,6.FRT,11.RA,16.D #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef RA_is_0
#line 30110 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Load_FloatingPoint_Double_with_Update(cpu *processor,
                                             instruction_word instruction,
                                             unsigned_word cia,
                                             idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Load_FloatingPoint_Double_with_Update
#undef MY_PREFIX
#define MY_PREFIX Load_FloatingPoint_Double_with_Update
#line 3546 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Load Floating-Point Double with Update\n"));
#line 30129 "icache.c"

  /* validate: 0.51,6.FRT,11.RA,16.D */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0xcc000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.51,6.FRT,11.RA,16.D #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.D.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.D.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 80 "./ppc-instructions"
    unsigned_word const EXTS_D UNUSED = cache_entry->crack.D.EXTS_D = ((signed_word)(signed16)(instruction)); /* in icache - extraction */
#line 30160 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Load_FloatingPoint_Double_with_Update;

    /* semantic routine */
#line 3546 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Load Floating-Point Double with Update\n"));
#line 30170 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Load_FloatingPoint_Double_with_Update, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3546 "./ppc-instructions"
    {
      unsigned_word EA;
      if (RA_is_0)
        program_interrupt(processor, cia,
                          illegal_instruction_program_interrupt);
      EA = *rA + EXTS(D);
      *frT = MEM(unsigned, EA, 8);
      *rA = EA;
      PPC_INSN_INT_FLOAT(RA_BITMASK, FRT_BITMASK, (RA_BITMASK & ~1), 0);
    }
#line 30190 "icache.c"
    return nia;
    /* extraction: 0.51,6.FRT,11.RA,16.D #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef RA_is_0
#line 30195 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Store_FloatingPoint_Single(cpu *processor,
                                  instruction_word instruction,
                                  unsigned_word cia,
                                  idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Store_FloatingPoint_Single
#undef MY_PREFIX
#define MY_PREFIX Store_FloatingPoint_Single
#line 3579 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Store Floating-Point Single\n"));
#line 30214 "icache.c"

  /* validate: 0.52,6.FRS,11.RA,16.D */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0xd0000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.52,6.FRS,11.RA,16.D #define get-values-from-icache|put-values-in-icache */
#define FRS EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 60 "./ppc-instructions"
    unsigned64 * const frS UNUSED = cache_entry->crack.D.frS = (cpu_registers(processor)->fpr + FRS); /* in icache - extraction */
#line 61 "./ppc-instructions"
    unsigned32 const FRS_BITMASK UNUSED = cache_entry->crack.D.FRS_BITMASK = (1 << FRS); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 80 "./ppc-instructions"
    unsigned_word const EXTS_D UNUSED = cache_entry->crack.D.EXTS_D = ((signed_word)(signed16)(instruction)); /* in icache - extraction */
#line 30245 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Store_FloatingPoint_Single;

    /* semantic routine */
#line 3579 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Store Floating-Point Single\n"));
#line 30255 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Store_FloatingPoint_Single, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3579 "./ppc-instructions"
    {
      unsigned_word b;
      unsigned_word EA;
      if (RA_is_0) b = 0;
      else         b = *rA;
      EA = b + EXTS(D);
      STORE(EA, 4, SINGLE(*frS));
      PPC_INSN_INT_FLOAT(0, 0, (RA_BITMASK & ~1), FRS_BITMASK);
    }
#line 30274 "icache.c"
    return nia;
    /* extraction: 0.52,6.FRS,11.RA,16.D #undef get-values-from-icache|put-values-in-icache */
#undef FRS
#undef RA_is_0
#line 30279 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Store_FloatingPoint_Single_with_Update(cpu *processor,
                                              instruction_word instruction,
                                              unsigned_word cia,
                                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Store_FloatingPoint_Single_with_Update
#undef MY_PREFIX
#define MY_PREFIX Store_FloatingPoint_Single_with_Update
#line 3605 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Store Floating-Point Single with Update\n"));
#line 30298 "icache.c"

  /* validate: 0.53,6.FRS,11.RA,16.D */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0xd4000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.53,6.FRS,11.RA,16.D #define get-values-from-icache|put-values-in-icache */
#define FRS EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 60 "./ppc-instructions"
    unsigned64 * const frS UNUSED = cache_entry->crack.D.frS = (cpu_registers(processor)->fpr + FRS); /* in icache - extraction */
#line 61 "./ppc-instructions"
    unsigned32 const FRS_BITMASK UNUSED = cache_entry->crack.D.FRS_BITMASK = (1 << FRS); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 80 "./ppc-instructions"
    unsigned_word const EXTS_D UNUSED = cache_entry->crack.D.EXTS_D = ((signed_word)(signed16)(instruction)); /* in icache - extraction */
#line 30329 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Store_FloatingPoint_Single_with_Update;

    /* semantic routine */
#line 3605 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Store Floating-Point Single with Update\n"));
#line 30339 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Store_FloatingPoint_Single_with_Update, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3605 "./ppc-instructions"
    {
      unsigned_word EA;
      if (RA_is_0)
        program_interrupt(processor, cia,
                          illegal_instruction_program_interrupt);
      EA = *rA + EXTS(D);
      STORE(EA, 4, SINGLE(*frS));
      *rA = EA;
      PPC_INSN_INT_FLOAT(RA_BITMASK, 0, (RA_BITMASK & ~1), FRS_BITMASK);
    }
#line 30359 "icache.c"
    return nia;
    /* extraction: 0.53,6.FRS,11.RA,16.D #undef get-values-from-icache|put-values-in-icache */
#undef FRS
#undef RA_is_0
#line 30364 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Store_FloatingPoint_Double(cpu *processor,
                                  instruction_word instruction,
                                  unsigned_word cia,
                                  idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Store_FloatingPoint_Double
#undef MY_PREFIX
#define MY_PREFIX Store_FloatingPoint_Double
#line 3633 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Store Floating-Point Double\n"));
#line 30383 "icache.c"

  /* validate: 0.54,6.FRS,11.RA,16.D */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0xd8000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.54,6.FRS,11.RA,16.D #define get-values-from-icache|put-values-in-icache */
#define FRS EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 60 "./ppc-instructions"
    unsigned64 * const frS UNUSED = cache_entry->crack.D.frS = (cpu_registers(processor)->fpr + FRS); /* in icache - extraction */
#line 61 "./ppc-instructions"
    unsigned32 const FRS_BITMASK UNUSED = cache_entry->crack.D.FRS_BITMASK = (1 << FRS); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 80 "./ppc-instructions"
    unsigned_word const EXTS_D UNUSED = cache_entry->crack.D.EXTS_D = ((signed_word)(signed16)(instruction)); /* in icache - extraction */
#line 30414 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Store_FloatingPoint_Double;

    /* semantic routine */
#line 3633 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Store Floating-Point Double\n"));
#line 30424 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Store_FloatingPoint_Double, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3633 "./ppc-instructions"
    {
      unsigned_word b;
      unsigned_word EA;
      if (RA_is_0) b = 0;
      else         b = *rA;
      EA = b + EXTS(D);
      STORE(EA, 8, *frS);
      PPC_INSN_INT_FLOAT(0, 0, (RA_BITMASK & ~1), FRS_BITMASK);
    }
#line 30443 "icache.c"
    return nia;
    /* extraction: 0.54,6.FRS,11.RA,16.D #undef get-values-from-icache|put-values-in-icache */
#undef FRS
#undef RA_is_0
#line 30448 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Store_FloatingPoint_Double_with_Update(cpu *processor,
                                              instruction_word instruction,
                                              unsigned_word cia,
                                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Store_FloatingPoint_Double_with_Update
#undef MY_PREFIX
#define MY_PREFIX Store_FloatingPoint_Double_with_Update
#line 3671 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Store Floating-Point Double with Update\n"));
#line 30467 "icache.c"

  /* validate: 0.55,6.FRS,11.RA,16.D */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0xdc000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.55,6.FRS,11.RA,16.D #define get-values-from-icache|put-values-in-icache */
#define FRS EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 60 "./ppc-instructions"
    unsigned64 * const frS UNUSED = cache_entry->crack.D.frS = (cpu_registers(processor)->fpr + FRS); /* in icache - extraction */
#line 61 "./ppc-instructions"
    unsigned32 const FRS_BITMASK UNUSED = cache_entry->crack.D.FRS_BITMASK = (1 << FRS); /* in icache - extraction */
#line 37 "./ppc-instructions"
    unsigned const RA UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 38 "./ppc-instructions"
    signed_word * const rA UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 39 "./ppc-instructions"
    unsigned32 const RA_BITMASK UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 80 "./ppc-instructions"
    unsigned_word const EXTS_D UNUSED = cache_entry->crack.D.EXTS_D = ((signed_word)(signed16)(instruction)); /* in icache - extraction */
#line 30498 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Store_FloatingPoint_Double_with_Update;

    /* semantic routine */
#line 3671 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Store Floating-Point Double with Update\n"));
#line 30508 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Store_FloatingPoint_Double_with_Update, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3671 "./ppc-instructions"
    {
      unsigned_word EA;
      if (RA_is_0)
        program_interrupt(processor, cia,
                          illegal_instruction_program_interrupt);
      EA = *rA + EXTS(D);
      STORE(EA, 8, *frS);
      *rA = EA;
      PPC_INSN_INT_FLOAT(RA_BITMASK, 0, (RA_BITMASK & ~1), FRS_BITMASK);
    }
#line 30528 "icache.c"
    return nia;
    /* extraction: 0.55,6.FRS,11.RA,16.D #undef get-values-from-icache|put-values-in-icache */
#undef FRS
#undef RA_is_0
#line 30533 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Divide_Single_Rc0(cpu *processor,
                                  instruction_word instruction,
                                  unsigned_word cia,
                                  idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Divide_Single
#undef MY_PREFIX
#define MY_PREFIX Floating_Divide_Single_Rc0
#line 3934 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Divide Single\n"));
#line 30552 "icache.c"

  /* validate: 0.59,6.FRT,11.FRA,16.FRB,21./,26.18,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0xec000024)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.59,6.FRT,11.FRA,16.FRB,21./,26.18,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 3934 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 30586 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Divide_Single_Rc0;

    /* semantic routine */
#line 3934 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Divide Single\n"));
#line 30596 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Divide_Single, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3934 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frB,
                               fpscr_vxsnan | fpscr_vxzdz,
                               1, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      1); /*single-precision*/
      }
      else if (is_invalid_zero_divide (processor, cia,
                                       *frA, *frB,
                                       1 /*single?*/)) {
        invalid_zero_divide_operation (processor, cia,
                                       frT, *frA, *frB,
                                       1 /*single?*/);
      }
      else {
        /*HACK!*/
        float s = *(double*)frA / *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK, Rc);
    }
#line 30635 "icache.c"
    return nia;
    /* extraction: 0.59,6.FRT,11.FRA,16.FRB,21./,26.18,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#line 30641 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Divide_Single_Rc1(cpu *processor,
                                  instruction_word instruction,
                                  unsigned_word cia,
                                  idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Divide_Single
#undef MY_PREFIX
#define MY_PREFIX Floating_Divide_Single_Rc1
#line 3934 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Divide Single\n"));
#line 30660 "icache.c"

  /* validate: 0.59,6.FRT,11.FRA,16.FRB,21./,26.18,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0xec000024)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.59,6.FRT,11.FRA,16.FRB,21./,26.18,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 3934 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 30694 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Divide_Single_Rc1;

    /* semantic routine */
#line 3934 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Divide Single\n"));
#line 30704 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Divide_Single, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3934 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frB,
                               fpscr_vxsnan | fpscr_vxzdz,
                               1, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      1); /*single-precision*/
      }
      else if (is_invalid_zero_divide (processor, cia,
                                       *frA, *frB,
                                       1 /*single?*/)) {
        invalid_zero_divide_operation (processor, cia,
                                       frT, *frA, *frB,
                                       1 /*single?*/);
      }
      else {
        /*HACK!*/
        float s = *(double*)frA / *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK, Rc);
    }
#line 30743 "icache.c"
    return nia;
    /* extraction: 0.59,6.FRT,11.FRA,16.FRB,21./,26.18,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#line 30749 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Subtract_Single_Rc0(cpu *processor,
                                    instruction_word instruction,
                                    unsigned_word cia,
                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Subtract_Single
#undef MY_PREFIX
#define MY_PREFIX Floating_Subtract_Single_Rc0
#line 3823 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Subtract Single\n"));
#line 30768 "icache.c"

  /* validate: 0.59,6.FRT,11.FRA,16.FRB,21./,26.20,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0xec000028)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.59,6.FRT,11.FRA,16.FRB,21./,26.20,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 3823 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 30802 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Subtract_Single_Rc0;

    /* semantic routine */
#line 3823 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Subtract Single\n"));
#line 30812 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Subtract_Single, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3823 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               1, /*single?*/
                               1) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      1); /*single-precision*/
      }
      else {
        /*HACK!*/
        float s = *(double*)frA - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK, Rc);
    }
#line 30844 "icache.c"
    return nia;
    /* extraction: 0.59,6.FRT,11.FRA,16.FRB,21./,26.20,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#line 30850 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Subtract_Single_Rc1(cpu *processor,
                                    instruction_word instruction,
                                    unsigned_word cia,
                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Subtract_Single
#undef MY_PREFIX
#define MY_PREFIX Floating_Subtract_Single_Rc1
#line 3823 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Subtract Single\n"));
#line 30869 "icache.c"

  /* validate: 0.59,6.FRT,11.FRA,16.FRB,21./,26.20,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0xec000028)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.59,6.FRT,11.FRA,16.FRB,21./,26.20,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 3823 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 30903 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Subtract_Single_Rc1;

    /* semantic routine */
#line 3823 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Subtract Single\n"));
#line 30913 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Subtract_Single, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3823 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               1, /*single?*/
                               1) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      1); /*single-precision*/
      }
      else {
        /*HACK!*/
        float s = *(double*)frA - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK, Rc);
    }
#line 30945 "icache.c"
    return nia;
    /* extraction: 0.59,6.FRT,11.FRA,16.FRB,21./,26.20,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#line 30951 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Add_Single_Rc0(cpu *processor,
                               instruction_word instruction,
                               unsigned_word cia,
                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Add_Single
#undef MY_PREFIX
#define MY_PREFIX Floating_Add_Single_Rc0
#line 3771 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Add Single\n"));
#line 30970 "icache.c"

  /* validate: 0.59,6.FRT,11.FRA,16.FRB,21./,26.21,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0xec00002a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.59,6.FRT,11.FRA,16.FRB,21./,26.21,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 3771 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 31004 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Add_Single_Rc0;

    /* semantic routine */
#line 3771 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Add Single\n"));
#line 31014 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Add_Single, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3771 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               1, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      1); /*single-precision*/
      }
      else {
        /*HACK!*/
        float s = *(double*)frA + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK, Rc);
    }
#line 31046 "icache.c"
    return nia;
    /* extraction: 0.59,6.FRT,11.FRA,16.FRB,21./,26.21,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#line 31052 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Add_Single_Rc1(cpu *processor,
                               instruction_word instruction,
                               unsigned_word cia,
                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Add_Single
#undef MY_PREFIX
#define MY_PREFIX Floating_Add_Single_Rc1
#line 3771 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Add Single\n"));
#line 31071 "icache.c"

  /* validate: 0.59,6.FRT,11.FRA,16.FRB,21./,26.21,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0xec00002a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.59,6.FRT,11.FRA,16.FRB,21./,26.21,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 3771 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 31105 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Add_Single_Rc1;

    /* semantic routine */
#line 3771 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Add Single\n"));
#line 31115 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Add_Single, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3771 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               1, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      1); /*single-precision*/
      }
      else {
        /*HACK!*/
        float s = *(double*)frA + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK, Rc);
    }
#line 31147 "icache.c"
    return nia;
    /* extraction: 0.59,6.FRT,11.FRA,16.FRB,21./,26.21,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#line 31153 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Square_Root_Single_Rc0(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Square_Root_Single
#undef MY_PREFIX
#define MY_PREFIX Floating_Square_Root_Single_Rc0
#line 4712 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Square Root Single\n"));
#line 31172 "icache.c"

  /* validate: 0.59,6.FRT,11./,16.FRB,21./,26.22,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc1f07fe) != 0xec00002c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.59,6.FRT,11./,16.FRB,21./,26.22,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 4712 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 31201 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Square_Root_Single_Rc0;

    /* semantic routine */
#line 4712 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Square Root Single\n"));
#line 31211 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Square_Root_Single, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4712 "./ppc-instructions"
    {
      program_interrupt(processor, cia, optional_instruction_program_interrupt);
    }
#line 31224 "icache.c"
    return nia;
    /* extraction: 0.59,6.FRT,11./,16.FRB,21./,26.22,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRB
#line 31229 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Square_Root_Single_Rc1(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Square_Root_Single
#undef MY_PREFIX
#define MY_PREFIX Floating_Square_Root_Single_Rc1
#line 4712 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Square Root Single\n"));
#line 31248 "icache.c"

  /* validate: 0.59,6.FRT,11./,16.FRB,21./,26.22,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc1f07fe) != 0xec00002c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.59,6.FRT,11./,16.FRB,21./,26.22,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 4712 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 31277 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Square_Root_Single_Rc1;

    /* semantic routine */
#line 4712 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Square Root Single\n"));
#line 31287 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Square_Root_Single, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4712 "./ppc-instructions"
    {
      program_interrupt(processor, cia, optional_instruction_program_interrupt);
    }
#line 31300 "icache.c"
    return nia;
    /* extraction: 0.59,6.FRT,11./,16.FRB,21./,26.22,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRB
#line 31305 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Reciprocal_Estimate_Single_Rc0(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Reciprocal_Estimate_Single
#undef MY_PREFIX
#define MY_PREFIX Floating_Reciprocal_Estimate_Single_Rc0
#line 4715 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Reciprocal Estimate Single\n"));
#line 31324 "icache.c"

  /* validate: 0.59,6.FRT,11./,16.FRB,21./,26.24,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc1f07fe) != 0xec000030)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.59,6.FRT,11./,16.FRB,21./,26.24,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 4715 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 31353 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Reciprocal_Estimate_Single_Rc0;

    /* semantic routine */
#line 4715 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Reciprocal Estimate Single\n"));
#line 31363 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Reciprocal_Estimate_Single, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4715 "./ppc-instructions"
    {
      program_interrupt(processor, cia, optional_instruction_program_interrupt);
    }
#line 31376 "icache.c"
    return nia;
    /* extraction: 0.59,6.FRT,11./,16.FRB,21./,26.24,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRB
#line 31381 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Reciprocal_Estimate_Single_Rc1(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Reciprocal_Estimate_Single
#undef MY_PREFIX
#define MY_PREFIX Floating_Reciprocal_Estimate_Single_Rc1
#line 4715 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Reciprocal Estimate Single\n"));
#line 31400 "icache.c"

  /* validate: 0.59,6.FRT,11./,16.FRB,21./,26.24,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc1f07fe) != 0xec000030)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.59,6.FRT,11./,16.FRB,21./,26.24,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 4715 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 31429 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Reciprocal_Estimate_Single_Rc1;

    /* semantic routine */
#line 4715 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Reciprocal Estimate Single\n"));
#line 31439 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Reciprocal_Estimate_Single, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4715 "./ppc-instructions"
    {
      program_interrupt(processor, cia, optional_instruction_program_interrupt);
    }
#line 31452 "icache.c"
    return nia;
    /* extraction: 0.59,6.FRT,11./,16.FRB,21./,26.24,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRB
#line 31457 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_Single_Rc0(cpu *processor,
                                    instruction_word instruction,
                                    unsigned_word cia,
                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply_Single
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_Single_Rc0
#line 3875 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply Single\n"));
#line 31476 "icache.c"

  /* validate: 0.59,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xec000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.59,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC EXTRACTED32(instruction, 21, 25) /* icache scratch - extraction */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3875 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 31510 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_Single_Rc0;

    /* semantic routine */
#line 3875 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply Single\n"));
#line 31520 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply_Single, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3875 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               1, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      1); /*single-precision*/
      }
      else {
        /*HACK!*/
        float s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 31552 "icache.c"
    return nia;
    /* extraction: 0.59,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 31558 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_Single_Rc1(cpu *processor,
                                    instruction_word instruction,
                                    unsigned_word cia,
                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply_Single
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_Single_Rc1
#line 3875 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply Single\n"));
#line 31577 "icache.c"

  /* validate: 0.59,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xec000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.59,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC EXTRACTED32(instruction, 21, 25) /* icache scratch - extraction */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3875 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 31611 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_Single_Rc1;

    /* semantic routine */
#line 3875 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply Single\n"));
#line 31621 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply_Single, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3875 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               1, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      1); /*single-precision*/
      }
      else {
        /*HACK!*/
        float s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 31653 "icache.c"
    return nia;
    /* extraction: 0.59,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 31659 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_Single_Rc0(cpu *processor,
                                            instruction_word instruction,
                                            unsigned_word cia,
                                            idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract_Single
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_Single_Rc0
#line 4102 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract Single\n"));
#line 31678 "icache.c"

  /* validate: 0.59,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xec000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.59,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC EXTRACTED32(instruction, 21, 25) /* icache scratch - extraction */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4102 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 31717 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_Single_Rc0;

    /* semantic routine */
#line 4102 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract Single\n"));
#line 31727 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract_Single, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4102 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      float product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               1, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               1, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        float s = product - *(double*)frB;
        *(double*)frT = (double)s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 31778 "icache.c"
    return nia;
    /* extraction: 0.59,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 31785 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_Single_Rc1(cpu *processor,
                                            instruction_word instruction,
                                            unsigned_word cia,
                                            idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract_Single
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_Single_Rc1
#line 4102 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract Single\n"));
#line 31804 "icache.c"

  /* validate: 0.59,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xec000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.59,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC EXTRACTED32(instruction, 21, 25) /* icache scratch - extraction */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4102 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 31843 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_Single_Rc1;

    /* semantic routine */
#line 4102 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract Single\n"));
#line 31853 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract_Single, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4102 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      float product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               1, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               1, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        float s = product - *(double*)frB;
        *(double*)frT = (double)s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 31904 "icache.c"
    return nia;
    /* extraction: 0.59,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 31911 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_Single_Rc0(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd_Single
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_Single_Rc0
#line 4012 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add Single\n"));
#line 31930 "icache.c"

  /* validate: 0.59,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xec00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.59,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC EXTRACTED32(instruction, 21, 25) /* icache scratch - extraction */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4012 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 31969 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_Single_Rc0;

    /* semantic routine */
#line 4012 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add Single\n"));
#line 31979 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd_Single, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4012 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      float product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               1, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               1, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        float s = product + *(double*)frB;
        *(double*)frT = (double)s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 32030 "icache.c"
    return nia;
    /* extraction: 0.59,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 32037 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_Single_Rc1(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd_Single
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_Single_Rc1
#line 4012 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add Single\n"));
#line 32056 "icache.c"

  /* validate: 0.59,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xec00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.59,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC EXTRACTED32(instruction, 21, 25) /* icache scratch - extraction */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4012 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 32095 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_Single_Rc1;

    /* semantic routine */
#line 4012 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add Single\n"));
#line 32105 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd_Single, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4012 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      float product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               1, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               1, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        float s = product + *(double*)frB;
        *(double*)frT = (double)s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 32156 "icache.c"
    return nia;
    /* extraction: 0.59,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 32163 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_Single_Rc0(cpu *processor,
                                                     instruction_word instruction,
                                                     unsigned_word cia,
                                                     idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract_Single
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_Single_Rc0
#line 4282 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract Single\n"));
#line 32182 "icache.c"

  /* validate: 0.59,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xec00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.59,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC EXTRACTED32(instruction, 21, 25) /* icache scratch - extraction */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4282 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 32221 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_Single_Rc0;

    /* semantic routine */
#line 4282 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract Single\n"));
#line 32231 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract_Single, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4282 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      float product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               1, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               1, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        float s = -(product - *(double*)frB);
        *(double*)frT = (double)s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 32282 "icache.c"
    return nia;
    /* extraction: 0.59,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 32289 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_Single_Rc1(cpu *processor,
                                                     instruction_word instruction,
                                                     unsigned_word cia,
                                                     idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract_Single
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_Single_Rc1
#line 4282 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract Single\n"));
#line 32308 "icache.c"

  /* validate: 0.59,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xec00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.59,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC EXTRACTED32(instruction, 21, 25) /* icache scratch - extraction */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4282 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 32347 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_Single_Rc1;

    /* semantic routine */
#line 4282 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract Single\n"));
#line 32357 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract_Single, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4282 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      float product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               1, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               1, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        float s = -(product - *(double*)frB);
        *(double*)frT = (double)s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 32408 "icache.c"
    return nia;
    /* extraction: 0.59,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 32415 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_Single_Rc0(cpu *processor,
                                                instruction_word instruction,
                                                unsigned_word cia,
                                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd_Single
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_Single_Rc0
#line 4192 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add Single\n"));
#line 32434 "icache.c"

  /* validate: 0.59,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xec00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.59,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC EXTRACTED32(instruction, 21, 25) /* icache scratch - extraction */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4192 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 32473 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_Single_Rc0;

    /* semantic routine */
#line 4192 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add Single\n"));
#line 32483 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd_Single, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4192 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      float product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               1, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               1, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        float s = -(product + *(double*)frB);
        *(double*)frT = (double)s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 32534 "icache.c"
    return nia;
    /* extraction: 0.59,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 32541 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_Single_Rc1(cpu *processor,
                                                instruction_word instruction,
                                                unsigned_word cia,
                                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd_Single
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_Single_Rc1
#line 4192 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add Single\n"));
#line 32560 "icache.c"

  /* validate: 0.59,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xec00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.59,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC EXTRACTED32(instruction, 21, 25) /* icache scratch - extraction */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4192 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 32599 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_Single_Rc1;

    /* semantic routine */
#line 4192 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add Single\n"));
#line 32609 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd_Single, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4192 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      float product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               1, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               1, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        float s = -(product + *(double*)frB);
        *(double*)frT = (double)s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 32660 "icache.c"
    return nia;
    /* extraction: 0.59,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 32667 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Compare_Unordered(cpu *processor,
                                  instruction_word instruction,
                                  unsigned_word cia,
                                  idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Compare_Unordered
#undef MY_PREFIX
#define MY_PREFIX Floating_Compare_Unordered
#line 4607 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Compare Unordered\n"));
#line 32686 "icache.c"

  /* validate: 0.63,6.BF,9./,11.FRA,16.FRB,21.0,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc6007ff) != 0xfc000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.BF,9./,11.FRA,16.FRB,21.0,31./ #define get-values-from-icache|put-values-in-icache */
#line 68 "./ppc-instructions"
    unsigned const BF UNUSED = cache_entry->crack.X.BF = EXTRACTED32(instruction, 6, 8); /* in icache - extraction */
#line 69 "./ppc-instructions"
    unsigned32 const BF_BITMASK UNUSED = cache_entry->crack.X.BF_BITMASK = (1 << BF); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.X.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.X.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.X.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.X.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 32717 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Compare_Unordered;

    /* semantic routine */
#line 4607 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Compare Unordered\n"));
#line 32727 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Compare_Unordered, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4607 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      unsigned c;
      if (is_NaN(*frA, 0) || is_NaN(*frB, 0))
        c = cr_i_summary_overflow; /* 0b0001 - (FRA) ? (FRB) */
      else if (is_less_than(frA, frB))
        c = cr_i_negative; /* 0b1000 - (FRA) < (FRB) */
      else if (is_greater_than(frA, frB))
        c = cr_i_positive; /* 0b0100 - (FRA) > (FRB) */
      else
        c = cr_i_zero; /* 0b0010 - (FRA) = (FRB) */
      FPSCR_SET_FPCC(c);
      CR_SET(BF, c); /* CR[4*BF..4*BF+3] = c */
      if (is_SNaN(*frA, 0) || is_SNaN(*frB, 0))
        FPSCR_OR_VX(fpscr_vxsnan);
      FPSCR_END(0);
      PPC_INSN_FLOAT_CR(0, FRA_BITMASK | FRB_BITMASK, BF_BITMASK);
    }
#line 32755 "icache.c"
    return nia;
    /* extraction: 0.63,6.BF,9./,11.FRA,16.FRB,21.0,31./ #undef get-values-from-icache|put-values-in-icache */
#undef FRA
#undef FRB
#line 32760 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Round_to_SinglePrecision_Rc0(cpu *processor,
                                             instruction_word instruction,
                                             unsigned_word cia,
                                             idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Round_to_SinglePrecision
#undef MY_PREFIX
#define MY_PREFIX Floating_Round_to_SinglePrecision_Rc0
#line 4332 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Round to Single-Precision\n"));
#line 32779 "icache.c"

  /* validate: 0.63,6.FRT,11./,16.FRB,21.12,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc1f07fe) != 0xfc000018)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11./,16.FRB,21.12,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.X.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.X.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.X.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.X.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 4332 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 32808 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Round_to_SinglePrecision_Rc0;

    /* semantic routine */
#line 4332 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Round to Single-Precision\n"));
#line 32818 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Round_to_SinglePrecision, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4332 "./ppc-instructions"
    {
      int sign;
      int exp;
      unsigned64 frac_grx;
      /***/
        /* split off cases for what to do */
        if (EXTRACTED64(*frB, 1, 11) < 897
            && EXTRACTED64(*frB, 1, 63) > 0) {
            if ((FPSCR & fpscr_ue) == 0) GOTO(Disabled_Exponent_Underflow);
            if ((FPSCR & fpscr_ue) != 0) GOTO(Enabled_Exponent_Underflow);
        }
        if (EXTRACTED64(*frB, 1, 11) > 1150
            && EXTRACTED64(*frB, 1, 11) < 2047) {
            if ((FPSCR & fpscr_oe) == 0) GOTO(Disabled_Exponent_Overflow);
            if ((FPSCR & fpscr_oe) != 0) GOTO(Enabled_Exponent_Overflow);
        }
        if (EXTRACTED64(*frB, 1, 11) > 896
            && EXTRACTED64(*frB, 1, 11) < 1151) GOTO(Normal_Operand);
        if (EXTRACTED64(*frB, 1, 63) == 0) GOTO(Zero_Operand);
        if (EXTRACTED64(*frB, 1, 11) == 2047) {
          if (EXTRACTED64(*frB, 12, 63) == 0) GOTO(Infinity_Operand);
          if (EXTRACTED64(*frB, 12, 12) == 1) GOTO(QNaN_Operand);
          if (EXTRACTED64(*frB, 12, 12) == 0
              && EXTRACTED64(*frB, 13, 63) > 0) GOTO(SNaN_Operand);
        }
      /**/
      LABEL(Disabled_Exponent_Underflow):
        sign = EXTRACTED64(*frB, 0, 0);
        if (EXTRACTED64(*frB, 1, 11) == 0) {
          exp = -1022;
          frac_grx = INSERTED64(EXTRACTED64(*frB, 12, 63), 1, 52);
        }
        if (EXTRACTED64(*frB, 1, 11) > 0) {
          exp = EXTRACTED64(*frB, 1, 11) - 1023;
          frac_grx = BIT64(0) | INSERTED64(EXTRACTED64(*frB, 12, 63), 1, 52);
        }
          /* G|R|X == zero from above */
          while (exp < -126) {
            exp = exp + 1;
            frac_grx = (INSERTED64(EXTRACTED64(frac_grx, 0, 54), 1, 55)
                        | MASKED64(frac_grx, 55, 55));
          }
        FPSCR_SET_UX(EXTRACTED64(frac_grx, 24, 55) > 0);
        Round_Single(processor, sign, &exp, &frac_grx);
        FPSCR_SET_XX(FPSCR & fpscr_fi);
        if (EXTRACTED64(frac_grx, 0, 52) == 0) {
          *frT = INSERTED64(sign, 0, 0);
          if (sign == 0) FPSCR_SET_FPRF(fpscr_rf_pos_zero);
          if (sign == 1) FPSCR_SET_FPRF(fpscr_rf_neg_zero);
        }
        if (EXTRACTED64(frac_grx, 0, 52) > 0) {
          if (EXTRACTED64(frac_grx, 0, 0) == 1) {
            if (sign == 0) FPSCR_SET_FPRF(fpscr_rf_pos_normal_number);
            if (sign == 1) FPSCR_SET_FPRF(fpscr_rf_neg_normal_number);
          }
          if (EXTRACTED64(frac_grx, 0, 0) == 0) {
            if (sign == 0) FPSCR_SET_FPRF(fpscr_rf_pos_denormalized_number);
            if (sign == 1) FPSCR_SET_FPRF(fpscr_rf_neg_denormalized_number);
          }
          /*Normalize_Operand:*/
            while (EXTRACTED64(frac_grx, 0, 0) == 0) {
              exp = exp - 1;
              frac_grx = INSERTED64(EXTRACTED64(frac_grx, 1,  52), 0, 51);
            }
          *frT = (INSERTED64(sign, 0, 0)
                  | INSERTED64(exp + 1023, 1, 11)
                  | INSERTED64(EXTRACTED64(frac_grx, 1, 52), 12, 63));
        }
        GOTO(Done);
      /**/
      LABEL(Enabled_Exponent_Underflow):
        FPSCR_SET_UX(1);
        sign = EXTRACTED64(*frB, 0, 0);
        if (EXTRACTED64(*frB, 1, 11) == 0) {
          exp = -1022;
          frac_grx = INSERTED64(EXTRACTED64(*frB, 12, 63), 1, 52);
        }
        if (EXTRACTED64(*frB, 1, 11) > 0) {
          exp = EXTRACTED64(*frB, 1, 11) - 1023;
          frac_grx = (BIT64(0) |
                      INSERTED64(EXTRACTED64(*frB, 12, 63), 1, 52));
        }
        /*Normalize_Operand:*/
          while (EXTRACTED64(frac_grx, 0, 0) == 0) {
            exp = exp - 1;
            frac_grx = INSERTED64(EXTRACTED64(frac_grx, 1, 52), 0, 51);
          }
        Round_Single(processor, sign, &exp, &frac_grx);
        FPSCR_SET_XX(FPSCR & fpscr_fi);
        exp = exp + 192;
        *frT = (INSERTED64(sign, 0, 0)
                | INSERTED64(exp + 1023, 1, 11)
                | INSERTED64(EXTRACTED64(frac_grx, 1, 52), 12, 63));
        if (sign == 0) FPSCR_SET_FPRF(fpscr_rf_pos_normal_number);
        if (sign == 1) FPSCR_SET_FPRF(fpscr_rf_neg_normal_number);
        GOTO(Done);
      /**/
      LABEL(Disabled_Exponent_Overflow):
        FPSCR_SET_OX(1);
        if ((FPSCR & fpscr_rn) == fpscr_rn_round_to_nearest) {
          if (EXTRACTED64(*frB, 0, 0) == 0) {
            *frT = INSERTED64(0x7FF00000, 0, 31) | 0x00000000;
            FPSCR_SET_FPRF(fpscr_rf_pos_infinity);
          }
          if (EXTRACTED64(*frB, 0, 0) == 1) {
            *frT = INSERTED64(0xFFF00000, 0, 31) | 0x00000000;
            FPSCR_SET_FPRF(fpscr_rf_neg_infinity);
          }
        }
        if ((FPSCR & fpscr_rn) == fpscr_rn_round_towards_zero) {
          if (EXTRACTED64(*frB, 0, 0) == 0) {
            *frT = INSERTED64(0x47EFFFFF, 0, 31) | 0xE0000000;
            FPSCR_SET_FPRF(fpscr_rf_pos_normal_number);
          }
          if (EXTRACTED64(*frB, 0, 0) == 1) {
            *frT = INSERTED64(0xC7EFFFFF, 0, 31) | 0xE0000000;
            FPSCR_SET_FPRF(fpscr_rf_neg_normal_number);
          }
        }
        if ((FPSCR & fpscr_rn) == fpscr_rn_round_towards_pos_infinity) {
          if (EXTRACTED64(*frB, 0, 0) == 0) {
            *frT = INSERTED64(0x7FF00000, 0, 31) | 0x00000000;
            FPSCR_SET_FPRF(fpscr_rf_pos_infinity);
          }
          if (EXTRACTED64(*frB, 0, 0) == 1) {
            *frT = INSERTED64(0xC7EFFFFF, 0, 31) | 0xE0000000;
            FPSCR_SET_FPRF(fpscr_rf_neg_normal_number);
          }
        }
        if ((FPSCR & fpscr_rn) == fpscr_rn_round_towards_neg_infinity) {
          if (EXTRACTED64(*frB, 0, 0) == 0) {
            *frT = INSERTED64(0x47EFFFFF, 0, 31) | 0xE0000000;
            FPSCR_SET_FPRF(fpscr_rf_pos_normal_number);
          }
          if (EXTRACTED64(*frB, 0, 0) == 1) {
            *frT = INSERTED64(0xFFF00000, 0, 31) | 0x00000000;
            FPSCR_SET_FPRF(fpscr_rf_neg_infinity);
          }
        }
        /* FPSCR[FR] <- undefined */
        FPSCR_SET_FI(1);
        FPSCR_SET_XX(1);
        GOTO(Done);
      /**/
      LABEL(Enabled_Exponent_Overflow):
        sign = EXTRACTED64(*frB, 0, 0);
        exp = EXTRACTED64(*frB, 1, 11) - 1023;
        frac_grx = BIT64(0) | INSERTED64(EXTRACTED64(*frB, 12, 63), 1, 52);
        Round_Single(processor, sign, &exp, &frac_grx);
        FPSCR_SET_XX(FPSCR & fpscr_fi);
      /**/
      LABEL(Enabled_Overflow):
        FPSCR_SET_OX(1);
        exp = exp - 192;
        *frT = (INSERTED64(sign, 0, 0)
                | INSERTED64(exp + 1023, 1, 11)
                | INSERTED64(EXTRACTED64(frac_grx, 1, 52), 12, 63));
        if (sign == 0) FPSCR_SET_FPRF(fpscr_rf_pos_normal_number);
        if (sign == 1) FPSCR_SET_FPRF(fpscr_rf_neg_normal_number);
        GOTO(Done);
      /**/
      LABEL(Zero_Operand):
        *frT = *frB;
        if (EXTRACTED64(*frB, 0, 0) == 0) FPSCR_SET_FPRF(fpscr_rf_pos_zero);
        if (EXTRACTED64(*frB, 0, 0) == 1) FPSCR_SET_FPRF(fpscr_rf_neg_zero);
        FPSCR_SET_FR(0);
        FPSCR_SET_FI(0);
        GOTO(Done);
      /**/
      LABEL(Infinity_Operand):
        *frT = *frB;
        if (EXTRACTED64(*frB, 0, 0) == 0) FPSCR_SET_FPRF(fpscr_rf_pos_infinity);
        if (EXTRACTED64(*frB, 0, 0) == 1) FPSCR_SET_FPRF(fpscr_rf_neg_infinity);
        FPSCR_SET_FR(0);
        FPSCR_SET_FI(0);
        GOTO(Done);
      /**/
      LABEL(QNaN_Operand):
        *frT = INSERTED64(EXTRACTED64(*frB, 0, 34), 0, 34);
        FPSCR_SET_FPRF(fpscr_rf_quiet_nan);
        FPSCR_SET_FR(0);
        FPSCR_SET_FI(0);
        GOTO(Done);
      /**/
      LABEL(SNaN_Operand):
        FPSCR_OR_VX(fpscr_vxsnan);
        if ((FPSCR & fpscr_ve) == 0) {
          *frT = (MASKED64(*frB, 0, 11)
                  | BIT64(12)
                  | MASKED64(*frB, 13, 34));
          FPSCR_SET_FPRF(fpscr_rf_quiet_nan);
        }
        FPSCR_SET_FR(0);
        FPSCR_SET_FI(0);
        GOTO(Done);
      /**/
      LABEL(Normal_Operand):
        sign = EXTRACTED64(*frB, 0, 0);
        exp = EXTRACTED64(*frB, 1, 11) - 1023;
        frac_grx = BIT64(0) | INSERTED64(EXTRACTED64(*frB, 12, 63), 1, 52);
        Round_Single(processor, sign, &exp, &frac_grx);
        FPSCR_SET_XX(FPSCR & fpscr_fi);
        if (exp > 127 && (FPSCR & fpscr_oe) == 0) GOTO(Disabled_Exponent_Overflow);
        if (exp > 127 && (FPSCR & fpscr_oe) != 0) GOTO(Enabled_Overflow);
        *frT = (INSERTED64(sign, 0, 0)
                | INSERTED64(exp + 1023, 1, 11)
                | INSERTED64(EXTRACTED64(frac_grx, 1, 52), 12, 63));
        if (sign == 0) FPSCR_SET_FPRF(fpscr_rf_pos_normal_number);
        if (sign == 1) FPSCR_SET_FPRF(fpscr_rf_neg_normal_number);
        GOTO(Done);
      /**/
      LABEL(Done):
        PPC_INSN_FLOAT(FRT_BITMASK, FRB_BITMASK, Rc);
    }
#line 33042 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11./,16.FRB,21.12,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRB
#line 33047 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Round_to_SinglePrecision_Rc1(cpu *processor,
                                             instruction_word instruction,
                                             unsigned_word cia,
                                             idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Round_to_SinglePrecision
#undef MY_PREFIX
#define MY_PREFIX Floating_Round_to_SinglePrecision_Rc1
#line 4332 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Round to Single-Precision\n"));
#line 33066 "icache.c"

  /* validate: 0.63,6.FRT,11./,16.FRB,21.12,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc1f07fe) != 0xfc000018)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11./,16.FRB,21.12,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.X.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.X.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.X.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.X.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 4332 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 33095 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Round_to_SinglePrecision_Rc1;

    /* semantic routine */
#line 4332 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Round to Single-Precision\n"));
#line 33105 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Round_to_SinglePrecision, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4332 "./ppc-instructions"
    {
      int sign;
      int exp;
      unsigned64 frac_grx;
      /***/
        /* split off cases for what to do */
        if (EXTRACTED64(*frB, 1, 11) < 897
            && EXTRACTED64(*frB, 1, 63) > 0) {
            if ((FPSCR & fpscr_ue) == 0) GOTO(Disabled_Exponent_Underflow);
            if ((FPSCR & fpscr_ue) != 0) GOTO(Enabled_Exponent_Underflow);
        }
        if (EXTRACTED64(*frB, 1, 11) > 1150
            && EXTRACTED64(*frB, 1, 11) < 2047) {
            if ((FPSCR & fpscr_oe) == 0) GOTO(Disabled_Exponent_Overflow);
            if ((FPSCR & fpscr_oe) != 0) GOTO(Enabled_Exponent_Overflow);
        }
        if (EXTRACTED64(*frB, 1, 11) > 896
            && EXTRACTED64(*frB, 1, 11) < 1151) GOTO(Normal_Operand);
        if (EXTRACTED64(*frB, 1, 63) == 0) GOTO(Zero_Operand);
        if (EXTRACTED64(*frB, 1, 11) == 2047) {
          if (EXTRACTED64(*frB, 12, 63) == 0) GOTO(Infinity_Operand);
          if (EXTRACTED64(*frB, 12, 12) == 1) GOTO(QNaN_Operand);
          if (EXTRACTED64(*frB, 12, 12) == 0
              && EXTRACTED64(*frB, 13, 63) > 0) GOTO(SNaN_Operand);
        }
      /**/
      LABEL(Disabled_Exponent_Underflow):
        sign = EXTRACTED64(*frB, 0, 0);
        if (EXTRACTED64(*frB, 1, 11) == 0) {
          exp = -1022;
          frac_grx = INSERTED64(EXTRACTED64(*frB, 12, 63), 1, 52);
        }
        if (EXTRACTED64(*frB, 1, 11) > 0) {
          exp = EXTRACTED64(*frB, 1, 11) - 1023;
          frac_grx = BIT64(0) | INSERTED64(EXTRACTED64(*frB, 12, 63), 1, 52);
        }
          /* G|R|X == zero from above */
          while (exp < -126) {
            exp = exp + 1;
            frac_grx = (INSERTED64(EXTRACTED64(frac_grx, 0, 54), 1, 55)
                        | MASKED64(frac_grx, 55, 55));
          }
        FPSCR_SET_UX(EXTRACTED64(frac_grx, 24, 55) > 0);
        Round_Single(processor, sign, &exp, &frac_grx);
        FPSCR_SET_XX(FPSCR & fpscr_fi);
        if (EXTRACTED64(frac_grx, 0, 52) == 0) {
          *frT = INSERTED64(sign, 0, 0);
          if (sign == 0) FPSCR_SET_FPRF(fpscr_rf_pos_zero);
          if (sign == 1) FPSCR_SET_FPRF(fpscr_rf_neg_zero);
        }
        if (EXTRACTED64(frac_grx, 0, 52) > 0) {
          if (EXTRACTED64(frac_grx, 0, 0) == 1) {
            if (sign == 0) FPSCR_SET_FPRF(fpscr_rf_pos_normal_number);
            if (sign == 1) FPSCR_SET_FPRF(fpscr_rf_neg_normal_number);
          }
          if (EXTRACTED64(frac_grx, 0, 0) == 0) {
            if (sign == 0) FPSCR_SET_FPRF(fpscr_rf_pos_denormalized_number);
            if (sign == 1) FPSCR_SET_FPRF(fpscr_rf_neg_denormalized_number);
          }
          /*Normalize_Operand:*/
            while (EXTRACTED64(frac_grx, 0, 0) == 0) {
              exp = exp - 1;
              frac_grx = INSERTED64(EXTRACTED64(frac_grx, 1,  52), 0, 51);
            }
          *frT = (INSERTED64(sign, 0, 0)
                  | INSERTED64(exp + 1023, 1, 11)
                  | INSERTED64(EXTRACTED64(frac_grx, 1, 52), 12, 63));
        }
        GOTO(Done);
      /**/
      LABEL(Enabled_Exponent_Underflow):
        FPSCR_SET_UX(1);
        sign = EXTRACTED64(*frB, 0, 0);
        if (EXTRACTED64(*frB, 1, 11) == 0) {
          exp = -1022;
          frac_grx = INSERTED64(EXTRACTED64(*frB, 12, 63), 1, 52);
        }
        if (EXTRACTED64(*frB, 1, 11) > 0) {
          exp = EXTRACTED64(*frB, 1, 11) - 1023;
          frac_grx = (BIT64(0) |
                      INSERTED64(EXTRACTED64(*frB, 12, 63), 1, 52));
        }
        /*Normalize_Operand:*/
          while (EXTRACTED64(frac_grx, 0, 0) == 0) {
            exp = exp - 1;
            frac_grx = INSERTED64(EXTRACTED64(frac_grx, 1, 52), 0, 51);
          }
        Round_Single(processor, sign, &exp, &frac_grx);
        FPSCR_SET_XX(FPSCR & fpscr_fi);
        exp = exp + 192;
        *frT = (INSERTED64(sign, 0, 0)
                | INSERTED64(exp + 1023, 1, 11)
                | INSERTED64(EXTRACTED64(frac_grx, 1, 52), 12, 63));
        if (sign == 0) FPSCR_SET_FPRF(fpscr_rf_pos_normal_number);
        if (sign == 1) FPSCR_SET_FPRF(fpscr_rf_neg_normal_number);
        GOTO(Done);
      /**/
      LABEL(Disabled_Exponent_Overflow):
        FPSCR_SET_OX(1);
        if ((FPSCR & fpscr_rn) == fpscr_rn_round_to_nearest) {
          if (EXTRACTED64(*frB, 0, 0) == 0) {
            *frT = INSERTED64(0x7FF00000, 0, 31) | 0x00000000;
            FPSCR_SET_FPRF(fpscr_rf_pos_infinity);
          }
          if (EXTRACTED64(*frB, 0, 0) == 1) {
            *frT = INSERTED64(0xFFF00000, 0, 31) | 0x00000000;
            FPSCR_SET_FPRF(fpscr_rf_neg_infinity);
          }
        }
        if ((FPSCR & fpscr_rn) == fpscr_rn_round_towards_zero) {
          if (EXTRACTED64(*frB, 0, 0) == 0) {
            *frT = INSERTED64(0x47EFFFFF, 0, 31) | 0xE0000000;
            FPSCR_SET_FPRF(fpscr_rf_pos_normal_number);
          }
          if (EXTRACTED64(*frB, 0, 0) == 1) {
            *frT = INSERTED64(0xC7EFFFFF, 0, 31) | 0xE0000000;
            FPSCR_SET_FPRF(fpscr_rf_neg_normal_number);
          }
        }
        if ((FPSCR & fpscr_rn) == fpscr_rn_round_towards_pos_infinity) {
          if (EXTRACTED64(*frB, 0, 0) == 0) {
            *frT = INSERTED64(0x7FF00000, 0, 31) | 0x00000000;
            FPSCR_SET_FPRF(fpscr_rf_pos_infinity);
          }
          if (EXTRACTED64(*frB, 0, 0) == 1) {
            *frT = INSERTED64(0xC7EFFFFF, 0, 31) | 0xE0000000;
            FPSCR_SET_FPRF(fpscr_rf_neg_normal_number);
          }
        }
        if ((FPSCR & fpscr_rn) == fpscr_rn_round_towards_neg_infinity) {
          if (EXTRACTED64(*frB, 0, 0) == 0) {
            *frT = INSERTED64(0x47EFFFFF, 0, 31) | 0xE0000000;
            FPSCR_SET_FPRF(fpscr_rf_pos_normal_number);
          }
          if (EXTRACTED64(*frB, 0, 0) == 1) {
            *frT = INSERTED64(0xFFF00000, 0, 31) | 0x00000000;
            FPSCR_SET_FPRF(fpscr_rf_neg_infinity);
          }
        }
        /* FPSCR[FR] <- undefined */
        FPSCR_SET_FI(1);
        FPSCR_SET_XX(1);
        GOTO(Done);
      /**/
      LABEL(Enabled_Exponent_Overflow):
        sign = EXTRACTED64(*frB, 0, 0);
        exp = EXTRACTED64(*frB, 1, 11) - 1023;
        frac_grx = BIT64(0) | INSERTED64(EXTRACTED64(*frB, 12, 63), 1, 52);
        Round_Single(processor, sign, &exp, &frac_grx);
        FPSCR_SET_XX(FPSCR & fpscr_fi);
      /**/
      LABEL(Enabled_Overflow):
        FPSCR_SET_OX(1);
        exp = exp - 192;
        *frT = (INSERTED64(sign, 0, 0)
                | INSERTED64(exp + 1023, 1, 11)
                | INSERTED64(EXTRACTED64(frac_grx, 1, 52), 12, 63));
        if (sign == 0) FPSCR_SET_FPRF(fpscr_rf_pos_normal_number);
        if (sign == 1) FPSCR_SET_FPRF(fpscr_rf_neg_normal_number);
        GOTO(Done);
      /**/
      LABEL(Zero_Operand):
        *frT = *frB;
        if (EXTRACTED64(*frB, 0, 0) == 0) FPSCR_SET_FPRF(fpscr_rf_pos_zero);
        if (EXTRACTED64(*frB, 0, 0) == 1) FPSCR_SET_FPRF(fpscr_rf_neg_zero);
        FPSCR_SET_FR(0);
        FPSCR_SET_FI(0);
        GOTO(Done);
      /**/
      LABEL(Infinity_Operand):
        *frT = *frB;
        if (EXTRACTED64(*frB, 0, 0) == 0) FPSCR_SET_FPRF(fpscr_rf_pos_infinity);
        if (EXTRACTED64(*frB, 0, 0) == 1) FPSCR_SET_FPRF(fpscr_rf_neg_infinity);
        FPSCR_SET_FR(0);
        FPSCR_SET_FI(0);
        GOTO(Done);
      /**/
      LABEL(QNaN_Operand):
        *frT = INSERTED64(EXTRACTED64(*frB, 0, 34), 0, 34);
        FPSCR_SET_FPRF(fpscr_rf_quiet_nan);
        FPSCR_SET_FR(0);
        FPSCR_SET_FI(0);
        GOTO(Done);
      /**/
      LABEL(SNaN_Operand):
        FPSCR_OR_VX(fpscr_vxsnan);
        if ((FPSCR & fpscr_ve) == 0) {
          *frT = (MASKED64(*frB, 0, 11)
                  | BIT64(12)
                  | MASKED64(*frB, 13, 34));
          FPSCR_SET_FPRF(fpscr_rf_quiet_nan);
        }
        FPSCR_SET_FR(0);
        FPSCR_SET_FI(0);
        GOTO(Done);
      /**/
      LABEL(Normal_Operand):
        sign = EXTRACTED64(*frB, 0, 0);
        exp = EXTRACTED64(*frB, 1, 11) - 1023;
        frac_grx = BIT64(0) | INSERTED64(EXTRACTED64(*frB, 12, 63), 1, 52);
        Round_Single(processor, sign, &exp, &frac_grx);
        FPSCR_SET_XX(FPSCR & fpscr_fi);
        if (exp > 127 && (FPSCR & fpscr_oe) == 0) GOTO(Disabled_Exponent_Overflow);
        if (exp > 127 && (FPSCR & fpscr_oe) != 0) GOTO(Enabled_Overflow);
        *frT = (INSERTED64(sign, 0, 0)
                | INSERTED64(exp + 1023, 1, 11)
                | INSERTED64(EXTRACTED64(frac_grx, 1, 52), 12, 63));
        if (sign == 0) FPSCR_SET_FPRF(fpscr_rf_pos_normal_number);
        if (sign == 1) FPSCR_SET_FPRF(fpscr_rf_neg_normal_number);
        GOTO(Done);
      /**/
      LABEL(Done):
        PPC_INSN_FLOAT(FRT_BITMASK, FRB_BITMASK, Rc);
    }
#line 33329 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11./,16.FRB,21.12,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRB
#line 33334 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Convert_To_Integer_Word_Rc0(cpu *processor,
                                            instruction_word instruction,
                                            unsigned_word cia,
                                            idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Convert_To_Integer_Word
#undef MY_PREFIX
#define MY_PREFIX Floating_Convert_To_Integer_Word_Rc0
#line 4553 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Convert To Integer Word\n"));
#line 33353 "icache.c"

  /* validate: 0.63,6.FRT,11./,16.FRB,21.14,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc1f07fe) != 0xfc00001c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11./,16.FRB,21.14,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.X.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.X.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.X.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.X.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 4553 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 33382 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Convert_To_Integer_Word_Rc0;

    /* semantic routine */
#line 4553 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Convert To Integer Word\n"));
#line 33392 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Convert_To_Integer_Word, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4553 "./ppc-instructions"
    {
      floating_point_assist_interrupt(processor, cia);
    }
#line 33405 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11./,16.FRB,21.14,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRB
#line 33410 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Convert_To_Integer_Word_Rc1(cpu *processor,
                                            instruction_word instruction,
                                            unsigned_word cia,
                                            idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Convert_To_Integer_Word
#undef MY_PREFIX
#define MY_PREFIX Floating_Convert_To_Integer_Word_Rc1
#line 4553 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Convert To Integer Word\n"));
#line 33429 "icache.c"

  /* validate: 0.63,6.FRT,11./,16.FRB,21.14,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc1f07fe) != 0xfc00001c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11./,16.FRB,21.14,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.X.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.X.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.X.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.X.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 4553 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 33458 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Convert_To_Integer_Word_Rc1;

    /* semantic routine */
#line 4553 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Convert To Integer Word\n"));
#line 33468 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Convert_To_Integer_Word, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4553 "./ppc-instructions"
    {
      floating_point_assist_interrupt(processor, cia);
    }
#line 33481 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11./,16.FRB,21.14,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRB
#line 33486 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Convert_To_Integer_Word_with_round_towards_Zero_Rc0(cpu *processor,
                                                                    instruction_word instruction,
                                                                    unsigned_word cia,
                                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Convert_To_Integer_Word_with_round_towards_Zero
#undef MY_PREFIX
#define MY_PREFIX Floating_Convert_To_Integer_Word_with_round_towards_Zero_Rc0
#line 4560 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Convert To Integer Word with round towards Zero\n"));
#line 33505 "icache.c"

  /* validate: 0.63,6.FRT,11./,16.FRB,21.15,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc1f07fe) != 0xfc00001e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11./,16.FRB,21.15,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.X.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.X.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.X.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.X.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 4560 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 33534 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Convert_To_Integer_Word_with_round_towards_Zero_Rc0;

    /* semantic routine */
#line 4560 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Convert To Integer Word with round towards Zero\n"));
#line 33544 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Convert_To_Integer_Word_with_round_towards_Zero, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4560 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      convert_to_integer(processor, cia,
                         frT, *frB,
                         fpscr_rn_round_towards_zero, 32);
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRB_BITMASK, Rc);
    }
#line 33562 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11./,16.FRB,21.15,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRB
#line 33567 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Convert_To_Integer_Word_with_round_towards_Zero_Rc1(cpu *processor,
                                                                    instruction_word instruction,
                                                                    unsigned_word cia,
                                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Convert_To_Integer_Word_with_round_towards_Zero
#undef MY_PREFIX
#define MY_PREFIX Floating_Convert_To_Integer_Word_with_round_towards_Zero_Rc1
#line 4560 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Convert To Integer Word with round towards Zero\n"));
#line 33586 "icache.c"

  /* validate: 0.63,6.FRT,11./,16.FRB,21.15,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc1f07fe) != 0xfc00001e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11./,16.FRB,21.15,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.X.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.X.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.X.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.X.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 4560 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 33615 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Convert_To_Integer_Word_with_round_towards_Zero_Rc1;

    /* semantic routine */
#line 4560 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Convert To Integer Word with round towards Zero\n"));
#line 33625 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Convert_To_Integer_Word_with_round_towards_Zero, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4560 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      convert_to_integer(processor, cia,
                         frT, *frB,
                         fpscr_rn_round_towards_zero, 32);
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRB_BITMASK, Rc);
    }
#line 33643 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11./,16.FRB,21.15,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRB
#line 33648 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Divide_Rc0(cpu *processor,
                           instruction_word instruction,
                           unsigned_word cia,
                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Divide
#undef MY_PREFIX
#define MY_PREFIX Floating_Divide_Rc0
#line 3901 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Divide\n"));
#line 33667 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21./,26.18,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0xfc000024)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21./,26.18,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 3901 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 33701 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Divide_Rc0;

    /* semantic routine */
#line 3901 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Divide\n"));
#line 33711 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Divide, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3901 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frB,
                               fpscr_vxsnan | fpscr_vxzdz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else if (is_invalid_zero_divide (processor, cia,
                                       *frA, *frB,
                                       0 /*single?*/)) {
        invalid_zero_divide_operation (processor, cia,
                                       frT, *frA, *frB,
                                       0 /*single?*/);
      }
      else {
        /*HACK!*/
        double s = *(double*)frA / *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK, Rc);
    }
#line 33750 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21./,26.18,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#line 33756 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Divide_Rc1(cpu *processor,
                           instruction_word instruction,
                           unsigned_word cia,
                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Divide
#undef MY_PREFIX
#define MY_PREFIX Floating_Divide_Rc1
#line 3901 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Divide\n"));
#line 33775 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21./,26.18,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0xfc000024)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21./,26.18,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 3901 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 33809 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Divide_Rc1;

    /* semantic routine */
#line 3901 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Divide\n"));
#line 33819 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Divide, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3901 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frB,
                               fpscr_vxsnan | fpscr_vxzdz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else if (is_invalid_zero_divide (processor, cia,
                                       *frA, *frB,
                                       0 /*single?*/)) {
        invalid_zero_divide_operation (processor, cia,
                                       frT, *frA, *frB,
                                       0 /*single?*/);
      }
      else {
        /*HACK!*/
        double s = *(double*)frA / *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK, Rc);
    }
#line 33858 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21./,26.18,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#line 33864 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Subtract_Rc0(cpu *processor,
                             instruction_word instruction,
                             unsigned_word cia,
                             idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Subtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Subtract_Rc0
#line 3797 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Subtract\n"));
#line 33883 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21./,26.20,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0xfc000028)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21./,26.20,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 3797 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 33917 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Subtract_Rc0;

    /* semantic routine */
#line 3797 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Subtract\n"));
#line 33927 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Subtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3797 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               1) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK, Rc);
    }
#line 33959 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21./,26.20,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#line 33965 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Subtract_Rc1(cpu *processor,
                             instruction_word instruction,
                             unsigned_word cia,
                             idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Subtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Subtract_Rc1
#line 3797 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Subtract\n"));
#line 33984 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21./,26.20,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0xfc000028)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21./,26.20,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 3797 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 34018 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Subtract_Rc1;

    /* semantic routine */
#line 3797 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Subtract\n"));
#line 34028 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Subtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3797 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               1) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK, Rc);
    }
#line 34060 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21./,26.20,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#line 34066 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Add_Rc0(cpu *processor,
                        instruction_word instruction,
                        unsigned_word cia,
                        idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Add
#undef MY_PREFIX
#define MY_PREFIX Floating_Add_Rc0
#line 3745 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Add\n"));
#line 34085 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21./,26.21,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0xfc00002a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21./,26.21,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 3745 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 34119 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Add_Rc0;

    /* semantic routine */
#line 3745 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Add\n"));
#line 34129 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Add, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3745 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK, Rc);
    }
#line 34161 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21./,26.21,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#line 34167 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Add_Rc1(cpu *processor,
                        instruction_word instruction,
                        unsigned_word cia,
                        idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Add
#undef MY_PREFIX
#define MY_PREFIX Floating_Add_Rc1
#line 3745 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Add\n"));
#line 34186 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21./,26.21,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0xfc00002a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21./,26.21,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 3745 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 34220 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Add_Rc1;

    /* semantic routine */
#line 3745 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Add\n"));
#line 34230 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Add, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3745 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK, Rc);
    }
#line 34262 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21./,26.21,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#line 34268 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Square_Root_Rc0(cpu *processor,
                                instruction_word instruction,
                                unsigned_word cia,
                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Square_Root
#undef MY_PREFIX
#define MY_PREFIX Floating_Square_Root_Rc0
#line 4709 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Square Root\n"));
#line 34287 "icache.c"

  /* validate: 0.63,6.FRT,11./,16.FRB,21./,26.22,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc1f07fe) != 0xfc00002c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11./,16.FRB,21./,26.22,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 4709 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 34316 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Square_Root_Rc0;

    /* semantic routine */
#line 4709 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Square Root\n"));
#line 34326 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Square_Root, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4709 "./ppc-instructions"
    {
      program_interrupt(processor, cia, optional_instruction_program_interrupt);
    }
#line 34339 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11./,16.FRB,21./,26.22,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRB
#line 34344 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Square_Root_Rc1(cpu *processor,
                                instruction_word instruction,
                                unsigned_word cia,
                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Square_Root
#undef MY_PREFIX
#define MY_PREFIX Floating_Square_Root_Rc1
#line 4709 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Square Root\n"));
#line 34363 "icache.c"

  /* validate: 0.63,6.FRT,11./,16.FRB,21./,26.22,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc1f07fe) != 0xfc00002c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11./,16.FRB,21./,26.22,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 4709 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 34392 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Square_Root_Rc1;

    /* semantic routine */
#line 4709 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Square Root\n"));
#line 34402 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Square_Root, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4709 "./ppc-instructions"
    {
      program_interrupt(processor, cia, optional_instruction_program_interrupt);
    }
#line 34415 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11./,16.FRB,21./,26.22,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRB
#line 34420 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC0_Rc0(cpu *processor,
                                instruction_word instruction,
                                unsigned_word cia,
                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC0_Rc0
#line 4729 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 34439 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 0 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4729 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 34478 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC0_Rc0;

    /* semantic routine */
#line 4729 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 34488 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4729 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 34510 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 34517 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC0_Rc1(cpu *processor,
                                instruction_word instruction,
                                unsigned_word cia,
                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC0_Rc1
#line 4729 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 34536 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 0 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4729 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 34575 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC0_Rc1;

    /* semantic routine */
#line 4729 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 34585 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4729 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 34607 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 34614 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC0_Rc0(cpu *processor,
                                  instruction_word instruction,
                                  unsigned_word cia,
                                  idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC0_Rc0
#line 3849 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 34633 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 0 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3849 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 34667 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC0_Rc0;

    /* semantic routine */
#line 3849 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 34677 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3849 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 34709 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 34715 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC0_Rc1(cpu *processor,
                                  instruction_word instruction,
                                  unsigned_word cia,
                                  idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC0_Rc1
#line 3849 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 34734 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 0 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3849 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 34768 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC0_Rc1;

    /* semantic routine */
#line 3849 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 34778 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3849 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 34810 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 34816 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Reciprocal_Square_Root_Estimate_Rc0(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Reciprocal_Square_Root_Estimate
#undef MY_PREFIX
#define MY_PREFIX Floating_Reciprocal_Square_Root_Estimate_Rc0
#line 4718 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Reciprocal Square Root Estimate\n"));
#line 34835 "icache.c"

  /* validate: 0.63,6.FRT,11./,16.FRB,21./,26.26,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc1f07fe) != 0xfc000034)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11./,16.FRB,21./,26.26,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 4718 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 34864 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Reciprocal_Square_Root_Estimate_Rc0;

    /* semantic routine */
#line 4718 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Reciprocal Square Root Estimate\n"));
#line 34874 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Reciprocal_Square_Root_Estimate, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4718 "./ppc-instructions"
    {
      program_interrupt(processor, cia, optional_instruction_program_interrupt);
    }
#line 34887 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11./,16.FRB,21./,26.26,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRB
#line 34892 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Reciprocal_Square_Root_Estimate_Rc1(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Reciprocal_Square_Root_Estimate
#undef MY_PREFIX
#define MY_PREFIX Floating_Reciprocal_Square_Root_Estimate_Rc1
#line 4718 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Reciprocal Square Root Estimate\n"));
#line 34911 "icache.c"

  /* validate: 0.63,6.FRT,11./,16.FRB,21./,26.26,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc1f07fe) != 0xfc000034)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11./,16.FRB,21./,26.26,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 4718 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 34940 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Reciprocal_Square_Root_Estimate_Rc1;

    /* semantic routine */
#line 4718 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Reciprocal Square Root Estimate\n"));
#line 34950 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Reciprocal_Square_Root_Estimate, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4718 "./ppc-instructions"
    {
      program_interrupt(processor, cia, optional_instruction_program_interrupt);
    }
#line 34963 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11./,16.FRB,21./,26.26,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRB
#line 34968 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC0_Rc0(cpu *processor,
                                          instruction_word instruction,
                                          unsigned_word cia,
                                          idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC0_Rc0
#line 4057 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 34987 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 0 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4057 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 35026 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC0_Rc0;

    /* semantic routine */
#line 4057 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 35036 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4057 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 35087 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 35094 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC0_Rc1(cpu *processor,
                                          instruction_word instruction,
                                          unsigned_word cia,
                                          idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC0_Rc1
#line 4057 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 35113 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 0 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4057 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 35152 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC0_Rc1;

    /* semantic routine */
#line 4057 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 35162 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4057 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 35213 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 35220 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC0_Rc0(cpu *processor,
                                     instruction_word instruction,
                                     unsigned_word cia,
                                     idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC0_Rc0
#line 3967 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 35239 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 0 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3967 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 35278 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC0_Rc0;

    /* semantic routine */
#line 3967 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 35288 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3967 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 35339 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 35346 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC0_Rc1(cpu *processor,
                                     instruction_word instruction,
                                     unsigned_word cia,
                                     idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC0_Rc1
#line 3967 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 35365 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 0 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3967 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 35404 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC0_Rc1;

    /* semantic routine */
#line 3967 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 35414 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3967 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 35465 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 35472 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC0_Rc0(cpu *processor,
                                                   instruction_word instruction,
                                                   unsigned_word cia,
                                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC0_Rc0
#line 4237 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 35491 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 0 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4237 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 35530 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC0_Rc0;

    /* semantic routine */
#line 4237 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 35540 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4237 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 35591 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 35598 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC0_Rc1(cpu *processor,
                                                   instruction_word instruction,
                                                   unsigned_word cia,
                                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC0_Rc1
#line 4237 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 35617 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 0 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4237 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 35656 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC0_Rc1;

    /* semantic routine */
#line 4237 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 35666 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4237 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 35717 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 35724 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC0_Rc0(cpu *processor,
                                              instruction_word instruction,
                                              unsigned_word cia,
                                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC0_Rc0
#line 4147 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 35743 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 0 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4147 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 35782 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC0_Rc0;

    /* semantic routine */
#line 4147 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 35792 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4147 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 35843 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 35850 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC0_Rc1(cpu *processor,
                                              instruction_word instruction,
                                              unsigned_word cia,
                                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC0_Rc1
#line 4147 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 35869 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 0 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4147 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 35908 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC0_Rc1;

    /* semantic routine */
#line 4147 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 35918 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4147 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 35969 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 35976 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Compare_Ordered(cpu *processor,
                                instruction_word instruction,
                                unsigned_word cia,
                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Compare_Ordered
#undef MY_PREFIX
#define MY_PREFIX Floating_Compare_Ordered
#line 4629 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Compare Ordered\n"));
#line 35995 "icache.c"

  /* validate: 0.63,6.BF,9./,11.FRA,16.FRB,21.32,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc6007ff) != 0xfc000040)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.BF,9./,11.FRA,16.FRB,21.32,31./ #define get-values-from-icache|put-values-in-icache */
#line 68 "./ppc-instructions"
    unsigned const BF UNUSED = cache_entry->crack.X.BF = EXTRACTED32(instruction, 6, 8); /* in icache - extraction */
#line 69 "./ppc-instructions"
    unsigned32 const BF_BITMASK UNUSED = cache_entry->crack.X.BF_BITMASK = (1 << BF); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.X.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.X.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.X.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.X.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 36026 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Compare_Ordered;

    /* semantic routine */
#line 4629 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Compare Ordered\n"));
#line 36036 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Compare_Ordered, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4629 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      unsigned c;
      if (is_NaN(*frA, 0) || is_NaN(*frB, 0))
        c = cr_i_summary_overflow; /* 0b0001 - (FRA) ? (FRB) */
      else if (is_less_than(frA, frB))
        c = cr_i_negative; /* 0b1000 - (FRA) < (FRB) */
      else if (is_greater_than(frA, frB))
        c = cr_i_positive; /* 0b0100 - (FRA) > (FRB) */
      else
        c = cr_i_zero; /* 0b0010 - (FRA) = (FRB) */
      FPSCR_SET_FPCC(c);
      CR_SET(BF, c); /* CR[4*BF..4*BF+3] = c */
      if (is_SNaN(*frA, 0) || is_SNaN(*frB, 0)) {
        FPSCR_OR_VX(fpscr_vxsnan);
        if ((FPSCR & fpscr_ve) == 0)
          FPSCR_OR_VX(fpscr_vxvc);
      }
      else if (is_QNaN(*frA, 0) || is_QNaN(*frB, 0)) {
        FPSCR_OR_VX(fpscr_vxvc);
      }
      FPSCR_END(0);
      PPC_INSN_FLOAT_CR(0, FRA_BITMASK | FRB_BITMASK, BF_BITMASK);
    }
#line 36070 "icache.c"
    return nia;
    /* extraction: 0.63,6.BF,9./,11.FRA,16.FRB,21.32,31./ #undef get-values-from-icache|put-values-in-icache */
#undef FRA
#undef FRB
#line 36075 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Move_To_FPSCR_Bit_1_Rc0(cpu *processor,
                               instruction_word instruction,
                               unsigned_word cia,
                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Move_To_FPSCR_Bit_1
#undef MY_PREFIX
#define MY_PREFIX Move_To_FPSCR_Bit_1_Rc0
#line 4692 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Move To FPSCR Bit 1\n"));
#line 36094 "icache.c"

  /* validate: 0.63,6.BT,11./,16./,21.38,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc1ffffe) != 0xfc00004c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.BT,11./,16./,21.38,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 76 "./ppc-instructions"
    unsigned const BT UNUSED = cache_entry->crack.X.BT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned32 const BT_BITMASK UNUSED = cache_entry->crack.X.BT_BITMASK = (1 << BT); /* in icache - extraction */
#line 4692 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 36117 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Move_To_FPSCR_Bit_1_Rc0;

    /* semantic routine */
#line 4692 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Move To FPSCR Bit 1\n"));
#line 36127 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Move_To_FPSCR_Bit_1, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4692 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      unsigned32 bit = BIT32(BT);
      if (bit & fpscr_fi)
        bit |= fpscr_xx;
      if ((bit & fpscr_vx_bits))
        bit |= fpscr_fx;
      /* note - omit vx bit */
      if ((bit & (fpscr_ox | fpscr_ux | fpscr_zx | fpscr_xx)))
        bit |= fpscr_fx;
      FPSCR |= bit;
      FPSCR_END(Rc);
    }
#line 36150 "icache.c"
    return nia;
    /* extraction: 0.63,6.BT,11./,16./,21.38,31.Rc #undef get-values-from-icache|put-values-in-icache */
#line 36153 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Move_To_FPSCR_Bit_1_Rc1(cpu *processor,
                               instruction_word instruction,
                               unsigned_word cia,
                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Move_To_FPSCR_Bit_1
#undef MY_PREFIX
#define MY_PREFIX Move_To_FPSCR_Bit_1_Rc1
#line 4692 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Move To FPSCR Bit 1\n"));
#line 36172 "icache.c"

  /* validate: 0.63,6.BT,11./,16./,21.38,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc1ffffe) != 0xfc00004c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.BT,11./,16./,21.38,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 76 "./ppc-instructions"
    unsigned const BT UNUSED = cache_entry->crack.X.BT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned32 const BT_BITMASK UNUSED = cache_entry->crack.X.BT_BITMASK = (1 << BT); /* in icache - extraction */
#line 4692 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 36195 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Move_To_FPSCR_Bit_1_Rc1;

    /* semantic routine */
#line 4692 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Move To FPSCR Bit 1\n"));
#line 36205 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Move_To_FPSCR_Bit_1, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4692 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      unsigned32 bit = BIT32(BT);
      if (bit & fpscr_fi)
        bit |= fpscr_xx;
      if ((bit & fpscr_vx_bits))
        bit |= fpscr_fx;
      /* note - omit vx bit */
      if ((bit & (fpscr_ox | fpscr_ux | fpscr_zx | fpscr_xx)))
        bit |= fpscr_fx;
      FPSCR |= bit;
      FPSCR_END(Rc);
    }
#line 36228 "icache.c"
    return nia;
    /* extraction: 0.63,6.BT,11./,16./,21.38,31.Rc #undef get-values-from-icache|put-values-in-icache */
#line 36231 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negate_Rc0(cpu *processor,
                           instruction_word instruction,
                           unsigned_word cia,
                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negate
#undef MY_PREFIX
#define MY_PREFIX Floating_Negate_Rc0
#line 3713 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negate\n"));
#line 36250 "icache.c"

  /* validate: 0.63,6.FRT,11./,16.FRB,21.40,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc1f07fe) != 0xfc000050)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11./,16.FRB,21.40,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.X.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.X.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.X.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.X.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 3713 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 36279 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negate_Rc0;

    /* semantic routine */
#line 3713 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negate\n"));
#line 36289 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negate, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3713 "./ppc-instructions"
    {
      *frT = *frB ^ BIT64(0);
      CR1_UPDATE(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRB_BITMASK, Rc);
    }
#line 36304 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11./,16.FRB,21.40,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRB
#line 36309 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negate_Rc1(cpu *processor,
                           instruction_word instruction,
                           unsigned_word cia,
                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negate
#undef MY_PREFIX
#define MY_PREFIX Floating_Negate_Rc1
#line 3713 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negate\n"));
#line 36328 "icache.c"

  /* validate: 0.63,6.FRT,11./,16.FRB,21.40,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc1f07fe) != 0xfc000050)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11./,16.FRB,21.40,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.X.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.X.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.X.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.X.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 3713 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 36357 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negate_Rc1;

    /* semantic routine */
#line 3713 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negate\n"));
#line 36367 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negate, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3713 "./ppc-instructions"
    {
      *frT = *frB ^ BIT64(0);
      CR1_UPDATE(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRB_BITMASK, Rc);
    }
#line 36382 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11./,16.FRB,21.40,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRB
#line 36387 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC1_Rc0(cpu *processor,
                                instruction_word instruction,
                                unsigned_word cia,
                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC1_Rc0
#line 4729 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 36406 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 1 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4729 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 36445 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC1_Rc0;

    /* semantic routine */
#line 4729 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 36455 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4729 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 36477 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 36484 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC1_Rc1(cpu *processor,
                                instruction_word instruction,
                                unsigned_word cia,
                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC1_Rc1
#line 4729 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 36503 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 1 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4729 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 36542 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC1_Rc1;

    /* semantic routine */
#line 4729 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 36552 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4729 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 36574 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 36581 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC1_Rc0(cpu *processor,
                                  instruction_word instruction,
                                  unsigned_word cia,
                                  idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC1_Rc0
#line 3849 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 36600 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 1 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3849 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 36634 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC1_Rc0;

    /* semantic routine */
#line 3849 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 36644 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3849 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 36676 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 36682 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC1_Rc1(cpu *processor,
                                  instruction_word instruction,
                                  unsigned_word cia,
                                  idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC1_Rc1
#line 3849 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 36701 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 1 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3849 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 36735 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC1_Rc1;

    /* semantic routine */
#line 3849 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 36745 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3849 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 36777 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 36783 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC1_Rc0(cpu *processor,
                                          instruction_word instruction,
                                          unsigned_word cia,
                                          idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC1_Rc0
#line 4057 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 36802 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 1 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4057 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 36841 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC1_Rc0;

    /* semantic routine */
#line 4057 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 36851 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4057 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 36902 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 36909 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC1_Rc1(cpu *processor,
                                          instruction_word instruction,
                                          unsigned_word cia,
                                          idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC1_Rc1
#line 4057 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 36928 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 1 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4057 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 36967 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC1_Rc1;

    /* semantic routine */
#line 4057 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 36977 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4057 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 37028 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 37035 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC1_Rc0(cpu *processor,
                                     instruction_word instruction,
                                     unsigned_word cia,
                                     idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC1_Rc0
#line 3967 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 37054 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 1 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3967 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 37093 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC1_Rc0;

    /* semantic routine */
#line 3967 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 37103 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3967 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 37154 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 37161 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC1_Rc1(cpu *processor,
                                     instruction_word instruction,
                                     unsigned_word cia,
                                     idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC1_Rc1
#line 3967 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 37180 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 1 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3967 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 37219 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC1_Rc1;

    /* semantic routine */
#line 3967 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 37229 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3967 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 37280 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 37287 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC1_Rc0(cpu *processor,
                                                   instruction_word instruction,
                                                   unsigned_word cia,
                                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC1_Rc0
#line 4237 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 37306 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 1 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4237 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 37345 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC1_Rc0;

    /* semantic routine */
#line 4237 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 37355 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4237 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 37406 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 37413 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC1_Rc1(cpu *processor,
                                                   instruction_word instruction,
                                                   unsigned_word cia,
                                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC1_Rc1
#line 4237 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 37432 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 1 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4237 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 37471 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC1_Rc1;

    /* semantic routine */
#line 4237 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 37481 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4237 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 37532 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 37539 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC1_Rc0(cpu *processor,
                                              instruction_word instruction,
                                              unsigned_word cia,
                                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC1_Rc0
#line 4147 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 37558 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 1 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4147 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 37597 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC1_Rc0;

    /* semantic routine */
#line 4147 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 37607 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4147 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 37658 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 37665 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC1_Rc1(cpu *processor,
                                              instruction_word instruction,
                                              unsigned_word cia,
                                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC1_Rc1
#line 4147 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 37684 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 1 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4147 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 37723 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC1_Rc1;

    /* semantic routine */
#line 4147 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 37733 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4147 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 37784 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 37791 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Move_to_Condition_Register_from_FPSCR(cpu *processor,
                                             instruction_word instruction,
                                             unsigned_word cia,
                                             idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Move_to_Condition_Register_from_FPSCR
#undef MY_PREFIX
#define MY_PREFIX Move_to_Condition_Register_from_FPSCR
#line 4663 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Move to Condition Register from FPSCR\n"));
#line 37810 "icache.c"

  /* validate: 0.63,6.BF,9./,11.BFA,14./,16./,21.64,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc63ffff) != 0xfc000080)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.BF,9./,11.BFA,14./,16./,21.64,31./ #define get-values-from-icache|put-values-in-icache */
#line 68 "./ppc-instructions"
    unsigned const BF UNUSED = cache_entry->crack.X.BF = EXTRACTED32(instruction, 6, 8); /* in icache - extraction */
#line 69 "./ppc-instructions"
    unsigned32 const BF_BITMASK UNUSED = cache_entry->crack.X.BF_BITMASK = (1 << BF); /* in icache - extraction */
#line 4663 "./ppc-instructions"
    unsigned const BFA UNUSED = cache_entry->crack.X.BFA = EXTRACTED32(instruction, 11, 13); /* default in icache - extraction */
#line 37833 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Move_to_Condition_Register_from_FPSCR;

    /* semantic routine */
#line 4663 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Move to Condition Register from FPSCR\n"));
#line 37843 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Move_to_Condition_Register_from_FPSCR, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4663 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      unsigned field = FPSCR_FIELD(BFA);
      CR_SET(BF, field);
      FPSCR_SET(BFA, 0); /* FPSCR_END fixes up FEX/VX */
      FPSCR_END(0);
    }
#line 37860 "icache.c"
    return nia;
    /* extraction: 0.63,6.BF,9./,11.BFA,14./,16./,21.64,31./ #undef get-values-from-icache|put-values-in-icache */
#line 37863 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Move_To_FPSCR_Bit_0_Rc0(cpu *processor,
                               instruction_word instruction,
                               unsigned_word cia,
                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Move_To_FPSCR_Bit_0
#undef MY_PREFIX
#define MY_PREFIX Move_To_FPSCR_Bit_0_Rc0
#line 4686 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Move To FPSCR Bit 0\n"));
#line 37882 "icache.c"

  /* validate: 0.63,6.BT,11./,16./,21.70,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc1ffffe) != 0xfc00008c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.BT,11./,16./,21.70,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 76 "./ppc-instructions"
    unsigned const BT UNUSED = cache_entry->crack.X.BT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned32 const BT_BITMASK UNUSED = cache_entry->crack.X.BT_BITMASK = (1 << BT); /* in icache - extraction */
#line 4686 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 37905 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Move_To_FPSCR_Bit_0_Rc0;

    /* semantic routine */
#line 4686 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Move To FPSCR Bit 0\n"));
#line 37915 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Move_To_FPSCR_Bit_0, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4686 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      unsigned32 bit = BIT32(BT);
      FPSCR &= ~bit;
      FPSCR_END(Rc);
    }
#line 37931 "icache.c"
    return nia;
    /* extraction: 0.63,6.BT,11./,16./,21.70,31.Rc #undef get-values-from-icache|put-values-in-icache */
#line 37934 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Move_To_FPSCR_Bit_0_Rc1(cpu *processor,
                               instruction_word instruction,
                               unsigned_word cia,
                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Move_To_FPSCR_Bit_0
#undef MY_PREFIX
#define MY_PREFIX Move_To_FPSCR_Bit_0_Rc1
#line 4686 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Move To FPSCR Bit 0\n"));
#line 37953 "icache.c"

  /* validate: 0.63,6.BT,11./,16./,21.70,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc1ffffe) != 0xfc00008c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.BT,11./,16./,21.70,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 76 "./ppc-instructions"
    unsigned const BT UNUSED = cache_entry->crack.X.BT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 77 "./ppc-instructions"
    unsigned32 const BT_BITMASK UNUSED = cache_entry->crack.X.BT_BITMASK = (1 << BT); /* in icache - extraction */
#line 4686 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 37976 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Move_To_FPSCR_Bit_0_Rc1;

    /* semantic routine */
#line 4686 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Move To FPSCR Bit 0\n"));
#line 37986 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Move_To_FPSCR_Bit_0, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4686 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      unsigned32 bit = BIT32(BT);
      FPSCR &= ~bit;
      FPSCR_END(Rc);
    }
#line 38002 "icache.c"
    return nia;
    /* extraction: 0.63,6.BT,11./,16./,21.70,31.Rc #undef get-values-from-icache|put-values-in-icache */
#line 38005 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Move_Register_Rc0(cpu *processor,
                                  instruction_word instruction,
                                  unsigned_word cia,
                                  idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Move_Register
#undef MY_PREFIX
#define MY_PREFIX Floating_Move_Register_Rc0
#line 3704 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Move Register\n"));
#line 38024 "icache.c"

  /* validate: 0.63,6.FRT,11./,16.FRB,21.72,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc1f07fe) != 0xfc000090)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11./,16.FRB,21.72,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.X.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.X.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.X.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.X.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 3704 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 38053 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Move_Register_Rc0;

    /* semantic routine */
#line 3704 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Move Register\n"));
#line 38063 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Move_Register, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3704 "./ppc-instructions"
    {
      *frT = *frB;
      CR1_UPDATE(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRB_BITMASK, Rc);
    }
#line 38078 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11./,16.FRB,21.72,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRB
#line 38083 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Move_Register_Rc1(cpu *processor,
                                  instruction_word instruction,
                                  unsigned_word cia,
                                  idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Move_Register
#undef MY_PREFIX
#define MY_PREFIX Floating_Move_Register_Rc1
#line 3704 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Move Register\n"));
#line 38102 "icache.c"

  /* validate: 0.63,6.FRT,11./,16.FRB,21.72,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc1f07fe) != 0xfc000090)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11./,16.FRB,21.72,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.X.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.X.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.X.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.X.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 3704 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 38131 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Move_Register_Rc1;

    /* semantic routine */
#line 3704 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Move Register\n"));
#line 38141 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Move_Register, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3704 "./ppc-instructions"
    {
      *frT = *frB;
      CR1_UPDATE(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRB_BITMASK, Rc);
    }
#line 38156 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11./,16.FRB,21.72,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRB
#line 38161 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC2_Rc0(cpu *processor,
                                instruction_word instruction,
                                unsigned_word cia,
                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC2_Rc0
#line 4729 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 38180 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 2 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4729 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 38219 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC2_Rc0;

    /* semantic routine */
#line 4729 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 38229 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4729 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 38251 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 38258 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC2_Rc1(cpu *processor,
                                instruction_word instruction,
                                unsigned_word cia,
                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC2_Rc1
#line 4729 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 38277 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 2 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4729 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 38316 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC2_Rc1;

    /* semantic routine */
#line 4729 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 38326 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4729 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 38348 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 38355 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC2_Rc0(cpu *processor,
                                  instruction_word instruction,
                                  unsigned_word cia,
                                  idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC2_Rc0
#line 3849 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 38374 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 2 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3849 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 38408 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC2_Rc0;

    /* semantic routine */
#line 3849 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 38418 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3849 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 38450 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 38456 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC2_Rc1(cpu *processor,
                                  instruction_word instruction,
                                  unsigned_word cia,
                                  idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC2_Rc1
#line 3849 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 38475 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 2 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3849 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 38509 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC2_Rc1;

    /* semantic routine */
#line 3849 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 38519 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3849 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 38551 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 38557 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC2_Rc0(cpu *processor,
                                          instruction_word instruction,
                                          unsigned_word cia,
                                          idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC2_Rc0
#line 4057 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 38576 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 2 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4057 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 38615 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC2_Rc0;

    /* semantic routine */
#line 4057 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 38625 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4057 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 38676 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 38683 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC2_Rc1(cpu *processor,
                                          instruction_word instruction,
                                          unsigned_word cia,
                                          idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC2_Rc1
#line 4057 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 38702 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 2 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4057 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 38741 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC2_Rc1;

    /* semantic routine */
#line 4057 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 38751 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4057 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 38802 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 38809 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC2_Rc0(cpu *processor,
                                     instruction_word instruction,
                                     unsigned_word cia,
                                     idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC2_Rc0
#line 3967 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 38828 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 2 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3967 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 38867 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC2_Rc0;

    /* semantic routine */
#line 3967 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 38877 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3967 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 38928 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 38935 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC2_Rc1(cpu *processor,
                                     instruction_word instruction,
                                     unsigned_word cia,
                                     idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC2_Rc1
#line 3967 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 38954 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 2 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3967 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 38993 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC2_Rc1;

    /* semantic routine */
#line 3967 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 39003 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3967 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 39054 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 39061 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC2_Rc0(cpu *processor,
                                                   instruction_word instruction,
                                                   unsigned_word cia,
                                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC2_Rc0
#line 4237 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 39080 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 2 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4237 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 39119 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC2_Rc0;

    /* semantic routine */
#line 4237 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 39129 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4237 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 39180 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 39187 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC2_Rc1(cpu *processor,
                                                   instruction_word instruction,
                                                   unsigned_word cia,
                                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC2_Rc1
#line 4237 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 39206 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 2 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4237 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 39245 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC2_Rc1;

    /* semantic routine */
#line 4237 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 39255 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4237 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 39306 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 39313 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC2_Rc0(cpu *processor,
                                              instruction_word instruction,
                                              unsigned_word cia,
                                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC2_Rc0
#line 4147 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 39332 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 2 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4147 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 39371 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC2_Rc0;

    /* semantic routine */
#line 4147 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 39381 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4147 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 39432 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 39439 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC2_Rc1(cpu *processor,
                                              instruction_word instruction,
                                              unsigned_word cia,
                                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC2_Rc1
#line 4147 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 39458 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 2 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4147 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 39497 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC2_Rc1;

    /* semantic routine */
#line 4147 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 39507 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4147 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 39558 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 39565 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC3_Rc0(cpu *processor,
                                instruction_word instruction,
                                unsigned_word cia,
                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC3_Rc0
#line 4729 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 39584 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 3 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4729 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 39623 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC3_Rc0;

    /* semantic routine */
#line 4729 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 39633 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4729 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 39655 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 39662 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC3_Rc1(cpu *processor,
                                instruction_word instruction,
                                unsigned_word cia,
                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC3_Rc1
#line 4729 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 39681 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 3 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4729 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 39720 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC3_Rc1;

    /* semantic routine */
#line 4729 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 39730 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4729 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 39752 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 39759 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC3_Rc0(cpu *processor,
                                  instruction_word instruction,
                                  unsigned_word cia,
                                  idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC3_Rc0
#line 3849 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 39778 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 3 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3849 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 39812 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC3_Rc0;

    /* semantic routine */
#line 3849 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 39822 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3849 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 39854 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 39860 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC3_Rc1(cpu *processor,
                                  instruction_word instruction,
                                  unsigned_word cia,
                                  idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC3_Rc1
#line 3849 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 39879 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 3 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3849 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 39913 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC3_Rc1;

    /* semantic routine */
#line 3849 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 39923 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3849 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 39955 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 39961 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC3_Rc0(cpu *processor,
                                          instruction_word instruction,
                                          unsigned_word cia,
                                          idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC3_Rc0
#line 4057 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 39980 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 3 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4057 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 40019 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC3_Rc0;

    /* semantic routine */
#line 4057 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 40029 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4057 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 40080 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 40087 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC3_Rc1(cpu *processor,
                                          instruction_word instruction,
                                          unsigned_word cia,
                                          idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC3_Rc1
#line 4057 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 40106 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 3 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4057 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 40145 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC3_Rc1;

    /* semantic routine */
#line 4057 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 40155 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4057 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 40206 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 40213 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC3_Rc0(cpu *processor,
                                     instruction_word instruction,
                                     unsigned_word cia,
                                     idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC3_Rc0
#line 3967 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 40232 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 3 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3967 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 40271 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC3_Rc0;

    /* semantic routine */
#line 3967 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 40281 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3967 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 40332 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 40339 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC3_Rc1(cpu *processor,
                                     instruction_word instruction,
                                     unsigned_word cia,
                                     idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC3_Rc1
#line 3967 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 40358 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 3 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3967 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 40397 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC3_Rc1;

    /* semantic routine */
#line 3967 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 40407 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3967 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 40458 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 40465 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC3_Rc0(cpu *processor,
                                                   instruction_word instruction,
                                                   unsigned_word cia,
                                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC3_Rc0
#line 4237 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 40484 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 3 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4237 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 40523 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC3_Rc0;

    /* semantic routine */
#line 4237 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 40533 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4237 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 40584 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 40591 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC3_Rc1(cpu *processor,
                                                   instruction_word instruction,
                                                   unsigned_word cia,
                                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC3_Rc1
#line 4237 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 40610 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 3 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4237 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 40649 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC3_Rc1;

    /* semantic routine */
#line 4237 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 40659 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4237 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 40710 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 40717 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC3_Rc0(cpu *processor,
                                              instruction_word instruction,
                                              unsigned_word cia,
                                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC3_Rc0
#line 4147 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 40736 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 3 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4147 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 40775 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC3_Rc0;

    /* semantic routine */
#line 4147 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 40785 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4147 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 40836 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 40843 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC3_Rc1(cpu *processor,
                                              instruction_word instruction,
                                              unsigned_word cia,
                                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC3_Rc1
#line 4147 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 40862 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 3 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4147 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 40901 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC3_Rc1;

    /* semantic routine */
#line 4147 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 40911 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4147 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 40962 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 40969 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Move_To_FPSCR_Field_Immediate_Rc0(cpu *processor,
                                         instruction_word instruction,
                                         unsigned_word cia,
                                         idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Move_To_FPSCR_Field_Immediate
#undef MY_PREFIX
#define MY_PREFIX Move_To_FPSCR_Field_Immediate_Rc0
#line 4670 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Move To FPSCR Field Immediate\n"));
#line 40988 "icache.c"

  /* validate: 0.63,6.BF,9./,11./,16.U,20./,21.134,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc7f0ffe) != 0xfc00010c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.BF,9./,11./,16.U,20./,21.134,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 68 "./ppc-instructions"
    unsigned const BF UNUSED = cache_entry->crack.X.BF = EXTRACTED32(instruction, 6, 8); /* in icache - extraction */
#line 69 "./ppc-instructions"
    unsigned32 const BF_BITMASK UNUSED = cache_entry->crack.X.BF_BITMASK = (1 << BF); /* in icache - extraction */
#line 4670 "./ppc-instructions"
    unsigned const U UNUSED = cache_entry->crack.X.U = EXTRACTED32(instruction, 16, 19); /* default in icache - extraction */
#line 4670 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 41013 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Move_To_FPSCR_Field_Immediate_Rc0;

    /* semantic routine */
#line 4670 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Move To FPSCR Field Immediate\n"));
#line 41023 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Move_To_FPSCR_Field_Immediate, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4670 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      FPSCR_SET(BF, U);
      FPSCR_END(Rc);
    }
#line 41038 "icache.c"
    return nia;
    /* extraction: 0.63,6.BF,9./,11./,16.U,20./,21.134,31.Rc #undef get-values-from-icache|put-values-in-icache */
#line 41041 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Move_To_FPSCR_Field_Immediate_Rc1(cpu *processor,
                                         instruction_word instruction,
                                         unsigned_word cia,
                                         idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Move_To_FPSCR_Field_Immediate
#undef MY_PREFIX
#define MY_PREFIX Move_To_FPSCR_Field_Immediate_Rc1
#line 4670 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Move To FPSCR Field Immediate\n"));
#line 41060 "icache.c"

  /* validate: 0.63,6.BF,9./,11./,16.U,20./,21.134,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc7f0ffe) != 0xfc00010c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.BF,9./,11./,16.U,20./,21.134,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 68 "./ppc-instructions"
    unsigned const BF UNUSED = cache_entry->crack.X.BF = EXTRACTED32(instruction, 6, 8); /* in icache - extraction */
#line 69 "./ppc-instructions"
    unsigned32 const BF_BITMASK UNUSED = cache_entry->crack.X.BF_BITMASK = (1 << BF); /* in icache - extraction */
#line 4670 "./ppc-instructions"
    unsigned const U UNUSED = cache_entry->crack.X.U = EXTRACTED32(instruction, 16, 19); /* default in icache - extraction */
#line 4670 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 41085 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Move_To_FPSCR_Field_Immediate_Rc1;

    /* semantic routine */
#line 4670 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Move To FPSCR Field Immediate\n"));
#line 41095 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Move_To_FPSCR_Field_Immediate, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4670 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      FPSCR_SET(BF, U);
      FPSCR_END(Rc);
    }
#line 41110 "icache.c"
    return nia;
    /* extraction: 0.63,6.BF,9./,11./,16.U,20./,21.134,31.Rc #undef get-values-from-icache|put-values-in-icache */
#line 41113 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_Absolute_Value_Rc0(cpu *processor,
                                            instruction_word instruction,
                                            unsigned_word cia,
                                            idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_Absolute_Value
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_Absolute_Value_Rc0
#line 3731 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Absolute Value\n"));
#line 41132 "icache.c"

  /* validate: 0.63,6.FRT,11./,16.FRB,21.136,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc1f07fe) != 0xfc000110)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11./,16.FRB,21.136,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.X.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.X.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.X.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.X.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 3731 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 41161 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_Absolute_Value_Rc0;

    /* semantic routine */
#line 3731 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Absolute Value\n"));
#line 41171 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_Absolute_Value, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3731 "./ppc-instructions"
    {
      *frT = *frB | BIT64(0);
      CR1_UPDATE(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRB_BITMASK, Rc);
    }
#line 41186 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11./,16.FRB,21.136,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRB
#line 41191 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_Absolute_Value_Rc1(cpu *processor,
                                            instruction_word instruction,
                                            unsigned_word cia,
                                            idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_Absolute_Value
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_Absolute_Value_Rc1
#line 3731 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Absolute Value\n"));
#line 41210 "icache.c"

  /* validate: 0.63,6.FRT,11./,16.FRB,21.136,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc1f07fe) != 0xfc000110)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11./,16.FRB,21.136,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.X.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.X.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.X.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.X.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 3731 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 41239 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_Absolute_Value_Rc1;

    /* semantic routine */
#line 3731 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Absolute Value\n"));
#line 41249 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_Absolute_Value, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3731 "./ppc-instructions"
    {
      *frT = *frB | BIT64(0);
      CR1_UPDATE(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRB_BITMASK, Rc);
    }
#line 41264 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11./,16.FRB,21.136,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRB
#line 41269 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC4_Rc0(cpu *processor,
                                instruction_word instruction,
                                unsigned_word cia,
                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC4_Rc0
#line 4729 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 41288 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 4 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4729 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 41327 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC4_Rc0;

    /* semantic routine */
#line 4729 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 41337 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4729 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 41359 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 41366 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC4_Rc1(cpu *processor,
                                instruction_word instruction,
                                unsigned_word cia,
                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC4_Rc1
#line 4729 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 41385 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 4 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4729 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 41424 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC4_Rc1;

    /* semantic routine */
#line 4729 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 41434 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4729 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 41456 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 41463 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC4_Rc0(cpu *processor,
                                  instruction_word instruction,
                                  unsigned_word cia,
                                  idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC4_Rc0
#line 3849 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 41482 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 4 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3849 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 41516 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC4_Rc0;

    /* semantic routine */
#line 3849 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 41526 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3849 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 41558 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 41564 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC4_Rc1(cpu *processor,
                                  instruction_word instruction,
                                  unsigned_word cia,
                                  idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC4_Rc1
#line 3849 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 41583 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 4 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3849 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 41617 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC4_Rc1;

    /* semantic routine */
#line 3849 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 41627 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3849 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 41659 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 41665 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC4_Rc0(cpu *processor,
                                          instruction_word instruction,
                                          unsigned_word cia,
                                          idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC4_Rc0
#line 4057 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 41684 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 4 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4057 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 41723 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC4_Rc0;

    /* semantic routine */
#line 4057 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 41733 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4057 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 41784 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 41791 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC4_Rc1(cpu *processor,
                                          instruction_word instruction,
                                          unsigned_word cia,
                                          idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC4_Rc1
#line 4057 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 41810 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 4 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4057 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 41849 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC4_Rc1;

    /* semantic routine */
#line 4057 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 41859 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4057 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 41910 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 41917 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC4_Rc0(cpu *processor,
                                     instruction_word instruction,
                                     unsigned_word cia,
                                     idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC4_Rc0
#line 3967 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 41936 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 4 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3967 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 41975 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC4_Rc0;

    /* semantic routine */
#line 3967 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 41985 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3967 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 42036 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 42043 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC4_Rc1(cpu *processor,
                                     instruction_word instruction,
                                     unsigned_word cia,
                                     idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC4_Rc1
#line 3967 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 42062 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 4 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3967 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 42101 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC4_Rc1;

    /* semantic routine */
#line 3967 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 42111 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3967 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 42162 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 42169 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC4_Rc0(cpu *processor,
                                                   instruction_word instruction,
                                                   unsigned_word cia,
                                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC4_Rc0
#line 4237 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 42188 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 4 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4237 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 42227 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC4_Rc0;

    /* semantic routine */
#line 4237 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 42237 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4237 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 42288 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 42295 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC4_Rc1(cpu *processor,
                                                   instruction_word instruction,
                                                   unsigned_word cia,
                                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC4_Rc1
#line 4237 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 42314 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 4 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4237 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 42353 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC4_Rc1;

    /* semantic routine */
#line 4237 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 42363 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4237 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 42414 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 42421 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC4_Rc0(cpu *processor,
                                              instruction_word instruction,
                                              unsigned_word cia,
                                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC4_Rc0
#line 4147 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 42440 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 4 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4147 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 42479 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC4_Rc0;

    /* semantic routine */
#line 4147 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 42489 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4147 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 42540 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 42547 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC4_Rc1(cpu *processor,
                                              instruction_word instruction,
                                              unsigned_word cia,
                                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC4_Rc1
#line 4147 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 42566 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 4 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4147 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 42605 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC4_Rc1;

    /* semantic routine */
#line 4147 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 42615 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4147 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 42666 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 42673 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC5_Rc0(cpu *processor,
                                instruction_word instruction,
                                unsigned_word cia,
                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC5_Rc0
#line 4729 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 42692 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 5 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4729 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 42731 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC5_Rc0;

    /* semantic routine */
#line 4729 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 42741 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4729 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 42763 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 42770 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC5_Rc1(cpu *processor,
                                instruction_word instruction,
                                unsigned_word cia,
                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC5_Rc1
#line 4729 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 42789 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 5 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4729 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 42828 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC5_Rc1;

    /* semantic routine */
#line 4729 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 42838 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4729 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 42860 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 42867 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC5_Rc0(cpu *processor,
                                  instruction_word instruction,
                                  unsigned_word cia,
                                  idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC5_Rc0
#line 3849 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 42886 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 5 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3849 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 42920 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC5_Rc0;

    /* semantic routine */
#line 3849 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 42930 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3849 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 42962 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 42968 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC5_Rc1(cpu *processor,
                                  instruction_word instruction,
                                  unsigned_word cia,
                                  idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC5_Rc1
#line 3849 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 42987 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 5 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3849 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 43021 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC5_Rc1;

    /* semantic routine */
#line 3849 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 43031 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3849 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 43063 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 43069 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC5_Rc0(cpu *processor,
                                          instruction_word instruction,
                                          unsigned_word cia,
                                          idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC5_Rc0
#line 4057 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 43088 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 5 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4057 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 43127 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC5_Rc0;

    /* semantic routine */
#line 4057 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 43137 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4057 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 43188 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 43195 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC5_Rc1(cpu *processor,
                                          instruction_word instruction,
                                          unsigned_word cia,
                                          idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC5_Rc1
#line 4057 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 43214 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 5 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4057 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 43253 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC5_Rc1;

    /* semantic routine */
#line 4057 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 43263 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4057 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 43314 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 43321 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC5_Rc0(cpu *processor,
                                     instruction_word instruction,
                                     unsigned_word cia,
                                     idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC5_Rc0
#line 3967 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 43340 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 5 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3967 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 43379 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC5_Rc0;

    /* semantic routine */
#line 3967 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 43389 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3967 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 43440 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 43447 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC5_Rc1(cpu *processor,
                                     instruction_word instruction,
                                     unsigned_word cia,
                                     idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC5_Rc1
#line 3967 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 43466 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 5 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3967 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 43505 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC5_Rc1;

    /* semantic routine */
#line 3967 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 43515 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3967 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 43566 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 43573 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC5_Rc0(cpu *processor,
                                                   instruction_word instruction,
                                                   unsigned_word cia,
                                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC5_Rc0
#line 4237 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 43592 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 5 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4237 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 43631 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC5_Rc0;

    /* semantic routine */
#line 4237 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 43641 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4237 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 43692 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 43699 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC5_Rc1(cpu *processor,
                                                   instruction_word instruction,
                                                   unsigned_word cia,
                                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC5_Rc1
#line 4237 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 43718 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 5 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4237 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 43757 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC5_Rc1;

    /* semantic routine */
#line 4237 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 43767 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4237 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 43818 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 43825 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC5_Rc0(cpu *processor,
                                              instruction_word instruction,
                                              unsigned_word cia,
                                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC5_Rc0
#line 4147 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 43844 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 5 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4147 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 43883 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC5_Rc0;

    /* semantic routine */
#line 4147 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 43893 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4147 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 43944 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 43951 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC5_Rc1(cpu *processor,
                                              instruction_word instruction,
                                              unsigned_word cia,
                                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC5_Rc1
#line 4147 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 43970 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 5 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4147 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 44009 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC5_Rc1;

    /* semantic routine */
#line 4147 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 44019 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4147 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 44070 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 44077 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC6_Rc0(cpu *processor,
                                instruction_word instruction,
                                unsigned_word cia,
                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC6_Rc0
#line 4729 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 44096 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 6 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4729 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 44135 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC6_Rc0;

    /* semantic routine */
#line 4729 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 44145 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4729 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 44167 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 44174 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC6_Rc1(cpu *processor,
                                instruction_word instruction,
                                unsigned_word cia,
                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC6_Rc1
#line 4729 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 44193 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 6 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4729 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 44232 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC6_Rc1;

    /* semantic routine */
#line 4729 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 44242 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4729 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 44264 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 44271 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC6_Rc0(cpu *processor,
                                  instruction_word instruction,
                                  unsigned_word cia,
                                  idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC6_Rc0
#line 3849 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 44290 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 6 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3849 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 44324 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC6_Rc0;

    /* semantic routine */
#line 3849 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 44334 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3849 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 44366 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 44372 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC6_Rc1(cpu *processor,
                                  instruction_word instruction,
                                  unsigned_word cia,
                                  idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC6_Rc1
#line 3849 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 44391 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 6 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3849 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 44425 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC6_Rc1;

    /* semantic routine */
#line 3849 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 44435 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3849 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 44467 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 44473 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC6_Rc0(cpu *processor,
                                          instruction_word instruction,
                                          unsigned_word cia,
                                          idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC6_Rc0
#line 4057 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 44492 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 6 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4057 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 44531 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC6_Rc0;

    /* semantic routine */
#line 4057 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 44541 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4057 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 44592 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 44599 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC6_Rc1(cpu *processor,
                                          instruction_word instruction,
                                          unsigned_word cia,
                                          idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC6_Rc1
#line 4057 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 44618 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 6 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4057 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 44657 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC6_Rc1;

    /* semantic routine */
#line 4057 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 44667 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4057 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 44718 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 44725 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC6_Rc0(cpu *processor,
                                     instruction_word instruction,
                                     unsigned_word cia,
                                     idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC6_Rc0
#line 3967 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 44744 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 6 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3967 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 44783 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC6_Rc0;

    /* semantic routine */
#line 3967 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 44793 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3967 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 44844 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 44851 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC6_Rc1(cpu *processor,
                                     instruction_word instruction,
                                     unsigned_word cia,
                                     idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC6_Rc1
#line 3967 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 44870 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 6 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3967 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 44909 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC6_Rc1;

    /* semantic routine */
#line 3967 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 44919 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3967 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 44970 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 44977 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC6_Rc0(cpu *processor,
                                                   instruction_word instruction,
                                                   unsigned_word cia,
                                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC6_Rc0
#line 4237 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 44996 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 6 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4237 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 45035 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC6_Rc0;

    /* semantic routine */
#line 4237 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 45045 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4237 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 45096 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 45103 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC6_Rc1(cpu *processor,
                                                   instruction_word instruction,
                                                   unsigned_word cia,
                                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC6_Rc1
#line 4237 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 45122 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 6 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4237 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 45161 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC6_Rc1;

    /* semantic routine */
#line 4237 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 45171 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4237 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 45222 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 45229 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC6_Rc0(cpu *processor,
                                              instruction_word instruction,
                                              unsigned_word cia,
                                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC6_Rc0
#line 4147 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 45248 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 6 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4147 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 45287 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC6_Rc0;

    /* semantic routine */
#line 4147 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 45297 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4147 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 45348 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 45355 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC6_Rc1(cpu *processor,
                                              instruction_word instruction,
                                              unsigned_word cia,
                                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC6_Rc1
#line 4147 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 45374 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 6 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4147 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 45413 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC6_Rc1;

    /* semantic routine */
#line 4147 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 45423 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4147 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 45474 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 45481 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC7_Rc0(cpu *processor,
                                instruction_word instruction,
                                unsigned_word cia,
                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC7_Rc0
#line 4729 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 45500 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 7 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4729 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 45539 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC7_Rc0;

    /* semantic routine */
#line 4729 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 45549 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4729 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 45571 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 45578 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC7_Rc1(cpu *processor,
                                instruction_word instruction,
                                unsigned_word cia,
                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC7_Rc1
#line 4729 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 45597 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 7 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4729 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 45636 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC7_Rc1;

    /* semantic routine */
#line 4729 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 45646 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4729 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 45668 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 45675 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC7_Rc0(cpu *processor,
                                  instruction_word instruction,
                                  unsigned_word cia,
                                  idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC7_Rc0
#line 3849 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 45694 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 7 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3849 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 45728 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC7_Rc0;

    /* semantic routine */
#line 3849 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 45738 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3849 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 45770 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 45776 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC7_Rc1(cpu *processor,
                                  instruction_word instruction,
                                  unsigned_word cia,
                                  idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC7_Rc1
#line 3849 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 45795 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 7 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3849 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 45829 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC7_Rc1;

    /* semantic routine */
#line 3849 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 45839 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3849 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 45871 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 45877 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC7_Rc0(cpu *processor,
                                          instruction_word instruction,
                                          unsigned_word cia,
                                          idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC7_Rc0
#line 4057 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 45896 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 7 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4057 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 45935 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC7_Rc0;

    /* semantic routine */
#line 4057 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 45945 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4057 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 45996 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 46003 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC7_Rc1(cpu *processor,
                                          instruction_word instruction,
                                          unsigned_word cia,
                                          idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC7_Rc1
#line 4057 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 46022 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 7 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4057 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 46061 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC7_Rc1;

    /* semantic routine */
#line 4057 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 46071 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4057 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 46122 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 46129 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC7_Rc0(cpu *processor,
                                     instruction_word instruction,
                                     unsigned_word cia,
                                     idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC7_Rc0
#line 3967 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 46148 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 7 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3967 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 46187 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC7_Rc0;

    /* semantic routine */
#line 3967 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 46197 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3967 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 46248 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 46255 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC7_Rc1(cpu *processor,
                                     instruction_word instruction,
                                     unsigned_word cia,
                                     idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC7_Rc1
#line 3967 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 46274 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 7 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3967 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 46313 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC7_Rc1;

    /* semantic routine */
#line 3967 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 46323 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3967 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 46374 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 46381 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC7_Rc0(cpu *processor,
                                                   instruction_word instruction,
                                                   unsigned_word cia,
                                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC7_Rc0
#line 4237 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 46400 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 7 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4237 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 46439 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC7_Rc0;

    /* semantic routine */
#line 4237 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 46449 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4237 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 46500 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 46507 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC7_Rc1(cpu *processor,
                                                   instruction_word instruction,
                                                   unsigned_word cia,
                                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC7_Rc1
#line 4237 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 46526 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 7 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4237 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 46565 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC7_Rc1;

    /* semantic routine */
#line 4237 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 46575 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4237 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 46626 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 46633 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC7_Rc0(cpu *processor,
                                              instruction_word instruction,
                                              unsigned_word cia,
                                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC7_Rc0
#line 4147 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 46652 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 7 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4147 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 46691 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC7_Rc0;

    /* semantic routine */
#line 4147 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 46701 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4147 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 46752 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 46759 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC7_Rc1(cpu *processor,
                                              instruction_word instruction,
                                              unsigned_word cia,
                                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC7_Rc1
#line 4147 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 46778 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 7 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4147 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 46817 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC7_Rc1;

    /* semantic routine */
#line 4147 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 46827 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4147 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 46878 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 46885 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Absolute_Value_Rc0(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Absolute_Value
#undef MY_PREFIX
#define MY_PREFIX Floating_Absolute_Value_Rc0
#line 3722 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Absolute Value\n"));
#line 46904 "icache.c"

  /* validate: 0.63,6.FRT,11./,16.FRB,21.264,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc1f07fe) != 0xfc000210)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11./,16.FRB,21.264,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.X.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.X.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.X.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.X.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 3722 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 46933 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Absolute_Value_Rc0;

    /* semantic routine */
#line 3722 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Absolute Value\n"));
#line 46943 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Absolute_Value, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3722 "./ppc-instructions"
    {
      *frT = *frB & ~BIT64(0);
      CR1_UPDATE(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRB_BITMASK, Rc);
    }
#line 46958 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11./,16.FRB,21.264,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRB
#line 46963 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Absolute_Value_Rc1(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Absolute_Value
#undef MY_PREFIX
#define MY_PREFIX Floating_Absolute_Value_Rc1
#line 3722 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Absolute Value\n"));
#line 46982 "icache.c"

  /* validate: 0.63,6.FRT,11./,16.FRB,21.264,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc1f07fe) != 0xfc000210)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11./,16.FRB,21.264,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.X.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.X.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.X.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.X.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 3722 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 47011 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Absolute_Value_Rc1;

    /* semantic routine */
#line 3722 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Absolute Value\n"));
#line 47021 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Absolute_Value, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3722 "./ppc-instructions"
    {
      *frT = *frB & ~BIT64(0);
      CR1_UPDATE(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRB_BITMASK, Rc);
    }
#line 47036 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11./,16.FRB,21.264,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRB
#line 47041 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC8_Rc0(cpu *processor,
                                instruction_word instruction,
                                unsigned_word cia,
                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC8_Rc0
#line 4729 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 47060 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 8 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4729 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 47099 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC8_Rc0;

    /* semantic routine */
#line 4729 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 47109 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4729 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 47131 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 47138 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC8_Rc1(cpu *processor,
                                instruction_word instruction,
                                unsigned_word cia,
                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC8_Rc1
#line 4729 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 47157 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 8 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4729 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 47196 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC8_Rc1;

    /* semantic routine */
#line 4729 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 47206 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4729 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 47228 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 47235 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC8_Rc0(cpu *processor,
                                  instruction_word instruction,
                                  unsigned_word cia,
                                  idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC8_Rc0
#line 3849 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 47254 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 8 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3849 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 47288 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC8_Rc0;

    /* semantic routine */
#line 3849 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 47298 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3849 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 47330 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 47336 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC8_Rc1(cpu *processor,
                                  instruction_word instruction,
                                  unsigned_word cia,
                                  idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC8_Rc1
#line 3849 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 47355 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 8 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3849 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 47389 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC8_Rc1;

    /* semantic routine */
#line 3849 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 47399 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3849 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 47431 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 47437 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC8_Rc0(cpu *processor,
                                          instruction_word instruction,
                                          unsigned_word cia,
                                          idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC8_Rc0
#line 4057 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 47456 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 8 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4057 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 47495 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC8_Rc0;

    /* semantic routine */
#line 4057 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 47505 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4057 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 47556 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 47563 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC8_Rc1(cpu *processor,
                                          instruction_word instruction,
                                          unsigned_word cia,
                                          idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC8_Rc1
#line 4057 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 47582 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 8 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4057 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 47621 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC8_Rc1;

    /* semantic routine */
#line 4057 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 47631 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4057 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 47682 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 47689 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC8_Rc0(cpu *processor,
                                     instruction_word instruction,
                                     unsigned_word cia,
                                     idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC8_Rc0
#line 3967 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 47708 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 8 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3967 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 47747 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC8_Rc0;

    /* semantic routine */
#line 3967 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 47757 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3967 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 47808 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 47815 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC8_Rc1(cpu *processor,
                                     instruction_word instruction,
                                     unsigned_word cia,
                                     idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC8_Rc1
#line 3967 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 47834 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 8 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3967 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 47873 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC8_Rc1;

    /* semantic routine */
#line 3967 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 47883 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3967 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 47934 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 47941 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC8_Rc0(cpu *processor,
                                                   instruction_word instruction,
                                                   unsigned_word cia,
                                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC8_Rc0
#line 4237 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 47960 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 8 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4237 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 47999 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC8_Rc0;

    /* semantic routine */
#line 4237 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 48009 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4237 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 48060 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 48067 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC8_Rc1(cpu *processor,
                                                   instruction_word instruction,
                                                   unsigned_word cia,
                                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC8_Rc1
#line 4237 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 48086 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 8 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4237 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 48125 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC8_Rc1;

    /* semantic routine */
#line 4237 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 48135 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4237 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 48186 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 48193 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC8_Rc0(cpu *processor,
                                              instruction_word instruction,
                                              unsigned_word cia,
                                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC8_Rc0
#line 4147 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 48212 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 8 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4147 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 48251 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC8_Rc0;

    /* semantic routine */
#line 4147 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 48261 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4147 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 48312 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 48319 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC8_Rc1(cpu *processor,
                                              instruction_word instruction,
                                              unsigned_word cia,
                                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC8_Rc1
#line 4147 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 48338 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 8 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4147 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 48377 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC8_Rc1;

    /* semantic routine */
#line 4147 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 48387 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4147 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 48438 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 48445 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC9_Rc0(cpu *processor,
                                instruction_word instruction,
                                unsigned_word cia,
                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC9_Rc0
#line 4729 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 48464 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 9 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4729 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 48503 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC9_Rc0;

    /* semantic routine */
#line 4729 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 48513 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4729 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 48535 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 48542 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC9_Rc1(cpu *processor,
                                instruction_word instruction,
                                unsigned_word cia,
                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC9_Rc1
#line 4729 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 48561 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 9 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4729 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 48600 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC9_Rc1;

    /* semantic routine */
#line 4729 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 48610 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4729 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 48632 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 48639 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC9_Rc0(cpu *processor,
                                  instruction_word instruction,
                                  unsigned_word cia,
                                  idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC9_Rc0
#line 3849 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 48658 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 9 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3849 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 48692 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC9_Rc0;

    /* semantic routine */
#line 3849 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 48702 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3849 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 48734 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 48740 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC9_Rc1(cpu *processor,
                                  instruction_word instruction,
                                  unsigned_word cia,
                                  idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC9_Rc1
#line 3849 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 48759 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 9 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3849 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 48793 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC9_Rc1;

    /* semantic routine */
#line 3849 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 48803 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3849 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 48835 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 48841 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC9_Rc0(cpu *processor,
                                          instruction_word instruction,
                                          unsigned_word cia,
                                          idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC9_Rc0
#line 4057 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 48860 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 9 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4057 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 48899 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC9_Rc0;

    /* semantic routine */
#line 4057 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 48909 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4057 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 48960 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 48967 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC9_Rc1(cpu *processor,
                                          instruction_word instruction,
                                          unsigned_word cia,
                                          idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC9_Rc1
#line 4057 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 48986 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 9 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4057 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 49025 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC9_Rc1;

    /* semantic routine */
#line 4057 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 49035 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4057 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 49086 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 49093 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC9_Rc0(cpu *processor,
                                     instruction_word instruction,
                                     unsigned_word cia,
                                     idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC9_Rc0
#line 3967 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 49112 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 9 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3967 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 49151 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC9_Rc0;

    /* semantic routine */
#line 3967 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 49161 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3967 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 49212 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 49219 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC9_Rc1(cpu *processor,
                                     instruction_word instruction,
                                     unsigned_word cia,
                                     idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC9_Rc1
#line 3967 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 49238 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 9 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3967 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 49277 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC9_Rc1;

    /* semantic routine */
#line 3967 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 49287 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3967 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 49338 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 49345 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC9_Rc0(cpu *processor,
                                                   instruction_word instruction,
                                                   unsigned_word cia,
                                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC9_Rc0
#line 4237 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 49364 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 9 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4237 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 49403 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC9_Rc0;

    /* semantic routine */
#line 4237 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 49413 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4237 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 49464 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 49471 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC9_Rc1(cpu *processor,
                                                   instruction_word instruction,
                                                   unsigned_word cia,
                                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC9_Rc1
#line 4237 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 49490 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 9 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4237 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 49529 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC9_Rc1;

    /* semantic routine */
#line 4237 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 49539 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4237 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 49590 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 49597 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC9_Rc0(cpu *processor,
                                              instruction_word instruction,
                                              unsigned_word cia,
                                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC9_Rc0
#line 4147 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 49616 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 9 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4147 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 49655 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC9_Rc0;

    /* semantic routine */
#line 4147 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 49665 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4147 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 49716 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 49723 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC9_Rc1(cpu *processor,
                                              instruction_word instruction,
                                              unsigned_word cia,
                                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC9_Rc1
#line 4147 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 49742 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 9 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4147 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 49781 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC9_Rc1;

    /* semantic routine */
#line 4147 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 49791 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4147 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 49842 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 49849 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC10_Rc0(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC10_Rc0
#line 4729 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 49868 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 10 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4729 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 49907 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC10_Rc0;

    /* semantic routine */
#line 4729 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 49917 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4729 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 49939 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 49946 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC10_Rc1(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC10_Rc1
#line 4729 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 49965 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 10 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4729 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 50004 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC10_Rc1;

    /* semantic routine */
#line 4729 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 50014 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4729 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 50036 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 50043 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC10_Rc0(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC10_Rc0
#line 3849 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 50062 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 10 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3849 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 50096 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC10_Rc0;

    /* semantic routine */
#line 3849 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 50106 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3849 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 50138 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 50144 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC10_Rc1(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC10_Rc1
#line 3849 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 50163 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 10 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3849 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 50197 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC10_Rc1;

    /* semantic routine */
#line 3849 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 50207 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3849 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 50239 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 50245 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC10_Rc0(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC10_Rc0
#line 4057 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 50264 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 10 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4057 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 50303 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC10_Rc0;

    /* semantic routine */
#line 4057 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 50313 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4057 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 50364 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 50371 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC10_Rc1(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC10_Rc1
#line 4057 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 50390 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 10 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4057 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 50429 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC10_Rc1;

    /* semantic routine */
#line 4057 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 50439 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4057 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 50490 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 50497 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC10_Rc0(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC10_Rc0
#line 3967 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 50516 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 10 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3967 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 50555 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC10_Rc0;

    /* semantic routine */
#line 3967 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 50565 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3967 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 50616 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 50623 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC10_Rc1(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC10_Rc1
#line 3967 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 50642 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 10 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3967 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 50681 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC10_Rc1;

    /* semantic routine */
#line 3967 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 50691 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3967 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 50742 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 50749 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC10_Rc0(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC10_Rc0
#line 4237 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 50768 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 10 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4237 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 50807 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC10_Rc0;

    /* semantic routine */
#line 4237 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 50817 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4237 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 50868 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 50875 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC10_Rc1(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC10_Rc1
#line 4237 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 50894 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 10 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4237 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 50933 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC10_Rc1;

    /* semantic routine */
#line 4237 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 50943 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4237 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 50994 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 51001 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC10_Rc0(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC10_Rc0
#line 4147 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 51020 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 10 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4147 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 51059 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC10_Rc0;

    /* semantic routine */
#line 4147 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 51069 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4147 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 51120 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 51127 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC10_Rc1(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC10_Rc1
#line 4147 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 51146 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 10 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4147 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 51185 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC10_Rc1;

    /* semantic routine */
#line 4147 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 51195 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4147 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 51246 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 51253 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC11_Rc0(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC11_Rc0
#line 4729 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 51272 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 11 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4729 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 51311 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC11_Rc0;

    /* semantic routine */
#line 4729 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 51321 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4729 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 51343 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 51350 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC11_Rc1(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC11_Rc1
#line 4729 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 51369 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 11 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4729 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 51408 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC11_Rc1;

    /* semantic routine */
#line 4729 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 51418 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4729 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 51440 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 51447 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC11_Rc0(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC11_Rc0
#line 3849 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 51466 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 11 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3849 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 51500 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC11_Rc0;

    /* semantic routine */
#line 3849 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 51510 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3849 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 51542 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 51548 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC11_Rc1(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC11_Rc1
#line 3849 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 51567 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 11 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3849 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 51601 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC11_Rc1;

    /* semantic routine */
#line 3849 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 51611 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3849 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 51643 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 51649 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC11_Rc0(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC11_Rc0
#line 4057 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 51668 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 11 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4057 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 51707 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC11_Rc0;

    /* semantic routine */
#line 4057 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 51717 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4057 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 51768 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 51775 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC11_Rc1(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC11_Rc1
#line 4057 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 51794 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 11 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4057 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 51833 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC11_Rc1;

    /* semantic routine */
#line 4057 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 51843 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4057 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 51894 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 51901 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC11_Rc0(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC11_Rc0
#line 3967 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 51920 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 11 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3967 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 51959 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC11_Rc0;

    /* semantic routine */
#line 3967 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 51969 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3967 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 52020 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 52027 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC11_Rc1(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC11_Rc1
#line 3967 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 52046 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 11 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3967 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 52085 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC11_Rc1;

    /* semantic routine */
#line 3967 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 52095 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3967 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 52146 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 52153 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC11_Rc0(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC11_Rc0
#line 4237 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 52172 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 11 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4237 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 52211 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC11_Rc0;

    /* semantic routine */
#line 4237 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 52221 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4237 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 52272 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 52279 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC11_Rc1(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC11_Rc1
#line 4237 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 52298 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 11 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4237 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 52337 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC11_Rc1;

    /* semantic routine */
#line 4237 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 52347 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4237 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 52398 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 52405 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC11_Rc0(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC11_Rc0
#line 4147 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 52424 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 11 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4147 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 52463 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC11_Rc0;

    /* semantic routine */
#line 4147 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 52473 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4147 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 52524 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 52531 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC11_Rc1(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC11_Rc1
#line 4147 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 52550 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 11 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4147 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 52589 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC11_Rc1;

    /* semantic routine */
#line 4147 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 52599 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4147 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 52650 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 52657 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC12_Rc0(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC12_Rc0
#line 4729 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 52676 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 12 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4729 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 52715 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC12_Rc0;

    /* semantic routine */
#line 4729 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 52725 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4729 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 52747 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 52754 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC12_Rc1(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC12_Rc1
#line 4729 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 52773 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 12 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4729 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 52812 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC12_Rc1;

    /* semantic routine */
#line 4729 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 52822 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4729 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 52844 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 52851 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC12_Rc0(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC12_Rc0
#line 3849 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 52870 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 12 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3849 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 52904 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC12_Rc0;

    /* semantic routine */
#line 3849 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 52914 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3849 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 52946 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 52952 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC12_Rc1(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC12_Rc1
#line 3849 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 52971 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 12 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3849 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 53005 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC12_Rc1;

    /* semantic routine */
#line 3849 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 53015 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3849 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 53047 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 53053 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC12_Rc0(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC12_Rc0
#line 4057 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 53072 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 12 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4057 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 53111 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC12_Rc0;

    /* semantic routine */
#line 4057 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 53121 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4057 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 53172 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 53179 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC12_Rc1(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC12_Rc1
#line 4057 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 53198 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 12 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4057 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 53237 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC12_Rc1;

    /* semantic routine */
#line 4057 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 53247 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4057 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 53298 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 53305 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC12_Rc0(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC12_Rc0
#line 3967 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 53324 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 12 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3967 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 53363 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC12_Rc0;

    /* semantic routine */
#line 3967 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 53373 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3967 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 53424 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 53431 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC12_Rc1(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC12_Rc1
#line 3967 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 53450 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 12 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3967 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 53489 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC12_Rc1;

    /* semantic routine */
#line 3967 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 53499 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3967 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 53550 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 53557 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC12_Rc0(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC12_Rc0
#line 4237 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 53576 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 12 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4237 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 53615 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC12_Rc0;

    /* semantic routine */
#line 4237 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 53625 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4237 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 53676 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 53683 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC12_Rc1(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC12_Rc1
#line 4237 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 53702 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 12 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4237 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 53741 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC12_Rc1;

    /* semantic routine */
#line 4237 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 53751 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4237 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 53802 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 53809 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC12_Rc0(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC12_Rc0
#line 4147 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 53828 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 12 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4147 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 53867 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC12_Rc0;

    /* semantic routine */
#line 4147 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 53877 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4147 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 53928 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 53935 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC12_Rc1(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC12_Rc1
#line 4147 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 53954 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 12 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4147 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 53993 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC12_Rc1;

    /* semantic routine */
#line 4147 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 54003 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4147 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 54054 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 54061 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC13_Rc0(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC13_Rc0
#line 4729 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 54080 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 13 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4729 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 54119 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC13_Rc0;

    /* semantic routine */
#line 4729 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 54129 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4729 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 54151 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 54158 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC13_Rc1(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC13_Rc1
#line 4729 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 54177 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 13 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4729 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 54216 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC13_Rc1;

    /* semantic routine */
#line 4729 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 54226 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4729 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 54248 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 54255 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC13_Rc0(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC13_Rc0
#line 3849 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 54274 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 13 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3849 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 54308 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC13_Rc0;

    /* semantic routine */
#line 3849 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 54318 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3849 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 54350 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 54356 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC13_Rc1(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC13_Rc1
#line 3849 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 54375 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 13 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3849 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 54409 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC13_Rc1;

    /* semantic routine */
#line 3849 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 54419 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3849 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 54451 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 54457 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC13_Rc0(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC13_Rc0
#line 4057 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 54476 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 13 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4057 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 54515 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC13_Rc0;

    /* semantic routine */
#line 4057 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 54525 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4057 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 54576 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 54583 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC13_Rc1(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC13_Rc1
#line 4057 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 54602 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 13 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4057 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 54641 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC13_Rc1;

    /* semantic routine */
#line 4057 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 54651 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4057 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 54702 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 54709 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC13_Rc0(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC13_Rc0
#line 3967 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 54728 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 13 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3967 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 54767 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC13_Rc0;

    /* semantic routine */
#line 3967 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 54777 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3967 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 54828 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 54835 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC13_Rc1(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC13_Rc1
#line 3967 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 54854 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 13 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3967 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 54893 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC13_Rc1;

    /* semantic routine */
#line 3967 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 54903 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3967 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 54954 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 54961 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC13_Rc0(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC13_Rc0
#line 4237 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 54980 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 13 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4237 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 55019 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC13_Rc0;

    /* semantic routine */
#line 4237 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 55029 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4237 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 55080 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 55087 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC13_Rc1(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC13_Rc1
#line 4237 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 55106 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 13 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4237 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 55145 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC13_Rc1;

    /* semantic routine */
#line 4237 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 55155 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4237 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 55206 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 55213 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC13_Rc0(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC13_Rc0
#line 4147 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 55232 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 13 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4147 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 55271 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC13_Rc0;

    /* semantic routine */
#line 4147 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 55281 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4147 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 55332 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 55339 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC13_Rc1(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC13_Rc1
#line 4147 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 55358 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 13 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4147 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 55397 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC13_Rc1;

    /* semantic routine */
#line 4147 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 55407 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4147 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 55458 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 55465 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC14_Rc0(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC14_Rc0
#line 4729 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 55484 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 14 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4729 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 55523 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC14_Rc0;

    /* semantic routine */
#line 4729 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 55533 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4729 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 55555 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 55562 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC14_Rc1(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC14_Rc1
#line 4729 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 55581 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 14 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4729 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 55620 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC14_Rc1;

    /* semantic routine */
#line 4729 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 55630 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4729 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 55652 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 55659 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC14_Rc0(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC14_Rc0
#line 3849 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 55678 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 14 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3849 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 55712 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC14_Rc0;

    /* semantic routine */
#line 3849 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 55722 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3849 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 55754 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 55760 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC14_Rc1(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC14_Rc1
#line 3849 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 55779 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 14 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3849 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 55813 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC14_Rc1;

    /* semantic routine */
#line 3849 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 55823 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3849 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 55855 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 55861 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC14_Rc0(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC14_Rc0
#line 4057 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 55880 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 14 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4057 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 55919 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC14_Rc0;

    /* semantic routine */
#line 4057 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 55929 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4057 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 55980 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 55987 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC14_Rc1(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC14_Rc1
#line 4057 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 56006 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 14 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4057 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 56045 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC14_Rc1;

    /* semantic routine */
#line 4057 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 56055 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4057 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 56106 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 56113 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC14_Rc0(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC14_Rc0
#line 3967 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 56132 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 14 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3967 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 56171 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC14_Rc0;

    /* semantic routine */
#line 3967 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 56181 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3967 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 56232 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 56239 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC14_Rc1(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC14_Rc1
#line 3967 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 56258 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 14 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3967 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 56297 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC14_Rc1;

    /* semantic routine */
#line 3967 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 56307 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3967 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 56358 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 56365 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC14_Rc0(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC14_Rc0
#line 4237 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 56384 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 14 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4237 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 56423 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC14_Rc0;

    /* semantic routine */
#line 4237 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 56433 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4237 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 56484 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 56491 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC14_Rc1(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC14_Rc1
#line 4237 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 56510 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 14 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4237 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 56549 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC14_Rc1;

    /* semantic routine */
#line 4237 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 56559 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4237 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 56610 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 56617 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC14_Rc0(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC14_Rc0
#line 4147 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 56636 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 14 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4147 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 56675 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC14_Rc0;

    /* semantic routine */
#line 4147 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 56685 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4147 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 56736 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 56743 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC14_Rc1(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC14_Rc1
#line 4147 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 56762 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 14 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4147 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 56801 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC14_Rc1;

    /* semantic routine */
#line 4147 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 56811 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4147 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 56862 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 56869 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC15_Rc0(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC15_Rc0
#line 4729 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 56888 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 15 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4729 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 56927 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC15_Rc0;

    /* semantic routine */
#line 4729 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 56937 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4729 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 56959 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 56966 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC15_Rc1(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC15_Rc1
#line 4729 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 56985 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 15 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4729 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 57024 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC15_Rc1;

    /* semantic routine */
#line 4729 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 57034 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4729 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 57056 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 57063 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC15_Rc0(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC15_Rc0
#line 3849 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 57082 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 15 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3849 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 57116 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC15_Rc0;

    /* semantic routine */
#line 3849 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 57126 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3849 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 57158 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 57164 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC15_Rc1(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC15_Rc1
#line 3849 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 57183 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 15 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3849 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 57217 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC15_Rc1;

    /* semantic routine */
#line 3849 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 57227 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3849 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 57259 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 57265 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC15_Rc0(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC15_Rc0
#line 4057 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 57284 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 15 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4057 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 57323 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC15_Rc0;

    /* semantic routine */
#line 4057 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 57333 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4057 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 57384 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 57391 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC15_Rc1(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC15_Rc1
#line 4057 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 57410 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 15 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4057 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 57449 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC15_Rc1;

    /* semantic routine */
#line 4057 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 57459 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4057 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 57510 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 57517 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC15_Rc0(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC15_Rc0
#line 3967 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 57536 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 15 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3967 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 57575 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC15_Rc0;

    /* semantic routine */
#line 3967 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 57585 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3967 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 57636 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 57643 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC15_Rc1(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC15_Rc1
#line 3967 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 57662 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 15 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3967 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 57701 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC15_Rc1;

    /* semantic routine */
#line 3967 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 57711 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3967 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 57762 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 57769 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC15_Rc0(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC15_Rc0
#line 4237 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 57788 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 15 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4237 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 57827 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC15_Rc0;

    /* semantic routine */
#line 4237 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 57837 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4237 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 57888 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 57895 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC15_Rc1(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC15_Rc1
#line 4237 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 57914 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 15 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4237 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 57953 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC15_Rc1;

    /* semantic routine */
#line 4237 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 57963 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4237 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 58014 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 58021 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC15_Rc0(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC15_Rc0
#line 4147 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 58040 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 15 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4147 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 58079 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC15_Rc0;

    /* semantic routine */
#line 4147 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 58089 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4147 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 58140 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 58147 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC15_Rc1(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC15_Rc1
#line 4147 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 58166 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 15 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4147 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 58205 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC15_Rc1;

    /* semantic routine */
#line 4147 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 58215 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4147 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 58266 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 58273 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC16_Rc0(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC16_Rc0
#line 4729 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 58292 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 16 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4729 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 58331 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC16_Rc0;

    /* semantic routine */
#line 4729 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 58341 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4729 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 58363 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 58370 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC16_Rc1(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC16_Rc1
#line 4729 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 58389 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 16 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4729 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 58428 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC16_Rc1;

    /* semantic routine */
#line 4729 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 58438 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4729 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 58460 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 58467 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC16_Rc0(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC16_Rc0
#line 3849 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 58486 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 16 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3849 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 58520 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC16_Rc0;

    /* semantic routine */
#line 3849 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 58530 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3849 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 58562 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 58568 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC16_Rc1(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC16_Rc1
#line 3849 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 58587 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 16 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3849 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 58621 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC16_Rc1;

    /* semantic routine */
#line 3849 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 58631 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3849 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 58663 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 58669 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC16_Rc0(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC16_Rc0
#line 4057 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 58688 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 16 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4057 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 58727 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC16_Rc0;

    /* semantic routine */
#line 4057 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 58737 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4057 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 58788 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 58795 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC16_Rc1(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC16_Rc1
#line 4057 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 58814 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 16 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4057 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 58853 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC16_Rc1;

    /* semantic routine */
#line 4057 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 58863 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4057 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 58914 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 58921 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC16_Rc0(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC16_Rc0
#line 3967 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 58940 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 16 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3967 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 58979 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC16_Rc0;

    /* semantic routine */
#line 3967 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 58989 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3967 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 59040 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 59047 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC16_Rc1(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC16_Rc1
#line 3967 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 59066 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 16 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3967 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 59105 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC16_Rc1;

    /* semantic routine */
#line 3967 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 59115 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3967 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 59166 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 59173 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC16_Rc0(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC16_Rc0
#line 4237 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 59192 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 16 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4237 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 59231 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC16_Rc0;

    /* semantic routine */
#line 4237 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 59241 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4237 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 59292 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 59299 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC16_Rc1(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC16_Rc1
#line 4237 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 59318 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 16 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4237 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 59357 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC16_Rc1;

    /* semantic routine */
#line 4237 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 59367 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4237 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 59418 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 59425 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC16_Rc0(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC16_Rc0
#line 4147 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 59444 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 16 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4147 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 59483 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC16_Rc0;

    /* semantic routine */
#line 4147 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 59493 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4147 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 59544 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 59551 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC16_Rc1(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC16_Rc1
#line 4147 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 59570 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 16 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4147 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 59609 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC16_Rc1;

    /* semantic routine */
#line 4147 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 59619 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4147 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 59670 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 59677 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC17_Rc0(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC17_Rc0
#line 4729 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 59696 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 17 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4729 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 59735 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC17_Rc0;

    /* semantic routine */
#line 4729 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 59745 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4729 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 59767 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 59774 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC17_Rc1(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC17_Rc1
#line 4729 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 59793 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 17 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4729 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 59832 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC17_Rc1;

    /* semantic routine */
#line 4729 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 59842 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4729 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 59864 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 59871 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC17_Rc0(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC17_Rc0
#line 3849 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 59890 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 17 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3849 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 59924 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC17_Rc0;

    /* semantic routine */
#line 3849 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 59934 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3849 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 59966 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 59972 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC17_Rc1(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC17_Rc1
#line 3849 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 59991 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 17 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3849 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 60025 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC17_Rc1;

    /* semantic routine */
#line 3849 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 60035 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3849 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 60067 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 60073 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC17_Rc0(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC17_Rc0
#line 4057 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 60092 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 17 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4057 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 60131 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC17_Rc0;

    /* semantic routine */
#line 4057 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 60141 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4057 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 60192 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 60199 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC17_Rc1(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC17_Rc1
#line 4057 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 60218 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 17 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4057 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 60257 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC17_Rc1;

    /* semantic routine */
#line 4057 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 60267 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4057 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 60318 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 60325 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC17_Rc0(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC17_Rc0
#line 3967 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 60344 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 17 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3967 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 60383 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC17_Rc0;

    /* semantic routine */
#line 3967 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 60393 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3967 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 60444 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 60451 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC17_Rc1(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC17_Rc1
#line 3967 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 60470 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 17 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3967 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 60509 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC17_Rc1;

    /* semantic routine */
#line 3967 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 60519 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3967 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 60570 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 60577 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC17_Rc0(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC17_Rc0
#line 4237 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 60596 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 17 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4237 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 60635 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC17_Rc0;

    /* semantic routine */
#line 4237 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 60645 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4237 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 60696 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 60703 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC17_Rc1(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC17_Rc1
#line 4237 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 60722 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 17 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4237 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 60761 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC17_Rc1;

    /* semantic routine */
#line 4237 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 60771 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4237 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 60822 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 60829 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC17_Rc0(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC17_Rc0
#line 4147 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 60848 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 17 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4147 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 60887 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC17_Rc0;

    /* semantic routine */
#line 4147 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 60897 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4147 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 60948 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 60955 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC17_Rc1(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC17_Rc1
#line 4147 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 60974 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 17 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4147 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 61013 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC17_Rc1;

    /* semantic routine */
#line 4147 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 61023 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4147 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 61074 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 61081 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Move_From_FPSCR_Rc0(cpu *processor,
                           instruction_word instruction,
                           unsigned_word cia,
                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Move_From_FPSCR
#undef MY_PREFIX
#define MY_PREFIX Move_From_FPSCR_Rc0
#line 4658 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Move From FPSCR\n"));
#line 61100 "icache.c"

  /* validate: 0.63,6.FRT,11./,16./,21.583,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc1ffffe) != 0xfc00048e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11./,16./,21.583,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.X.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.X.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#line 4658 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 61124 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Move_From_FPSCR_Rc0;

    /* semantic routine */
#line 4658 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Move From FPSCR\n"));
#line 61134 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Move_From_FPSCR, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4658 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      *frT = FPSCR;
      FPSCR_END(Rc);
    }
#line 61149 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11./,16./,21.583,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#line 61153 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Move_From_FPSCR_Rc1(cpu *processor,
                           instruction_word instruction,
                           unsigned_word cia,
                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Move_From_FPSCR
#undef MY_PREFIX
#define MY_PREFIX Move_From_FPSCR_Rc1
#line 4658 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Move From FPSCR\n"));
#line 61172 "icache.c"

  /* validate: 0.63,6.FRT,11./,16./,21.583,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc1ffffe) != 0xfc00048e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11./,16./,21.583,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.X.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.X.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#line 4658 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 61196 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Move_From_FPSCR_Rc1;

    /* semantic routine */
#line 4658 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Move From FPSCR\n"));
#line 61206 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Move_From_FPSCR, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4658 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      *frT = FPSCR;
      FPSCR_END(Rc);
    }
#line 61221 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11./,16./,21.583,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#line 61225 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC18_Rc0(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC18_Rc0
#line 4729 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 61244 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 18 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4729 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 61283 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC18_Rc0;

    /* semantic routine */
#line 4729 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 61293 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4729 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 61315 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 61322 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC18_Rc1(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC18_Rc1
#line 4729 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 61341 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 18 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4729 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 61380 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC18_Rc1;

    /* semantic routine */
#line 4729 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 61390 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4729 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 61412 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 61419 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC18_Rc0(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC18_Rc0
#line 3849 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 61438 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 18 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3849 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 61472 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC18_Rc0;

    /* semantic routine */
#line 3849 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 61482 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3849 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 61514 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 61520 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC18_Rc1(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC18_Rc1
#line 3849 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 61539 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 18 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3849 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 61573 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC18_Rc1;

    /* semantic routine */
#line 3849 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 61583 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3849 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 61615 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 61621 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC18_Rc0(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC18_Rc0
#line 4057 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 61640 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 18 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4057 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 61679 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC18_Rc0;

    /* semantic routine */
#line 4057 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 61689 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4057 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 61740 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 61747 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC18_Rc1(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC18_Rc1
#line 4057 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 61766 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 18 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4057 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 61805 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC18_Rc1;

    /* semantic routine */
#line 4057 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 61815 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4057 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 61866 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 61873 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC18_Rc0(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC18_Rc0
#line 3967 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 61892 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 18 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3967 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 61931 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC18_Rc0;

    /* semantic routine */
#line 3967 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 61941 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3967 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 61992 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 61999 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC18_Rc1(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC18_Rc1
#line 3967 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 62018 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 18 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3967 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 62057 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC18_Rc1;

    /* semantic routine */
#line 3967 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 62067 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3967 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 62118 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 62125 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC18_Rc0(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC18_Rc0
#line 4237 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 62144 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 18 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4237 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 62183 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC18_Rc0;

    /* semantic routine */
#line 4237 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 62193 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4237 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 62244 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 62251 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC18_Rc1(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC18_Rc1
#line 4237 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 62270 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 18 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4237 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 62309 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC18_Rc1;

    /* semantic routine */
#line 4237 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 62319 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4237 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 62370 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 62377 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC18_Rc0(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC18_Rc0
#line 4147 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 62396 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 18 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4147 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 62435 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC18_Rc0;

    /* semantic routine */
#line 4147 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 62445 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4147 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 62496 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 62503 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC18_Rc1(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC18_Rc1
#line 4147 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 62522 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 18 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4147 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 62561 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC18_Rc1;

    /* semantic routine */
#line 4147 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 62571 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4147 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 62622 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 62629 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC19_Rc0(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC19_Rc0
#line 4729 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 62648 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 19 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4729 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 62687 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC19_Rc0;

    /* semantic routine */
#line 4729 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 62697 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4729 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 62719 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 62726 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC19_Rc1(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC19_Rc1
#line 4729 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 62745 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 19 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4729 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 62784 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC19_Rc1;

    /* semantic routine */
#line 4729 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 62794 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4729 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 62816 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 62823 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC19_Rc0(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC19_Rc0
#line 3849 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 62842 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 19 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3849 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 62876 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC19_Rc0;

    /* semantic routine */
#line 3849 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 62886 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3849 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 62918 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 62924 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC19_Rc1(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC19_Rc1
#line 3849 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 62943 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 19 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3849 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 62977 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC19_Rc1;

    /* semantic routine */
#line 3849 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 62987 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3849 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 63019 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 63025 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC19_Rc0(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC19_Rc0
#line 4057 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 63044 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 19 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4057 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 63083 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC19_Rc0;

    /* semantic routine */
#line 4057 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 63093 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4057 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 63144 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 63151 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC19_Rc1(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC19_Rc1
#line 4057 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 63170 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 19 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4057 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 63209 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC19_Rc1;

    /* semantic routine */
#line 4057 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 63219 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4057 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 63270 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 63277 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC19_Rc0(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC19_Rc0
#line 3967 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 63296 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 19 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3967 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 63335 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC19_Rc0;

    /* semantic routine */
#line 3967 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 63345 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3967 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 63396 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 63403 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC19_Rc1(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC19_Rc1
#line 3967 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 63422 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 19 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3967 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 63461 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC19_Rc1;

    /* semantic routine */
#line 3967 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 63471 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3967 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 63522 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 63529 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC19_Rc0(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC19_Rc0
#line 4237 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 63548 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 19 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4237 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 63587 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC19_Rc0;

    /* semantic routine */
#line 4237 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 63597 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4237 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 63648 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 63655 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC19_Rc1(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC19_Rc1
#line 4237 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 63674 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 19 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4237 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 63713 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC19_Rc1;

    /* semantic routine */
#line 4237 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 63723 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4237 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 63774 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 63781 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC19_Rc0(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC19_Rc0
#line 4147 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 63800 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 19 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4147 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 63839 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC19_Rc0;

    /* semantic routine */
#line 4147 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 63849 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4147 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 63900 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 63907 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC19_Rc1(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC19_Rc1
#line 4147 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 63926 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 19 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4147 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 63965 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC19_Rc1;

    /* semantic routine */
#line 4147 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 63975 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4147 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 64026 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 64033 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC20_Rc0(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC20_Rc0
#line 4729 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 64052 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 20 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4729 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 64091 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC20_Rc0;

    /* semantic routine */
#line 4729 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 64101 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4729 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 64123 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 64130 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC20_Rc1(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC20_Rc1
#line 4729 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 64149 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 20 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4729 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 64188 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC20_Rc1;

    /* semantic routine */
#line 4729 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 64198 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4729 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 64220 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 64227 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC20_Rc0(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC20_Rc0
#line 3849 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 64246 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 20 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3849 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 64280 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC20_Rc0;

    /* semantic routine */
#line 3849 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 64290 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3849 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 64322 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 64328 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC20_Rc1(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC20_Rc1
#line 3849 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 64347 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 20 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3849 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 64381 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC20_Rc1;

    /* semantic routine */
#line 3849 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 64391 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3849 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 64423 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 64429 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC20_Rc0(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC20_Rc0
#line 4057 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 64448 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 20 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4057 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 64487 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC20_Rc0;

    /* semantic routine */
#line 4057 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 64497 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4057 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 64548 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 64555 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC20_Rc1(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC20_Rc1
#line 4057 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 64574 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 20 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4057 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 64613 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC20_Rc1;

    /* semantic routine */
#line 4057 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 64623 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4057 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 64674 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 64681 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC20_Rc0(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC20_Rc0
#line 3967 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 64700 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 20 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3967 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 64739 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC20_Rc0;

    /* semantic routine */
#line 3967 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 64749 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3967 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 64800 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 64807 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC20_Rc1(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC20_Rc1
#line 3967 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 64826 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 20 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3967 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 64865 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC20_Rc1;

    /* semantic routine */
#line 3967 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 64875 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3967 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 64926 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 64933 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC20_Rc0(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC20_Rc0
#line 4237 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 64952 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 20 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4237 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 64991 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC20_Rc0;

    /* semantic routine */
#line 4237 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 65001 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4237 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 65052 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 65059 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC20_Rc1(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC20_Rc1
#line 4237 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 65078 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 20 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4237 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 65117 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC20_Rc1;

    /* semantic routine */
#line 4237 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 65127 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4237 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 65178 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 65185 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC20_Rc0(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC20_Rc0
#line 4147 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 65204 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 20 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4147 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 65243 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC20_Rc0;

    /* semantic routine */
#line 4147 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 65253 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4147 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 65304 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 65311 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC20_Rc1(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC20_Rc1
#line 4147 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 65330 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 20 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4147 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 65369 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC20_Rc1;

    /* semantic routine */
#line 4147 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 65379 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4147 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 65430 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 65437 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC21_Rc0(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC21_Rc0
#line 4729 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 65456 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 21 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4729 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 65495 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC21_Rc0;

    /* semantic routine */
#line 4729 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 65505 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4729 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 65527 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 65534 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC21_Rc1(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC21_Rc1
#line 4729 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 65553 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 21 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4729 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 65592 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC21_Rc1;

    /* semantic routine */
#line 4729 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 65602 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4729 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 65624 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 65631 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC21_Rc0(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC21_Rc0
#line 3849 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 65650 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 21 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3849 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 65684 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC21_Rc0;

    /* semantic routine */
#line 3849 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 65694 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3849 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 65726 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 65732 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC21_Rc1(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC21_Rc1
#line 3849 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 65751 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 21 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3849 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 65785 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC21_Rc1;

    /* semantic routine */
#line 3849 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 65795 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3849 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 65827 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 65833 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC21_Rc0(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC21_Rc0
#line 4057 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 65852 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 21 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4057 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 65891 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC21_Rc0;

    /* semantic routine */
#line 4057 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 65901 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4057 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 65952 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 65959 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC21_Rc1(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC21_Rc1
#line 4057 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 65978 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 21 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4057 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 66017 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC21_Rc1;

    /* semantic routine */
#line 4057 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 66027 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4057 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 66078 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 66085 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC21_Rc0(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC21_Rc0
#line 3967 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 66104 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 21 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3967 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 66143 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC21_Rc0;

    /* semantic routine */
#line 3967 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 66153 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3967 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 66204 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 66211 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC21_Rc1(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC21_Rc1
#line 3967 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 66230 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 21 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3967 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 66269 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC21_Rc1;

    /* semantic routine */
#line 3967 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 66279 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3967 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 66330 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 66337 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC21_Rc0(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC21_Rc0
#line 4237 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 66356 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 21 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4237 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 66395 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC21_Rc0;

    /* semantic routine */
#line 4237 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 66405 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4237 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 66456 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 66463 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC21_Rc1(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC21_Rc1
#line 4237 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 66482 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 21 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4237 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 66521 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC21_Rc1;

    /* semantic routine */
#line 4237 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 66531 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4237 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 66582 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 66589 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC21_Rc0(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC21_Rc0
#line 4147 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 66608 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 21 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4147 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 66647 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC21_Rc0;

    /* semantic routine */
#line 4147 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 66657 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4147 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 66708 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 66715 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC21_Rc1(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC21_Rc1
#line 4147 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 66734 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 21 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4147 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 66773 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC21_Rc1;

    /* semantic routine */
#line 4147 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 66783 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4147 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 66834 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 66841 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Move_To_FPSCR_Fields_Rc0(cpu *processor,
                                instruction_word instruction,
                                unsigned_word cia,
                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Move_To_FPSCR_Fields
#undef MY_PREFIX
#define MY_PREFIX Move_To_FPSCR_Fields_Rc0
#line 4675 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Move To FPSCR Fields\n"));
#line 66860 "icache.c"

  /* validate: 0.63,6./,7.FLM,15./,16.FRB,21.711,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfe0107fe) != 0xfc00058e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6./,7.FLM,15./,16.FRB,21.711,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 4675 "./ppc-instructions"
    unsigned const FLM UNUSED = cache_entry->crack.XFL.FLM = EXTRACTED32(instruction, 7, 14); /* default in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.XFL.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.XFL.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 4675 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 66886 "icache.c"

#define insn cache_entry->crack.XFL.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Move_To_FPSCR_Fields_Rc0;

    /* semantic routine */
#line 4675 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Move To FPSCR Fields\n"));
#line 66896 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Move_To_FPSCR_Fields, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4675 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      int i;
      for (i = 0; i < 8; i++) {
        if ((FLM & BIT8(i))) {
          FPSCR &= ~MASK32(i*4, i*4+3);
          FPSCR |= MASKED32(*frB, i*4, i*4+3);
        }
      }
      FPSCR_END(Rc);
    }
#line 66917 "icache.c"
    return nia;
    /* extraction: 0.63,6./,7.FLM,15./,16.FRB,21.711,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRB
#line 66921 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Move_To_FPSCR_Fields_Rc1(cpu *processor,
                                instruction_word instruction,
                                unsigned_word cia,
                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Move_To_FPSCR_Fields
#undef MY_PREFIX
#define MY_PREFIX Move_To_FPSCR_Fields_Rc1
#line 4675 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Move To FPSCR Fields\n"));
#line 66940 "icache.c"

  /* validate: 0.63,6./,7.FLM,15./,16.FRB,21.711,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfe0107fe) != 0xfc00058e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6./,7.FLM,15./,16.FRB,21.711,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 4675 "./ppc-instructions"
    unsigned const FLM UNUSED = cache_entry->crack.XFL.FLM = EXTRACTED32(instruction, 7, 14); /* default in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.XFL.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.XFL.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 4675 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 66966 "icache.c"

#define insn cache_entry->crack.XFL.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Move_To_FPSCR_Fields_Rc1;

    /* semantic routine */
#line 4675 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Move To FPSCR Fields\n"));
#line 66976 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Move_To_FPSCR_Fields, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4675 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      int i;
      for (i = 0; i < 8; i++) {
        if ((FLM & BIT8(i))) {
          FPSCR &= ~MASK32(i*4, i*4+3);
          FPSCR |= MASKED32(*frB, i*4, i*4+3);
        }
      }
      FPSCR_END(Rc);
    }
#line 66997 "icache.c"
    return nia;
    /* extraction: 0.63,6./,7.FLM,15./,16.FRB,21.711,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRB
#line 67001 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC22_Rc0(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC22_Rc0
#line 4729 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 67020 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 22 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4729 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 67059 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC22_Rc0;

    /* semantic routine */
#line 4729 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 67069 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4729 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 67091 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 67098 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC22_Rc1(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC22_Rc1
#line 4729 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 67117 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 22 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4729 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 67156 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC22_Rc1;

    /* semantic routine */
#line 4729 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 67166 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4729 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 67188 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 67195 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC22_Rc0(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC22_Rc0
#line 3849 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 67214 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 22 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3849 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 67248 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC22_Rc0;

    /* semantic routine */
#line 3849 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 67258 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3849 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 67290 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 67296 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC22_Rc1(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC22_Rc1
#line 3849 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 67315 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 22 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3849 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 67349 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC22_Rc1;

    /* semantic routine */
#line 3849 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 67359 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3849 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 67391 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 67397 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC22_Rc0(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC22_Rc0
#line 4057 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 67416 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 22 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4057 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 67455 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC22_Rc0;

    /* semantic routine */
#line 4057 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 67465 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4057 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 67516 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 67523 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC22_Rc1(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC22_Rc1
#line 4057 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 67542 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 22 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4057 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 67581 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC22_Rc1;

    /* semantic routine */
#line 4057 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 67591 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4057 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 67642 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 67649 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC22_Rc0(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC22_Rc0
#line 3967 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 67668 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 22 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3967 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 67707 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC22_Rc0;

    /* semantic routine */
#line 3967 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 67717 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3967 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 67768 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 67775 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC22_Rc1(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC22_Rc1
#line 3967 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 67794 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 22 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3967 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 67833 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC22_Rc1;

    /* semantic routine */
#line 3967 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 67843 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3967 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 67894 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 67901 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC22_Rc0(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC22_Rc0
#line 4237 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 67920 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 22 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4237 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 67959 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC22_Rc0;

    /* semantic routine */
#line 4237 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 67969 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4237 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 68020 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 68027 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC22_Rc1(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC22_Rc1
#line 4237 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 68046 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 22 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4237 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 68085 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC22_Rc1;

    /* semantic routine */
#line 4237 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 68095 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4237 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 68146 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 68153 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC22_Rc0(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC22_Rc0
#line 4147 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 68172 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 22 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4147 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 68211 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC22_Rc0;

    /* semantic routine */
#line 4147 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 68221 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4147 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 68272 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 68279 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC22_Rc1(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC22_Rc1
#line 4147 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 68298 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 22 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4147 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 68337 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC22_Rc1;

    /* semantic routine */
#line 4147 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 68347 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4147 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 68398 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 68405 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC23_Rc0(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC23_Rc0
#line 4729 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 68424 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 23 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4729 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 68463 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC23_Rc0;

    /* semantic routine */
#line 4729 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 68473 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4729 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 68495 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 68502 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC23_Rc1(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC23_Rc1
#line 4729 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 68521 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 23 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4729 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 68560 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC23_Rc1;

    /* semantic routine */
#line 4729 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 68570 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4729 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 68592 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 68599 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC23_Rc0(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC23_Rc0
#line 3849 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 68618 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 23 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3849 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 68652 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC23_Rc0;

    /* semantic routine */
#line 3849 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 68662 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3849 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 68694 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 68700 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC23_Rc1(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC23_Rc1
#line 3849 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 68719 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 23 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3849 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 68753 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC23_Rc1;

    /* semantic routine */
#line 3849 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 68763 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3849 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 68795 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 68801 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC23_Rc0(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC23_Rc0
#line 4057 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 68820 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 23 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4057 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 68859 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC23_Rc0;

    /* semantic routine */
#line 4057 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 68869 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4057 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 68920 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 68927 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC23_Rc1(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC23_Rc1
#line 4057 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 68946 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 23 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4057 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 68985 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC23_Rc1;

    /* semantic routine */
#line 4057 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 68995 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4057 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 69046 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 69053 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC23_Rc0(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC23_Rc0
#line 3967 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 69072 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 23 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3967 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 69111 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC23_Rc0;

    /* semantic routine */
#line 3967 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 69121 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3967 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 69172 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 69179 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC23_Rc1(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC23_Rc1
#line 3967 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 69198 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 23 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3967 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 69237 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC23_Rc1;

    /* semantic routine */
#line 3967 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 69247 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3967 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 69298 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 69305 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC23_Rc0(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC23_Rc0
#line 4237 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 69324 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 23 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4237 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 69363 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC23_Rc0;

    /* semantic routine */
#line 4237 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 69373 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4237 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 69424 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 69431 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC23_Rc1(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC23_Rc1
#line 4237 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 69450 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 23 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4237 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 69489 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC23_Rc1;

    /* semantic routine */
#line 4237 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 69499 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4237 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 69550 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 69557 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC23_Rc0(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC23_Rc0
#line 4147 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 69576 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 23 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4147 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 69615 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC23_Rc0;

    /* semantic routine */
#line 4147 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 69625 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4147 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 69676 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 69683 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC23_Rc1(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC23_Rc1
#line 4147 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 69702 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 23 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4147 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 69741 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC23_Rc1;

    /* semantic routine */
#line 4147 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 69751 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4147 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 69802 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 69809 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC24_Rc0(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC24_Rc0
#line 4729 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 69828 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 24 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4729 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 69867 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC24_Rc0;

    /* semantic routine */
#line 4729 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 69877 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4729 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 69899 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 69906 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC24_Rc1(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC24_Rc1
#line 4729 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 69925 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 24 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4729 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 69964 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC24_Rc1;

    /* semantic routine */
#line 4729 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 69974 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4729 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 69996 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 70003 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC24_Rc0(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC24_Rc0
#line 3849 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 70022 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 24 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3849 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 70056 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC24_Rc0;

    /* semantic routine */
#line 3849 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 70066 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3849 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 70098 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 70104 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC24_Rc1(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC24_Rc1
#line 3849 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 70123 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 24 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3849 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 70157 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC24_Rc1;

    /* semantic routine */
#line 3849 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 70167 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3849 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 70199 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 70205 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC24_Rc0(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC24_Rc0
#line 4057 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 70224 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 24 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4057 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 70263 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC24_Rc0;

    /* semantic routine */
#line 4057 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 70273 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4057 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 70324 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 70331 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC24_Rc1(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC24_Rc1
#line 4057 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 70350 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 24 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4057 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 70389 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC24_Rc1;

    /* semantic routine */
#line 4057 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 70399 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4057 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 70450 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 70457 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC24_Rc0(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC24_Rc0
#line 3967 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 70476 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 24 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3967 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 70515 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC24_Rc0;

    /* semantic routine */
#line 3967 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 70525 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3967 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 70576 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 70583 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC24_Rc1(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC24_Rc1
#line 3967 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 70602 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 24 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3967 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 70641 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC24_Rc1;

    /* semantic routine */
#line 3967 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 70651 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3967 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 70702 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 70709 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC24_Rc0(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC24_Rc0
#line 4237 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 70728 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 24 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4237 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 70767 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC24_Rc0;

    /* semantic routine */
#line 4237 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 70777 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4237 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 70828 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 70835 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC24_Rc1(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC24_Rc1
#line 4237 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 70854 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 24 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4237 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 70893 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC24_Rc1;

    /* semantic routine */
#line 4237 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 70903 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4237 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 70954 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 70961 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC24_Rc0(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC24_Rc0
#line 4147 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 70980 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 24 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4147 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 71019 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC24_Rc0;

    /* semantic routine */
#line 4147 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 71029 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4147 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 71080 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 71087 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC24_Rc1(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC24_Rc1
#line 4147 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 71106 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 24 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4147 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 71145 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC24_Rc1;

    /* semantic routine */
#line 4147 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 71155 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4147 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 71206 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 71213 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC25_Rc0(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC25_Rc0
#line 4729 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 71232 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 25 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4729 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 71271 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC25_Rc0;

    /* semantic routine */
#line 4729 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 71281 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4729 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 71303 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 71310 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC25_Rc1(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC25_Rc1
#line 4729 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 71329 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 25 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4729 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 71368 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC25_Rc1;

    /* semantic routine */
#line 4729 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 71378 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4729 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 71400 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 71407 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC25_Rc0(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC25_Rc0
#line 3849 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 71426 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 25 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3849 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 71460 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC25_Rc0;

    /* semantic routine */
#line 3849 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 71470 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3849 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 71502 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 71508 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC25_Rc1(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC25_Rc1
#line 3849 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 71527 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 25 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3849 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 71561 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC25_Rc1;

    /* semantic routine */
#line 3849 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 71571 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3849 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 71603 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 71609 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC25_Rc0(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC25_Rc0
#line 4057 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 71628 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 25 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4057 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 71667 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC25_Rc0;

    /* semantic routine */
#line 4057 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 71677 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4057 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 71728 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 71735 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC25_Rc1(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC25_Rc1
#line 4057 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 71754 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 25 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4057 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 71793 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC25_Rc1;

    /* semantic routine */
#line 4057 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 71803 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4057 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 71854 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 71861 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC25_Rc0(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC25_Rc0
#line 3967 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 71880 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 25 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3967 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 71919 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC25_Rc0;

    /* semantic routine */
#line 3967 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 71929 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3967 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 71980 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 71987 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC25_Rc1(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC25_Rc1
#line 3967 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 72006 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 25 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3967 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 72045 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC25_Rc1;

    /* semantic routine */
#line 3967 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 72055 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3967 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 72106 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 72113 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC25_Rc0(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC25_Rc0
#line 4237 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 72132 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 25 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4237 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 72171 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC25_Rc0;

    /* semantic routine */
#line 4237 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 72181 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4237 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 72232 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 72239 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC25_Rc1(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC25_Rc1
#line 4237 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 72258 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 25 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4237 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 72297 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC25_Rc1;

    /* semantic routine */
#line 4237 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 72307 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4237 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 72358 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 72365 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC25_Rc0(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC25_Rc0
#line 4147 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 72384 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 25 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4147 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 72423 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC25_Rc0;

    /* semantic routine */
#line 4147 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 72433 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4147 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 72484 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 72491 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC25_Rc1(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC25_Rc1
#line 4147 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 72510 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 25 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4147 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 72549 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC25_Rc1;

    /* semantic routine */
#line 4147 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 72559 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4147 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 72610 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 72617 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC26_Rc0(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC26_Rc0
#line 4729 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 72636 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 26 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4729 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 72675 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC26_Rc0;

    /* semantic routine */
#line 4729 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 72685 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4729 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 72707 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 72714 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC26_Rc1(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC26_Rc1
#line 4729 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 72733 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 26 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4729 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 72772 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC26_Rc1;

    /* semantic routine */
#line 4729 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 72782 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4729 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 72804 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 72811 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC26_Rc0(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC26_Rc0
#line 3849 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 72830 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 26 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3849 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 72864 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC26_Rc0;

    /* semantic routine */
#line 3849 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 72874 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3849 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 72906 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 72912 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC26_Rc1(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC26_Rc1
#line 3849 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 72931 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 26 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3849 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 72965 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC26_Rc1;

    /* semantic routine */
#line 3849 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 72975 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3849 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 73007 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 73013 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC26_Rc0(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC26_Rc0
#line 4057 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 73032 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 26 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4057 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 73071 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC26_Rc0;

    /* semantic routine */
#line 4057 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 73081 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4057 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 73132 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 73139 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC26_Rc1(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC26_Rc1
#line 4057 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 73158 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 26 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4057 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 73197 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC26_Rc1;

    /* semantic routine */
#line 4057 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 73207 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4057 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 73258 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 73265 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC26_Rc0(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC26_Rc0
#line 3967 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 73284 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 26 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3967 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 73323 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC26_Rc0;

    /* semantic routine */
#line 3967 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 73333 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3967 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 73384 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 73391 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC26_Rc1(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC26_Rc1
#line 3967 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 73410 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 26 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3967 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 73449 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC26_Rc1;

    /* semantic routine */
#line 3967 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 73459 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3967 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 73510 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 73517 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC26_Rc0(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC26_Rc0
#line 4237 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 73536 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 26 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4237 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 73575 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC26_Rc0;

    /* semantic routine */
#line 4237 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 73585 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4237 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 73636 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 73643 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC26_Rc1(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC26_Rc1
#line 4237 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 73662 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 26 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4237 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 73701 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC26_Rc1;

    /* semantic routine */
#line 4237 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 73711 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4237 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 73762 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 73769 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC26_Rc0(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC26_Rc0
#line 4147 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 73788 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 26 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4147 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 73827 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC26_Rc0;

    /* semantic routine */
#line 4147 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 73837 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4147 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 73888 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 73895 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC26_Rc1(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC26_Rc1
#line 4147 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 73914 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 26 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4147 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 73953 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC26_Rc1;

    /* semantic routine */
#line 4147 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 73963 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4147 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 74014 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 74021 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC27_Rc0(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC27_Rc0
#line 4729 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 74040 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 27 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4729 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 74079 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC27_Rc0;

    /* semantic routine */
#line 4729 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 74089 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4729 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 74111 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 74118 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC27_Rc1(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC27_Rc1
#line 4729 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 74137 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 27 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4729 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 74176 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC27_Rc1;

    /* semantic routine */
#line 4729 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 74186 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4729 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 74208 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 74215 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC27_Rc0(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC27_Rc0
#line 3849 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 74234 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 27 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3849 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 74268 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC27_Rc0;

    /* semantic routine */
#line 3849 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 74278 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3849 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 74310 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 74316 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC27_Rc1(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC27_Rc1
#line 3849 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 74335 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 27 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3849 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 74369 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC27_Rc1;

    /* semantic routine */
#line 3849 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 74379 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3849 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 74411 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 74417 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC27_Rc0(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC27_Rc0
#line 4057 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 74436 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 27 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4057 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 74475 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC27_Rc0;

    /* semantic routine */
#line 4057 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 74485 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4057 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 74536 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 74543 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC27_Rc1(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC27_Rc1
#line 4057 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 74562 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 27 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4057 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 74601 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC27_Rc1;

    /* semantic routine */
#line 4057 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 74611 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4057 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 74662 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 74669 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC27_Rc0(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC27_Rc0
#line 3967 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 74688 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 27 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3967 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 74727 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC27_Rc0;

    /* semantic routine */
#line 3967 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 74737 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3967 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 74788 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 74795 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC27_Rc1(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC27_Rc1
#line 3967 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 74814 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 27 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3967 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 74853 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC27_Rc1;

    /* semantic routine */
#line 3967 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 74863 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3967 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 74914 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 74921 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC27_Rc0(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC27_Rc0
#line 4237 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 74940 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 27 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4237 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 74979 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC27_Rc0;

    /* semantic routine */
#line 4237 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 74989 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4237 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 75040 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 75047 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC27_Rc1(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC27_Rc1
#line 4237 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 75066 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 27 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4237 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 75105 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC27_Rc1;

    /* semantic routine */
#line 4237 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 75115 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4237 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 75166 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 75173 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC27_Rc0(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC27_Rc0
#line 4147 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 75192 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 27 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4147 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 75231 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC27_Rc0;

    /* semantic routine */
#line 4147 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 75241 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4147 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 75292 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 75299 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC27_Rc1(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC27_Rc1
#line 4147 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 75318 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 27 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4147 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 75357 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC27_Rc1;

    /* semantic routine */
#line 4147 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 75367 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4147 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 75418 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 75425 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC28_Rc0(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC28_Rc0
#line 4729 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 75444 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 28 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4729 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 75483 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC28_Rc0;

    /* semantic routine */
#line 4729 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 75493 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4729 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 75515 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 75522 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC28_Rc1(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC28_Rc1
#line 4729 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 75541 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 28 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4729 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 75580 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC28_Rc1;

    /* semantic routine */
#line 4729 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 75590 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4729 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 75612 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 75619 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC28_Rc0(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC28_Rc0
#line 3849 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 75638 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 28 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3849 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 75672 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC28_Rc0;

    /* semantic routine */
#line 3849 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 75682 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3849 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 75714 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 75720 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC28_Rc1(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC28_Rc1
#line 3849 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 75739 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 28 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3849 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 75773 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC28_Rc1;

    /* semantic routine */
#line 3849 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 75783 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3849 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 75815 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 75821 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC28_Rc0(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC28_Rc0
#line 4057 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 75840 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 28 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4057 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 75879 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC28_Rc0;

    /* semantic routine */
#line 4057 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 75889 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4057 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 75940 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 75947 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC28_Rc1(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC28_Rc1
#line 4057 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 75966 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 28 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4057 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 76005 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC28_Rc1;

    /* semantic routine */
#line 4057 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 76015 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4057 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 76066 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 76073 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC28_Rc0(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC28_Rc0
#line 3967 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 76092 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 28 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3967 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 76131 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC28_Rc0;

    /* semantic routine */
#line 3967 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 76141 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3967 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 76192 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 76199 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC28_Rc1(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC28_Rc1
#line 3967 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 76218 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 28 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3967 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 76257 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC28_Rc1;

    /* semantic routine */
#line 3967 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 76267 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3967 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 76318 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 76325 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC28_Rc0(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC28_Rc0
#line 4237 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 76344 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 28 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4237 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 76383 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC28_Rc0;

    /* semantic routine */
#line 4237 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 76393 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4237 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 76444 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 76451 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC28_Rc1(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC28_Rc1
#line 4237 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 76470 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 28 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4237 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 76509 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC28_Rc1;

    /* semantic routine */
#line 4237 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 76519 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4237 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 76570 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 76577 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC28_Rc0(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC28_Rc0
#line 4147 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 76596 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 28 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4147 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 76635 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC28_Rc0;

    /* semantic routine */
#line 4147 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 76645 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4147 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 76696 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 76703 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC28_Rc1(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC28_Rc1
#line 4147 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 76722 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 28 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4147 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 76761 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC28_Rc1;

    /* semantic routine */
#line 4147 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 76771 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4147 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 76822 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 76829 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC29_Rc0(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC29_Rc0
#line 4729 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 76848 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 29 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4729 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 76887 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC29_Rc0;

    /* semantic routine */
#line 4729 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 76897 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4729 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 76919 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 76926 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC29_Rc1(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC29_Rc1
#line 4729 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 76945 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 29 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4729 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 76984 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC29_Rc1;

    /* semantic routine */
#line 4729 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 76994 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4729 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 77016 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 77023 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC29_Rc0(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC29_Rc0
#line 3849 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 77042 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 29 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3849 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 77076 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC29_Rc0;

    /* semantic routine */
#line 3849 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 77086 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3849 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 77118 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 77124 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC29_Rc1(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC29_Rc1
#line 3849 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 77143 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 29 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3849 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 77177 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC29_Rc1;

    /* semantic routine */
#line 3849 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 77187 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3849 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 77219 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 77225 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC29_Rc0(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC29_Rc0
#line 4057 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 77244 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 29 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4057 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 77283 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC29_Rc0;

    /* semantic routine */
#line 4057 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 77293 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4057 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 77344 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 77351 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC29_Rc1(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC29_Rc1
#line 4057 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 77370 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 29 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4057 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 77409 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC29_Rc1;

    /* semantic routine */
#line 4057 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 77419 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4057 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 77470 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 77477 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC29_Rc0(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC29_Rc0
#line 3967 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 77496 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 29 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3967 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 77535 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC29_Rc0;

    /* semantic routine */
#line 3967 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 77545 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3967 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 77596 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 77603 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC29_Rc1(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC29_Rc1
#line 3967 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 77622 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 29 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3967 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 77661 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC29_Rc1;

    /* semantic routine */
#line 3967 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 77671 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3967 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 77722 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 77729 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC29_Rc0(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC29_Rc0
#line 4237 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 77748 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 29 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4237 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 77787 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC29_Rc0;

    /* semantic routine */
#line 4237 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 77797 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4237 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 77848 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 77855 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC29_Rc1(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC29_Rc1
#line 4237 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 77874 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 29 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4237 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 77913 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC29_Rc1;

    /* semantic routine */
#line 4237 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 77923 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4237 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 77974 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 77981 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC29_Rc0(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC29_Rc0
#line 4147 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 78000 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 29 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4147 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 78039 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC29_Rc0;

    /* semantic routine */
#line 4147 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 78049 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4147 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 78100 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 78107 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC29_Rc1(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC29_Rc1
#line 4147 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 78126 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 29 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4147 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 78165 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC29_Rc1;

    /* semantic routine */
#line 4147 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 78175 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4147 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 78226 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 78233 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC30_Rc0(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC30_Rc0
#line 4729 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 78252 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 30 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4729 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 78291 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC30_Rc0;

    /* semantic routine */
#line 4729 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 78301 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4729 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 78323 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 78330 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC30_Rc1(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC30_Rc1
#line 4729 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 78349 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 30 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4729 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 78388 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC30_Rc1;

    /* semantic routine */
#line 4729 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 78398 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4729 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 78420 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 78427 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC30_Rc0(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC30_Rc0
#line 3849 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 78446 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 30 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3849 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 78480 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC30_Rc0;

    /* semantic routine */
#line 3849 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 78490 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3849 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 78522 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 78528 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC30_Rc1(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC30_Rc1
#line 3849 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 78547 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 30 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3849 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 78581 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC30_Rc1;

    /* semantic routine */
#line 3849 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 78591 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3849 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 78623 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 78629 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC30_Rc0(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC30_Rc0
#line 4057 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 78648 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 30 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4057 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 78687 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC30_Rc0;

    /* semantic routine */
#line 4057 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 78697 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4057 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 78748 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 78755 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC30_Rc1(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC30_Rc1
#line 4057 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 78774 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 30 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4057 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 78813 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC30_Rc1;

    /* semantic routine */
#line 4057 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 78823 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4057 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 78874 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 78881 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC30_Rc0(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC30_Rc0
#line 3967 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 78900 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 30 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3967 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 78939 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC30_Rc0;

    /* semantic routine */
#line 3967 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 78949 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3967 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 79000 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 79007 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC30_Rc1(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC30_Rc1
#line 3967 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 79026 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 30 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3967 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 79065 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC30_Rc1;

    /* semantic routine */
#line 3967 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 79075 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3967 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 79126 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 79133 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC30_Rc0(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC30_Rc0
#line 4237 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 79152 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 30 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4237 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 79191 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC30_Rc0;

    /* semantic routine */
#line 4237 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 79201 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4237 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 79252 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 79259 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC30_Rc1(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC30_Rc1
#line 4237 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 79278 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 30 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4237 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 79317 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC30_Rc1;

    /* semantic routine */
#line 4237 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 79327 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4237 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 79378 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 79385 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC30_Rc0(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC30_Rc0
#line 4147 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 79404 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 30 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4147 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 79443 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC30_Rc0;

    /* semantic routine */
#line 4147 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 79453 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4147 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 79504 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 79511 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC30_Rc1(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC30_Rc1
#line 4147 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 79530 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 30 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4147 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 79569 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC30_Rc1;

    /* semantic routine */
#line 4147 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 79579 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4147 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 79630 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 79637 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC31_Rc0(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC31_Rc0
#line 4729 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 79656 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 31 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4729 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 79695 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC31_Rc0;

    /* semantic routine */
#line 4729 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 79705 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4729 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 79727 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 79734 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC31_Rc1(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC31_Rc1
#line 4729 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 79753 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 31 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4729 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 79792 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC31_Rc1;

    /* semantic routine */
#line 4729 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 79802 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4729 "./ppc-instructions"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        unsigned64 zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 79824 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 79831 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC31_Rc0(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC31_Rc0
#line 3849 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 79850 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 31 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3849 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 79884 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC31_Rc0;

    /* semantic routine */
#line 3849 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 79894 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3849 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 79926 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 79932 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC31_Rc1(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC31_Rc1
#line 3849 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 79951 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 31 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3849 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 79985 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC31_Rc1;

    /* semantic routine */
#line 3849 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 79995 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3849 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 80027 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 80033 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC31_Rc0(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC31_Rc0
#line 4057 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 80052 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 31 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4057 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 80091 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC31_Rc0;

    /* semantic routine */
#line 4057 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 80101 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4057 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 80152 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 80159 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC31_Rc1(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC31_Rc1
#line 4057 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 80178 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 31 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4057 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 80217 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC31_Rc1;

    /* semantic routine */
#line 4057 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 80227 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4057 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 80278 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 80285 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC31_Rc0(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC31_Rc0
#line 3967 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 80304 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 31 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3967 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 80343 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC31_Rc0;

    /* semantic routine */
#line 3967 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 80353 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3967 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 80404 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 80411 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC31_Rc1(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC31_Rc1
#line 3967 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 80430 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 31 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3967 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 80469 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC31_Rc1;

    /* semantic routine */
#line 3967 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 80479 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3967 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 80530 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 80537 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC31_Rc0(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC31_Rc0
#line 4237 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 80556 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 31 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4237 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 80595 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC31_Rc0;

    /* semantic routine */
#line 4237 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 80605 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4237 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 80656 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 80663 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC31_Rc1(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC31_Rc1
#line 4237 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 80682 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 31 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4237 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 80721 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC31_Rc1;

    /* semantic routine */
#line 4237 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 80731 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4237 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 80782 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 80789 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC31_Rc0(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC31_Rc0
#line 4147 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 80808 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 31 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4147 "./ppc-instructions"
    unsigned const Rc UNUSED = 0; /* default in icache - constant field */
#line 80847 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC31_Rc0;

    /* semantic routine */
#line 4147 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 80857 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4147 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 80908 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 80915 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC31_Rc1(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC31_Rc1
#line 4147 "./ppc-instructions"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 80934 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 63 "./ppc-instructions"
    unsigned64 * const frT UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 64 "./ppc-instructions"
    unsigned32 const FRT_BITMASK UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 51 "./ppc-instructions"
    unsigned64 * const frA UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 52 "./ppc-instructions"
    unsigned32 const FRA_BITMASK UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 54 "./ppc-instructions"
    unsigned64 * const frB UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 55 "./ppc-instructions"
    unsigned32 const FRB_BITMASK UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 31 /* icache scratch - constant field */
#line 57 "./ppc-instructions"
    unsigned64 * const frC UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 58 "./ppc-instructions"
    unsigned32 const FRC_BITMASK UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4147 "./ppc-instructions"
    unsigned const Rc UNUSED = 1; /* default in icache - constant field */
#line 80973 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC31_Rc1;

    /* semantic routine */
#line 4147 "./ppc-instructions"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 80983 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4147 "./ppc-instructions"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      (unsigned64*)&product, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 81034 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 81041 "icache.c"

#undef insn
  }
}

#endif /* _ICACHE_C_*/
